
var SHOW_DIV = "block";
var HIDE_DIV = "none";
var SERVLET = "pecs";

dojo.require("dojo.hash");

dojo.ready(function() {

	var hashString = dojo.hash();	
	if (hashString != null && hashString != "") {
		var arrayParam = hashString.split(",");
		dojo.byId("drugA").value = arrayParam[0];
		dojo.byId("drugB").value = arrayParam[1];
		dojo.byId("interaction").value = arrayParam[2];
		dojo.byId("severityLevel").value = arrayParam[3];
		dplProcess();
	}



});

function dplProcess() {
	var vaExportButton = getVaExportButton();
	var fdbExportButton = getFdbExportButton();
	
	if ( valid() ) {
		showVaResults(true);
		showFdbResults(true);
		var criteria = buildCriteria();
		dataGridInit(SERVLET, "vaQueryResults", "VA", "DRUG_PAIR", "single", "drugPairLookup/stagingQuery?json=" + dojo.toJson(criteria), true, null);
		dataGridInit(SERVLET, "fdbQueryResults", "FDB", "DRUG_PAIR", "single", "drugPairLookup/fdbQuery?json=" + dojo.toJson(criteria), true, null);
		dojo.hash(createHashString());
	} else {
		vaExportButton.disabled = true;
		fdbExportButton.disable = true;
		showVaResults(false);
		showFdbResults(false);
		dojo.hash("");
	}
}

function createHashString(){
	var hashString = "";
	var drugA = dojo.byId("drugA");
	var drugB = dojo.byId("drugB");
	var interaction = dojo.byId("interaction");
	var severityLevel = dojo.byId("severityLevel");
	
	hashString = (drugA == null || drugA.value == "") ?  hashString + "," : hashString + drugA.value +",";
	hashString = (drugB == null || drugB.value == "") ? hashString  + "," : hashString + drugB.value + ",";
	hashString = (interaction == null || interaction.value == "") ? hashString + "," : hashString + interaction.value + ",";
	hashString = (severityLevel == null || severityLevel.value == "") ? hashString + "" : hashString + severityLevel.value;

	return hashString;
}


function exportData(recordType) {
	if ( valid() ) {
		var criteria = buildCriteria();
		var url = "/ct/pecs/export/" + recordType + "/DRUG_PAIR?json=" + dojo.toJson(criteria) + "&history=false";
		location.href = url;
	}
}

function getVaExportButton() {
	var button = dojo.byId("vaExport");
	if (button != null) {
		button.disabled = false;
		dojo.connect(button, "onclick", null, function(event) {
			event.preventDefault();
			exportData("VA");
		});
	}
	return button;
}

function getFdbExportButton() {
	var button = dojo.byId("fdbExport");
	if (button != null) {
		button.disabled = false;
		dojo.connect(button, "onclick", null, function(event) {
			event.preventDefault();
			exportData("FDB");
		});
	}
	return button;
}

function valid() {
	var drugA = dojo.byId("drugA");
	var drugB = dojo.byId("drugB");
	var interaction = dojo.byId("interaction");
	var severityLevel = dojo.byId("severityLevel");

	if (( drugA == null || drugA.value == "")
			&& (drugB == null || drugB.value == "")
			&& (interaction == null || interaction.value == "")
			&& (severityLevel == null || severityLevel.value == "")) {
		showErrors(true);
		displayError(findMessage("ScriptAssert.drugPairLookupCriteria"));
		return false;
	}
	showErrors(false);
	return true;
}

function displayError(error) {
	var errorDiv = dojo.byId("errorMessageText");
	errorDiv.innerHTML = "<li>" + error + "</li>";
}

function buildCriteria() {
	var list = new Array();
	if ( dojo.byId("drugA").value != "" ) {
		var drugA = dojo.byId("drugA").value;
		var obj = new Object();
		if ( isNaN(drugA)) {
			obj.field = "RTGENID1_DESC";
			obj.operator = "CONTAINS";
		} else {
			obj.field = "RTGENID1";
			obj.operator = "EQ";
		}
		obj.value = drugA;
		list.push(obj);
		list.push("AND");
	}
	if ( dojo.byId("drugB").value != "" ) {
		var drugB = dojo.byId("drugB").value;
		var obj = new Object();
		if ( isNaN(drugB)) {
			obj.field = "RTGENID2_DESC";
			obj.operator = "CONTAINS";
		} else {
			obj.field = "RTGENID2";
			obj.operator = "EQ";
		}
		obj.value = drugB;
		list.push(obj);
		list.push("AND");
	}
	if ( dojo.byId("interaction").value != "" ) {
		var interaction = dojo.byId("interaction").value;
		var obj = new Object();
		if ( isNaN(interaction)) {
			obj.field="INTERACTIONID_DESC";
			obj.operator="CONTAINS";
		} else {
			obj.field="INTERACTIONID";
			obj.operator="EQ";
		}
		obj.value = interaction;
		list.push(obj);
		list.push("AND");
	}
	if ( dojo.byId("severityLevel").value != "" ) {
		var severityLevel = dojo.byId("severityLevel").value;
		var obj = new Object();
		obj.field="SEVERITYLEVELCODE";
		obj.operator="EQ";
		obj.value=severityLevel;
		list.push(obj);
		list.push("AND");
	}
	return list;
}

function showErrors(show) {
	var errorDiv = dojo.byId("errorMessages");
	if ( show ) {
		errorDiv.style.display = SHOW_DIV;
	} else {
		errorDiv.style.display = HIDE_DIV;
	}
}
function showVaResults(show) {
	var vaResults = dojo.byId("vaResultsDiv");
	if ( show ) {
		vaResults.style.display = SHOW_DIV;
	} else {
		vaResults.style.display = HIDE_DIV;
	}
}

function showFdbResults(show) {
	var fdbResults = dojo.byId("fdbResultsDiv");
	if ( show ) {
		fdbResults.style.display = SHOW_DIV;
	} else {
		fdbResults.style.display = HIDE_DIV;
	}
}

function findMessage(key) {
	var message = "";
    var xhrArgs = {
            url : "/ct/" + SERVLET + "/property/" + key + ".",
            sync : true,
            load : function(data) {
                message = data;
            },
            error : function(error) {
                alert("An unexpected error occurred: " + error.message);
            }
        };

    // Call xhrGet
    var deferred = dojo.xhrGet(xhrArgs);
	if ( message.indexOf("LoginForm") > -1 ) {
		parent.location = "/ct/login/login.jsp";
	}
    
    return message;
}

/**
 * Allows the user to press the Enter key while entering criteria.  The form should not be submitted.
 * @param field
 * @param e
 * @returns {Boolean}
 */
function queryOnEnter(event) {
	var keycode;
	if ( window.event ) {
		keycode = window.event.keyCode;
	} else if ( event ) {
		keycode = event.which;
	} else {
		return false;
	}
	
	if ( keycode == 13 ) {
		dplProcess();
		return false;
	}
	
	return false;
}