var SHOW_DIV = "block";
var HIDE_DIV = "none";

dojo.addOnLoad(function() {
	
	dojo.declare("pecs.QueryType", null, {
		constructor : function() {
			var conceptSelect = dojo.byId("conceptSelection");
			var recordSelect = dojo.byId("recordSelection");

			this.conceptOptions = conceptSelect.options;
			this.recordOptions = recordSelect.options;
			this.conceptType = conceptSelect.options[conceptSelect.selectedIndex].value;
			this.recordType = recordSelect.options[recordSelect.selectedIndex].value;
		}
	});

	var myQueries = dojo.query(".savedQuery");

	myQueries.connect("onclick", function(event) {
		event.preventDefault();
		showSavedQueries(this);
	});

	var path = "/ct/pecs/customization/query/";
	var url = location.pathname;
	var query = location.search;
	var pathArray = url.split("/");

	if (pathArray[4] == "query" && pathArray.length == 7) {

		var conceptSelect = dojo.byId("conceptSelection");
		var recordSelect = dojo.byId("recordSelection");
		for ( var i = 0; i <= conceptSelect.options.length; i++) {
			if (conceptSelect.options[i].value == pathArray[6]) {
				conceptSelect.selectedIndex = i;
				break;
			}
		}
		for ( var x = 0; x <= recordSelect.options.length; x++) {
			if (recordSelect.options[x].value == pathArray[5]) {
				recordSelect.selectedIndex = x;
				break;
			}
		}

		if (query != null) {
			query = query.substring(1);
			var queryArray = query.split("&");
			var history;
			var jsonQuery;
			for ( var i = 0; i < queryArray.length; i++) {
				var params = queryArray[i].split("=");
				if (params[0] == "history") {
					history = params[1];
				}
				if (params[0] == "json") {
					jsonQuery = params[1];
				}
			}
			showQueryBuilder(jsonQuery, history);
			var queryType = new pecs.QueryType();

			sendQuery(queryType, history, jsonQuery);

		}
	}
	dojo.declare("pecs.BlankPath", null, {
		constructor : function() {
			this.DRUG_DRUG_INTERACTION = "/ct/pecs/drugdrug/blank";
			this.DOSE_RANGE = "/ct/pecs/doseRangeCustomization/blank/";
			this.PROFESSIONAL_MONOGRAPH = "/ct/pecs/monograph/professional/blank";
		}});
});

function createQuery(json) {
	var fieldId = "columnSelection-";
	var operatorId = "constraintSelection-";
	var valueId = "columnValue-";
	var conjunctionId = "conjunction-";
	var counter = 0;
	dojo.forEach(json, function(entry, i) {
		if (i % 2 == 0) {
			if (i != 0) {
				cloneForm(counter);
			}
			var field = dojo.byId(fieldId + counter);
			var operator = dojo.byId(operatorId + counter);
			var value = dojo.byId(valueId + counter);

			value.value = entry.value;
			operator.selectedIndex = getOptionIndex(operator.options, entry.operator);
			field.selectedIndex = getOptionIndex(field.options, entry.field);
			counter++;
		}
		
		var conj = dojo.byId(conjunctionId + (counter - 1));
		conj.selectedIndex = getOptionIndex(conj.options, entry);
	});
}

function getOptionIndex(options, value) {
	var index = 0;
	dojo.forEach(options, function(entry, i) {
		if (entry.value == value) {
			index = i;
		}
	});
	return index;
}

function showQueryBuilder(jsonQuery, history) {
	// Look up the node we'll stick the text under.
	var targetNode = dojo.byId("queryBuilder");
	var concept = dojo.byId("conceptSelection");
	var recordType = dojo.byId("recordSelection");

	if (concept.value == "" || recordType.value == "") {
		targetNode.innerHTML = "";
		return;
	}
	// The parameters to pass to xhrGet, the url, how to handle it, and the
	// callbacks.
	var xhrArgs = {
		url : "/ct/pecs/customization/criteria",
		handleAs : "text",
		form : "queryBuilderForm",
		handle: fetchError,
		load : function(data) {
			var basicNode = dojo.place(dojo.trim(data), targetNode, "replace");
			connectSetup();
			
			dojo.style(basicNode, "opacity", "0");
			dojo.style(basicNode, "display", "block");
			var fadeArgs = {
				node : basicNode
			};
			dojo.fadeIn(fadeArgs).play();
			if (jsonQuery != null) {
				var passedQ = unescape(jsonQuery);
				passedQ = dojo.fromJson(passedQ);
				createQuery(passedQ);
				if (dojo.byId("history") != null &&  history == "true") {
					dojo.byId("history").checked = true;
				}
			}
		},
		error : function(error) {
			targetNode.innerHTML = formatAsError("The list of columns set for display could not be loaded from the database.  Resubmit query.  If problem persists, report this error to the support team.");
		}
	};

	// Call the asynchronous xhrGet
	var deferred = dojo.xhrPost(xhrArgs);
	var myQueries = dojo.byId("myQueries");
	showSavedQueries(myQueries);
}

function showSavedQueries(node) {
	// Look up the node we'll stick the text under.
	var targetNode = dojo.byId("savedQueriesList");
	var queryType = pecs.QueryType();
	// The parameters to pass to xhrGet, the url, how to handle it, and the
	// callbacks.
	var xhrArgs = {
		url : node.href + "/" + queryType.recordType + "/" + queryType.conceptType,
		handleAs : "text",
		handle: fetchError,
		sync: true,
		load : function(data) {

			var basicNode = dojo.place(dojo.trim(data), targetNode, "replace");

			dojo.style(basicNode, "opacity", "0");
			dojo.style(basicNode, "display", "block");
			var fadeArgs = {
				node : basicNode
			};
			dojo.fadeIn(fadeArgs).play();

		},
		error : function(error, ioargs) {
			var status = ioargs.xhr.status;
			if ( ioargs.xhr.responseText.indexOf("LoginForm") > -1 ) {
				window.location = "/ct/login/login.jsp";
				return;
			}
		
			targetNode.innerHTML = formatAsError("One or more saved queries were unable to be retrieved at this time.  Resubmit query.  If problem persists, report this error to the support team.");

		}
	};

	// Call the asynchronous xhrGet
	var deferred = dojo.xhrPost(xhrArgs);
}
function saveQuery() {
	// Look up the node we'll stick the text under.
	var targetNode = dojo.byId("customQueryName");
	var errorNode = dojo.byId("customQueryNameError");
	var queryType = pecs.QueryType();
	if ( errorNode != null ) {
		errorNode.innerHTML = "";
	}
	// The parameters to pass to xhrGet, the url, how to handle it, and the
	// callbacks.
	var xhrArgs = {
		url : "/ct/pecs/customization/query/save/" + queryType.recordType + "/" + queryType.conceptType + "?query=" + buildExportJson() + "&queryName=" + targetNode.value,
		handleAs : "text",
		handle: fetchError,
		sync: true,
		load : function(data) {
			// Nothing to do, need to reload the queries later

		},
		error : function(error, ioargs) {
			if ( errorNode != null ) {
				var status = ioargs.xhr.status;
				if (status == 404) {
					return;
				}
				errorNode.innerHTML = formatAsError("This query was not correctly saved and must be rebuilt after it is deleted.  Please report this query to the support team to ensure it is deleted properly.");
			}
		}
	};

	// Call the asynchronous xhrGet
	var deferred = dojo.xhrPost(xhrArgs);
	showSavedQueries(dojo.byId("myQueries"));
}

function loadQuery() {
	var selected = findSelectedQuery();
	if ( selected != null ) {
		clickClear();
		// The parameters to pass to xhrGet, the url, how to handle it, and the
		// callbacks.
		var xhrArgs = {
			url : "/ct/pecs/customization/query/load/" + selected.value,
			handleAs : "json",
			handle: fetchError,
			sync: true,
			load : function(data) {
				if ( data != null ) {
					var passedQ = unescape(data["querySql"]);
					passedQ = dojo.fromJson(passedQ);
					createQuery(passedQ);
				} else {
					alert("The selected query could not be loaded.  Please reselect the query parameters to refresh the saved searches.");
				}
			},
			error : function(error, ioargs) {
				var status = ioargs.xhr.status;
				if (status == 404) {
					return;
				}
				alert("Either a system error occurred or the query operation timed out, and the operation to execute the Others query could not be executed.  Resubmit query.  If problem persists, report this error to the support team.");
			}
		};

		// Call the asynchronous xhrGet
		var deferred = dojo.xhrGet(xhrArgs);
	}
}
function deleteQuery() {
	var selected = findSelectedQuery();
	if ( selected != null ) {
		// The parameters to pass to xhrGet, the url, how to handle it, and the
		// callbacks.
		var xhrArgs = {
			url : "/ct/pecs/customization/query/delete/" + selected.value,
			handleAs : "text",
			handle: fetchError,
			sync: true,
			load : function(data) {
				// Nothing to do, need to reload the queries later
			},
			error : function(error, ioargs) {
				var status = ioargs.xhr.status;
				if (status == 404) {
					return;
				}
				alert("Saved query was not found and is unable to be deleted at this time.  Resubmit query.  If problem persists, report this error to the support team.");

			}
		};
	
		// Call the asynchronous xhrGet
		var deferred = dojo.xhrPost(xhrArgs);
		showSavedQueries(dojo.byId("myQueries"));
	}
}

function findSelectedQuery() {
	var existingQueries = dojo.query(".radio");
	var index = -1;
	dojo.forEach(existingQueries, function(entry, i) {
		if (entry.checked) {
			index = i;
		}
	});
	if ( index != -1 ) {
		return existingQueries[index];
	}
}

function enableSaveQueryButton() {
	var customQueryName = dojo.byId("customQueryName");
	var customQueryNameError = dojo.byId("customQueryNameError");
	var saveQueryButton = dojo.byId("saveQueryBtn");
	var existingQueries = dojo.query(".radio");
	customQueryNameError.innerHTML = "";
	
	if ( buildExportJson() != null 
			&& (customQueryName.value != null && customQueryName.value.length > 3)
			&& existingQueries.length < 10) {
		saveQueryButton.disabled = false;
		return;
	}
	if ( existingQueries.length >= 10 ) {
		dojo.byId("customQueryNameError").innerHTML = "The maximum of 10 saved queries already exist.  Delete a query before attempting to save.";
	}
	saveQueryButton.disabled = true;
};

function connectSetup() {
	addQueryPram();
	querySubmit();
	clearQuery();
	connectDRC();
	
}

function connectDRC(){
	var drc = dojo.byId("addDefaultDRCQuery");
	var json = '[{ "field":"CONCEPTTYPE", "operator":"EQ", "value":"6" },"AND",{ "field":"AGEHIGHINDAYS", "operator":"GE", "value":"6570" }]';
	if (drc!= null){
		dojo.connect(drc, "onclick", null, function(event){
			var oldJson = createJson();
			if (oldJson != "[]"){
				oldJson = oldJson.substring(0, oldJson.length -1);
				oldJson = oldJson + ",\"AND\"," + json.substring(1, json.length);
			}else{
				oldJson = json;
				//console.log(oldJson);
			}
			
			dojo.byId('clearAction').click();
			createQuery(dojo.fromJson(oldJson));
			drc.disabled=true;
		});
	}
}

function addQueryPram() {
	var conj = dojo.byId("conjunction-0");
	dojo.connect(conj, "onchange", null, "addQueryConnect");
}

function addQueryConnect(event) {
	var element = event.target.id;
	var conj = event.target;
	var curIndex = element.charAt(element.indexOf("-") + 1);

	var valueObject = dojo.byId("columnValue-" + curIndex);
	var value = valueObject.value;

	if (value == "") {
		alert("You must enter a value before selecting AND/OR");
		conj.selectedIndex = 0;
		return;
	}

	var nodelist = dojo.query(".control");
	var size = nodelist.length;

	if (conj.options[conj.selectedIndex].value == "") {
		
		var button = dojo.byId("butCtnr");

		nodelist.forEach(function(entry, i) {
			if (i == curIndex) {
				dojo.place(button, entry, "last");
			}
			if (i > curIndex) {
				dojo.destroy(entry);
			}
		});
	} else {
		var num = new Number(curIndex) + 1;
		var next = dojo.byId("columnValue-" + num);
		if (next  != null){
			return
		}
		cloneForm(size);
	}

}

function cloneForm(size) {
	var controls = dojo.byId("controls-" + (size - 1));
	var button = dojo.byId("butCtnr");
	var qControls = dojo.byId("queryControls");
	dojo.place(button, dojo.body(), "last");
	var clone = dojo.clone(controls);
	clone.id = "controls-" + (size);
	var controlsString = clone.innerHTML;
	var patt1 = /-\d/g;

	controlsString = controlsString.replace(patt1, "-" + size);
	clone.innerHTML = controlsString;
	dojo.place(button, clone, "last");
	
	var basicNode = dojo.place(clone, qControls, "last");
	var text = dojo.byId("columnValue-" + size);
	text.value = "";
	var conj = dojo.byId("conjunction-" + size);
	
	dojo.connect(conj, "onchange", null, "addQueryConnect");
}

function clearQuery() {
	var targetNode = dojo.byId("queryCriteria");
	var clear = dojo.byId("clearAction");
	var button = dojo.byId("butCtnr");
	var vaExport = dojo.byId("vaExport");
	var fdbExport = dojo.byId("fdbExport");
	var drc = dojo.byId("addDefaultDRCQuery");
	
	dojo.connect(clear, "onclick", null, function(event) {
		vaExport.disabled = true;
		fdbExport.disabled = true;
		if (dojo.byId("history") != null ) {
			 dojo.byId("history").checked = false;
		}
		var nodelist = dojo.query(".control");
		nodelist.forEach(function(entry, i) {
			if (i == 0){
			dojo.place(button, entry, "last");
			}
			if (i > 0) {
				dojo.destroy(entry);
			}
		});
		targetNode.reset();
		
	if(drc != null){
		drc.disabled=false;
	}	
	});
}

/**
 * A method to simulate a user clicking the Clear button.
 */
function clickClear() {
	var clear = dojo.byId("clearAction");
	if ( clear != null ) {
		clear.click();
	}
}
function showVaResults(show) {
	var vaResults = dojo.byId("vaResultsDiv");
	if (show) {
		vaResults.style.display = SHOW_DIV;
		enableExportButton("vaExport", "VA");
	} else {
		vaResults.style.display = HIDE_DIV;
	}
}

function showFdbResults(show) {
	var fdbResults = dojo.byId("fdbResultsDiv");
	if (show) {
		fdbResults.style.display = SHOW_DIV;
		enableExportButton("fdbExport", "FDB");
	} else {
		fdbResults.style.display = HIDE_DIV;
	}
}
function querySubmit() {
	var addButton = dojo.byId("addAction");
	
	dojo.connect(dojo.byId("queryCriteria"), "onsubmit", null, function(event) {
		event.preventDefault();
		
		var json = createJson();
		if (json == "[]") {
			alert("you must enter query criteria");
			return;
		}

		var queryType = new pecs.QueryType();
		var history = "false";
		if (dojo.byId("history") != null && dojo.byId("history").checked == true) {
			var history = "true";
		}

		var url = location.protocol + "//" + location.hostname + ":" + location.port + "/ct/pecs/customization/query/"
				+ queryType.recordType + "/" + queryType.conceptType + "?json=" + json + "&history=" + history;

		window.location = url;
	});

}
function createJson(){
	var json = "[";
	var string = "";
	var conj = "";
	var nodes = dojo.query(".formControl");
	for(var i = 0; i < nodes.length; i++) {
		element = nodes[i];
		
		if (element.name == "conjunction" && element.options[element.selectedIndex].value != ""
				&& element.disabled == false) {
				conj = ',"' + element.options[element.selectedIndex].value + '",';
				json = json + conj;
				continue;
		}
		// [{"":"DESCRIPTION1","operator":"CONTAINS","value":"anti"}]/[:500]
		// status:500
		if (element.name == "columnSelection") {
			string = string + '{ "field":"' + element.options[element.selectedIndex].value + '"';
			continue;
		}
		if (element.name == "constraintSelection") {
			string = string + ', "operator":"' + element.options[element.selectedIndex].value + '"';
			continue;
		}

		if (element.name == "columnValue") {
			string = string + ', "value":"' + dojo.trim(element.value) + '" }';

			if (element.value != null && element.value != ""){
				json = json + string;
			} else {
				var pos = json.lastIndexOf(conj);
				json = json.substring(0, pos);
				
			}
			string = "";
		}
		
	};
	
	json = json + "]";
	
	return json;
}
function sendQuery(queryType, history, jsonQuery) {
	if (queryType.recordType == "VA" || queryType.recordType == "BOTH") {

		showVaResults(true);
		dataGridInit("pecs", "vaQueryResults", "VA", queryType.conceptType, "single", "customization/query/VA/"
				+ queryType.conceptType + "?history=" + history + "&json=" + jsonQuery, true, "There were no Custom records found that matched your query parameters.");
	}
	if (queryType.recordType == "FDB" || queryType.recordType == "BOTH") {
		showFdbResults(true);
		dataGridInit("pecs", "fdbQueryResults", "FDB", queryType.conceptType, "single", "customization/query/FDB/"
				+ queryType.conceptType + "?history=" + history + "&json=" + jsonQuery, true, "There were no FDB records found that matched your query parameters.");
	}
	if ((queryType.recordType == "BOTH" && queryType.conceptType != "DUPLICATE_THERAPY") 
			&& (queryType.recordType == "BOTH" &&  queryType.conceptType != "DRUG_PAIR")){
		var blankButton = dojo.byId("openBlankForm");
		blankButton.style.display = SHOW_DIV;
		
		var form = dojo.byId("blankForm");
	
		dojo.connect(form, "onclick", function(event){
			var blankPath = new pecs.BlankPath();
			var pathString = blankPath[queryType.conceptType];
			 
			window.location.href = pathString;
		});
	}else {
		var blankButton = dojo.byId("openBlankForm");
		blankButton.style.display = HIDE_DIV;
	}
}
function fetchError(errorData, ioargs) {
	 var status = ioargs.xhr.status;
	// alert(status);
	if (errorData.stack != null && errorData.stack.indexOf("LoginForm") > -1) {
		parent.location = "/ct/login/login.jsp";
	}
}

/**
 * Builds the URL to export the current query results and sends that URL to the
 * server.
 * 
 * @param recordType -
 *            Passed in and not taken from queryType because of querying on
 *            BOTH.
 */
function sendExport(recordType) {
	var queryType = new pecs.QueryType();
	var history = "false";
	var message = "#" + recordType.toLowerCase() + "QueryResults" + " div.dojoxGridMasterMessages";
	var node = dojo.query(message);
	if (node[0].innerHTML == "Your Query Returned No results" || node[0].innerHTML.match("Sorry, an error occurred") != null){
		alert("The export has no results");
		return
	}
	
	if (dojo.byId("history") != null && dojo.byId("history").checked == true) {
		var history = "true";
	}
	var jsonQuery = buildExportJson();
	var url = "/ct/pecs/export/" + recordType + "/" + queryType.conceptType + "?json=" + jsonQuery + "&history="
			+ history;
	location.href = url;
}

function enableExportButton(buttonName, recordType) {
	var button = dojo.byId(buttonName);
	if (button != null) {
		button.disabled = false;
		dojo.connect(button, "onclick", null, function(event) {
			event.preventDefault();
			sendExport(recordType);
		});
	}
}

/*
 * Pulls the query from the URL for exporting purposes.
 */
function buildExportJson() {
	var jsonQuery;
	var query = location.search;
	if (query != null) {
		query = query.substring(1);
		var queryArray = query.split("&");
		for ( var i = 0; i < queryArray.length; i++) {
			var params = queryArray[i].split("=");
			if (params[0] == "json") {
				jsonQuery = params[1];
			}
		}
	}
	return jsonQuery;
}

function formatAsError(errorMsg) {
	return "<span class=\"error\">" + errorMsg + "</span>";
}