--create Dose Range cross reference between PECS Dose Range customizations and FDB originating Dose Range
SELECT 'START : ' || CURRENT_TIMESTAMP AS "FdbDoseRangeXref_DATA.sql" FROM DUAL;

INSERT INTO ctstaging.fdb_dose_range_xref (fdb_reference_id
                                          ,concepttype
                                          ,conceptid
                                          ,agelowindays
                                          ,agehighindays
                                          ,doserouteid
                                          ,dosetypeid
                                          ,fdbdx
                                          ,hittype)
   SELECT ctstaging.fdb_doseroute_ref_id_seq.NEXTVAL
         ,concepttype
         ,conceptid
         ,agelowindays
         ,agehighindays
         ,routecode
         ,typecode
         ,fdbdx
         ,hittype
     FROM (  SELECT DISTINCT custom.concepttype
                            ,custom.conceptid
                            ,custom.agelowindays
                            ,custom.agehighindays
                            ,custom.routecode
                            ,custom.typecode
                            ,custom.fdbdx
                            ,fdb.hittype
               FROM (  SELECT dr.va_dose_range_id
                             ,dr.concepttype
                             ,dr.conceptid
                             ,dr.agelowindays
                             ,dr.agehighindays
                             ,droute.code AS routecode
                             ,dtype.code AS typecode
                             ,dr.fdbdx
                             ,COUNT (*) cntrecs
                         FROM ctstaging.pecs_dose_range_version_va dr
                              INNER JOIN ctstaging.dose_route droute
                                 ON droute.dose_route_id = dr.dose_route_id
                              INNER JOIN ctstaging.dose_type dtype
                                 ON dtype.dose_type_id = dr.dose_type_id
                        WHERE dr.version_id =
                                 (SELECT MIN (version_id)
                                    FROM ctstaging.pecs_dose_range_version_va subdr
                                   WHERE subdr.va_dose_range_id =
                                            dr.va_dose_range_id)
                     GROUP BY dr.va_dose_range_id
                             ,dr.concepttype
                             ,dr.conceptid
                             ,dr.agelowindays
                             ,dr.agehighindays
                             ,droute.code
                             ,dtype.code
                             ,dr.fdbdx) custom
                    INNER JOIN fdb_dif.fdb_dosing fdb
                       ON     custom.agehighindays = fdb.agehighindays
                          AND custom.agelowindays = fdb.agelowindays
                          AND custom.conceptid = fdb.conceptid
                          AND custom.concepttype = fdb.concepttype
                          AND custom.routecode = fdb.doserouteid
                          AND custom.typecode = fdb.dosetypeid
                          AND custom.fdbdx = fdb.fdbdx
              WHERE NOT EXISTS
                           (SELECT *
                              FROM ctstaging.fdb_dose_range_xref xref
                             WHERE     custom.agehighindays =
                                          xref.agehighindays
                                   AND custom.agelowindays = xref.agelowindays
                                   AND custom.conceptid = xref.conceptid
                                   AND custom.concepttype = xref.concepttype
                                   AND custom.routecode = xref.doserouteid
                                   AND custom.typecode = xref.dosetypeid
                                   AND custom.fdbdx = xref.fdbdx
                                   AND fdb.hittype = xref.hittype)
           ORDER BY custom.concepttype
                   ,custom.conceptid
                   ,custom.agelowindays
                   ,custom.agehighindays
                   ,custom.routecode
                   ,custom.typecode
                   ,custom.fdbdx
                   ,fdb.hittype);

COMMIT;

SELECT 'END : ' || CURRENT_TIMESTAMP AS "FdbDoseRangeXref_DATA.sql" FROM DUAL;
