SELECT 'START : ' || CURRENT_TIMESTAMP AS "PECS5_Create_Tables_Only.sql" FROM DUAL;

CREATE TABLE ctstaging.clinical_effect
(
   clinical_effect_id           NUMBER (11) NOT NULL
  ,reverse_clinical_effect_id   NUMBER (11) NOT NULL
  ,code                         VARCHAR2 (50 BYTE) NOT NULL
  ,name                         VARCHAR2 (80 BYTE) NULL
  ,description                  VARCHAR2 (250 BYTE) NULL
  ,sort_order                   NUMBER (6) NULL
  ,isactive                     VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate                DATE NULL
  ,expirationdate               DATE NULL
  ,record_created_by            VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date          DATE NOT NULL
  ,record_modified_by           VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date         DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.comparison_report_concept
(
   comparison_report_concept_id   INTEGER NOT NULL
  ,code                           VARCHAR2 (50 BYTE) NULL
  ,name                           VARCHAR2 (80 BYTE) NULL
  ,description                    VARCHAR2 (250 BYTE) NULL
  ,sort_order                     NUMBER (6) NULL
  ,effectivedate                  DATE NULL
  ,expirationdate                 DATE NULL
  ,record_created_by              VARCHAR2 (30 BYTE) NULL
  ,record_create_date             DATE NULL
  ,record_modified_by             VARCHAR2 (30 BYTE) NULL
  ,record_modified_date           DATE NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.ct_users_va
(
   ct_users_va_id   NUMBER (11) NOT NULL
  ,username         VARCHAR2 (255 BYTE) NOT NULL
  ,created_date     TIMESTAMP (6) NULL
  ,modified_date    TIMESTAMP (6) NULL
  ,user_enabled     VARCHAR2 (1 BYTE) DEFAULT 'Y' NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.ct_version_va
(
   ct_version_key      NUMBER (6) NOT NULL
  ,fdb_versionkey      NUMBER (6) NOT NULL
  ,ct_dbversion        VARCHAR2 (5 BYTE) NULL
  ,ct_buildversion     VARCHAR2 (5 BYTE) NULL
  ,ct_frequency        VARCHAR2 (1 BYTE) NULL
  ,ct_versioncomment   VARCHAR2 (80 BYTE) NULL
  ,ct_dbtype           VARCHAR2 (10 BYTE) NULL
  ,ct_issuedate        TIMESTAMP(6) NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.custom_update_file_va
(
   custom_update_file_va_id   NUMBER (11) NOT NULL
  ,created_date               TIMESTAMP (6) NULL
  ,update_version             VARCHAR2 (32 BYTE) NULL
  ,created_by                 VARCHAR2 (64 BYTE) NULL
  ,zip_file_name              VARCHAR2 (64 BYTE) NULL
  ,zip_file_content           BLOB NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
LOB (zip_file_content) STORE AS lob_custom_update_file_va
   (TABLESPACE ctstaging_data
    ENABLE STORAGE IN ROW
    STORAGE (INITIAL 65536
             NEXT 1048576
             MINEXTENTS 1
             MAXEXTENTS 2147483645
             PCTINCREASE 0
             BUFFER_POOL DEFAULT)
    CHUNK 8192
    PCTVERSION 0
    NOCACHE LOGGING)
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.dose_route
(
   dose_route_id          NUMBER (11) NOT NULL
  ,code                   VARCHAR2 (50 BYTE) NOT NULL
  ,name                   VARCHAR2 (80 BYTE) NULL
  ,description            VARCHAR2 (250 BYTE) NULL
  ,sort_order             NUMBER (6) NULL
  ,isactive               VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate          DATE NULL
  ,expirationdate         DATE NULL
  ,record_created_by      VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date    DATE NOT NULL
  ,record_modified_by     VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.dose_type
(
   dose_type_id           NUMBER (11) NOT NULL
  ,code                   VARCHAR2 (50 BYTE) NOT NULL
  ,name                   VARCHAR2 (80 BYTE) NULL
  ,description            VARCHAR2 (250 BYTE) NULL
  ,sort_order             NUMBER (6) NULL
  ,isactive               VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate          DATE NULL
  ,expirationdate         DATE NULL
  ,record_created_by      VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date    DATE NOT NULL
  ,record_modified_by     VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.dose_unit_type
(
   dose_unit_type_id      NUMBER (11) NOT NULL
  ,code                   VARCHAR2 (50 BYTE) NOT NULL
  ,name                   VARCHAR2 (80 BYTE) NULL
  ,description            VARCHAR2 (250 BYTE) NULL
  ,sort_order             NUMBER (6) NULL
  ,isactive               VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate          DATE NULL
  ,expirationdate         DATE NULL
  ,record_created_by      VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date    DATE NOT NULL
  ,record_modified_by     VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE CTSTAGING.DUPLICATE_ALLOWANCE
(
  DUPLICATE_ALLOWANCE_ID  NUMBER(11)            NOT NULL,
  CODE                    INTEGER               NOT NULL,
  NAME                    VARCHAR2(80 BYTE),
  DESCRIPTION             VARCHAR2(250 BYTE),
  SORT_ORDER              NUMBER(6),
  ISACTIVE                VARCHAR2(1 BYTE)      NOT NULL,
  EFFECTIVEDATE           DATE,
  EXPIRATIONDATE          DATE,
  RECORD_CREATED_BY       VARCHAR2(30 BYTE)     NOT NULL,
  RECORD_CREATED_DATE     DATE                  NOT NULL,
  RECORD_MODIFIED_BY      VARCHAR2(30 BYTE)     NOT NULL,
  RECORD_MODIFIED_DATE    DATE                  NOT NULL
)
TABLESPACE CTSTAGING_DATA;

CREATE TABLE ctstaging.easy_search_select_type
(
   easy_search_select_type_id   NUMBER (11) NOT NULL
  ,code                         VARCHAR2 (50 BYTE) NOT NULL
  ,name                         VARCHAR2 (80 BYTE) NULL
  ,description                  VARCHAR2 (250 BYTE) NULL
  ,sort_order                   NUMBER (6) NULL
  ,isactive                     VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate                DATE NULL
  ,expirationdate               DATE NULL
  ,record_created_by            VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date          DATE NOT NULL
  ,record_modified_by           VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date         DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.fdb_update_report
(
   fdb_update_report_id           INTEGER NOT NULL
  ,fdb_update_date                DATE NULL
  ,content                        BLOB NULL
  ,comparison_report_concept_id   INTEGER NOT NULL
  ,file_name                      VARCHAR2 (100) NOT NULL
)
LOB (
   content)
   STORE AS content_lob_seg
   (
      TABLESPACE lob_data
      NOCACHE LOGGING
      STORAGE (INITIAL 1000 K NEXT 1000 K PCTINCREASE 0)
      CHUNK 8192
      INDEX image_lob_idx ( TABLESPACE lob_index ))
TABLESPACE ctstaging_data;

CREATE TABLE ctstaging.field_metadata
(
   field_id        NUMBER (11) NOT NULL
  ,concept_name    VARCHAR2 (200 BYTE) NOT NULL
  ,field_name      VARCHAR2 (200 BYTE) NOT NULL
  ,display_order   NUMBER (11) NOT NULL
  ,is_query        VARCHAR2 (1 BYTE) DEFAULT 'N' NOT NULL
  ,is_report       VARCHAR2 (1 BYTE) DEFAULT 'N' NOT NULL
  ,is_detail       VARCHAR2 (1 BYTE) DEFAULT 'N' NOT NULL
  ,display_name    VARCHAR2 (50 BYTE) DEFAULT 'N' NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.forward_reverse_monograph
(
   forward_monograph_id   NUMBER (11) NOT NULL
  ,reverse_monograph_id   NUMBER (11) NOT NULL
  ,description            VARCHAR2 (150 BYTE) NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.half_life_unit
(
   half_life_unit_id      NUMBER (11) NOT NULL
  ,code                   VARCHAR2 (50 BYTE) NOT NULL
  ,name                   VARCHAR2 (80 BYTE) NULL
  ,description            VARCHAR2 (250 BYTE) NULL
  ,sort_order             NUMBER (6) NULL
  ,isactive               VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate          DATE NULL
  ,expirationdate         DATE NULL
  ,record_created_by      VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date    DATE NOT NULL
  ,record_modified_by     VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.monograph_section_order_va
(
   sectioncode    VARCHAR2 (1 BYTE) NOT NULL
  ,setion_order   NUMBER (6) NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.pecs_ddi_va
(
   pecs_ddi_va_id          NUMBER (11) NOT NULL
  ,interaction_id          INTEGER NOT NULL
  ,requestor               VARCHAR2 (50 BYTE) NOT NULL
  ,fdb_id                  INTEGER NULL
  ,fdb_reference_id        INTEGER
  ,reverse_fdb_id          NUMBER
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_ddi_version_va
(
   version_id                     NUMBER (11) NOT NULL
  ,pecs_ddi_va_id                 NUMBER (11) NOT NULL
  ,status_id                      NUMBER (11) NOT NULL
  ,severity_level_id              NUMBER (11) NOT NULL
  ,clinical_effect1_id            NUMBER (11) NOT NULL
  ,clinical_effect2_id            NUMBER (11) NULL
  ,description                    VARCHAR2 (60 BYTE) NOT NULL
  ,monograph_id                   INTEGER NULL
  ,edi_number_id                  NUMBER (11) NULL
  ,di_fact_number_id              NUMBER (11) NULL
  ,di_fact_documentation_id       NUMBER (11) NULL
  ,di_fact_onset_id               NUMBER (11) NULL
  ,di_fact_severity_id            NUMBER (11) NULL
  ,micromedex_onset_id            NUMBER (11) NULL
  ,micromedex_severity_id         NUMBER (11) NULL
  ,micromedex_substantiation_id   NUMBER (11) NULL
  ,medline_hit_id                 NUMBER (11) NULL
  ,package_insert_id              NUMBER (11) NULL
  ,pbm_criteria_id                NUMBER (11) NULL
  ,aids_guideline_id              NUMBER (11) NULL
  ,interaction_source_id          NUMBER (11) NULL
  ,interaction_type_id            NUMBER (11) NULL
  ,level_of_evidence_id           NUMBER (11) NULL
  ,edi_text                       VARCHAR2 (4000 BYTE) NULL
  ,di_fact_text                   VARCHAR2 (4000 BYTE) NULL
  ,micromedex_text                VARCHAR2 (4000 BYTE) NULL
  ,medline_text                   VARCHAR2 (4000 BYTE) NULL
  ,package_insert_text            VARCHAR2 (4000 BYTE) NULL
  ,pbm_criteria_text              VARCHAR2 (4000 BYTE) NULL
  ,aids_guideline_text            VARCHAR2 (4000 BYTE) NULL
  ,group_discussion               VARCHAR2 (4000 BYTE) NULL
  ,version_comment                VARCHAR2 (4000 BYTE) NULL
  ,version_date                   TIMESTAMP (6) NULL
  ,version_user                   VARCHAR2 (50 BYTE) NOT NULL
  ,action_by                      VARCHAR2 (50 BYTE) NOT NULL
  ,assigned_user                  VARCHAR2 (50 BYTE) NULL
  ,reference_text                 VARCHAR2 (1024 BYTE) NULL
  ,version_num                    NUMBER (6) NOT NULL
  ,export_date                    TIMESTAMP
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_dose_range_va
(
   va_dose_range_id        NUMBER (11) NOT NULL
  ,requestor               VARCHAR2 (50 BYTE) NOT NULL
  ,fdb_reference_id        INTEGER
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_dose_range_version_va
(
   version_id                   NUMBER (11) NOT NULL
  ,va_dose_range_id             NUMBER (11) NOT NULL
  ,status_id                    NUMBER (11) NOT NULL
  ,reference_text               VARCHAR2 (1024 BYTE) NULL
  ,category                     VARCHAR2 (10 BYTE) NOT NULL
  ,concepttype                  SMALLINT NOT NULL
  ,conceptid                    INTEGER NOT NULL
  ,agelowindays                 INTEGER NOT NULL
  ,agehighindays                INTEGER NOT NULL
  ,dose_route_id                NUMBER (11) NOT NULL
  ,dose_type_id                 NUMBER (11) NOT NULL
  ,fdbdx                        VARCHAR2 (9 BYTE) NOT NULL
  ,doselow                      NUMBER (16, 6) NULL
  ,doselowunits                 VARCHAR2 (30 BYTE) NULL
  ,dosehigh                     NUMBER (16, 6) NULL
  ,dosehighunits                VARCHAR2 (30 BYTE) NULL
  ,doseformlow                  NUMBER (16, 6) NULL
  ,doseformlowunits             VARCHAR2 (30 BYTE) NULL
  ,doseformhigh                 NUMBER (16, 6) NULL
  ,doseformhighunits            VARCHAR2 (30 BYTE) NULL
  ,maxsingledose                NUMBER (16, 6) NULL
  ,maxsingledoseunits           VARCHAR2 (30 BYTE) NULL
  ,maxsingledoseform            NUMBER (16, 6) NULL
  ,maxsingledoseformunits       VARCHAR2 (30 BYTE) NULL
  ,maxdailydose                 NUMBER (16, 6) NULL
  ,maxdailydoseunits            VARCHAR2 (30 BYTE) NULL
  ,maxdailydoseform             NUMBER (16, 6) NULL
  ,maxdailydoseformunits        VARCHAR2 (30 BYTE) NULL
  ,maxlifetimedose              NUMBER (16, 6) NULL
  ,maxlifetimedoseunits         VARCHAR2 (30 BYTE) NULL
  ,maxlifetimedoseform          NUMBER (16, 6) NULL
  ,maxlifetimedoseformunits     VARCHAR2 (30 BYTE) NULL
  ,loweliminationhalflife       NUMBER (16, 6) NULL
  ,higheliminationhalflife      NUMBER (16, 6) NULL
  ,frequencylow                 NUMBER (16, 6) NULL
  ,frequencyhigh                NUMBER (16, 6) NULL
  ,durationlow                  INTEGER NULL
  ,durationhigh                 INTEGER NULL
  ,maxduration                  INTEGER NULL
  ,hepaticimpairmentind         SMALLINT NULL
  ,renalimpairmentind           SMALLINT NULL
  ,crclthreshhold               INTEGER NULL
  ,crclthreshholdunits          VARCHAR2 (30 BYTE) NULL
  ,weightrequiredind            SMALLINT NULL
  ,bsarequiredind               INTEGER NULL
  ,doseratelow                  NUMBER (16, 6) NULL
  ,doseratelowunits             VARCHAR2 (30 BYTE) NULL
  ,doseratehigh                 NUMBER (16, 6) NULL
  ,doseratehighunits            VARCHAR2 (30 BYTE) NULL
  ,doseformratelow              NUMBER (16, 6) NULL
  ,doseformratelowunits         VARCHAR2 (30 BYTE) NULL
  ,doseformratehigh             NUMBER (16, 6) NULL
  ,doseformratehighunits        VARCHAR2 (30 BYTE) NULL
  ,maxsingledoserate            NUMBER (16, 6) NULL
  ,maxsingledoserateunits       VARCHAR2 (30 BYTE) NULL
  ,maxsingledoseformrate        NUMBER (16, 6) NULL
  ,maxsingledoseformrateunits   VARCHAR2 (30 BYTE) NULL
  ,maxdailydoserate             NUMBER (16, 6) NULL
  ,maxdailydoserateunits        VARCHAR2 (30 BYTE) NULL
  ,maxdailydoseformrate         NUMBER (16, 6) NULL
  ,maxdailydoseformrateunits    VARCHAR2 (30 BYTE) NULL
  ,dxid                         VARCHAR2 (8 BYTE) NULL
  ,version_num                  NUMBER (6) NOT NULL
  ,version_comment              VARCHAR2 (4000 BYTE) NULL
  ,version_date                 TIMESTAMP (6) NULL
  ,version_user                 VARCHAR2 (50 BYTE) NOT NULL
  ,action_by                    VARCHAR2 (50 BYTE) NOT NULL
  ,assigned_user                VARCHAR2 (50 BYTE) NULL
  ,ntesingledose                NUMERIC(16,6)
  ,ntesingledoseunits           VARCHAR(30)
  ,ntesingledoseform            NUMERIC(16,6)
  ,ntesingledoseformunits       VARCHAR(30)
  ,export_date                  TIMESTAMP
  ,half_life_unit_id            NUMBER(11,0)
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_drug_pair_va
(
   va_drug_pair_id         NUMBER (11) NOT NULL
  ,routed_generic_id1      INTEGER NOT NULL
  ,routed_generic_id2      INTEGER NOT NULL
  ,pecs_ddi_va_id          NUMBER (11) NOT NULL
  ,requestor               VARCHAR2 (50 BYTE) NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_drug_pair_version_va
(
   version_id        NUMBER (11) NOT NULL
  ,va_drug_pair_id   NUMBER (11) NOT NULL
  ,version_num       NUMBER (6) NOT NULL
  ,version_comment   VARCHAR2 (4000 BYTE) NULL
  ,version_date      TIMESTAMP (6) NULL
  ,version_user      VARCHAR2 (50 BYTE) NOT NULL
  ,action_by         VARCHAR2 (50 BYTE) NOT NULL
  ,status_id         NUMBER (3) NOT NULL
  ,assigned_user     VARCHAR2 (50 BYTE) NULL
  ,reference_text    VARCHAR2 (1024 BYTE) NULL
  ,export_date       TIMESTAMP
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_dupl_therapy_va
(
   va_dupl_therapy_id      NUMBER (11) NOT NULL
  ,dtcid                   INTEGER NOT NULL
  ,category_code           VARCHAR2 (10 BYTE) NOT NULL
  ,requestor               VARCHAR2 (50 BYTE) NOT NULL
  ,fdb_reference_id        INTEGER
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_dupl_therapy_version_va
(
   version_id                 NUMBER (11) NOT NULL
  ,va_dupl_therapy_id         NUMBER (11) NOT NULL
  ,version_num                NUMBER (6) NOT NULL
  ,version_comment            VARCHAR2 (4000 BYTE) NULL
  ,version_date               TIMESTAMP (6) NULL
  ,version_user               VARCHAR2 (50 BYTE) NOT NULL
  ,action_by                  VARCHAR2 (50 BYTE) NOT NULL
  ,status_id                  NUMBER (3) NOT NULL
  ,custom_dup_allowance_ind   INTEGER NOT NULL
  ,custom_string              VARCHAR2 (255 BYTE) NOT NULL
  ,assigned_user              VARCHAR2 (50 BYTE) NULL
  ,reference_text             VARCHAR2 (1024 BYTE) NULL
  ,export_date                TIMESTAMP
  ,custom_dup_allowance_id    NUMBER(11,0) NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_locking
(
   record_id     VARCHAR2 (75 BYTE) NOT NULL
  ,concept       VARCHAR2 (50 BYTE) NOT NULL
  ,record_type   VARCHAR2 (3 BYTE) NOT NULL
  ,user_name     VARCHAR2 (50 BYTE) NOT NULL
  ,date_locked   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.pecs_monograph_section_line_va
(
   line_id        NUMBER (11) NOT NULL
  ,section_code   VARCHAR2 (1 BYTE) NOT NULL
  ,format_code    VARCHAR2 (1 BYTE) NOT NULL
  ,line_text      VARCHAR2 (4000 BYTE) NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_monograph_va
(
   va_monograph_id         INTEGER NOT NULL
  ,monograph_id            INTEGER NOT NULL
  ,language_id             INTEGER NOT NULL
  ,category_code           VARCHAR2 (10 BYTE) NOT NULL
  ,requestor               VARCHAR2 (50 BYTE) NOT NULL
  ,fdb_reference_id        INTEGER
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_monograph_version_line_va
(
   version_id        NUMBER (11) NOT NULL
  ,line_id           NUMBER (11) NOT NULL
  ,sequence_number   SMALLINT NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_monograph_version_va
(
   version_id        NUMBER (11) NOT NULL
  ,va_monograph_id   INTEGER NOT NULL
  ,version_num       NUMBER (6) NOT NULL
  ,version_comment   VARCHAR2 (4000 BYTE) NULL
  ,version_date      TIMESTAMP (6) NULL
  ,version_user      VARCHAR2 (50 BYTE) NOT NULL
  ,action_by         VARCHAR2 (50 BYTE) NOT NULL
  ,status_id         NUMBER (3) NOT NULL
  ,assigned_user     VARCHAR2 (50 BYTE) NULL
  ,reference_text    VARCHAR2 (1024 BYTE) NULL
  ,title             VARCHAR2 (4000 BYTE) NOT NULL
  ,export_date       TIMESTAMP
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.pecs_status
(
   status_id     NUMBER (3) NOT NULL
  ,code          VARCHAR2 (50 BYTE) NOT NULL
  ,name          VARCHAR2 (50 BYTE) NOT NULL
  ,description   VARCHAR2 (255 BYTE) NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.period_of_time
(
   period_of_time_id      NUMBER (11) NOT NULL
  ,code                   VARCHAR2 (50 BYTE) NOT NULL
  ,name                   VARCHAR2 (80 BYTE) NULL
  ,description            VARCHAR2 (250 BYTE) NULL
  ,sort_order             NUMBER (6) NULL
  ,isactive               VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate          DATE NULL
  ,expirationdate         DATE NULL
  ,record_created_by      VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date    DATE NOT NULL
  ,record_modified_by     VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

CREATE TABLE ctstaging.reference_text_va
(
   reference_text_va_id         NUMBER (11) NOT NULL
  ,reference_text_type          VARCHAR2 (32 BYTE) NOT NULL
  ,reference_text_name          VARCHAR2 (64 BYTE) NULL
  ,reference_text_description   VARCHAR2 (128 BYTE) NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.saved_query_va
(
   saved_query_va_id   NUMBER (11) NOT NULL
  ,saved_query_name    VARCHAR2 (64 BYTE) NULL
  ,created_date        TIMESTAMP (6) NOT NULL
  ,created_by          VARCHAR2 (64 BYTE) NOT NULL
  ,query_table_name    VARCHAR2 (64 BYTE) NOT NULL
  ,query_sql           VARCHAR2 (1024 BYTE) NULL
  ,query_is_enabled    VARCHAR2 (1 BYTE) NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;


CREATE TABLE ctstaging.severity_level
(
   severity_level_id      NUMBER (11) NOT NULL
  ,code                   VARCHAR2 (50 BYTE) NOT NULL
  ,name                   VARCHAR2 (80 BYTE) NULL
  ,description            VARCHAR2 (250 BYTE) NULL
  ,sort_order             NUMBER (6) NULL
  ,isactive               VARCHAR2 (1 BYTE) NOT NULL
  ,effectivedate          DATE NULL
  ,expirationdate         DATE NULL
  ,record_created_by      VARCHAR2 (30 BYTE) NOT NULL
  ,record_created_date    DATE NOT NULL
  ,record_modified_by     VARCHAR2 (30 BYTE) NOT NULL
  ,record_modified_date   DATE NOT NULL
)
TABLESPACE ctstaging_data
LOGGING
PCTFREE 10
INITRANS 1
STORAGE (INITIAL 64 K
         NEXT 1024 K
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         BUFFER_POOL DEFAULT)
NOCOMPRESS
NOCACHE
NOPARALLEL
NOROWDEPENDENCIES
DISABLE ROW MOVEMENT;

-- CCR 5514 - Null Drug Pair Removal Process
--
-- DESCRIPTION:
-- Create copy of the FDB_GENERIC_ROUTEDDRUG table to support display
-- of deleted routed drug description.

--DROP TABLE CTSTAGING.GENERIC_ROUTEDDRUG CASCADE CONSTRAINTS;

CREATE TABLE ctstaging.generic_routeddrug
(
   rtgenid                 INTEGER NOT NULL
  ,rtgenidstring           VARCHAR (8)
  ,description1            VARCHAR (155)
  ,description2            VARCHAR (255)
  ,description3            VARCHAR (255)
  ,description4            VARCHAR (255)
  ,description5            VARCHAR (255)
  ,description6            VARCHAR (255)
  ,rtid                    INTEGER
  ,hicl                    INTEGER
  ,gcnseqno_unique         INTEGER
  ,rtdfgenid_unique        INTEGER
  ,singleingredientind     SMALLINT
  ,medicaldeviceind        SMALLINT
  ,regionalind             SMALLINT
  ,rxotccode               VARCHAR (1)
  ,statusddcm              SMALLINT
  ,statusddim              SMALLINT
  ,statusdfim              SMALLINT
  ,statusdt                SMALLINT
  ,statusprecgeri          SMALLINT
  ,statuspreclact          SMALLINT
  ,statusprecpedi          SMALLINT
  ,statusprecpreg          SMALLINT
  ,statusside              SMALLINT
  ,statusdrc               SMALLINT
  ,singledoserouteind      SMALLINT
  ,canbedosedind           SMALLINT
  ,haspackageddrugsind     SMALLINT
  ,neocanbedosedind        SMALLINT
  ,neosingledoserouteind   SMALLINT
);

-- Table CTSTAGING.DDIMINTERACTION_COMMENT

CREATE TABLE ctstaging.ddiminteraction_comment
(
   comment_id         INTEGER NOT NULL
  ,fdb_reference_id   INTEGER
  ,comment_text       VARCHAR2 (4000 BYTE)
  ,comment_date       TIMESTAMP (6)
  ,comment_user       VARCHAR2 (50 BYTE) NOT NULL
)
TABLESPACE ctstaging_data;

-- Table CTSTAGING.FDB_DDIM_XREF

CREATE TABLE ctstaging.fdb_ddim_xref
(
   fdb_reference_id   INTEGER NOT NULL
  ,interaction_id     INTEGER
)
TABLESPACE ctstaging_data;

COMMENT ON COLUMN ctstaging.fdb_ddim_xref.interaction_id IS
   'Interaction ID from FDB_DDIMINTERATION table ';


-- Table CTSTAGING.FDB_DOSE_RANGE_XREF

CREATE TABLE ctstaging.fdb_dose_range_xref
(
   fdb_reference_id   INTEGER NOT NULL
  ,concepttype        INTEGER
  ,conceptid          INTEGER
  ,agelowindays       INTEGER
  ,agehighindays      INTEGER
  ,doserouteid        VARCHAR2 (3 BYTE)
  ,dosetypeid         VARCHAR2 (2 BYTE)
  ,fdbdx              VARCHAR2 (9 BYTE)
  ,hittype            INTEGER
)
TABLESPACE ctstaging_data;

-- Table CTSTAGING.DOSERANGE_COMMENT

CREATE TABLE ctstaging.doserange_comment
(
   comment_id         INTEGER NOT NULL
  ,fdb_reference_id   INTEGER
  ,comment_text       VARCHAR2 (4000 BYTE)
  ,comment_date       TIMESTAMP (6)
  ,comment_user       VARCHAR2 (50 BYTE) NOT NULL
)
TABLESPACE ctstaging_data;

-- Table CTSTAGING.FDB_DUPTHERAPY_XREF

CREATE TABLE ctstaging.fdb_duptherapy_xref
(
   fdb_reference_id   INTEGER NOT NULL
  ,dtcid              INTEGER
)
TABLESPACE ctstaging_data;

-- Table CTSTAGING.DUPLTHERAPY_COMMENT

CREATE TABLE ctstaging.dupltherapy_comment
(
   comment_id         INTEGER NOT NULL
  ,fdb_reference_id   INTEGER
  ,comment_text       VARCHAR2 (4000 BYTE)
  ,comment_date       TIMESTAMP (6)
  ,comment_user       VARCHAR2 (50 BYTE) NOT NULL
)
TABLESPACE ctstaging_data;

-- Table CTSTAGING.FDB_MONOGRAPH_XREF

CREATE TABLE ctstaging.fdb_monograph_xref
(
   fdb_reference_id   INTEGER NOT NULL
  ,languageid         INTEGER
  ,versioncode        VARCHAR2 (10)
  ,monographid        INTEGER
  ,sequencenumber     INTEGER
)
TABLESPACE ctstaging_data;

-- Table CTSTAGING.MONOGRAPH_COMMENT

CREATE TABLE ctstaging.monograph_comment
(
   comment_id         INTEGER NOT NULL
  ,fdb_reference_id   INTEGER
  ,comment_text       VARCHAR2 (4000 BYTE)
  ,comment_date       TIMESTAMP (6)
  ,comment_user       VARCHAR2 (50 BYTE) NOT NULL
)
TABLESPACE ctstaging_data;

CREATE TABLE ctstaging.pecs_contact
(
   pecs_contact_id        NUMBER (11) NOT NULL
  ,created_date           TIMESTAMP (6)
  ,created_by             VARCHAR2 (64 BYTE)
  ,pecs_contact_content   BLOB
)
LOB (pecs_contact_content) STORE AS
   (TABLESPACE ctstaging_data
    ENABLE STORAGE IN ROW
    CHUNK 8192
    RETENTION
    NOCACHE LOGGING
    STORAGE (INITIAL 64 K
             NEXT 1 M
             MINEXTENTS 1
             MAXEXTENTS UNLIMITED
             PCTINCREASE 0
             BUFFER_POOL DEFAULT
             FLASH_CACHE DEFAULT
             CELL_FLASH_CACHE DEFAULT))
TABLESPACE ctstaging_data
RESULT_CACHE (MODE DEFAULT)
PCTUSED 0
PCTFREE 10
INITRANS 1
MAXTRANS 255
STORAGE (INITIAL 64 K
         NEXT 1 M
         MINEXTENTS 1
         MAXEXTENTS UNLIMITED
         PCTINCREASE 0
         BUFFER_POOL DEFAULT
         FLASH_CACHE DEFAULT
         CELL_FLASH_CACHE DEFAULT)
LOGGING
NOCOMPRESS
NOCACHE
NOPARALLEL
MONITORING;

CREATE TABLE ctstaging.pecs_ddi_export_va
(
   pecs_ddi_va_id          NUMBER (11) NOT NULL
  ,exportable_version_id   NUMBER (11)
  ,exported_version_id     NUMBER (11)
  ,active_version_id       NUMBER (11) NOT NULL
)
TABLESPACE ctstaging_data;

CREATE TABLE ctstaging.pecs_drug_pair_export_va
(
   va_drug_pair_id         NUMBER (11) NOT NULL
  ,exportable_version_id   NUMBER (11)
  ,exported_version_id     NUMBER (11)
  ,active_version_id       NUMBER (11) NOT NULL
)
TABLESPACE ctstaging_data;

CREATE TABLE ctstaging.pecs_dose_range_export_va
(
   va_dose_range_id        NUMBER (11) NOT NULL
  ,exportable_version_id   NUMBER (11)
  ,exported_version_id     NUMBER (11)
  ,active_version_id       NUMBER (11) NOT NULL
)
TABLESPACE ctstaging_data;

CREATE TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA
(
  VA_DUPL_THERAPY_ID  NUMBER(11)             NOT NULL,
  EXPORTABLE_VERSION_ID  NUMBER(11),
  EXPORTED_VERSION_ID    NUMBER(11),
  ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL
)
TABLESPACE CTSTAGING_DATA;

CREATE TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA
(
  VA_MONOGRAPH_ID  NUMBER(11)             NOT NULL,
  EXPORTABLE_VERSION_ID  NUMBER(11),
  EXPORTED_VERSION_ID    NUMBER(11),
  ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL
)
TABLESPACE CTSTAGING_DATA;



SELECT 'END : ' || CURRENT_TIMESTAMP AS "PECS5_Create_Tables_Only.sql" FROM DUAL;