--
-- Header: PECS5_create_user_modified.sql 04-4-2012
--
-- Created By: Alicia Ray
--
-- FILE NAME: PECS5_create_user_modified.sql
--
-- DESCRIPTION:
--  This script creates the PECS application database user, and
--  grants permissions.
--
--
-- NOTES:
--
--
-- MODIFIED	 (MM-DD-YYYY)	  DESCRIPTION
---------------------------------------
-- Ray		  04-04-2012	  Initial version for PECS 2.2
-- Shelley    10-21-2013     Updated for PECS 3.3
-- Shelley    12-05-2013     Updated for PECS 5.0
--
SELECT 'START : ' || CURRENT_TIMESTAMP AS "PECS5_create_user_modified.sql" FROM DUAL;

DROP USER ctstaging CASCADE;

CREATE USER ctstaging
  IDENTIFIED BY ctstaging_123
  DEFAULT TABLESPACE ctstaging_data
  TEMPORARY TABLESPACE temp
  PROFILE service_account
  ACCOUNT UNLOCK;
-- Roles for CTSTAGING
GRANT CONNECT TO ctstaging;
GRANT RESOURCE TO ctstaging;
GRANT SELECT_CATALOG_ROLE TO ctstaging;
ALTER USER ctstaging DEFAULT ROLE CONNECT, RESOURCE, select_catalog_role;
-- 12 System Privileges for CTSTAGING
GRANT CREATE PROCEDURE TO ctstaging;
GRANT CREATE SYNONYM TO ctstaging;
GRANT DROP PUBLIC SYNONYM TO ctstaging;
GRANT CREATE SESSION TO ctstaging;
GRANT CREATE SEQUENCE TO ctstaging;
GRANT CREATE ANY VIEW TO ctstaging;
GRANT CREATE PUBLIC SYNONYM TO ctstaging;
GRANT CREATE TABLE TO ctstaging;
GRANT UNLIMITED TABLESPACE TO ctstaging;
GRANT EXPORT FULL DATABASE TO ctstaging;
GRANT CREATE TRIGGER TO ctstaging;
GRANT DROP ANY VIEW TO ctstaging;
-- 2 Tablespace Quotas for CTSTAGING
ALTER USER ctstaging QUOTA UNLIMITED ON ctstaging_data;
ALTER USER ctstaging QUOTA UNLIMITED ON ctstaging_index;
ALTER USER ctstaging QUOTA UNLIMITED ON lob_data;
ALTER USER ctstaging QUOTA UNLIMITED ON lob_index;

SET TERM      OFF
SET ECHO      OFF
SET HEADING   OFF
SET FEEDBACK  OFF
SET PAGESIZE  0
SET VERIFY    OFF
SET LINESIZE  250

SPOOL fdb_dif5ctstaging.sql

SELECT 'GRANT SELECT ON FDB_DIF.' || table_name || ' TO  CTSTAGING; '
  FROM all_tables
 WHERE owner = 'FDB_DIF';

SPOOL OFF

SET TERM ON
SET ECHO ON
SET HEADING ON
SET FEEDBACK ON
SET PAGESIZE 9999
SET VERIFY ON



--spool fdb_dif3ctstaging.log
SPOOL PECS5_creation_pkg1.log APPEND

START  fdb_dif5ctstaging.sql

-- Grant FDB_DIF level privileges

REVOKE SELECT ON fdb_dif.fdb_monograph_ddim FROM ctstaging;
GRANT SELECT ON fdb_dif.fdb_monograph_ddim TO ctstaging WITH GRANT OPTION;

SELECT 'END : ' || CURRENT_TIMESTAMP AS "PECS5_create_user_modified.sql" FROM DUAL;
