ALTER TABLE CTSTAGING.DUPLICATE_ALLOWANCE
 DROP PRIMARY KEY CASCADE;

DROP TABLE CTSTAGING.DUPLICATE_ALLOWANCE CASCADE CONSTRAINTS;

CREATE TABLE CTSTAGING.DUPLICATE_ALLOWANCE
(
  DUPLICATE_ALLOWANCE_ID  NUMBER(11)            NOT NULL,
  CODE                    INTEGER               NOT NULL,
  NAME                    VARCHAR2(80 BYTE),
  DESCRIPTION             VARCHAR2(250 BYTE),
  SORT_ORDER              NUMBER(6),
  ISACTIVE                VARCHAR2(1 BYTE)      NOT NULL,
  EFFECTIVEDATE           DATE,
  EXPIRATIONDATE          DATE,
  RECORD_CREATED_BY       VARCHAR2(30 BYTE)     NOT NULL,
  RECORD_CREATED_DATE     DATE                  NOT NULL,
  RECORD_MODIFIED_BY      VARCHAR2(30 BYTE)     NOT NULL,
  RECORD_MODIFIED_DATE    DATE                  NOT NULL
)
TABLESPACE CTSTAGING_DATA
PCTUSED    0
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX CTSTAGING.XPKDUPLICATE_ALLOWANCE ON CTSTAGING.DUPLICATE_ALLOWANCE
(DUPLICATE_ALLOWANCE_ID)
LOGGING
TABLESPACE CTSTAGING_INDEX
PCTFREE    10
INITRANS   2
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
NOPARALLEL;


CREATE OR REPLACE TRIGGER CTSTAGING.DUPLICATEALLOWANCE_TRIGGER 
BEFORE INSERT ON CTSTAGING.DUPLICATE_ALLOWANCE
REFERENCING NEW AS NEW
FOR EACH ROW
BEGIN
  IF (NVL(:NEW.DUPLICATE_ALLOWANCE_ID, 0) < 1) THEN
    SELECT REFTABLE_SEQ.nextval INTO :NEW.DUPLICATE_ALLOWANCE_ID FROM dual;
  END IF;
END;
/


CREATE OR REPLACE PUBLIC SYNONYM DUPLICATE_ALLOWANCE FOR CTSTAGING.DUPLICATE_ALLOWANCE;


ALTER TABLE CTSTAGING.DUPLICATE_ALLOWANCE ADD (
  CONSTRAINT PK_DUPLICATE_ALLOWANCE
  PRIMARY KEY
  (DUPLICATE_ALLOWANCE_ID)
  USING INDEX CTSTAGING.XPKDUPLICATE_ALLOWANCE
  ENABLE VALIDATE);

GRANT SELECT ON CTSTAGING.DUPLICATE_ALLOWANCE TO CTSTAGING_READ_ONLY_ROLE;

GRANT DELETE, INSERT, UPDATE ON CTSTAGING.DUPLICATE_ALLOWANCE TO CTSTAGING_UPDATE_USER_ROLE;
