CREATE OR REPLACE PROCEDURE concept_export_migration (
     source_table      IN VARCHAR2, 
     destination IN VARCHAR2,
     key_field IN VARCHAR2 ) IS 

    
     id_var             NUMBER; 
     exportable_id_var  NUMBER; 
     exported_id_var    NUMBER; 
     active_id_var      NUMBER;
     alter_cursor      INTEGER; 
     source_cursor      INTEGER; 
     destination_cursor INTEGER; 
     ignore             INTEGER; 
     row_count          INTEGER := 0;
     sql_stmt           varchar2(150);
     
  BEGIN 
 

    -- Prepare a cursor to select from the source table: 
     sql_stmt := 'SELECT ' || key_field || ', exportable_version_id, exported_version_id, ' ||
     case when (source_table in ('CTSTAGING.PECS_DUPL_THERAPY_VA', 'CTSTAGING.PECS_DOSE_RANGE_VA', 'CTSTAGING.PECS_MONOGRAPH_VA')) 
        then 'CRT_VERSION_ID'
        when (source_table = 'CTSTAGING.PECS_DRUG_PAIR_VA') then 'CURRENT_VERSION_ID' else 'ACTIVE_VERSION_ID' end 
     ||' FROM ' || source_table ;
     source_cursor := dbms_sql.open_cursor; 
     DBMS_SQL.PARSE(source_cursor, sql_stmt, DBMS_SQL.NATIVE); 
     DBMS_SQL.DEFINE_COLUMN(source_cursor, 1, id_var); 
     DBMS_SQL.DEFINE_COLUMN(source_cursor, 2, exportable_id_var); 
     DBMS_SQL.DEFINE_COLUMN(source_cursor, 3, exported_id_var); 
     DBMS_SQL.DEFINE_COLUMN(source_cursor, 4, active_id_var); 
     ignore := DBMS_SQL.EXECUTE(source_cursor); 
 
  -- Prepare a cursor to insert into the destination table: 
     destination_cursor := DBMS_SQL.OPEN_CURSOR; 
     DBMS_SQL.PARSE(destination_cursor, 
                  'INSERT INTO ' || destination || 
                  ' VALUES (:id_bind, :exportable_bind, :exported_bind, :active_bind)', 
                   DBMS_SQL.NATIVE); 
 
  -- Fetch a row from the source table and insert it into the destination table: 
     LOOP 
       IF DBMS_SQL.FETCH_ROWS(source_cursor)>0 THEN 
         -- get column values of the row 
         DBMS_SQL.COLUMN_VALUE(source_cursor, 1, id_var); 
         DBMS_SQL.COLUMN_VALUE(source_cursor, 2, exportable_id_var); 
         DBMS_SQL.COLUMN_VALUE(source_cursor, 3, exported_id_var); 
         DBMS_SQL.COLUMN_VALUE(source_cursor, 4, active_id_var); 
 
  -- Bind the row into the cursor that inserts into the destination table. You 
  -- could alter this example to require the use of dynamic SQL by inserting an 
  -- if condition before the bind. 
        DBMS_SQL.BIND_VARIABLE(destination_cursor, ':id_bind', id_var); 
        DBMS_SQL.BIND_VARIABLE(destination_cursor, ':exportable_bind', exportable_id_var); 
        DBMS_SQL.BIND_VARIABLE(destination_cursor, ':exported_bind', exported_id_var);
        DBMS_SQL.BIND_VARIABLE(destination_cursor, ':active_bind', active_id_var); 
        ignore := DBMS_SQL.EXECUTE(destination_cursor); 
        row_count := row_count + 1;
      ELSE 
 
  -- No more rows to copy: 
        EXIT; 
      END IF; 
    END LOOP; 
     DBMS_OUTPUT.PUT_LINE('Concept table created: '||destination||'.  Number of rows migrated: '||to_char(row_count));
    -- create cursor to remove column from destination table
       
       
       
       select 'ALTER TABLE '|| source_table || ' DROP (' ||
        case when (source_table in ('CTSTAGING.PECS_DUPL_THERAPY_VA', 'CTSTAGING.PECS_DOSE_RANGE_VA', 'CTSTAGING.PECS_MONOGRAPH_VA')) 
        then 'CRT_VERSION_ID'
        when (source_table = 'CTSTAGING.PECS_DRUG_PAIR_VA') then 'CURRENT_VERSION_ID' else 'ACTIVE_VERSION_ID' end ||', EXPORTED_VERSION_ID, EXPORTABLE_VERSION_ID) CASCADE CONSTRAINTS' 
        into sql_stmt 
       from dual;
       
       alter_cursor := DBMS_SQL.OPEN_CURSOR;
      DBMS_SQL.PARSE(alter_cursor, sql_stmt, DBMS_SQL.NATIVE);
      ignore := DBMS_SQL.EXECUTE(alter_cursor);
      DBMS_OUTPUT.PUT_LINE('Concept table modified, version columns removed from: '||source_table||'.');
  -- Commit and close all cursors: 
     COMMIT; 
     DBMS_SQL.CLOSE_CURSOR(source_cursor); 
     DBMS_SQL.CLOSE_CURSOR(destination_cursor);
     DBMS_SQL.CLOSE_CURSOR(alter_cursor);

   EXCEPTION 
     WHEN OTHERS THEN 
       IF DBMS_SQL.IS_OPEN(source_cursor) THEN 
         DBMS_SQL.CLOSE_CURSOR(source_cursor); 
       END IF; 
       IF DBMS_SQL.IS_OPEN(destination_cursor) THEN 
         DBMS_SQL.CLOSE_CURSOR(destination_cursor); 
       END IF; 
       RAISE; 
  END; 
/ 
