ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA
 DROP PRIMARY KEY CASCADE;

DROP TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA CASCADE CONSTRAINTS;

CREATE TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA
(
  VA_DRUG_PAIR_ID  NUMBER(11)             NOT NULL,
  EXPORTABLE_VERSION_ID  NUMBER(11),
  EXPORTED_VERSION_ID    NUMBER(11),
  ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL
)
TABLESPACE CTSTAGING_DATA
PCTUSED    0
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_DP_EXPORT_VA ON CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA
(VA_DRUG_PAIR_ID)
LOGGING
TABLESPACE CTSTAGING_INDEX
PCTFREE    10
INITRANS   2
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
NOPARALLEL;




CREATE OR REPLACE PUBLIC SYNONYM PECS_DRUG_PAIR_EXPORT_VA FOR CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA;


ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD (
  CONSTRAINT PK_PECS_DP_EXPORT_VA
  PRIMARY KEY
  (VA_DRUG_PAIR_ID)
  USING INDEX CTSTAGING.XPK_PECS_DP_EXPORT_VA
  ENABLE VALIDATE);


-- Add unique keys
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT UKDPEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT UKDPEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT UKDPACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID);
-- Add foreign keys
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DP_EXPORT_VA_ID FOREIGN KEY (VA_DRUG_PAIR_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VA (VA_DRUG_PAIR_ID);
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DPV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VERSION_VA (VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DPV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VERSION_VA (VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DPV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VERSION_VA (VERSION_ID);


GRANT SELECT ON CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE;

GRANT DELETE, INSERT, UPDATE, SELECT ON CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE;

commit;