ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA
 DROP PRIMARY KEY CASCADE;

DROP TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA CASCADE CONSTRAINTS;

CREATE TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA
(
  VA_DUPL_THERAPY_ID  NUMBER(11)             NOT NULL,
  EXPORTABLE_VERSION_ID  NUMBER(11),
  EXPORTED_VERSION_ID    NUMBER(11),
  ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL
)
TABLESPACE CTSTAGING_DATA
PCTUSED    0
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_DT_EXPORT_VA ON CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA
(VA_DUPL_THERAPY_ID)
LOGGING
TABLESPACE CTSTAGING_INDEX
PCTFREE    10
INITRANS   2
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
NOPARALLEL;




CREATE OR REPLACE PUBLIC SYNONYM PECS_DUPL_THERAPY_EXPORT_VA FOR CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA;


ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD (
  CONSTRAINT PK_PECS_DT_EXPORT_VA
  PRIMARY KEY
  (VA_DUPL_THERAPY_ID)
  USING INDEX CTSTAGING.XPK_PECS_DT_EXPORT_VA
  ENABLE VALIDATE);


-- Add unique keys
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT UKDTEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT UKDTEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT UKDTACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID);
-- Add foreign keys
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DT_EXPORT_VA_ID FOREIGN KEY (VA_DUPL_THERAPY_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VA (VA_DUPL_THERAPY_ID);
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DTV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA (VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DTV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA (VERSION_ID);
ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DTV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA (VERSION_ID);


GRANT SELECT ON CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE;

GRANT DELETE, INSERT, UPDATE ON CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE;

commit;