ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA
 DROP PRIMARY KEY CASCADE;

DROP TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA CASCADE CONSTRAINTS;

CREATE TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA
(
  VA_MONOGRAPH_ID  NUMBER(11)             NOT NULL,
  EXPORTABLE_VERSION_ID  NUMBER(11),
  EXPORTED_VERSION_ID    NUMBER(11),
  ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL
)
TABLESPACE CTSTAGING_DATA
PCTUSED    0
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_MONO_EXPORT_VA ON CTSTAGING.PECS_MONOGRAPH_EXPORT_VA
(VA_MONOGRAPH_ID)
LOGGING
TABLESPACE CTSTAGING_INDEX
PCTFREE    10
INITRANS   2
MAXTRANS   255
STORAGE    (
            INITIAL          64K
            NEXT             1M
            MINEXTENTS       1
            MAXEXTENTS       UNLIMITED
            PCTINCREASE      0
            BUFFER_POOL      DEFAULT
           )
NOPARALLEL;




CREATE OR REPLACE PUBLIC SYNONYM PECS_MONOGRAPH_EXPORT_VA FOR CTSTAGING.PECS_MONOGRAPH_EXPORT_VA;


ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD (
  CONSTRAINT PK_PECS_MONO_EXPORT_VA
  PRIMARY KEY
  (VA_MONOGRAPH_ID)
  USING INDEX CTSTAGING.XPK_PECS_MONO_EXPORT_VA
  ENABLE VALIDATE);


-- Add unique keys
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT UKMNEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID);
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT UKMNEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID);
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT UKMNACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID);
-- Add foreign keys
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONO_EXPORT_VA_ID FOREIGN KEY (VA_MONOGRAPH_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VA (VA_MONOGRAPH_ID);
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONOV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID);
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONOV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID);
ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONOV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID);


GRANT SELECT ON CTSTAGING.PECS_MONOGRAPH_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE;

GRANT DELETE, INSERT, UPDATE, SELECT ON CTSTAGING.PECS_MONOGRAPH_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE;

commit;