package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import java.io.Serializable;
import javax.annotation.Resource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.transaction.annotation.Transactional;
import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVersionVa;
import gov.va.med.pharmacy.pecs.persistence.DrugDrugInteractionDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.TerminologyDAO;
import gov.va.med.pharmacy.pecs.state.PecsStateMachineFactory;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

/**
 * Junit for DrugPair.
 * 
 * @author DOMAINOVERSD
 * 
 */
public class DrugPairDdimIdTest extends AbstractUnitTest {

    @Resource
    private DrugPairDAOImpl drugPairDao;
    @Resource
    private DrugDrugInteractionDAOImpl drugDrugDAOImpl;
    
    @Resource
    private TerminologyDAO termDAO;
 
    @Resource
    private PecsStateMachineFactory stateMachineFactory;

    private static final Log LOGGER = LogFactory.getLog(DrugPairDdimIdTest.class);

    /**
     * Test Drug Pair Save and Call from FDB.
     * 
     */
    @Test
    @Transactional
    public final void testsavesuccesful() {

        DrugDrugInteraction ddi = createDrugDrugInteraction();
        Serializable ddiObject = drugDrugDAOImpl.save(ddi);
        Assert.assertNotNull(ddiObject);
        
        
        
        DrugPairVa drugPair = createDrugPair();
        drugPair.setId(15L);
        drugPair.setInteraction(ddi);
        
        Serializable object = drugPairDao.save(drugPair);
        Assert.assertNotNull(object);
        
        DrugPairVa retrievedRecord = drugPairDao.findById(drugPair.getId());
        
        Assert.assertEquals(retrievedRecord.getId(), drugPair.getId());
        

    }
    /**Test to save a null object.
     * 
     */
    @Test
    @Transactional
    public final void testSaveNull() {
        try {

            Serializable object = drugPairDao.save(null);
            Assert.assertNull(object);

        } catch (Exception e) {
            if (LOGGER.is   ugEnabled()) {
                System.out.print(e);
            }
            Assert.fail(e.getMessage());
        }
    }

    /** Test Save Failure.
     * 
     */
    @Test
    @Transactional
    public final void testSaveFailure() {

        try {
            DrugDrugInteraction ddi = createDrugDrugInteraction();
            Serializable ddiObject = drugDrugDAOImpl.save(ddi);
            Assert.assertNotNull(ddiObject);

          
            
            DrugPairVa record = createDrugPair();
           record.setInteraction(null);
           record.setId(null);

            Serializable object = drugPairDao.save(record);

            Assert.assertNull(object);

        } catch (org.hibernate.PropertyValueException e) {

            if (LOGGER.is   ugEnabled()) {

                System.out.print(e);
            }
            Assert.assertNotNull(e.getMessage()); 

        }

    }

    /**
     * My Preset DrugPair.
     * 
     * @return doc
     */
    public final DrugPairVa createDrugPair() {

        
        
        DrugPairVa drugPair = DrugPairVa.createDrugPair(createGenericDrug1(), 
                createGenericDrug2(), createDrugDrugInteraction());

        drugPair.setAuditUser("Tester");
        drugPair.setStatus(createStatus());
        drugPair.setActionBy("Tester");
       
        

        return drugPair;
    }

    /**
     * Generic Drug 1 for test.
     * 
     * @return genericDrug1
     */

    public final RoutedGenericFdb createGenericDrug1() {

        RoutedGenericFdb routeGeneric = new RoutedGenericFdb();
        routeGeneric.setRtGenId(34619L);
        routeGeneric.setDescription("My Basic Drug");
        
        
        routeGeneric.setCanBeDosedInd(true);
        routeGeneric.setDescription2("Basic Drug");
        routeGeneric.setDescription3("String");
        routeGeneric.setDescription4("Hi");
        routeGeneric.setDescription5("Boo");
        routeGeneric.setDescription6("ahhh!!!");
        routeGeneric.setGcnSeqNoUnique(98);
        routeGeneric.setHasPackagedDrugsInd(false);
        routeGeneric.setHicl(1);
        routeGeneric.setMedicalDeviceInd(true);
        routeGeneric.setNeoCanBeDosedInd(true);
        routeGeneric.setNeoSingleDoseRouteInd(false);
        routeGeneric.setRegionalInd(true);
        routeGeneric.setRtDfGenIdUnique(2);
        routeGeneric.setRtGenIdString("my Id");
        routeGeneric.setRtid(4);
        routeGeneric.setRxotcCode('t');
        routeGeneric.setSingleDoseRouteInd(true);
        
        
        return routeGeneric;

    }

    /**
     * Generic Drug 2 for test.
     * 
     * @return genericDrug2
     */
    public final RoutedGenericFdb createGenericDrug2() {

        RoutedGenericFdb routeGeneric = new RoutedGenericFdb();
        routeGeneric.setRtGenId(35890L);
        routeGeneric.setDescription("My Basic Drug");
        

        return routeGeneric;
    }

    /**
     * DrugDrug DDI Creation.
     * 
     * @return ddi
     */
    public final DrugDrugInteractionVa createDrugDrugInteraction() {
        DrugDrugInteractionVa ddi = DrugDrugInteractionVa.create();
        
        ddi.setActionBy("Food");
        ddi.setDescription("TesterDDI");
        ddi.setRequestor("Test User");
        ClinicalEffect clinicalEffect = new ClinicalEffect();
        ddi.setClinicalEffect1(clinicalEffect);
        ddi.setId(1L);
        ddi.setSeverityLevel(createSeverityLevel());
        ddi.setClinicalEffect1(createClinicalEffect());
        
        ddi.setStatus(createStatus());
        ddi.setAuditUser("Tester");

        return ddi;
    }

    /**Create Severity Level.
     * 
     * @return severitylevel
     */
    private SeverityLevel createSeverityLevel() {
        //SeverityLevel severity = new SeverityLevel();
        
        //severity.setCode("1");
        //severity.setCreatedBy("CCR3714");
        //severity.setModifiedBy("CCR3714");
        return (SeverityLevel)termDAO.getByCode(SeverityLevel.class, "1");
        
    }
    
    /**Create Clinical Effect.
     * 
     * @return clinicalEffect
     */
    private ClinicalEffect createClinicalEffect() {
        //SeverityLevel severity = new SeverityLevel();
        
        //severity.setCode("1");
        //severity.setCreatedBy("CCR3714");
        //severity.setModifiedBy("CCR3714");
        return (ClinicalEffect)termDAO.getByCode(ClinicalEffect.class, "INF");
        
    }
    /**Create Status.
     * 
     * @return status
     */
    public final Status createStatus()
    {
        Status status = new Status();
        status.setCode("2");
        status.setDescription("2");
        status.setName("Testing...");
        status.setStatusId(1);
        return status;
    }

    /**Create Drugpair.
     * 
     * @return version
     */
    @SuppressWarnings("unused")
    private final DrugPairVersionVa createDrugPairVersionVa() {
        DrugPairVersionVa version = new DrugPairVersionVa();
        version.setActionBy("actionUser");
        version.setUser("user");
        version.setStatus(new Status());
        version.setVersionId(123L);
        return version;
    }

    /**
     * Drug Pair DAO.
     * 
     * @return drugPair
     */
    public final DrugPairDAOImpl getdrugPairDao() {
        return this.drugPairDao;
    }

    /**
     * Setter Drug Pair Dao.
     * 
     * @param drugPairDao -DrugPairDAOImpl
     */
    public final void setdrugPairDao(DrugPairDAOImpl drugPairDao) {
        this.drugPairDao = drugPairDao;
    }

    /**
     * Get Pecs State machine Factory.
     * 
     * @return stateMachineFactory
     */

    public final PecsStateMachineFactory getPecsStateMachineFactory() {
        return this.stateMachineFactory;
    }

    /**
     * Setter SMF.
     * 
     * @param stateMachineFactory
     */
    public final void setStateMachineFactory(PecsStateMachineFactory stateMachineFactory) {
        this.stateMachineFactory = stateMachineFactory;
    }

}
