package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import java.util.Iterator;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.Test;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

/**
 * JUnit test class for FdbDrugPairDAO.
 * 
 */
public class FdbDrugPairDAOTest extends AbstractUnitTest {

    @Resource
    private FdbDrugPairDAO fdbDrugPairDao;

    /**
     * Tests the paging capabilities in HibernateDAOImpl and Page. Cycles through the pages of a multi-page query, ensuring
     * total count equals the reported query size and that the last row of one page is not repeated as the first row of the
     * second page. Note that the query includes sorting.
     */
    @SuppressWarnings("unchecked")
    @Test
    @Transactional
    public void testFindByQueryByPage() {
        String query =
                "from FdbDdimdruglink where lower(rtgenid1Desc) like '%zith%' AND lower(severitylevelcode) like '%3%' order by LOWER(rtgenid2Desc) asc, id asc";
        int pageNum = 1;
        int totCount = 0;
        Page page = fdbDrugPairDao.findByQueryByPage(query, pageNum);

        Assert.assertNotNull("page is null", page);
        Assert.assertTrue("The query does not require multiple pages", 1 < page.getTotalPages());

        totCount += page.getResults().size();

        // we want to avoid any indexing with +1 or -1 in our tests since that could just mask erroneous +1/-1 in the real code,
        // so we iterate to find the last item.
        FdbDdimdruglink lastFromPreviousPage = null;

        Iterator<FdbDdimdruglink> it = page.getResults().iterator();
        while (it.hasNext()) {
            lastFromPreviousPage = it.next();
        }

        while (page.isNextPage()) {
            pageNum++;
            page = fdbDrugPairDao.findByQueryByPage(query, pageNum);
            Assert.assertNotNull(page);
            totCount += page.getResults().size();
            FdbDdimdruglink firstFromCurrentPage = (FdbDdimdruglink) page.getResults().get(0);
            Assert.assertFalse("Last row from prevous page is same as first from this page!",
                    compareIds(lastFromPreviousPage.getId(), firstFromCurrentPage.getId()));
            
            FdbDdimdruglink newLastFromPreviousPage = null; 
            
            it = page.getResults().iterator();
            while (it.hasNext()) {
                newLastFromPreviousPage = it.next();
                Assert.assertFalse("Last row from prevous page is same as first from this page!",
                        compareIds(lastFromPreviousPage.getId(), newLastFromPreviousPage.getId()));                                
            }            
            
            lastFromPreviousPage = newLastFromPreviousPage;
        }

        Assert.assertTrue("Number of records returned from all the pages does not equal the querySize.",
                totCount == page.getQuerySize());
    }  
    
    /**
     * Compares two FdbDdimdruglinkIds.
     * 
     * @param id
     * @param id2
     * @return true if they are the same; false if they are not the same
     */
    private boolean compareIds(FdbDdimdruglinkId id, FdbDdimdruglinkId id2) {
        if (id.getInteractionid() != id2.getInteractionid()) {
            return false;
        }

        if (id.getRtgenid1() != id2.getRtgenid1()) {
            return false;
        }

        if (id.getRtgenid2() != id2.getRtgenid2()) {
            return false;
        }

        return true;
    }

}
