package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.persistence.DrugDrugInteractionDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.TerminologyDAO;
import gov.va.med.pharmacy.pecs.state.PecsStateMachineFactory;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import java.io.Serializable;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;

import org.hibernate.Query;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StopWatch;

/**
 * A test case for the Drug Drug Interaction DAO.
 * 
 * @author vhaispmetela
 * 
 */
public class DrugDrugInteractionDAOTest extends AbstractUnitTest {
    /**
     * Object Variables.
     * 
     */
    Version<DrugDrugInteraction> v;
    List<Version<DrugDrugInteraction>> history;

    @Resource
    private DrugDrugInteractionDAOImpl drugDrugInteractionDAO;

    @Resource
    private PecsStateMachineFactory stateMachineFactory;
    @Resource
    private TerminologyDAO terminologyDAO;

    @Test
    @Transactional
    public void testfindDDIByFdbId() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        System.out.println("starting @@@@@@@@@@@@@@@@@@@@@@@@@@@");
        List<DrugDrugInteractionVa> list = getDrugDrugInteractionDAO().findDDIByFdbId(1745L);
        System.out.println("ending @@@@@@@@@@@@@@@@@@@@@@@@@@@");
        stopWatch.stop();
        System.out.println(stopWatch.getLastTaskTimeMillis());
        Assert.assertTrue(!list.isEmpty());
    }

    @Test
    @Transactional
    public void testRetrieve() {
        DrugDrugInteractionVa ddim = drugDrugInteractionDAO.findById(409L);
        Assert.assertEquals("2", ddim.getSeverityLevel().getCode());
        Assert.assertNull(ddim.getConceptXref());
        Set<UserComment> userCommentHistory = ddim.getUserCommentHistory();
        Assert.assertNotNull(userCommentHistory);
        // Assert.assertNotNull(ddim.getExportableVersion());
        Assert.assertEquals(0, userCommentHistory.size());
    }

    @Test
    @Transactional
    public void testQuery() {
        String query =
                "from DrugDrugInteractionVersionVa as ddi left join fetch ddi.clinicalEffect2 "
                        + "where ddi.versionId = ddi.versionedObject.exportManager.currentVersion.versionId "
                        + "ANDddi.versionedObject.fdbId like '%1%' " + " order by LOWER(ddi.clinicalEffect2.code) asc";

        Query q = getDrugDrugInteractionDAO().getSession().createQuery(query);
        @SuppressWarnings("unchecked")
        List<DrugDrugInteractionVersionVa> list = q.list();
        System.out.println(list.size());
        for (DrugDrugInteractionVersionVa drugDrugInteractionVersionVa : list) {

            System.out.println(drugDrugInteractionVersionVa.getClinicalEffect2());
        }
    }

    @Test
    @Transactional
    public void testVersionRetrieve() {
        DrugDrugInteractionVersionVa version = drugDrugInteractionDAO.findVersionById(1689L);
        Assert.assertEquals("2", version.getSeverityLevel().getCode());
        Set<UserComment> userCommentHistory = version.getUserCommentHistory();
        Assert.assertNotNull(userCommentHistory);
        Assert.assertEquals(0, userCommentHistory.size());
        Assert.assertEquals(false, drugDrugInteractionDAO.getSession().isDirty());
    }

    /**
     * Tests the Save method for the DAO by persisting a record then finding that same record by Id.
     */
    @Test
    @Transactional()
    @Rollback(value = false)
    public final void testSaveSuccessful() {

        // Create a new record.
        DrugDrugInteractionVa record = createDrugDrugInteractionVA();

        // Modify anything your test needs specifically.
        // record.setConceptid(new Long(CONCEPT_ID));

        // Use the DAO to save to database.
        Serializable object = drugDrugInteractionDAO.save(record);

        // Verify the save action returned something.
        Assert.assertNotNull(object);

        // Find the record in the database by the id that was assigned during persistance.
        DrugDrugInteractionVa retrievedRecord = drugDrugInteractionDAO.findById(record.getId());

        // Verify the record was retrieved successfully.
        Assert.assertNotNull(retrievedRecord.getExportManager());
        // Assert.assertEquals("DTCId are not equal", retrievedRecord.getConceptid(), record.getConceptid());

    }

    /**
     * Convenience method to create a populated DuplicateTherapyVA object.
     * 
     * @return DuplicateTherapyVa -
     */
    private final DrugDrugInteractionVa createDrugDrugInteractionVA() {

        DrugDrugInteractionVa record = DrugDrugInteractionVa.create();

        record.setActionBy("APPROVER_ONE");
        record.setAuditUser("APPROVER_ONE");

        createDrugDrugInteractionVersionVA(record);

        // drugDrugInteractionDAO.save(record.getCurrentVersion());
        // record.getVersionHistory().add(record.getCurrentVersion());
        record.setRequestor("APPROVER_ONE");
        // record.setVersionHistory(history);
        record.setStatus(stateMachineFactory.getStatusByCode(Status.STATUS_NEW));

        return record;

    }

    /**
     * @param record
     */
    private void createDrugDrugInteractionVersionVA(DrugDrugInteractionVa record) {
        record.getCurrentVersion().setSeverityLevel((SeverityLevel) terminologyDAO.getById(SeverityLevel.class, 1195L));
        record.getCurrentVersion().setClinicalEffect1((ClinicalEffect) terminologyDAO.getById(ClinicalEffect.class, 1L));
        record.getCurrentVersion().setStatus((Status) terminologyDAO.getByCode(Status.class, "New"));
        record.getCurrentVersion().setDescription("test");
    }

    /**
     * Gets the DrugDrugInteractionDAO.
     * 
     * @return the DrugDrugInteractionDAO
     */
    public final DrugDrugInteractionDAOImpl getDrugDrugInteractionDAO() {
        return drugDrugInteractionDAO;
    }

    /**
     * Sets the DrugDrugInteractionDAO.
     * 
     * @param DrugDrugInteractionDAO
     *            the DrugDrugInteractionDAO to set
     */
    public final void setDrugDrugInteractionDAO(DrugDrugInteractionDAOImpl DrugDrugInteractionDAO) {
        this.drugDrugInteractionDAO = DrugDrugInteractionDAO;
    }

    /**
     * Gets the stateMachineFactory.
     * 
     * @return the stateMachineFactory
     */
    public final PecsStateMachineFactory getStateMachineFactory() {
        return stateMachineFactory;
    }

    /**
     * Sets the stateMachineFactory.
     * 
     * @param stateMachineFactory
     *            the stateMachineFactory to set
     */
    public final void setStateMachineFactory(PecsStateMachineFactory stateMachineFactory) {
        this.stateMachineFactory = stateMachineFactory;
    }

    /**
     * Gets the terminologyDAO.
     * 
     * @return the terminologyDAO
     */
    public TerminologyDAO getTerminologyDAO() {
        return terminologyDAO;
    }

    /**
     * Sets the terminologyDAO.
     * 
     * @param terminologyDAO
     *            the terminologyDAO to set
     */
    public void setTerminologyDAO(TerminologyDAO terminologyDAO) {
        this.terminologyDAO = terminologyDAO;
    }

}
