/**
 * 
 */
package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.DrugPair;
import gov.va.med.pharmacy.pecs.domain.RoutedGeneric;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdb;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdbLite;
import gov.va.med.pharmacy.pecs.persistence.DrugDrugInteractionDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.TerminologyDAOImpl;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.hibernate.ObjectNotFoundException;
import org.hibernate.stat.SecondLevelCacheStatistics;
import org.hibernate.stat.Statistics;
import org.junit.Before;
import org.junit.Test;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author VHAISACARUSR
 * 
 * Note, some of the tests are commented out at the annotations because we may go back and forth between automated 
 * tests and tests that require some manual intervention. 
 * 
 */
public class DrugPairDAOTest extends AbstractUnitTest {
    @Resource
    private DrugPairDAOImpl drugPairDAOImpl;
    @Resource
    private DrugDrugInteractionDAOImpl drugInteractionDAOImpl;

    @Resource
    private TerminologyDAOImpl terminologyDAOImpl;

    @Before
    public void beforeClassLoad() {
        Statistics stats = getTerminologyDAOImpl().getSessionFactory().getStatistics();
        stats.setStatisticsEnabled(true);
    }

    @Test
    @Transactional
    public void testFindDrugPair() {
        DrugPairVa drugPairVa = drugPairDAOImpl.findDrugPair(1048627, 1050000, 2004892);
        Assert.assertNotNull(drugPairVa);
    }

    @Test
    @Transactional
    public void testFindDrugPairStatusCountsByInteractionId() {
        Map<String, Long> results = drugPairDAOImpl.findDrugPairsStatusCountsByInteractionId(2004892);
        Assert.assertNotNull(results);
        for (String status : results.keySet()) {
            System.out.println(status + " count: " + results.get(status));
        }
    }

    @Test
    @Transactional
    public void testSaveDrugPair() {
        RoutedGeneric rtg1 = drugPairDAOImpl.findById(1048593L, RoutedGenericFdb.class);
        RoutedGeneric rtg2 = drugPairDAOImpl.findById(1050008L, RoutedGenericFdb.class);
        DrugDrugInteraction interactionVa = drugInteractionDAOImpl.findDDIByInteractionId(2004892L);

        DrugPairVa drugPairVa = DrugPairVa.createDrugPair(rtg1, rtg2, interactionVa);
        Assert.assertNotNull(drugPairVa);
        drugPairVa.setStatus((Status) terminologyDAOImpl.getByCode(Status.class, "New"));
        drugPairVa.setActionBy("APPROVER_ONE");
        drugPairVa.getCurrentVersion().setActionBy("APPROVER_ONE");
        drugPairVa.setRequestor("APPROVER_ONE");

        drugPairVa.setAuditUser("APPROVER_ONE");
        drugPairDAOImpl.saveOrUpdate(drugPairVa);
        Assert.assertNotNull(drugPairVa.getCurrentVersion());
    }

    @Test
    @Transactional
    public void testGetRoutedGenerics() {
        long start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugs();
        long end = System.currentTimeMillis();
        System.out.println("run 1  " + (end - start));

        Statistics stats = getTerminologyDAOImpl().getSessionFactory().getStatistics();
        String[] caches = stats.getSecondLevelCacheRegionNames();
        for (String string : caches) {
            System.out.println(string);
        }
        // SecondLevelCacheStatistics cacheStats =
        // stats.getSecondLevelCacheStatistics("gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdb");
        SecondLevelCacheStatistics cacheStats = stats.getSecondLevelCacheStatistics("RoutedGenericFdbQuery");
        System.out.println("count in memory" + cacheStats.getElementCountInMemory());
        System.out.println("element count on disk" + cacheStats.getElementCountOnDisk());

        System.out.println("hit count " + cacheStats.getHitCount());
        System.out.println(" miss count" + cacheStats.getMissCount());
        System.out.println("put count " + cacheStats.getPutCount());
        System.out.println("size in memory" + cacheStats.getSizeInMemory());
        start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugs();
        end = System.currentTimeMillis();
        System.out.println("run 2  " + (end - start));

        Map entries = cacheStats.getEntries();
        Iterator iterator = entries.keySet().iterator();
        while (iterator.hasNext()) {
            Object key = iterator.next();
            System.out.println(entries.get(key));

        }
    }

    //@Test
    @Transactional
    /**
     * Loads the routed generics into cache, then pauses, allowing the tester to manually delete data from the DB, 
     * before attempting an operation that will depend on the cached data self-healing. 
     */
    public void testGetRoutedGenericsCachingManuallyDeleteInMiddle() {

        getDrugPairDAOImpl().getRoutedGenericDrugs();
        pauseUntilEnter();
        try{
            //getDrugPairDAOImpl().findDrugPairsByInteractionId(2003752L);
            getDrugPairDAOImpl().getRoutedGenericDrugs();
        }
        catch (ObjectNotFoundException onfe)
        {
            Assert.fail("ObjectNotFoundException caught!");
        }
    }

    public static void pauseUntilEnter() {
        System.out.println("Press enter to continue...");
        Scanner keyboard = new Scanner(System.in);
        keyboard.nextLine();
    }

    @Test
    @Transactional
    public void testGetRoutedGenericsCaching() {
        final String CACHE_REGION = "RoutedGenericFdbLiteQuery";
        long start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugs();
        long end = System.currentTimeMillis();
        System.out.println("run 1  " + (end - start) + "ms");

        Statistics stats = getDrugPairDAOImpl().getSessionFactory().getStatistics();
        String[] caches = stats.getSecondLevelCacheRegionNames();
        for (String string : caches) {
            System.out.println(string);
        }
        System.out.println("=============================================================================");

        printCacheStats(stats, CACHE_REGION);

        start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugs();
        end = System.currentTimeMillis();
        System.out.println("run 2  " + (end - start) + "ms");

        printCacheStats(stats, CACHE_REGION);

        System.out.println("Evicting Caches");
        getDrugPairDAOImpl().getSessionFactory().getCache().evictQueryRegion("RoutedGenericFdbQuery");
        // getDrugPairDAOImpl().getSessionFactory().getCache().evictEntityRegion(RoutedGenericFdb.class.getName());
        printCacheStats(stats, CACHE_REGION);

        start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugs();
        end = System.currentTimeMillis();
        System.out.println("run 3  " + (end - start) + "ms");
        printCacheStats(stats, CACHE_REGION);
    }

    private void printCacheStats(Statistics stats, String region) {
        SecondLevelCacheStatistics statsL2 = stats.getSecondLevelCacheStatistics(region);
        System.out.println("entity load count: " + stats.getEntityLoadCount());
        System.out.println("entity fetch count: " + stats.getEntityFetchCount());
        System.out.println("query hit count: " + stats.getQueryCacheHitCount());
        System.out.println("query miss count: " + stats.getQueryCacheMissCount());
        System.out.println("query put count: " + stats.getQueryCachePutCount());
        System.out.println("L2 element count in memory: " + statsL2.getElementCountInMemory());
        System.out.println("L2 element count on disk: " + statsL2.getElementCountOnDisk());
        System.out.println("L2 hit count: " + statsL2.getHitCount());
        System.out.println("L2 miss count:" + statsL2.getMissCount());
        System.out.println("L2 put count: " + statsL2.getPutCount());
        System.out.println("L2 size in memory: " + statsL2.getSizeInMemory());
        System.out.println("=============================================================================");
    }

    @Test
    @Transactional
    public void testGetRoutedGenericsLiteCaching() {
        final String CACHE_REGION = "RoutedGenericFdbLiteQuery";
        long start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugsLite();
        long end = System.currentTimeMillis();
        System.out.println("run 1  " + (end - start) + "ms");

        Statistics stats = getDrugPairDAOImpl().getSessionFactory().getStatistics();
        printCacheStats(stats, CACHE_REGION);
        SecondLevelCacheStatistics statsL2 = stats.getSecondLevelCacheStatistics(CACHE_REGION);
        //expects misses and puts to increase and element counts and size to show items stored in cache
        Assert.assertTrue("entity load count: ", stats.getEntityLoadCount() > 10000);
        Assert.assertEquals("entity fetch count: ", 0, stats.getEntityFetchCount());
        Assert.assertEquals("query hit count: ", 0, stats.getQueryCacheHitCount());
        Assert.assertEquals("query miss count: ", 1, stats.getQueryCacheMissCount());
        Assert.assertEquals("query put count: ", 1, stats.getQueryCachePutCount());
        Assert.assertEquals("L2 element count in memory: ", 1, statsL2.getElementCountInMemory());
        Assert.assertEquals("L2 element count on disk: ", 0, statsL2.getElementCountOnDisk());
        Assert.assertEquals("L2 hit count: ", 0, statsL2.getHitCount());
        Assert.assertEquals("L2 miss count:", 1, statsL2.getMissCount());
        Assert.assertEquals("L2 put count: ", 1, statsL2.getPutCount());
        Assert.assertTrue("L2 size in memory: ", statsL2.getSizeInMemory() > 100000);

        start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugsLite();
        end = System.currentTimeMillis();
        System.out.println("run 2  " + (end - start) + "ms");

        printCacheStats(stats, CACHE_REGION);
        //expects hits to increase and other stats to stay the same
        Assert.assertTrue("entity load count: ", stats.getEntityLoadCount() > 10000);
        Assert.assertEquals("entity fetch count: ", 0, stats.getEntityFetchCount());
        Assert.assertEquals("query hit count: ", 1, stats.getQueryCacheHitCount());
        Assert.assertEquals("query miss count: ", 1, stats.getQueryCacheMissCount());
        Assert.assertEquals("query put count: ", 1, stats.getQueryCachePutCount());
        Assert.assertEquals("L2 element count in memory: ", 1, statsL2.getElementCountInMemory());
        Assert.assertEquals("L2 element count on disk: ", 0, statsL2.getElementCountOnDisk());
        Assert.assertEquals("L2 hit count: ", 1, statsL2.getHitCount());
        Assert.assertEquals("L2 miss count:", 1, statsL2.getMissCount());
        Assert.assertEquals("L2 put count: ", 1, statsL2.getPutCount());
        Assert.assertTrue("L2 size in memory: ", statsL2.getSizeInMemory() > 100000);

        System.out.println("Evicting Caches");
        getDrugPairDAOImpl().getSessionFactory().getCache().evictQueryRegion("RoutedGenericFdbLiteQuery");
        // getDrugPairDAOImpl().getSessionFactory().getCache().evictEntityRegion(RoutedGenericFdb.class.getName());
        printCacheStats(stats, CACHE_REGION);
        //expects element counts and memory to be zeroed out
        Assert.assertEquals("L2 element count in memory: ", 0, statsL2.getElementCountInMemory());
        Assert.assertEquals("L2 element count on disk: ", 0, statsL2.getElementCountOnDisk());
        Assert.assertEquals("L2 size in memory: ", 0, statsL2.getSizeInMemory());

        start = System.currentTimeMillis();
        getDrugPairDAOImpl().getRoutedGenericDrugsLite();
        end = System.currentTimeMillis();
        System.out.println("run 3  " + (end - start) + "ms");
        printCacheStats(stats, CACHE_REGION);
        //expects misses and puts to increase and element counts and size to show items stored in cache again
        Assert.assertTrue("entity load count: ", stats.getEntityLoadCount() > 10000);
        Assert.assertEquals("entity fetch count: ", 0, stats.getEntityFetchCount());
        Assert.assertEquals("query hit count: ", 1, stats.getQueryCacheHitCount());
        Assert.assertEquals("query miss count: ", 2, stats.getQueryCacheMissCount());
        Assert.assertEquals("query put count: ", 2, stats.getQueryCachePutCount());
        Assert.assertEquals("L2 element count in memory: ", 1, statsL2.getElementCountInMemory());
        Assert.assertEquals("L2 element count on disk: ", 0, statsL2.getElementCountOnDisk());
        Assert.assertEquals("L2 hit count: ", 1, statsL2.getHitCount());
        Assert.assertEquals("L2 miss count:", 2, statsL2.getMissCount());
        Assert.assertEquals("L2 put count: ", 2, statsL2.getPutCount());
        Assert.assertTrue("L2 size in memory: ", statsL2.getSizeInMemory() > 100000);
    }

    /**
     * Gets the drugPairDAOImpl.
     * 
     * @return the drugPairDAOImpl
     */
    public DrugPairDAOImpl getDrugPairDAOImpl() {
        return drugPairDAOImpl;
    }

    /**
     * Sets the drugPairDAOImpl.
     * 
     * @param drugPairDAOImpl
     *            the drugPairDAOImpl to set
     */
    public void setDrugPairDAOImpl(DrugPairDAOImpl drugPairDAOImpl) {
        this.drugPairDAOImpl = drugPairDAOImpl;
    }

    /**
     * Gets the drugInteractionDAOImpl.
     * 
     * @return the drugInteractionDAOImpl
     */
    public DrugDrugInteractionDAOImpl getDrugInteractionDAOImpl() {
        return drugInteractionDAOImpl;
    }

    /**
     * Sets the drugInteractionDAOImpl.
     * 
     * @param drugInteractionDAOImpl
     *            the drugInteractionDAOImpl to set
     */
    public void setDrugInteractionDAOImpl(DrugDrugInteractionDAOImpl drugInteractionDAOImpl) {
        this.drugInteractionDAOImpl = drugInteractionDAOImpl;
    }

    /**
     * Gets the terminologyDAOImpl.
     * 
     * @return the terminologyDAOImpl
     */
    public TerminologyDAOImpl getTerminologyDAOImpl() {
        return terminologyDAOImpl;
    }

    /**
     * Sets the terminologyDAOImpl.
     * 
     * @param terminologyDAOImpl
     *            the terminologyDAOImpl to set
     */
    public void setTerminologyDAOImpl(TerminologyDAOImpl terminologyDAOImpl) {
        this.terminologyDAOImpl = terminologyDAOImpl;
    }

}
