package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosing;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosingDAO;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeUserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeXref;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.Test;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;

public class DoseRangeUserCommentDAOTest extends AbstractUnitTest {
    @Resource(name = "fdbDosingDAO")
    private FdbDosingDAO fdbDosingDAO;

    @Resource
    private ConceptXrefDAO conceptXrefDao;

    /**
     * /11|6|6570|23725|999|99|999|3
     * 
     * private static final int CONCEPT_ID = 0; private static final int CONCEPT_TYPE = 1; private static final int
     * AGE_LOW_IN_DAYS = 2; private static final int AGE_HIGH_IN_DAYS = 3;
     * 
     * private static final int DOSE_ROUTE_ID = 4; private static final int DOSE_TYPE_ID = 5; private static final int FDBDX =
     * 6; private static final int HIT_TYPE = 7;
     */
    @Test
    @Transactional
    @Rollback(false)
    public void testNewXref() {
        DoseRangeXref xref = new DoseRangeXref();
        DoseRangeKey key = populateKey();
        xref.setDoseRangeKey(key);
        DoseRangeUserComment comment = new DoseRangeUserComment();
        xref.addUserComment(comment);
        comment.setCommentDate(new Date());
        comment.setComment("comment number 1");
        comment.setUserName("ROB");

        conceptXrefDao.save(xref);
        Assert.assertNotNull(xref.getFdbReferenceId());
        // drugDrugInteractionUserCommentDAO.save(comment);
        Assert.assertNotNull(comment.getCommentId());

    }

    /**
     * @return
     */
    private DoseRangeKey populateKey() {
        DoseRangeKey key = new DoseRangeKey();
        key.setAgeHighInDays(3L);
        key.setAgeLowInDays(1L);
        key.setConceptId(11L);
        key.setConceptType(6L);
        key.setDoseRouteId("999");
        key.setDoseTypeId("99");
        key.setFdbdx("999");
        key.setHitType(0L);
        return key;
    }

    @Test
    @Transactional
    @Rollback(false)
    public void testSaveExistingXref() {

        ConceptXref xref = conceptXrefDao.findDoseRangexRefByFDBId(populateKey());
        DoseRangeUserComment comment = new DoseRangeUserComment();
        xref.addUserComment(comment);
        comment.setCommentDate(new Date());
        comment.setComment("Test Comments Number 6");
        comment.setUserName("   ");
        conceptXrefDao.save(xref);
        Assert.assertNotNull(comment.getCommentId());
    }

    @Transactional
    private ConceptXref getXref(DoseRangeKey key) {

        FdbDosing fdbDosing = fdbDosingDAO.find(key);

        return fdbDosing.getConceptXref();
    }

    @Test
    @Transactional
    @Rollback(false)
    public void testFindCustomByxRef() {
        List<DoseRangeVersionVa> customRecords = conceptXrefDao.findCustomByXref(populateKey());

        Assert.assertEquals(false, customRecords.isEmpty());
    }

}
