package gov.va.med.pharmacy.pecs.persistence;

import static org.quartz.DateBuilder.newDate;
import static org.quartz.JobBuilder.newJob;
import static org.quartz.SimpleScheduleBuilder.simpleSchedule;
import static org.quartz.TriggerBuilder.newTrigger;
import static org.quartz.TriggerKey.triggerKey;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Test;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.SchedulerFactory;
import org.quartz.Trigger;
import org.quartz.impl.StdSchedulerFactory;

public class QuartzSchedulerTest extends AbstractUnitTest {

    private static final Log LOGGER = LogFactory.getLog(QuartzSchedulerTest.class);

    @Test
    public void testScheduledJob() throws SchedulerException, InterruptedException {
        LOGGER.info("------- Initializing -------------------");

        SchedulerFactory sf = new StdSchedulerFactory();

        Scheduler scheduler = sf.getScheduler();

        LOGGER.info("------- Initialization Complete --------");

        LOGGER.info("------- Jobs ----------------");

        JobDetail job = newJob(FDBComparisonReportJob.class).withIdentity("fdbComparisonReportJob").build();

        LOGGER.info("------- Triggers ----------------");
        Trigger trigger =
                newTrigger().withIdentity(triggerKey("fdbComparisonReportTrigger"))
                        .withSchedule(simpleSchedule().withIntervalInHours(24).repeatForever()) // forever
                        .startAt(newDate().atHourMinuteAndSecond(11, 20, 00).build()).forJob("fdbComparisonReportJob").build(); // 12:20
                                                                                                                                // (0-23
        // hour)

        LOGGER.info("------- Schedule Jobs ----------------");
        scheduler.scheduleJob(job, trigger);

        LOGGER.info("------- Schedule Start ----------------");
        scheduler.start();

        Thread.sleep(90L * 1000L);

        scheduler.shutdown(true);

    }
}