package gov.va.med.pharmacy.pecs.persistence.contact;

import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;
import gov.va.med.pharmacy.pecs.util.BlobHandler;

import java.util.Date;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.hibernate.Hibernate;
import org.junit.Test;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;

public class ContactUsDAOTest extends AbstractUnitTest implements InitializingBean {
    @Resource
    ContactUsDAOImpl contactUsDAOImpl;
    @Resource
    BlobHandler blobHandler;

    private static final String CONTACT_US_STRING = "<a>   </a><br><a>Don</a>";

    @Test
    @Transactional
    @Rollback(value = false)
    public void testSave() {
        ContactUs contactUs = new ContactUs();
        contactUs.setCreatedBy("ME");
        contactUs.setCreatedDate(new Date());
        contactUs.setContent(blobHandler.createBlob(CONTACT_US_STRING.getBytes()));
        contactUsDAOImpl.save(contactUs);
        Assert.assertNotNull(contactUs.getContactId());
        Assert.assertNotSame(0, contactUs.getContactId());
    }

    @Test
    @Transactional
    @Rollback(value = false)
    public void testRetrieve() throws Exception {
        ContactUs contactUs = contactUsDAOImpl.findById(7);
        Assert.assertNotNull(contactUs.getContent());
        Assert.assertEquals(CONTACT_US_STRING, new String(blobHandler.toByteArray(contactUs.getContent())));
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (contactUsDAOImpl == null) {
            throw new Exception("ContactUsDaoImpl has not been injected into " + this.getClass().getName());
        }
        if (blobHandler == null) {
            throw new Exception("BlobHandler has not been injected into " + this.getClass().getName());
        }
    }

    @Test
    @Transactional
    public void testSaveDAO() {
        ContactUs contactUs = new ContactUs();
        contactUs.setContent(Hibernate.createBlob("<b>Test Content 1 </b>".getBytes(), contactUsDAOImpl.getSession()));

        contactUsDAOImpl.save(contactUs);
    }
}
