package gov.va.med.pharmacy.pecs.persistence.contact;

import java.util.Date;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.Test;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;
import gov.va.med.pharmacy.pecs.util.BlobHandler;

public class ContactUsTest extends AbstractUnitTest implements InitializingBean {

    @Resource
    private ContactUsDAOImpl contactUsDaoImpl;

    @Resource
    private BlobHandler blobHandler;
    
    private static final String CONTACT_US_STRING = "<a>   </a><br><a>Don</a>";
    
    @Test
    @Transactional
    @Rollback(value=false)
    public void testSave() {
        ContactUs contactUs = new ContactUs();
        contactUs.setCreatedBy("ME");
        contactUs.setCreatedDate(new Date());
        contactUs.setContent(blobHandler.createBlob(CONTACT_US_STRING.getBytes()));
        contactUsDaoImpl.save(contactUs);
        Assert.assertNotNull(contactUs.getContactId());
        Assert.assertNotSame(0, contactUs.getContactId());
    }

    @Test
    @Transactional
    @Rollback(value=false)
    public void testRetrieve() throws Exception {
        ContactUs contactUs = contactUsDaoImpl.findById(1);
        Assert.assertNotNull(contactUs.getContent());
        Assert.assertEquals(CONTACT_US_STRING, new String(blobHandler.toByteArray(contactUs.getContent())));
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if ( contactUsDaoImpl == null ) {
            throw new Exception("ContactUsDaoImpl has not been injected into " + this.getClass().getName());
        }
        if ( blobHandler == null ) {
            throw new Exception("BlobHandler has not been injected into " + this.getClass().getName());
        }
    }
}
