package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.ConceptQueryFactory;
import gov.va.med.pharmacy.pecs.query.DoJoGridSuport;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.Test;
import org.springframework.transaction.annotation.Transactional;

public class DrugDrugInteractionServiceTest extends AbstractUnitTest {

    private String QUERYCUSTOMWITHCOMMENTSBYFDBID = "[{ \"field\":\"FDB_ID\", \"operator\":\"EQ\", \"value\":\"1\" }]/[:100]";
    private String QUERYFDBWITHCOMMENTSBYINTERACTIONID = "[{ \"field\":\"INTERACTIONID\", \"operator\":\"EQ\", \"value\":\"1\" }]/[:100]";
    private String QUERYFDBWITHNOCOMMENTSBYINTERACTIONID = "[{ \"field\":\"INTERACTIONID\", \"operator\":\"EQ\", \"value\":\"2\" }]/[:100]";
    private boolean INCLUDEHISTORY = false;
    
    @Resource
    private DrugDrugInteractionService service;
    @Resource(name="jsonConceptQueryFactory")
    private ConceptQueryFactory factory;
    
    @Test
    @Transactional
    public void testFDBAdvancedQueryWithComments() throws Exception {
        Page page = executeQuery(QUERYFDBWITHCOMMENTSBYINTERACTIONID, false);
        Assert.assertNotNull(page);
        Assert.assertEquals(1, page.getResults().size());
        DrugDrugInteractionDTO ddi = (DrugDrugInteractionDTO) page.getResults().get(0);
        Assert.assertEquals(6, ddi.getUserCommentHistory().size());
    }

    @Test
    @Transactional
    public void testFDBAdvancedQueryWithNoComments() throws Exception {
        Page page = executeQuery(QUERYFDBWITHNOCOMMENTSBYINTERACTIONID, false);
        Assert.assertNotNull(page);
        Assert.assertEquals(1, page.getResults().size());
        DrugDrugInteractionDTO ddi = (DrugDrugInteractionDTO) page.getResults().get(0);
        Assert.assertEquals(0, ddi.getUserCommentHistory().size());
    }

    @Test
    @Transactional
    public void testCustomAdvancedQueryWithComments() throws Exception {
        Page page = executeQuery(QUERYCUSTOMWITHCOMMENTSBYFDBID, true);
        Assert.assertNotNull(page);
        Assert.assertEquals(1, page.getResults().size());
        DrugDrugInteractionDTO ddi = (DrugDrugInteractionDTO) page.getResults().get(0);
        Assert.assertEquals(6, ddi.getUserCommentHistory().size());
    }
    
    private Page executeQuery(String jsonQuery, boolean isCustom ) throws Exception {
        Page page = null;
        DoJoGridSuport support = new DoJoGridSuport(jsonQuery);
        if ( support.isParsed() ) {
            ConceptQuery query = factory.createQuery(service.getMetadata(), support.getJson());
            query.setIncludeHistory(INCLUDEHISTORY);
            if ( isCustom ) {
                page = service.getStagingData(query, support.getPageNumber());
            } else {
                page = service.getFdbData(query, support.getPageNumber());
            }
        } else {
            throw new Exception("Cannot parse JSON query string.");
        }
        return page;
    }
}
