package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateParser;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;

import javax.annotation.Resource;

import org.junit.Test;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.transaction.annotation.Transactional;

public class RoutedGenericsProcessorTest extends AbstractUnitTest implements InitializingBean {

    @Resource
    private RoutedGenericsProcessor routedGenericsProcessor;
    @Resource
    private DrugPairService drugPairService;
    
    // File location is relative to project home.
    private String ROUTED_GENERIC_UPDATE_FILE = "bin/files/incremental/fdb/RoutedGenericUpdates.zip";

    @Test
    @Transactional
    //@Rollback(value = false)
    public void testProcessAdd() throws Exception {
        RoutedGenericUpdateDataAugmentation aug = getAugmentation(ROUTED_GENERIC_UPDATE_FILE);
        routedGenericsProcessor.setDataAugmentation(aug);
        routedGenericsProcessor.process();
    }

    /**
     * Opens the requested updateFileName, creates the parser and input streams and returns
     * the RoutedGenericUpdateDataAugmentation object.
     * 
     * @param updateFileName
     * @return RoutedGenericUpdateDataAugmentation
     * @throws Exception
     * @throws IOException
     * @throws FileNotFoundException
     */
    private RoutedGenericUpdateDataAugmentation getAugmentation(String updateFileName) throws Exception, IOException, FileNotFoundException {
        FdbUpdateParser parser = new FdbUpdateParser( new File(updateFileName));
        File f = parser.parseConcept(FdbUpdateParser.FDB_GENERIC_ROUTEDDRUG);
        ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
        RoutedGenericUpdateDataAugmentation aug = new RoutedGenericUpdateDataAugmentation(drugPairService);
        aug.setInput(in);
        return aug;
    }

    /**
     * Makes sure all Spring Beans have been injected.
     * 
     * @throws Exception if a bean has not been injected
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if ( routedGenericsProcessor == null ) {
            throw new Exception("RoutedGenericsProcessor has not been injected into " + this.getClass().getName());
        }
        if ( drugPairService == null ) {
            throw new Exception("DrugPairService has not been injected into " + this.getClass().getName());
        }
    }
}
