package gov.va.med.pharmacy.pecs.customupdate;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * The Class CtVersionVa.
 */
public final class CtVersionVa {

    /** The va date format. */
    private static DateFormat VA_DATE_FORMAT = new SimpleDateFormat("MM-dd-yyyy");

    /** The db date format. */
    private static DateFormat DB_DATE_FORMAT = new SimpleDateFormat("yyyyMMdd");

    /** The ct version key. */
    private long ctVersionKey;

    /** The fdb versionkey. */
    private long fdbVersionkey;

    /** The ct dbversion. */
    private String ctDbversion;

    /** The ct buildversion. */
    private String ctBuildversion;

    /** The ct frequency. */
    private String ctFrequency;

    /** The ct issuedate. */
    private Date ctIssuedate;

    /** The ct versioncomment. */
    private String ctVersioncomment;

    /** The ct dbtype. */
    private String ctDbtype;

    /**
     * default constructor.
     */
    public CtVersionVa() {
    }

    /**
     * minimal constructor.
     * 
     * @param fdbVersionkey
     *            the fdb versionkey
     */
    public CtVersionVa(long fdbVersionkey) {
        this.fdbVersionkey = fdbVersionkey;
    }

    /**
     * Gets the ct issuedate display.
     * 
     * @return the ct issuedate display
     */
    public String getCtIssuedateDisplay() {
        if (ctIssuedate != null) {
            try {
                return VA_DATE_FORMAT.format(getCtIssuedate());
            } catch (Exception pex) {
                return DB_DATE_FORMAT.format(getCtIssuedate());
            }
        }

        return DB_DATE_FORMAT.format(getCtIssuedate());
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "CtVersionVa [ctVersionKey=" + ctVersionKey + ", fdbVersionkey=" + fdbVersionkey + ", ctDbversion="
                + ctDbversion + ", ctBuildversion=" + ctBuildversion + ", ctFrequency=" + ctFrequency + ", ctIssuedate="
                + ctIssuedate + ", ctVersioncomment=" + ctVersioncomment + ", ctDbtype=" + ctDbtype + "]";
    }

    /**
     * Gets the ct version key.
     * 
     * @return the ct version key
     */
    public long getCtVersionKey() {
        return this.ctVersionKey;
    }

    /**
     * Sets the ct version key.
     * 
     * @param ctVersionKey
     *            the new ct version key
     */
    public void setCtVersionKey(long ctVersionKey) {
        this.ctVersionKey = ctVersionKey;
    }

    /**
     * Gets the fdb versionkey.
     * 
     * @return the fdb versionkey
     */
    public long getFdbVersionkey() {
        return this.fdbVersionkey;
    }

    /**
     * Sets the fdb versionkey.
     * 
     * @param fdbVersionkey
     *            the new fdb versionkey
     */
    public void setFdbVersionkey(long fdbVersionkey) {
        this.fdbVersionkey = fdbVersionkey;
    }

    /**
     * Gets the ct dbversion.
     * 
     * @return the ct dbversion
     */
    public String getCtDbversion() {
        return this.ctDbversion;
    }

    /**
     * Sets the ct dbversion.
     * 
     * @param ctDbversion
     *            the new ct dbversion
     */
    public void setCtDbversion(String ctDbversion) {
        this.ctDbversion = ctDbversion;
    }

    /**
     * Gets the ct buildversion.
     * 
     * @return the ct buildversion
     */
    public String getCtBuildversion() {
        return this.ctBuildversion;
    }

    /**
     * Sets the ct buildversion.
     * 
     * @param ctBuildversion
     *            the new ct buildversion
     */
    public void setCtBuildversion(String ctBuildversion) {
        this.ctBuildversion = ctBuildversion;
    }

    /**
     * Gets the ct frequency.
     * 
     * @return the ct frequency
     */
    public String getCtFrequency() {
        return this.ctFrequency;
    }

    /**
     * Sets the ct frequency.
     * 
     * @param ctFrequency
     *            the new ct frequency
     */
    public void setCtFrequency(String ctFrequency) {
        this.ctFrequency = ctFrequency;
    }

    /**
     * Gets the ct issuedate.
     * 
     * @return the ct issuedate
     */
    public Date getCtIssuedate() {
        return this.ctIssuedate;
    }

    /**
     * Sets the ct issuedate.
     * 
     * @param ctIssuedate
     *            the new ct issuedate
     */
    public void setCtIssuedate(Date ctIssuedate) {
        this.ctIssuedate = ctIssuedate;
    }

    /**
     * Gets the ct versioncomment.
     * 
     * @return the ct versioncomment
     */
    public String getCtVersioncomment() {
        return this.ctVersioncomment;
    }

    /**
     * Sets the ct versioncomment.
     * 
     * @param ctVersioncomment
     *            the new ct versioncomment
     */
    public void setCtVersioncomment(String ctVersioncomment) {
        this.ctVersioncomment = ctVersioncomment;
    }

    /**
     * Gets the ct dbtype.
     * 
     * @return the ct dbtype
     */
    public String getCtDbtype() {
        return this.ctDbtype;
    }

    /**
     * Sets the ct dbtype.
     * 
     * @param ctDbtype
     *            the new ct dbtype
     */
    public void setCtDbtype(String ctDbtype) {
        this.ctDbtype = ctDbtype;
    }

}
