package gov.va.med.pharmacy.pecs.customupdate;

/**
 * The Class CustomUpdateRecordCounts.
 */
public final class CustomUpdateRecordCounts {

    /** The duplicate therapy counts. */
    private RecordCounts duplicateTherapyCounts;

    /** The dose range counts. */
    private RecordCounts doseRangeCounts;

    /** The professional monograph counts. */
    private RecordCounts professionalMonographCounts;

    /** The drug pair counts. */
    private RecordCounts drugPairCounts;

    /** The drug drug interaction counts. */
    private RecordCounts drugDrugInteractionCounts;

    /** The drug drug interaction string counts. */
    private RecordCounts drugDrugInteractionStringsCounts;

    /**
     * Gets the duplicate therapy counts.
     * 
     * @return the duplicate therapy counts
     */
    public RecordCounts getDuplicateTherapyCounts() {
        return duplicateTherapyCounts;
    }

    /**
     * Sets the duplicate therapy counts.
     * 
     * @param duplicateTherapyCounts
     *            the new duplicate therapy counts
     */
    public void setDuplicateTherapyCounts(RecordCounts duplicateTherapyCounts) {
        this.duplicateTherapyCounts = duplicateTherapyCounts;
    }

    /**
     * Gets the formated duplicate therapy total.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the formated duplicate therapy total
     */
    public String getFormatedDuplicateTherapyTotal(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DTCAT", fileName, delimiter, this.duplicateTherapyCounts.getTotalCount(),
                terminator);
    }

    /**
     * Gets the dose range counts.
     * 
     * @return the dose range counts
     */
    public RecordCounts getDoseRangeCounts() {
        return doseRangeCounts;
    }

    /**
     * Sets the dose range counts.
     * 
     * @param doseRangeCounts
     *            the new dose range counts
     */
    public void setDoseRangeCounts(RecordCounts doseRangeCounts) {
        this.doseRangeCounts = doseRangeCounts;
    }

    /**
     * Gets the formated dose range total.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the formated dose range total
     */
    public String getFormatedDoseRangeTotal(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DOSING", fileName, delimiter, this.doseRangeCounts.getTotalCount(), terminator);
    }

    /**
     * Gets the professional monograph counts.
     * 
     * @return the professional monograph counts
     */
    public RecordCounts getProfessionalMonographCounts() {
        return professionalMonographCounts;

    }

    /**
     * Sets the professional monograph counts.
     * 
     * @param professionalMonographCounts
     *            the new professional monograph counts
     */
    public void setProfessionalMonographCounts(RecordCounts professionalMonographCounts) {
        this.professionalMonographCounts = professionalMonographCounts;
    }

    /**
     * Gets the formated professional monograph total.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the formated professional monograph total
     */
    public String getFormatedProfessionalMonographTotal(String fileName, String delimiter, String terminator) {
        //@formatter:off
        return formatUpdateRecord("FDB_CUSTOM_MONOGRAPH", fileName, delimiter, 
                this.professionalMonographCounts.getTotalCount(), terminator);
        //@formatter:on
    }

    /**
     * Gets the drug pair counts.
     * 
     * @return the drug pair counts
     */
    public RecordCounts getDrugPairCounts() {
        return drugPairCounts;
    }

    /**
     * Sets the drug pair counts.
     * 
     * @param drugPairCounts
     *            the new drug pair counts
     */
    public void setDrugPairCounts(RecordCounts drugPairCounts) {
        this.drugPairCounts = drugPairCounts;
    }

    /**
     * Gets the formated drug pairs total.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the formated drug pairs total
     */
    public String getFormatedDrugPairsTotal(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DDIM", fileName, delimiter, this.drugPairCounts.getTotalCount(), terminator);
    }

    /**
     * Gets the drug drug interaction counts.
     * 
     * @return the drug drug interaction counts
     */
    public RecordCounts getDrugDrugInteractionCounts() {
        return drugDrugInteractionCounts;
    }

    /**
     * Sets the drug drug interaction counts.
     * 
     * @param drugDrugInteractionCounts
     *            the new drug drug interaction counts
     */
    public void setDrugDrugInteractionCounts(RecordCounts drugDrugInteractionCounts) {
        this.drugDrugInteractionCounts = drugDrugInteractionCounts;
    }

    /**
     * Gets the formated drug drug interaction total.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the formated drug drug interaction total
     */
    public String getFormatedDrugDrugInteractionTotal(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DDIMINTERACTION", fileName, delimiter,
                this.drugDrugInteractionCounts.getTotalCount(), terminator);
    }

    /**
     * Gets the drug drug interaction strings counts.
     * 
     * @return the drug drug interaction strings counts
     */
    public RecordCounts getDrugDrugInteractionStringsCounts() {
        return drugDrugInteractionStringsCounts;
    }

    /**
     * Sets the drug drug interaction strings counts.
     * 
     * @param drugDrugInteractionStringsCounts
     *            the new drug drug interaction string counts
     */
    public void setDrugDrugInteractionStringsCounts(RecordCounts drugDrugInteractionStringsCounts) {
        this.drugDrugInteractionStringsCounts = drugDrugInteractionStringsCounts;
    }

    /**
     * Gets the formated drug drug interaction strings total.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the formated drug drug interaction strings total
     */
    public String getFormatedDrugDrugInteractionStringsTotal(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DDIMSTRINGS", fileName, delimiter,
                this.drugDrugInteractionStringsCounts.getTotalCount(), terminator);
    }

    /**
     * Gets the ct version update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the ct version update record
     */
    public String getCtVersionUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("CT_VERSION", fileName, delimiter, 1L, terminator);
    }

    /**
     * Format update record.
     * 
     * @param tableName
     *            the table name
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param counts
     *            the counts
     * @param terminator
     *            the terminator
     * @return the string
     */
    private String formatUpdateRecord(String tableName, String fileName, String delimiter, Long counts, String terminator) {
        StringBuffer sb = new StringBuffer();
        sb.append(fileName);
        sb.append(delimiter);
        sb.append(tableName);
        sb.append(delimiter);
        sb.append(counts.longValue());
        sb.append(terminator);
        return sb.toString();
    }
}
