/*
 * 
 */
package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVA;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

;

/**
 * The Class DoseRangeUpdateFileSupport.
 */
@Service
public final class DoseRangeUpdateFileSupport extends BaseUpdateFileSupport {

    /** The Constant logger. */
    private static final Logger LOGGER = Logger.getLogger(DoseRangeUpdateFileSupport.class);

    /** The Constant TABLE_NAME. */
    public static final String TABLE_NAME = "fdb_custom_dosing";

    /** The Constant DOSE_RANGE_FILE_NAME. */
    public static final String DOSE_RANGE_FILE_NAME = "FDBCUSTOMDOSERANGE";

    /** The Constant FIELDS_IN_RECORD. */
    private static final int FIELDS_IN_RECORD = 64;

    /** The Constant FIELD_DEF_RECORD. */
    private static final String FIELD_DEF_RECORD = "F|" + FIELDS_IN_RECORD
            + "|1|category|2|concepttype|3|conceptid|4|agelowindays|5|agehighindays|6|doserouteid|7|dosetypeid|8|fdbdx|9"
            + "|doselow|10|"
            + "doselowunits|11|dosehigh|12|dosehighunits|13|doseformlow|14|doseformlowunits|15|doseformhigh|16"
            + "|doseformhighunits|17|maxsingledose|18|maxsingledoseunits|19|maxsingledoseform|20|"
            + "maxsingledoseformunits|21|maxdailydose|22|maxdailydoseunits|23|maxdailydoseform|24"
            + "|maxdailydoseformunits|25|maxlifetimedose|26|maxlifetimedoseunits|27|maxlifetimedoseform"
            + "|28|maxlifetimedoseformunits|29|loweliminationhalflife|30|"
            + "higheliminationhalflife|31|halflifeunits|32|frequencylow|33|frequencyhigh|34|durationlow|35|"
            + "durationhigh|36|maxduration|37|hepaticimpairmentindicator|38|renalimpairmentindicator|39|"
            + "crclthreshhold|40|" + "crclthreshholdunits|41|weightimpairmentindicator|42|bsarequiredindicator|43|doseratelow"
            + "|44|doseratelowunits|45|doseratehigh|46|doseratehighunits|47|doseformratelow|48|doseformratelowunits"
            + "|49|doseformratehigh|50|"
            + "doseformratehighunits|51|maxsingledoserate|52|maxsingledoserateunits|53|maxsingledoseformrate"
            + "|54|maxsingledoseformrateunits|55|maxdailydoserate|56|maxdailydoserateunits|57|maxdailydoseformrate"
            + "|58|maxdailydoseformrateunits|59|dxid|60|ntesingledose|61|ntesingledoseunits"
            + "|62|ntesingledoseform|63|ntesingledoseformunits|64|agesourcecode";

    /** The Constant PRIMARY_KEY_RECORD. */
    private static final String PRIMARY_KEY_RECORD = "P|8|1|2|3|4|5|6|7|8";

    /**
     * This instance method returns the FDB update file Field Definition Record. The record will be specific to the Order Check
     * type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Field Definition Record for this Order Check type.
     */
    @Override
    public String generateFieldDefRecord() {
        return FIELD_DEF_RECORD;
    }

    /**
     * this method returns the FDB update file Primary Key Record. The record will be specific to the Order Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Primary Key Record for this Order Check type.
     */
    @Override
    public String generatePrimaryKeyRecord() {
        return PRIMARY_KEY_RECORD;
    }

    /**
     * Method to generate the full record file.
     * 
     * @param out
     *            the out
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#doGenerateFullRecords(java.io.OutputStream)
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateFullRecords(OutputStream out) {

        List<DoseRangeVA> list = (List<DoseRangeVA>) getGenericDao().findCustomUpdateRecords(getDomainObjectName());
        LOGGER.info("Inside doGenerateFullRecords() method");

        if (!list.isEmpty()) {
            for (DoseRangeVA m : list) {
                // for full updates we're using the same field order as for ADD
                for (Object[] values : getAddFields(m.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, true);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write FULL record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }

        return;
    }

    /**
     * Do generate add records.
     * 
     * @param zout
     *            the zout
     * @param fileDate
     *            the file date
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateAddRecords(ZipOutputStream zout, Date fileDate) {

        List<DoseRangeVA> list = (List<DoseRangeVA>) getGenericDao().findCustomUpdateAddRecords(getDomainObjectName());
        LOGGER.info("Inside doGenerateAddRecords() method");
        if (!list.isEmpty()) {
            for (DoseRangeVA record : list) {
                for (Object[] values : getAddFields(record.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
    }

    /**
     * Do generate change records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateChangeRecords(ZipOutputStream out, Date fileDate) {
        LOGGER.info("inside doGenerateChangeRecords() method");
        List<DoseRangeVA> list = (List<DoseRangeVA>) getGenericDao().findCustomUpdateChangeRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DoseRangeVA record : list) {
                // For doseRanges, CHANGE is implemented as DELETE+ADD.
                // The fact that a change could cause the number of lines
                // for a given section to change makes it tricky to implement
                // this otherwise (e.g. if a section goes from n lines to n-1,
                // we'd have to generate n-1 CHANGE records and 1 DELETE).
                // We're not concerned about the run-time performance and
                // functionally this does the job
                for (Object[] values : getDeleteFields(record.getExportedVersion())) {
                    // delete all lines of the previously exported version
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                for (Object[] values : getAddFields(record.getExportableVersion())) {
                    // add the lines of the currently exportable version
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
    }

    /**
     * Extract the values needed for a CHANGE record from this doseRange version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getAddFields(DoseRangeVersionVa version) {
        LOGGER.info("Inside getAddFields() method");

        ArrayList<Object[]> res = new ArrayList<Object[]>();
        Object[] v = new Object[FIELDS_IN_RECORD];
        v[0] = version.getCategory();
        LOGGER.debug("Inside getAddFields() method value of v[0] category" + v[0]);
        v[1] = version.getConceptType();
        LOGGER.debug("Inside getAddFields() method value of v[1] concept type" + v[1]);
        v[2] = version.getConceptid();
        LOGGER.debug("Inside getAddFields() method value of v[2] concept id" + v[2]);
        v[3] = version.getAgeLowInDays();
        LOGGER.debug("Inside getAddFields() method value of v[3] agelowindays" + v[3]);
        v[4] = version.getAgeHighInDays();
        LOGGER.debug("Inside getAddFields() method value of v[4] agehighindays" + v[4]);
        v[5] = version.getDoseRoute().getCode();
        LOGGER.debug("Inside getAddFields() method value of v[5] dose route" + v[5]);
        v[6] = version.getDoseType().getCode();
        LOGGER.debug("Inside getAddFields() method value of v[6] dosetype" + v[6]);
        v[7] = version.getFdbdx();
        LOGGER.debug("Inside getAddFields() method value of v[7] fdbdx" + v[7]);
        v[8] = version.getDoseLow();
        LOGGER.debug("Inside getAddFields() method value of v[8] doselow" + v[8]);
        v[9] = version.getDoseLowUnits();
        LOGGER.debug("Inside getAddFields() method value of v[9] doselowunit" + v[9]);
        v[10] = version.getDoseHigh();
        LOGGER.debug("Inside getAddFields() method value of v[10] dosehigh" + v[10]);
        v[11] = version.getDoseHighUnits();
        LOGGER.debug("Inside getAddFields() method value of v[11] dosehighunits" + v[11]);
        v[12] = version.getDoseFormLow();
        LOGGER.debug("Inside getAddFields() method value of v[12] doseformlow" + v[12]);
        v[13] = version.getDoseFormLowUnits();
        LOGGER.debug("Inside getAddFields() method value of v[13] doseformlowunits" + v[13]);
        v[14] = version.getDoseFormHigh();
        LOGGER.debug("Inside getAddFields() method value of v[14]doseformhigh" + v[14]);
        v[15] = version.getDoseFormHighUnits();
        LOGGER.debug("Inside getAddFields() method value of v[15]doseformhighunits" + v[15]);
        v[16] = version.getMaxSingleDose();
        LOGGER.debug("Inside getAddFields() method value of v[16]maxsingledose" + v[16]);
        v[17] = version.getMaxSingleDoseUnits();
        LOGGER.debug("Inside getAddFields() method value of v[17]maxsingledoseunits" + v[17]);
        v[18] = version.getMaxSingleDoseForm();
        LOGGER.debug("Inside getAddFields() method value of v[18]maxsingledoseform" + v[18]);
        v[19] = version.getMaxSingleDoseFormUnits();
        LOGGER.debug("Inside getAddFields() method value of v[19]maxsingledoseformunits" + v[19]);

        v[20] = version.getMaxDailyDose();
        LOGGER.debug("Inside getAddFields() method value of v[20]maxdailydose" + v[20]);
        v[21] = version.getMaxDailyDoseUnits();
        LOGGER.debug("Inside getAddFields() method value of v[21] maxdailydoseunits" + v[21]);
        v[22] = version.getMaxDailyDoseForm();
        LOGGER.debug("Inside getAddFields() method value of v[22]maxdailydoseform" + v[22]);
        v[23] = version.getMaxDailyDoseFormUnits();
        LOGGER.debug("Inside getAddFields() method value of v[23]maxdailydoseformunits" + v[23]);
        v[24] = version.getMaxLifeTimeDose();
        LOGGER.debug("Inside getAddFields() method value of v[24]maxlifetimedose" + v[24]);
        v[25] = version.getMaxLifeTimeDoseUnits();
        LOGGER.debug("Inside getAddFields() method value of v[25]maxlifetimedoseunits" + v[25]);
        v[26] = version.getMaxLifeTimeDoseForm();
        LOGGER.debug("Inside getAddFields() method value of v[26]maxlifetimedoseform" + v[26]);
        v[27] = version.getMaxLifeTimeDoseFormUnits();
        LOGGER.debug("Inside getAddFields() method value of v[27]maxlifetimedoseformunits" + v[27]);
        v[28] = version.getLowEliminationHalfLife();
        LOGGER.debug("Inside getAddFields() method value of v[28]loweliminationhalflife" + v[28]);
        v[29] = version.getHighEliminationHalfLife();
        LOGGER.debug("Inside getAddFields() method value of v[29]higheliminationhalflife" + v[29]);

        v[30] = version.getHalfLifeUnits();
        LOGGER.debug("Inside getAddFields() method value of v[30]" + v[30]);
        v[31] = version.getFrequencyLow();
        LOGGER.debug("Inside getAddFields() method value of v[31]" + v[31]);
        v[32] = version.getFrequencyHigh();
        LOGGER.debug("Inside getAddFields() method value of v[32]" + v[32]);
        v[33] = version.getDurationLow();
        LOGGER.debug("Inside getAddFields() method value of v[33]" + v[33]);
        v[34] = version.getDurationHigh();
        LOGGER.debug("Inside getAddFields() method value of v[34]" + v[34]);
        v[35] = version.getMaxDuration();
        LOGGER.debug("Inside getAddFields() method value of v[35]" + v[35]);
        v[36] = version.getHepaticImpairmentIndicator();
        LOGGER.debug("Inside getAddFields() method value of v[36]" + v[36]);
        v[37] = version.getRenalImpairmentIndicator();
        LOGGER.debug("Inside getAddFields() method value of v[37]" + v[37]);
        v[38] = version.getCrclThreshold();
        LOGGER.debug("Inside getAddFields() method value of v[38]" + v[38]);
        v[39] = version.getCrclThresholdUnits();
        LOGGER.debug("Inside getAddFields() method value of v[39]" + v[39]);
        v[40] = version.getWeightImpairmentIndicator();
        LOGGER.debug("Inside getAddFields() method value of v[40]" + v[40]);
        v[41] = version.getBsaRequiredIndicator();
        LOGGER.debug("Inside getAddFields() method value of v[41]" + v[41]);
        v[42] = version.getDoseRateLow();
        LOGGER.debug("Inside getAddFields() method value of v[42]" + v[42]);
        v[43] = version.getDoseRateLowUnits();
        LOGGER.debug("Inside getAddFields() method value of v[43]" + v[43]);
        v[44] = version.getDoseRateHigh();
        LOGGER.debug("Inside getAddFields() method value of v[44]" + v[44]);
        v[45] = version.getDoseRateHighUnits();
        LOGGER.debug("Inside getAddFields() method value of v[45]" + v[45]);
        v[46] = version.getDoseFormRateLow();
        LOGGER.debug("Inside getAddFields() method value of v[46]" + v[46]);
        v[47] = version.getDoseFormRateLowUnits();
        LOGGER.debug("Inside getAddFields() method value of v[47]" + v[47]);
        v[48] = version.getDoseFormRateHigh();
        LOGGER.debug("Inside getAddFields() method value of v[48]" + v[48]);
        v[49] = version.getDoseFormRateHighUnits();
        LOGGER.debug("Inside getAddFields() method value of v[49]" + v[49]);
        v[50] = version.getMaxSingleDoseRate();
        LOGGER.debug("Inside getAddFields() method value of v[50]" + v[50]);
        v[51] = version.getMaxSingleDoseRateUnits();
        LOGGER.debug("Inside getAddFields() method value of v[51]" + v[51]);
        v[52] = version.getMaxSingleDoseFormRate();
        LOGGER.debug("Inside getAddFields() method value of v[52]" + v[52]);
        v[53] = version.getMaxSingleDoseFormRateUnits();
        LOGGER.debug("Inside getAddFields() method value of v[53]" + v[53]);
        v[54] = version.getMaxDailyDoseRate();
        LOGGER.debug("Inside getAddFields() method value of v[54]" + v[54]);
        v[55] = version.getMaxDailyDoseRateUnits();
        LOGGER.debug("Inside getAddFields() method value of v[55]" + v[55]);
        v[56] = version.getMaxDailyDoseFormRate();
        LOGGER.debug("Inside getAddFields() method value of v[56]" + v[56]);
        v[57] = version.getMaxDailyDoseFormRateUnits();
        LOGGER.debug("Inside getAddFields() method value of v[57]" + v[57]);
        v[58] = version.getDxid();
        LOGGER.debug("Inside getAddFields() method value of v[58]" + v[58]);

        v[59] = version.getMaxSingleNTEDose();

        v[60] = version.getMaxSingleNTEDoseUnit();

        v[61] = version.getMaxSingleNTEDoseForm();

        v[62] = version.getMaxSingleNTEDoseFormUnit();
        v[63] = null; // Have to account for the AgeSourceCode in the update file.

        res.add(v);
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Method to generate the delete records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#doGenerateDeleteRecords(java.util.zip.ZipOutputStream,
     *      java.util.Date)
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateDeleteRecords(ZipOutputStream out, Date fileDate) {

        // List<DoseRangeVA> list =
        // doseRangeDAO.getSession().createQuery(DELETE_HQL).list();
        List<DoseRangeVA> list = (List<DoseRangeVA>) getGenericDao().findCustomUpdateDeleteRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DoseRangeVA record : list) {
                for (Object[] values : getDeleteFields(record.getExportedVersion())) {
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
    }

    /**
     * Extract the values needed for a DELETE record from this doseRange version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getDeleteFields(DoseRangeVersionVa version) {
        ArrayList<Object[]> res = new ArrayList<Object[]>();
        Object[] v = new Object[8];
        v[0] = version.getCategory();
        v[1] = version.getConceptType();
        v[2] = version.getConceptid();
        v[3] = version.getAgeLowInDays();
        v[4] = version.getAgeHighInDays();
        v[5] = version.getDoseRoute().getCode();
        v[6] = version.getDoseType().getCode();
        v[7] = version.getFdbdx();
        res.add(v);
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Method to retrieve the Update File Name.
     * 
     * @return the update file name
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getUpdateFileName()
     */
    @Override
    public String getUpdateFileName() {
        return DOSE_RANGE_FILE_NAME;
    }

    /**
     * Method to return the Key Field Count.
     * 
     * @return the key field count
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getKeyFieldCount()
     */
    @Override
    public int getKeyFieldCount() {
        return 5;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getTableName()
     */
    @Override
    public String getTableName() {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getDomainObjectName()
     */
    @Override
    public String getDomainObjectName() {

        return "DoseRangeVA";
    }

    /**
     * Gets the incremental record counts. DoseRange treats Changed Records as Deleted + Add. We Add the number of record counts
     * to add and delete and zero out changed.
     * 
     * @return the incremental record counts
     */
    @Override
    public RecordCounts getIncrementalRecordCounts() {
        RecordCounts recordCounts = getGenericDao().findCustomUpdateCounts(getDomainObjectName());
        if (recordCounts.getChangeCount() != 0) {
            recordCounts.setAddCount(recordCounts.getAddCount() + recordCounts.getChangeCount());
            recordCounts.setDeleteCount(recordCounts.getDeleteCount() + recordCounts.getChangeCount());
            recordCounts.setChangeCount(0L);
        }
        return recordCounts;
    }
}
