package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.DrugPair;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipOutputStream;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

/**
 * DrugPairUpdateFileSupport Entity.
 * 
 * 
 */
@Service
public final class DrugPairUpdateFileSupport extends BaseUpdateFileSupport {

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(DrugPairUpdateFileSupport.class);

    /** The Constant FIELD_DEF_RECORD. */
    private static final String FIELD_DEF_RECORD = "F|10|1|rtgenid1|2|rtgenid2|3|interactionid|4|seqno|5|uicategory1"
            + "|6|uicategory2|7|uicategory3|8|uicategory4|9|uicategory5|10|uicategory6";

    /** The Constant PRIMARY_KEY_RECORD. */
    private static final String PRIMARY_KEY_RECORD = "P|4|1|2|3|4";

    /** The Constant PRIMARY_KEY_RECORD_FIELD_CNT. */
    private static final int PRIMARY_KEY_RECORD_FIELD_CNT = 4;

    /** The Constant TABLE_NAME. */
    private static final String TABLE_NAME = "fdb_custom_ddim";

    /** DPFILENAME. */
    public static final String DP_FILE_NAME = "FDBCUSTOMDDIM";

    /** The Constant UPDATE_EXPORTED_HQL. */
    private static final String UPDATE_EXPORTED_HQL = "update gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairExportManager m "
            + "set m.exportedVersion = m.exportableVersion";

    /** The drug pair dao. */
    @Resource
    private DrugPairDAOImpl drugPairDAO;

    /**
     * Generate Full Records.
     * 
     * @param out
     *            - Output stream
     */
    @SuppressWarnings("unchecked")
    @Override
    public final void doGenerateFullRecords(OutputStream out) {
        List<Object[]> list = (List<Object[]>) getGenericDao().findDrugPairsCustomUpdateRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (Object[] dp : list) {
                for (Object[] values : getAddChangeFields(dp)) {
                    StringBuffer buf = formatAddRecord(values, true);
                    try {
                        out.write(buf.toString().getBytes());
                    } catch (IOException ioe) {
                        String msg = "Unable to write FULL record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }
        return;
    }

    /**
     * Add Change to Fields.
     * 
     * @param dp
     *            - Object []
     * @return res
     */
    private Object[][] getAddChangeFields(Object[] dp) {

        Object[][] res = new Object[2][];

        Object[] forward = new Object[10];
        forward[0] = dp[0];
        forward[1] = dp[1];
        forward[2] = dp[2];
        forward[3] = 1;
        forward[4] = "VA";
        forward[5] = "";
        forward[6] = "";
        forward[7] = "";
        forward[8] = "";
        forward[9] = "";
        res[0] = forward;

        Object[] reverse = new Object[10];
        reverse[0] = dp[1];
        reverse[1] = dp[0];
        reverse[2] = dp[2];
        reverse[3] = 2;
        reverse[4] = "VA";
        reverse[5] = "";
        reverse[6] = "";
        reverse[7] = "";
        reverse[8] = "";
        reverse[9] = "";
        res[1] = reverse;

        return res;
    }

    /**
     * Gets the total record counts.
     * 
     * @return Long the total record counts
     */
    @Override
    public Long getTotalRecordCounts() {
        return (getGenericDao().findCustomUpdateFullCounts(getDomainObjectName()) * 2);

    }

    /**
     * Gets the incremental record counts.
     * 
     * @return RecordCount populated with the incremental record counts
     */
    @Override
    public RecordCounts getIncrementalRecordCounts() {
        RecordCounts recordCounts = getGenericDao().findCustomUpdateCounts(getDomainObjectName());
        recordCounts.setAddCount(recordCounts.getAddCount() * 2);
        recordCounts.setChangeCount(recordCounts.getChangeCount() * 2);
        recordCounts.setDeleteCount(recordCounts.getDeleteCount() * 2);
        return recordCounts;
    }

    /**
     * Add Change Fields.
     * 
     * @param version
     *            - Version<DrugPair>
     * @return res
     */
    private Object[][] getAddChangeFields(Version<DrugPair> version) {
        // the field order is the following:
        // rtgenid1|rtgenid2|interactionid|seqno|uicategory1|uicategory2|uicategory3|uicategory4|uicategory5|uicategory6
        Object[][] res = new Object[2][];

        Object[] forward = new Object[10];
        forward[0] = version.getVersionedObject().getRoutedGeneric1().getIdentifier();
        forward[1] = version.getVersionedObject().getRoutedGeneric2().getIdentifier();
        forward[2] = version.getVersionedObject().getInteraction().getInteractionId();
        forward[3] = 1;
        forward[4] = "VA";
        forward[5] = "";
        forward[6] = "";
        forward[7] = "";
        forward[8] = "";
        forward[9] = "";
        res[0] = forward;

        Object[] reverse = new Object[10];
        reverse[0] = version.getVersionedObject().getRoutedGeneric2().getIdentifier();
        reverse[1] = version.getVersionedObject().getRoutedGeneric1().getIdentifier();
        reverse[2] = version.getVersionedObject().getInteraction().getInteractionId();
        reverse[3] = 2;
        reverse[4] = "VA";
        reverse[5] = "";
        reverse[6] = "";
        reverse[7] = "";
        reverse[8] = "";
        reverse[9] = "";
        res[1] = reverse;

        return res;
    }

    /**
     * Gets the delete fields.
     * 
     * @param version
     *            the version
     * @return the delete fields
     */
    private Object[][] getDeleteFields(DrugPairVa version) {
        // the field order is the following:
        // rtgenid1|rtgenid2|interactionid|seqno
        Object[][] res = new Object[2][];

        Object[] forward = new Object[4];
        forward[0] = version.getRoutedGeneric1().getIdentifier();
        forward[1] = version.getRoutedGeneric2().getIdentifier();
        forward[2] = version.getInteraction().getInteractionId();
        forward[3] = 1;
        res[0] = forward;

        Object[] reverse = new Object[4];
        reverse[0] = version.getRoutedGeneric2().getIdentifier();
        reverse[1] = version.getRoutedGeneric1().getIdentifier();
        reverse[2] = version.getInteraction().getInteractionId();
        reverse[3] = 2;
        res[1] = reverse;

        return res;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#generateFieldDefRecord()
     */
    @Override
    public final String generateFieldDefRecord() {
        return FIELD_DEF_RECORD;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#generatePrimaryKeyRecord()
     */
    @Override
    public final String generatePrimaryKeyRecord() {
        return PRIMARY_KEY_RECORD;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getKeyFieldCount()
     */
    @Override
    public final int getKeyFieldCount() {
        return PRIMARY_KEY_RECORD_FIELD_CNT;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getUpdateFileName()
     */
    @Override
    public final String getUpdateFileName() {
        return DP_FILE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getTableName()
     */
    @Override
    public final String getTableName() {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getDomainObjectName()
     */
    @Override
    public String getDomainObjectName() {

        return "DrugPairVa";
    }

    /**
     * Generates the added records and writes them to the zip file for incremental updates.
     * 
     * @param zout
     *            ZipOutputStream
     * @param fileDate
     *            Date
     */
    @Override
    protected void doGenerateAddRecords(ZipOutputStream zout, Date fileDate) {

        @SuppressWarnings("unchecked")
        List<DrugPairVa> list = (List<DrugPairVa>) getGenericDao().findCustomUpdateAddRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugPairVa record : list) {
                for (Object[] values : getAddChangeFields(record.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        zout.write(buf.toString().getBytes());
                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
        return;

    }

    /**
     * Generates the changed records and writes them to the zip file for incremental updates.
     * 
     * @param zout
     *            ZipOutputStream
     * @param fileDate
     *            Date
     **/
    @SuppressWarnings("unchecked")
    @Override
    protected void doGenerateChangeRecords(ZipOutputStream zout, Date fileDate) {

        List<DrugPairVa> list = (List<DrugPairVa>) getGenericDao().findCustomUpdateChangeRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugPairVa record : list) {
                for (Object[] values : getAddChangeFields(record.getExportableVersion())) {
                    StringBuffer buf = formatChangeRecord(values);
                    try {
                        zout.write(buf.toString().getBytes());
                    } catch (IOException ioe) {
                        String msg = "Unable to write CHANGE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
        return;
    }

    /**
     * Generates the deleted records and writes them to the zip file for incremental updates.
     * 
     * @param zout
     *            ZipOutputStream
     * @param fileDate
     *            Date
     * 
     **/
    @SuppressWarnings("unchecked")
    @Override
    protected void doGenerateDeleteRecords(ZipOutputStream zout, Date fileDate) {

        List<Object[]> list = (List<Object[]>) getGenericDao().findDrugPairsCustomUpdateDeleteRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (Object[] record : list) {

                for (Object[] values : getDeleteFields(record)) {
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        zout.write(buf.toString().getBytes());
                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
        return;

    }

    /**
     * Save version update.
     * 
     * @param fileDate
     *            the file date
     * @param record
     *            the record
     */
    public void saveVersionUpdate(Date fileDate, Object[] record) {
        drugPairDAO.updateDrugPairAsExported((Long) record[2], (Long) record[0], (Long) record[1], fileDate);
    }

    /**
     * Deleted Fields.
     * 
     * @param dp
     *            - Object
     * @return res
     */
    private Object[][] getDeleteFields(Object[] dp) {
        // the field order is the following:
        // rtgenid1|rtgenid2|interactionid|seqno
        Object[][] res = new Object[2][];

        Object[] forward = new Object[4];
        forward[0] = dp[0];
        forward[1] = dp[1];
        forward[2] = dp[2];
        forward[3] = 1;
        res[0] = forward;

        Object[] reverse = new Object[4];
        reverse[0] = dp[1];
        reverse[1] = dp[0];
        reverse[2] = dp[2];
        reverse[3] = 2;
        res[1] = reverse;

        return res;
    }

    /**
     * Generate records all statuses.
     * 
     * @param fileDate
     *            the file date
     * @param recordCounts
     *            the record counts
     * @param zout
     *            the zout
     */
    @Override
    public void generateRecordsAllStatuses(Date fileDate, RecordCounts recordCounts, ZipOutputStream zout) {
        try {
            doGenerateDeleteRecords(zout, fileDate);
            doGenerateChangeRecords(zout, fileDate);
            doGenerateAddRecords(zout, fileDate);
        } catch (Exception e) {
            String msg = "Failed to generate incremental Records.";
            LOGGER.error(msg, e);
            throw new CustomizationServiceException(msg, e);
        }

    }
}
