package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVersionVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

/**
 * The Class DuplicateTherapyUpdateFileSupport.
 * 
 */
@Service
public final class DuplicateTherapyUpdateFileSupport extends BaseUpdateFileSupport {

    /** The Constant DUPLICATE_THERAPY_VA. */
    private static final String DUPLICATE_THERAPY_VA = "DuplicateTherapyVa";

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(DuplicateTherapyUpdateFileSupport.class);

    /** The Constant TABLE_NAME. */
    private static final String TABLE_NAME = "fdb_custom_dtcat";
    /**
     * Static DUPLICATE_THERAPY_FILE_NAME set to value "FDBCUSTOMDUPLICATETHERAPY".
     */
    public static final String DUPLICATE_THERAPY_FILE_NAME = "FDBCUSTOMDUPLICATETHERAPY";

    /** The Constant FIELD_DEF_RECORD. */
    private static final String FIELD_DEF_RECORD =
            "F|5|1|category|2|dtcid|3|customstring|4|customdupallowance|5|customdupallowind";

    /** The Constant PRIMARY_KEY_RECORD. */
    private static final String PRIMARY_KEY_RECORD = "P|2|1|2";

    /** The Constant DUPLICATE_ALLOWANCE_IND. */
    private static final String DUPLICATE_ALLOWANCE_IND = "1";

    /**
     * This instance method returns the FDB update file Field Definition Record. The record will be specific to the Order Check
     * type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Field Definition Record for this Order Check type.
     */
    public String generateFieldDefRecord() {
        return FIELD_DEF_RECORD;
    }

    /**
     * this method returns the FDB update file Primary Key Record. The record will be specific to the Order Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Primary Key Record for this Order Check type.
     */
    public String generatePrimaryKeyRecord() {
        return PRIMARY_KEY_RECORD;
    }

    /**
     * Method to generate all the records for a concept.
     * 
     * @param zout
     *            of type OutputStream
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateFullRecords(OutputStream zout) {
        List<DuplicateTherapyVa> list =
                (List<DuplicateTherapyVa>) getGenericDao().findCustomUpdateRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DuplicateTherapyVa m : list) {
                // for full updates we're using the same field order as for ADD
                for (Object[] values : getAddFields(m.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, true);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write FULL record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }

        return;
    }

    /**
     * Do generate add records.
     * 
     * @param zout
     *            of type OutputStream
     * @param fileDate
     *            the file date
     */

    protected void doGenerateAddRecords(ZipOutputStream zout, Date fileDate) {
        @SuppressWarnings("unchecked")
        List<DuplicateTherapyVa> list =
                (List<DuplicateTherapyVa>) getGenericDao().findCustomUpdateAddRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DuplicateTherapyVa m : list) {
                for (Object[] values : getAddFields(m.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, m);
            }
        }
    }

    /**
     * Do generate change records.
     * 
     * @param out
     *            of type OutputStream
     * @param fileDate
     *            Date
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateChangeRecords(ZipOutputStream out, Date fileDate) {
        List<DuplicateTherapyVa> list =
                (List<DuplicateTherapyVa>) getGenericDao().findCustomUpdateChangeRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DuplicateTherapyVa record : list) {
                for (Object[] values : getAddFields(record.getExportableVersion())) {
                    // add the lines of the currently exportable version
                    StringBuffer buf = formatChangeRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
    }

    /**
     * Extract the values needed for a CHANGE record from this duplicateTherapy version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getAddFields(DuplicateTherapyVersionVa version) {
        ArrayList<Object[]> res = new ArrayList<Object[]>();
        Object[] v = new Object[5];

        v[0] = version.getCategory();
        v[1] = version.getVersionedObject().getDtcid();
        v[2] = version.getCustomString();
        v[3] = version.getCustomDupAllowance().getCode();
        v[4] = DUPLICATE_ALLOWANCE_IND;
        res.add(v);

        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Do generate delete records.
     * 
     * @param out
     *            of type OutputStream
     * @param fileDate
     *            recordDate
     */

    protected void doGenerateDeleteRecords(ZipOutputStream out, Date fileDate) {
        @SuppressWarnings("unchecked")
        List<DuplicateTherapyVa> list =
                (List<DuplicateTherapyVa>) getGenericDao().findCustomUpdateDeleteRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DuplicateTherapyVa record : list) {
                for (Object[] values : getDeleteFields(record.getExportedVersion())) {
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, record);
            }
        }
    }

    /**
     * Extract the values needed for a DELETE record from this duplicateTherapy version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getDeleteFields(DuplicateTherapyVersionVa version) {
        ArrayList<Object[]> res = new ArrayList<Object[]>();
        Object[] v = new Object[2];
        v[0] = version.getCategory();
        v[1] = version.getVersionedObject().getDtcid();
        res.add(v);

        return res.toArray(new Object[res.size()][]);

    }

    @Override
    public String getUpdateFileName() {
        return DUPLICATE_THERAPY_FILE_NAME;
    }

    @Override
    public int getKeyFieldCount() {
        return 2;
    }

    @Override
    public String getDomainObjectName() {
        return DUPLICATE_THERAPY_VA;
    }

    @Override
    public String getTableName() {
        return TABLE_NAME;
    }

}
