package gov.va.med.pharmacy.pecs.domain;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.DuplicateAllowance;

import java.util.Set;

/**
 * Custom duplicate therapy allows the user to associate user-defined information with a First DataBank duplicate therapy class.
 * The user cannot create a custom duplicate therapy class. The user may assign a custom duplicate allowance value to a
 * duplicate therapy class that can override the First DataBank assigned duplicate allowance value for the class.
 * 
 * @see {@link gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVersionVa}
 * @see {@link gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVa}
 * 
 */
public interface DuplicateTherapy {

    /**
     * Retrieves the Custom Duplicate Allowance.
     * 
     * User-defined number of duplications allowed (defaults to 0)
     * 
     * @return Duplicate Allowance - custom duplicate allowance
     */
    DuplicateAllowance getCustomDupAllowance();

    /**
     * Sets the Custom Duplicate Allowance.
     * 
     * User-defined number of duplications allowed (defaults to 0)
     * 
     * @param allowance
     *            - of type Duplicate Allowance
     */
    void setCustomDupAllowance(DuplicateAllowance allowance);

    /**
     * Retrieves the Custom Duplicate Allowance Indicator.
     * 
     * Indicates whether or not to use custom duplicate allowance
     * 
     * @return Long - custom duplicate allowance indicator
     */
    Long getCustomDupAllowanceInd();

    /**
     * Sets the Custom Duplicate Allowance Indicator.
     * 
     * Indicates whether or not to use custom duplicate allowance
     * 
     * @param indicator
     *            of type Long
     */
    void setCustomDupAllowanceInd(Long indicator);

    /**
     * Retrieves the Custom String.
     * 
     * User-defined information
     * 
     * @return String - custom string
     */
    String getCustomString();

    /**
     * Sets the Custom String.
     * 
     * User-defined information
     * 
     * @param customString
     *            of type String
     */
    void setCustomString(String customString);

    /**
     * Retrieves the Category.
     * 
     * User-defined category code
     * 
     * @return String - category
     */
    String getCategory();

    /**
     * Sets the Category.
     * 
     * User-defined category code
     * 
     * @param category
     *            of type String
     */
    void setCategory(String category);

    /**
     * Sets the user comment history.
     * 
     * @param userCommentHistory
     *            the new user comment history
     */
    void setUserCommentHistory(Set<UserComment> userCommentHistory);

    /**
     * Gets the user comment history.
     * 
     * @return the user comment history
     */
    Set<UserComment> getUserCommentHistory();
}
