package gov.va.med.pharmacy.pecs.domain;

import gov.va.med.pharmacy.pecs.domain.MonographSection.SectionCode;

import java.util.Map;
import java.util.Set;

/**
 * A description of a drug or of a drug interaction etc. Monographs are organized in sections.
 */
public interface Monograph {
    /**
     * The title of this monograph.
     * 
     * @return the title
     */
    String getTitle();

    /**
     * The sections of this monograph.
     * 
     * @return a set of objects representing monograph sections
     */
    Set<MonographSection> getSections();

    /**
     * The version code is a string indicating the type of monograph (professional or consumer). This has no relationship with
     * the version history.
     * 
     * @return the version code
     */
    String getVersionCode();

    /**
     * The category code is a string indicating what the monograph describes (drug-drug interactions, drug-food interactions
     * etc). version history.
     * 
     * @return the category code
     */
    String getCategoryCode();

    /**
     * The sections supported by this type of monograph.
     * 
     * @return a map of section codes and associated section labels
     */
    Map<SectionCode, String> getSupportedSections();

    /**
     * Sets the user comment history.
     * 
     * @param userCommentHistory
     *            the new user comment history
     */
    void setUserCommentHistory(Set<UserComment> userCommentHistory);

    /**
     * Gets the user comment history.
     * 
     * @return the user comment history
     */
    Set<UserComment> getUserCommentHistory();
}
