package gov.va.med.pharmacy.pecs.domain.impl.easy;

/**
 * Identifies a drug.
 * 
 * @author vhaisacarusr
 * 
 */
public final class DrugIdentifier {
    private DrugIdentifierType type;

    /**
     * Convenience constructor.
     * 
     * 
     * @param value
     *            the value of the identifier
     * @param type
     *            the drug identifier type
     */
    public DrugIdentifier(Object value, DrugIdentifierType type) {

        this.type = type;
        this.value = value;
    }

    private Object value;

    /**
     * Gets the type.
     * 
     * @return the type
     */
    public DrugIdentifierType getType() {
        return type;
    }

    /**
     * Sets the type.
     * 
     * @param type
     *            the type to set
     */
    public void setType(DrugIdentifierType type) {
        this.type = type;
    }

    /**
     * Gets the value.
     * 
     * @return the value
     */
    public Object getValue() {
        return value;
    }

    /**
     * Sets the value.
     * 
     * @param value
     *            the value to set
     */
    public void setValue(Object value) {
        this.value = value;
    }

    /**
     * An alternastive to the to string method.
     * 
     * @return the value as a string null if the value is null;
     */
    public String getValueAsString() {
        return getValue() == null ? null : getValue().toString();
    }

    @Override
    public String toString() {
        return getType().toString() + "=" + getValueAsString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((type == null) ? 0 : type.hashCode());
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        DrugIdentifier other = (DrugIdentifier) obj;
        if (type != other.type)
            return false;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        return true;
    }
}
