package gov.va.med.pharmacy.pecs.domain.impl.easy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The Class PecsDrugInfo.
 */
public class PecsDrugInfo {

    /** The description. */
    private String description;

    /** The id. */
    private DrugIdentifier id;

    /** The therapeutic classes. */
    private List<TherapeuticClass> therapeuticClasses = new ArrayList<TherapeuticClass>();

    /** The alternate ids. */
    private Map<DrugIdentifierType, DrugIdentifier> alternateIds = new HashMap<DrugIdentifierType, DrugIdentifier>();

    /**
     * Instantiates a new pecs drug info.
     */
    public PecsDrugInfo() {
    }

    /**
     * Instantiates a new pecs drug info.
     * 
     * @param identifier
     *            the identifier
     */
    public PecsDrugInfo(DrugIdentifier identifier) {
        id = identifier;
        alternateIds.put(id.getType(), id);
    }

    /**
     * Gets the description.
     * 
     * @return the description
     */
    public String getDescription() {
        return description;
    }

    /**
     * Sets the description.
     * 
     * @param name
     *            the new description
     */
    public void setDescription(String name) {
        this.description = name;
    }

    /**
     * Gets the id.
     * 
     * @return the id
     */
    public DrugIdentifier getId() {
        return id;
    }

    /**
     * Sets the id.
     * 
     * @param id
     *            the new id
     */
    public void setId(DrugIdentifier id) {
        this.id = id;
    }

    /**
     * Adds the alternate id.
     * 
     * @param id
     *            the id
     */
    public void addAlternateId(DrugIdentifier id) {
        alternateIds.put(id.getType(), id);
    }

    /**
     * Gets the id.
     * 
     * @param type
     *            the type
     * @return the id
     */
    public DrugIdentifier getId(DrugIdentifierType type) {
        if (id.getType() == type) {
            return id;
        } else {
            return alternateIds.get(type);
        }
    }

    /**
     * Gets the therapeutic classes.
     * 
     * @return the therapeutic classes
     */
    public List<TherapeuticClass> getTherapeuticClasses() {
        return therapeuticClasses;
    }

    /**
     * Sets the therapeutic classes.
     * 
     * @param dtc
     *            the new therapeutic classes
     */
    public void setTherapeuticClasses(List<TherapeuticClass> dtc) {
        therapeuticClasses = dtc;
    }

    /**
     * Gets the ui label.
     * 
     * @return the ui label
     */
    public String getUiLabel() {
        return getDescription() + " (GCN: " + getId(DrugIdentifierType.GCN_SEQNO).getValueAsString() + ")";
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "[" + description + " (" + id + ")]";
    }
}
