package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;

import java.util.Set;

/**
 * Allows a user to add comments to selected FDB Concepts.
 * 
 * @author vhaisaduffg
 * 
 */
public abstract class AbstractCommentable {

    private transient ConceptXref conceptXref;

    /**
     * Abstract method that must be overriden by realization classes to create a new ConceptXref object for the concept.
     * 
     * @return ConceptXref
     */
    protected abstract ConceptXref createConceptXref();

    /**
     * Returns the ConceptXref for this concept.
     * 
     * @return ConceptXref
     */
    public final ConceptXref getConceptXref() {
        if (conceptXref == null) {
            setConceptXref(createConceptXref());
        }
        return conceptXref;
    }

    /**
     * Sets the ConceptXref to use.
     * 
     * @param conceptXref
     *            the concept specific Xref
     */
    public final void setConceptXref(ConceptXref conceptXref) {
        this.conceptXref = conceptXref;
    }

    /**
     * Convenience method to return the user comments from the ConceptXref.
     * 
     * @return Set<IUserComment>
     */
    public final Set<UserComment> getUserCommentHistory() {
        return getConceptXref().getUserCommentHistory();
    }

    /**
     * Convenience method to set the user comments in the ConceptXref object. Should only be used by Hibernate.
     * 
     * @param userCommentHistory
     *            Set of IUserComment
     */
    public final void setUserCommentHistory(Set<UserComment> userCommentHistory) {
        getConceptXref().setUserCommentHistory(userCommentHistory);
    }

}
