package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.Version;

import java.text.SimpleDateFormat;
import java.util.ListIterator;

/**
 * The Class CustomizationSupport.
 * 
 * @param <T>
 *            the generic type
 */
public class CustomizationSupport<T> {

    /** The df. */
    private SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd kk:mm:ss");

    /**
     * Format the comments obtained from the version history of the given concept, up to the specified version.
     * 
     * @param concept
     *            the customized record for which the comment history is desired
     * @param version
     *            the version up to which we want comments for
     * @return the formatted comment history (descending chronological order)
     */
    public String formatComments(Customizable<T> concept, Version<T> version) {
        StringBuffer res = new StringBuffer();
        int index = concept.getVersionHistory().indexOf(version);
        for (ListIterator<Version<T>> iterator = concept.getVersionHistory().listIterator(index + 1); iterator.hasPrevious();) {
            // for (Version<T> v : concept.getVersionHistory()) {
            Version<T> v = iterator.previous();
            if (v.getComment() == null || v.getComment().trim().length() == 0) {
                continue;
            }
            res.append(df.format(v.getDate()));
            res.append(" ");
            res.append(v.getUser());
            res.append(": ");
            res.append(v.getComment());
            res.append("\r");
            // if (v.equals(version)) {
            // break;
            // }
        }
        return res.toString();
    }
}
