package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;

import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * Data Access Object for FDB Dose Range.
 * 
 * @author VHAISACARUSR
 * 
 */
@Repository
public final class FdbDosingDAO extends FdbHibernateDAOImpl<FdbDosing> {
    private static final Logger LOGGER = Logger.getLogger(FdbDosingDAO.class);
    private static final String FIND_DOSING_BY_ID = "findDosingById";

    @Override
    protected Class<FdbDosing> getPersistentClass() {
        return FdbDosing.class;
    }

    /**
     * Finds the Dose Range record given the Key provided.
     * 
     * @param doseRangeKey
     *            a key for the dose range records
     * @return a fdb dosing record. <code>null</code> if none exists.
     * 
     */
    @SuppressWarnings("unchecked")
    public FdbDosing find(DoseRangeKey doseRangeKey) {
        FdbDosing dosing = null;
        Query query = getSession().getNamedQuery(FIND_DOSING_BY_ID);
        query.setParameter("conceptId", doseRangeKey.getConceptId());
        query.setParameter("conceptType", doseRangeKey.getConceptType());
        query.setParameter("ageLowInDays", doseRangeKey.getAgeLowInDays());
        query.setParameter("ageHighInDays", doseRangeKey.getAgeHighInDays());
        query.setParameter("doseRouteId", doseRangeKey.getDoseRouteId());
        query.setParameter("doseTypeId", doseRangeKey.getDoseTypeId());
        query.setParameter("fdbdx", doseRangeKey.getFdbdx());
        query.setParameter("hitType", doseRangeKey.getHitType());
        List<FdbDosing> dosings = query.list();
        if (dosings.size() > 1) {
            LOGGER.error("more than on dose range returned expected 1 or 0 and got " + dosings.size());
        }
        if (!dosings.isEmpty()) {
            dosing = dosings.get(0);
        }
        return dosing;
    }

}
