package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;

import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * FDB Drug pair DAO.
 * 
 * @author DOMAINOVERSD
 * 
 */
@Repository
public final class FdbDrugPairDAO extends FdbHibernateDAOImpl<FdbDdimdruglink> {

    /** The Constant FIND_BY_ID. */
    private static final String FIND_BY_ID = "findById";

    /** The Constant FIND_BY_INTERACTION_ID. */
    private static final String FIND_BY_INTERACTION_ID = "findByInteractionId";

    /** The Constant FIND_BY_DRUG_ID. */
    private static final String FIND_BY_DRUG_ID = "findByDrugId";

    /**
     * Persistent Class.
     * 
     * @return FdbDdimDrugLink.class
     */
    @Override
    protected Class<FdbDdimdruglink> getPersistentClass() {
        return FdbDdimdruglink.class;
    }

    /**
     * find Fdb by Id.
     * 
     * @param interactionId
     *            The FDB Interaction ID
     * @param rtgenId1
     *            Routed Generic ID 1
     * @param rtgenId2
     *            Routed Generic ID 1
     * @return a FDB drug pair
     */
    public FdbDdimdruglink findById(Long interactionId, Long rtgenId1, Long rtgenId2) {
        Query query = getSession().getNamedQuery(FIND_BY_ID);
        query.setParameter("rtgenId1", rtgenId1);
        query.setParameter("rtgenId2", rtgenId2);
        query.setParameter("interactionId", interactionId);
        return (FdbDdimdruglink) query.uniqueResult();
    }

    /**
     * find by interactionId.
     * 
     * @param interactionId
     *            the FDB interaction ID
     * @return a list of FDB drug pairs
     */
    @SuppressWarnings("unchecked")
    public List<FdbDdimdruglink> findByInteractionId(Long interactionId) {
        Query query = getSession().getNamedQuery(FIND_BY_INTERACTION_ID);
        query.setParameter("interactionId", interactionId);
        return query.list();
    }

    // shahid 12-3-2012
    /**
     * Find by drug id.
     * 
     * @param drugId
     *            the drug id
     * @return the list
     */
    @SuppressWarnings("unchecked")
    public List<FdbDdimdruglink> findByDrugId(Long drugId) {
        Query query = getSession().getNamedQuery(FIND_BY_DRUG_ID);
        query.setParameter("rtgenId1", drugId);
        return query.list();
    }
}
