package gov.va.med.pharmacy.pecs.domain.impl.fdb;

import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;

import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * FdbMonographDdimDAO entity.
 * 
 * @author DOMAINOVERSD
 * @version Pecs 3.0
 */
@Repository
public final class FdbMonographDdimDAO extends FdbHibernateDAOImpl<FdbMonographDdim> {
    private static final String ALL_MONOGRAPHS_QUERY = "allMonographsQuery";
    private static final String ALL_MONOGRAPH_LINES_QUERY = "allMonographLinesQuery";

    // private static final String MONOGRAPH_BY_ID = "monographById";

    /**
     * Calls Persistent Class.
     * 
     * @return FdbMonographDdim.class
     */
    @Override
    protected Class<FdbMonographDdim> getPersistentClass() {
        return FdbMonographDdim.class;
    }

    /**
     * Returns a list of containing all monograph versions (only monograph id and title).
     * 
     * @return a list of monograph versions
     */
    @SuppressWarnings("unchecked")
    public List<Object[]> getAllMonographs() {
        Query query = getSession().getNamedQuery(ALL_MONOGRAPHS_QUERY);
        return query.list();
    }

    /**
     * A List of FdbMonographDbim using the fdbID from professional monographs.
     * 
     * @param fdbId
     *            - Long Object type
     * @return a queried list
     */
    @SuppressWarnings("unchecked")
    public List<FdbMonographDdim> getMonographLines(long fdbId) {
        Query query = getSession().getNamedQuery(ALL_MONOGRAPH_LINES_QUERY);
        query.setLong("monographid", fdbId);
        List<FdbMonographDdim> monographs = query.list();
        return monographs;
    }

}
