package gov.va.med.pharmacy.pecs.domain.impl.lookup;

import gov.va.med.pharmacy.pecs.domain.impl.va.FdbUpdateReport;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * A Lookup object that holds a Comparison Report Concept.
 * 
 * @author vhaisaduffg
 * 
 */
public final class ComparisonReportConcept extends LookupImpl {

    private Set<FdbUpdateReport> reports;

    /**
     * The code for a Dose Range comparison report.
     */
    public static final String DOSE_RANGE_CONCEPT = "DR";
    /**
     * The code for a Drug-Drug Interaction/Drug Pair comparison report.
     */
    public static final String DDI_DP_CONCEPT = "DDIDP";
    /**
     * The code for a Duplicate Therapy comparison report.
     */
    public static final String DUPLICATE_THERAPY_CONCEPT = "DT";

    /**
     * Returns the FDB Update Reports associated with this concept.
     * 
     * @return Set<FdbUpdateReport>
     */
    public Set<FdbUpdateReport> getReports() {
        return reports;
    }

    /**
     * Allows the reports associated with this concept to be set.
     * 
     * @param reports
     *            the Set of associated reports.
     */
    public void setReports(Set<FdbUpdateReport> reports) {
        this.reports = reports;
    }

    /**
     * Adds the FdbUpdateReport to this concept's reports and sets this concept in the FdbUpdateReport.
     * 
     * @param report
     *            FdbUpdateReport
     */
    public void addReport(FdbUpdateReport report) {
        reports.add(report);
        report.setConcept(this);
    }

    /**
     * Returns a list of Sorted FdbUpdateReport objects. The sort order is left up to the FdbUpdateReport sort order.
     * 
     * @return the list of reports
     * 
     */
    public List<FdbUpdateReport> getSortedReports() {
        List<FdbUpdateReport> unsortedList = Arrays.asList(reports.toArray(new FdbUpdateReport[0]));
        Collections.sort(unsortedList);
        return unsortedList;
    }
}
