package gov.va.med.pharmacy.pecs.domain.impl.lookup;

import gov.va.med.pharmacy.pecs.domain.Lookup;

import java.io.Serializable;
import java.sql.Timestamp;

import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * A default implementation of the Lookup interface.
 * 
 * @author vhaisaduffg
 * 
 */
public abstract class LookupImpl implements Lookup, Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 2424662883308232802L;
    private Long id;
    private String code = null;
    private String name = null;
    private String description;
    private Integer sortOrder;
    private Timestamp effectiveDate;
    private Timestamp expirationDate;
    private String createdBy;
    private String modifiedBy;
    private Timestamp created;
    private Timestamp modified;

    /**
     * Returns the identifier for this object.
     * 
     * @return Long
     */
    @Override
    public Long getId() {
        return id;
    }

    /**
     * Sets the identifier this object should use. This method should only be used by Hibernate which is why it is private.
     * 
     * @param id
     *            identifier
     */
    @SuppressWarnings("unused")
    private void setId(Long id) {
        this.id = id;
    }

    /**
     * Returns the code assigned to this Lookup object.
     * 
     * @return String
     */
    @Override
    public String getCode() {
        return code;
    }

    /**
     * Sets the code assigned to this Lookup object.
     * 
     * @param code
     *            String
     */
    @Override
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Returns the name assigned to this Lookup object.
     * 
     * @return String
     */
    @Override
    public String getName() {
        return name;
    }

    /**
     * Sets the name for this Lookup object.
     * 
     * @param name
     *            String
     */
    @Override
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns the description of this Lookup object.
     * 
     * @return String
     */
    @Override
    public String getDescription() {
        return description;
    }

    /**
     * Sets the description for this Lookup object.
     * 
     * @param description
     *            String
     */
    @Override
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Returns the sort order of this Lookup object.
     * 
     * @return Integer
     */
    @Override
    public Integer getSortOrder() {
        return sortOrder;
    }

    /**
     * Sets the sort order for this Lookup object.
     * 
     * @param sortOrder
     *            Integer
     */
    @Override
    public void setSortOrder(Integer sortOrder) {
        this.sortOrder = sortOrder;
    }

    /**
     * Returns whether this Lookup object is active or not. The effectiveDate and expirationDate are used to determine the
     * return value.
     * 
     * @return Boolean
     */
    @Override
    public Boolean isActive() {
        final Timestamp today = new Timestamp(System.currentTimeMillis());

        Boolean active = getEffectiveDate().before(today);
        if (getExpirationDate() != null) {
            active = active && today.before(getExpirationDate());
        }
        return active;
    }

    /**
     * Allows the active flag to be set for this Lookup object. This method should only be executed by Hibernate.
     * 
     * @param active
     *            Boolean
     */
    @SuppressWarnings("unused")
    private void setActive(Boolean active) {
        // Do Nothing
    }

    /**
     * Returns the effective date of this Lookup object.
     * 
     * @return Timestamp
     */
    @Override
    public Timestamp getEffectiveDate() {
        return effectiveDate;
    }

    /**
     * Sets the effective date for this Lookup object.
     * 
     * @param effectiveDate
     *            Timestamp
     */
    @Override
    public void setEffectiveDate(Timestamp effectiveDate) {
        this.effectiveDate = effectiveDate;
    }

    /**
     * Sets the expiration date for this Lookup object.
     * 
     * @return Timestamp
     */
    @Override
    public Timestamp getExpirationDate() {
        return expirationDate;
    }

    /**
     * Sets the expiration date for this Lookup object.
     * 
     * @param expirationDate
     *            Timestamp
     */
    @Override
    public void setExpirationDate(Timestamp expirationDate) {
        this.expirationDate = expirationDate;
    }

    /**
     * Returns the user who created this Lookup object.
     * 
     * @return String
     */
    @Override
    public String getCreatedBy() {
        return createdBy;
    }

    /**
     * Sets the user who created this Lookup object.
     * 
     * @param createdBy
     *            String
     */
    @Override
    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    /**
     * Returns the created date/time of this Lookup object.
     * 
     * @return Timestamp
     */
    @Override
    public Timestamp getCreated() {
        return created;
    }

    /**
     * Sets the created date/time of this Lookup object. This method is marked as private since it should only be used by
     * Hibernate.
     * 
     * @param created
     *            Timestamp
     */
    @SuppressWarnings("unused")
    private void setCreated(Timestamp created) {
        this.created = created;
    }

    /**
     * Returns the user who last modified this Lookup object.
     * 
     * @return String
     */
    @Override
    public String getModifiedBy() {
        return modifiedBy;
    }

    /**
     * Sets the user who modified this Lookup object.
     * 
     * @param modifiedBy
     *            Timestamp
     */
    @Override
    public void setModifiedBy(String modifiedBy) {
        this.modifiedBy = modifiedBy;
    }

    /**
     * Returns the modified date/time of the Lookup object.
     * 
     * @return Timestamp
     */
    @Override
    public Timestamp getModified() {
        return modified;
    }

    /**
     * Sets the modified date/time of this Lookup object. This method is marked as private since it should only be used by
     * Hibernate.
     * 
     * @param modified
     *            Timestamp
     */
    @SuppressWarnings("unused")
    private void setModified(Timestamp modified) {
        this.modified = modified;
    }

    /**
     * Returns a String that can be displayed on the UI. This method is used so that all Lookup objects are displayed the same
     * way on the UI.
     * 
     * @return String
     */
    @Override
    public String getUiLabel() {
        if (getCode() == null && getName() == null) {
            return "";
        }
        return getCode() + " - " + getName();
    }

    /**
     * Sets the UI label to use for this Lookup object. Since the UI label is calculated, this method is marked as private.
     * 
     * @param uiLabel
     *            String
     */
    @SuppressWarnings("unused")
    private void setUiLabel(String uiLabel) {
        // Do nothing since UiLabel is a derived property. This method is only
        // here for JSON parsing.
    }

    /**
     * Checks to see if two LookupImpl objects are equal based upon the Code attribute.
     * 
     * @param o
     *            the object to check
     * @return boolean true if equal, otherwise false
     */
    @Override
    public final boolean equals(Object o) {
        if (o instanceof LookupImpl) {
            String otherCode = ((LookupImpl) o).getCode();
            return this.getCode().equals(otherCode);
        }
        return false;
    }

    /**
     * Generates a Hash Code based upon the attributes in this object. Do not use the reflection hash code builder as this may
     * cause infinite loops and cause a StackOverflowException.
     * 
     * @return int
     */
    @Override
    public final int hashCode() {
        HashCodeBuilder hb = new HashCodeBuilder();
        hb.append(id);
        hb.append(code);
        hb.append(name);
        hb.append(description);
        hb.append(sortOrder);
        hb.append(effectiveDate);
        hb.append(expirationDate);
        hb.append(createdBy);
        hb.append(modifiedBy);
        hb.append(created);
        hb.append(modified);
        return hb.toHashCode();
    }

}
