package gov.va.med.pharmacy.pecs.domain.impl.va;

// Generated Sep 29, 2010 3:29:35 PM by Hibernate Tools 3.2.4.GA

import gov.va.med.pharmacy.pecs.domain.Monograph;
import gov.va.med.pharmacy.pecs.domain.MonographSection;
import gov.va.med.pharmacy.pecs.domain.Version;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * An abstract superclass for monographs of all types.
 */
public abstract class AbstractMonographVa extends AbstractCustomizable<Monograph> implements Monograph, java.io.Serializable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -3651242675299583295L;

    /** The id. */
    private long id;

    private MonographExportManager exportManager;

    /** The customized monograph id. */
    private Long customizedMonographId;

    /** The language id. */
    private int languageId;

    /** The version history. */
    private List<Version<Monograph>> versionHistory = new ArrayList<Version<Monograph>>(0);

    /* private Set<IUserComment> userCommentHistory; */

    /**
     * Instantiates a new abstract monograph va.
     */
    public AbstractMonographVa() {
    }

    /**
     * This constructor is intended to be used by subclasses, when creating a new instance from scratch (i.e. not loading a
     * persistent one).
     * 
     * @param initHistory
     *            whether to create a version (should be true when not loading from persistent store)
     */
    protected AbstractMonographVa(boolean initHistory) {
        // default to English
        this.languageId = 1;
        if (initHistory) {
            this.exportManager = new MonographExportManager();
            this.exportManager.setProfessionalMonographVa((ProfessionalDrugDrugMonographVa) this);
            setCurrentVersion(new MonographVersion());
            getCurrentVersion().setDate(new Date());
            getCurrentVersion().setVersionedObject(this);
            versionHistory.add(getCurrentVersion());
        }
    }

    /**
     * Gets the id.
     * 
     * @return the id
     */
    public long getId() {
        return this.id;
    }

    /**
     * Sets the id.
     * 
     * @param vaMonographId
     *            the new id
     */
    public void setId(long vaMonographId) {
        this.id = vaMonographId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getExportableVersion()
     */
    @Override
    public MonographVersion getExportableVersion() {
        return this.getExportManager().getExportableVersion();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.domain.impl.va.AbstractCustomizable#setExportableVersion(gov.va.med.pharmacy.pecs.domain.Version
     * )
     */
    @Override
    public void setExportableVersion(Version<Monograph> v) {
        this.getExportManager().setExportableVersion((MonographVersion) v);
    }

    /**
     * Gets the exported version.
     * 
     * @return the exported version
     */
    public MonographVersion getExportedVersion() {
        return this.getExportManager().getExportedVersion();
    }

    /**
     * Sets the exported version.
     * 
     * @param v
     *            the new exported version
     */
    public void setExportedVersion(MonographVersion v) {
        this.getExportManager().setExportedVersion(v);
    }

    /**
     * Sets the exported version.
     * 
     * @param v
     *            the new exported version
     */
    @Override
    public void setExportedVersion(Version<Monograph> v) {
        this.getExportManager().setExportedVersion((MonographVersion) v);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getCurrentVersion()
     */
    @Override
    public MonographVersion getCurrentVersion() {
        return this.getExportManager().getCurrentVersion();
    }

    /**
     * Sets the current version.
     * 
     * @param v
     *            the new current version
     */
    public void setCurrentVersion(MonographVersion v) {
        this.getExportManager().setCurrentVersion(v);
    }

    /**
     * Gets the customized monograph id.
     * 
     * @return the customized monograph id
     */
    public Long getCustomizedMonographId() {
        return this.customizedMonographId;
    }

    /**
     * Sets the customized monograph id.
     * 
     * @param monographId
     *            the new customized monograph id
     */
    public void setCustomizedMonographId(Long monographId) {
        this.customizedMonographId = monographId;
    }

    /**
     * Gets the language id.
     * 
     * @return the language id
     */
    public int getLanguageId() {
        return this.languageId;
    }

    /**
     * Sets the language id.
     * 
     * @param languageId
     *            the new language id
     */
    public void setLanguageId(int languageId) {
        this.languageId = languageId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getVersionHistory()
     */
    @Override
    public List<Version<Monograph>> getVersionHistory() {
        return this.versionHistory;
    }

    /**
     * Sets the version history.
     * 
     * @param pecsMonographVersionVas
     *            the new version history
     */
    public void setVersionHistory(List<Version<Monograph>> pecsMonographVersionVas) {
        this.versionHistory = pecsMonographVersionVas;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getTitle()
     */
    @Override
    public String getTitle() {
        return getCurrentVersion().getTitle();
    }

    /**
     * Set the monograph's title.
     * 
     * @param title
     *            the title to set
     */
    public void setTitle(String title) {
        getModifiableCurrentVersion().setTitle(title);
    }

    /**
     * Medical references justifying the customization request.
     * 
     * @return the text
     */
    public String getReferenceText() {
        return getCurrentVersion().getReferenceText();
    }

    /**
     * Provide reference text.
     * 
     * @param referenceText
     *            the text
     */
    public void setReferenceText(String referenceText) {
        getModifiableCurrentVersion().setReferenceText(referenceText);
    }

    /**
     * Set the text of the specified section. It's the caller's responsibility to prepend a label (e.g. TITLE: ) to the text, if
     * desired.
     * 
     * @param code
     *            the section code
     * @param text
     *            the section text
     */
    protected void setSectionText(MonographSection.SectionCode code, String text) {
        getModifiableCurrentVersion().getSection(code).setText(text);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getSections()
     */
    @Override
    public Set<MonographSection> getSections() {
        return getCurrentVersion().getSections();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#rollBack()
     */
    @Override
    public void rollBack() {
        if (!getCurrentVersion().isModifiable()) {
            MonographVersion v = newVersion();
            setCurrentVersion(v);
        }
        getCurrentVersion().copy(getExportableVersion());
        // setExportableVersion();
        setRolledBack(true);
    }

    /**
     * Get a version that can be modified. Once a version is persisted, It can no longer be modified (for audit purposes). If
     * the current version of this monograph is persistent, this method will create a copy of it, make it the current version
     * and return it.
     * 
     * @return the current non-persistent version
     */
    @Override
    protected MonographVersion getModifiableCurrentVersion() {
        if (!getCurrentVersion().isModifiable()) {
            MonographVersion v = newVersion();
            v.copy(getCurrentVersion());
            setCurrentVersion(v);
            setRolledBack(false);
        }
        return getCurrentVersion();
    }

    /**
     * New version.
     * 
     * @return the monograph version
     */
    private MonographVersion newVersion() {
        MonographVersion v = new MonographVersion();
        v.setVersionedObject(this);
        versionHistory.add(v);
        return v;
    }

    /**
     * Gets the exportManager.
     * 
     * @return the exportManager
     */
    public MonographExportManager getExportManager() {
        return exportManager;
    }

    /**
     * Sets the exportManager.
     * 
     * @param exportManager
     *            the exportManager to set
     */
    public void setExportManager(MonographExportManager exportManager) {
        this.exportManager = exportManager;
    }
}
