package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.UserComment;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.codehaus.jackson.annotate.JsonIgnore;

/**
 * Class AbstractUserComment<T> is a container object for the FDB User Comment.
 * 
 * @param <T>
 *            the generic type
 */
public abstract class AbstractUserComment<T> extends Object implements UserComment {

    /** The df. */
    private final SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd kk:mm:ss");
    /** The comment. */
    private String comment;

    /** The comment id is the unique identifier. */
    private Long commentId;

    /** The comment date. */
    private Date commentDate;

    /** The fdb reference. */

    private transient ConceptXref fdbReference;

    /** The user name that created the comment. */
    private String userName;

    /**
     * Gets the comment.
     * 
     * @return the comment
     */
    @Override
    public final String getComment() {
        return comment;
    }

    /**
     * Sets the comment.
     * 
     * @param comment
     *            the comment to set
     */
    @Override
    public final void setComment(String comment) {
        this.comment = comment;
    }

    /**
     * Gets the comment id.
     * 
     * @return the comment id
     */
    @Override
    public final Long getCommentId() {
        return commentId;
    }

    /**
     * Sets the comment id.
     * 
     * @param commentId
     *            the new comment id
     */
    @Override
    public final void setCommentId(Long commentId) {
        this.commentId = commentId;
    }

    /**
     * Gets the date the comment was created.
     * 
     * @return the comment date
     */
    @Override
    public final Date getCommentDate() {
        return commentDate;
    }

    /**
     * Sets the date the comment was created.
     * 
     * @param commentDate
     *            - the date of the user comment.
     */
    @Override
    public final void setCommentDate(Date commentDate) {
        this.commentDate = commentDate;
    }

    /**
     * Gets the fdb reference id. FDB id or suid for the concept.
     * 
     * @return the fdb reference id
     */
    @Override
    @JsonIgnore
    public final ConceptXref getFdbReference() {
        return fdbReference;
    }

    /**
     * Sets the fdb reference. FDB id or suid for the concept.
     * 
     * @param fdbReference
     *            the new fdb reference
     */
    @Override
    public final void setFdbReference(ConceptXref fdbReference) {
        this.fdbReference = fdbReference;
    }

    /**
     * Gets the user name.
     * 
     * @return the userName
     */
    @Override
    public final String getUserName() {
        return userName;
    }

    /**
     * Sets the user name.
     * 
     * @param userName
     *            the userName to set
     */
    @Override
    public final void setUserName(String userName) {
        this.userName = userName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public final String toString() {

        StringBuffer commentString = new StringBuffer();
        commentString.append(df.format(getCommentDate()));
        commentString.append(" ");
        commentString.append(getUserName());
        commentString.append(": ");
        commentString.append(getComment());
        commentString.append("\n");

        return commentString.toString();
    }
}
