package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.MonographSection;
import gov.va.med.pharmacy.pecs.domain.MonographSection.SectionCode;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

/**
 * The Class ConsumerDrugDrugMonographVa.
 */
public class ConsumerDrugDrugMonographVa extends AbstractMonographVa {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -2490771573597811161L;

    /** The supported sections. */
    private static HashMap<SectionCode, String> supportedSections = new HashMap<SectionCode, String>();
    static {
        supportedSections.put(SectionCode.TITLE, "MONOGRAPH TITLE:  ");
        supportedSections.put(SectionCode.MECHANISM_OF_ACTION, "HOW THE INTERACTION OCCURS:  ");
        supportedSections.put(SectionCode.CLINICAL_EFFECTS, "WHAT MIGHT HAPPEN:  ");
        supportedSections.put(SectionCode.SEVERITY_LEVEL, "MEDICAL WARNING:  ");
        supportedSections.put(SectionCode.PATIENT_MANAGEMENT, "WHAT YOU SHOULD DO ABOUT THIS INTERACTION:  ");
        supportedSections.put(SectionCode.REFERENCE, "REFERENCES:\n");
        supportedSections.put(SectionCode.DISCLAIMER, "DISCLAIMER:  ");
    }

    /**
     * Instantiates a new consumer drug drug monograph va.
     */
    public ConsumerDrugDrugMonographVa() {
    }

    /**
     * Instantiates a new consumer drug drug monograph va.
     * 
     * @param initHistory
     *            the init history
     */
    protected ConsumerDrugDrugMonographVa(boolean initHistory) {
        super(initHistory);
    }

    /**
     * Sets the Section Text.
     * 
     * @param section
     *            of type SectionCode
     * @param text
     *            of type String
     */
    protected void setSectionText(MonographSection.SectionCode section, String text) {
        String label = getSupportedSections().get(section);
        String old = getCurrentVersion().getSection(section).getText();

        if (!StringUtils.isBlank(text) && !text.equalsIgnoreCase(label)) {
            text = text.trim();
            // check if the text came over prefixed with the section label
            if (text.toLowerCase().startsWith(label.toLowerCase())) {
                // we don't want to store the label
                text = text.substring(label.length());
            }
            // don't write if there are no changes
            if (text.equals(old)) {
                return;
            }
        }
        getModifiableCurrentVersion().getSection(section).setText(text);
    }

    @Override
    public String getCategoryCode() {
        return "DDIM";
    }

    @Override
    public String getVersionCode() {
        return "FDB-C";
    }

    @Override
    public Map<SectionCode, String> getSupportedSections() {
        return supportedSections;
    }

    @Override
    protected ConceptXref createConceptXref() {

        return new MonographXref();
    }

}
