package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.DoseRange;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.HalfLifeUnit;

import java.io.Serializable;

/**
 * represents a version of a Dose Range object.
 * 
 * @author vhaisacarusr
 * 
 */
//@formatter:off
public final class DoseRangeVersionVa extends AbstractVersion<DoseRange> 
    implements DoseRange, Version<DoseRange>, Serializable {
//@formatter:on

    private static final long serialVersionUID = 5372839643960083662L;

    private Long conceptid;
    private Long conceptType;
    private String conceptDescription;

    // private long versionId;
    private DoseRoute doseRoute; // dwd temp
    private DoseType doseType; // dwd temp
    private String referenceText;
    private String category;

    private String fdbdx;
    private double lowEliminationHalfLife;
    private double highEliminationHalfLife;

    private HalfLifeUnit halfLifeUnits;
    private double frequencyLow;
    private double frequencyHigh;

    private long durationLow;
    private long durationHigh;
    private long maxDuration;
    private long renalImpairmentIndicator;
    private long weightImpairmentIndicator;
    private long bsaRequiredIndicator;
    private long hepaticImpairmentIndicator;
    private String warningCode;
    private double doseLow;

    private double doseRateLow;
    private double doseRateHigh;
    private double doseFormRateLow;
    private double doseFormRateHigh;
    private double maxSingleDoseRate;
    private double maxSingleDoseFormRate;
    private double maxDailyDoseRate;
    private double maxDailyDoseFormRate;
    private String dxid;
    private long crclThreshold;
    private long ageLowInDays;
    private long ageHighInDays;
    // private String assignedUser;

    private String doseLowUnits;
    private String doseHighUnits;
    private String doseFormLowUnits;
    private String doseFormHighUnits;
    private String maxSingleDoseUnits;
    private String maxSingleDoseFormUnits;
    private String maxDailyDoseUnits;
    private String maxDailyDoseFormUnits;
    private String maxLifeTimeDoseUnits;
    private String maxLifeTimeDoseFormUnits;
    private String crclThresholdUnits;
    private String doseRateLowUnits;
    private String doseRateHighUnits;
    private String doseFormRateLowUnits;
    private String doseFormRateHighUnits;
    private String maxSingleDoseRateUnits;
    private String maxSingleDoseFormRateUnits;
    private String maxDailyDoseRateUnits;
    private String maxDailyDoseFormRateUnits;

    private DoseRangeVA doseRangeVa;

    private double doseFormLow;
    private double doseHigh;
    private double doseFormHigh;
    private double maxSingleDose;
    private double maxSingleDoseForm;
    private double maxDailyDose;
    private double maxDailyDoseForm;
    private double maxLifeTimeDose;
    private double maxLifeTimeDoseForm;

    private double maxSingleNTEDose;

    private String maxSingleNTEDoseUnit;

    private double maxSingleNTEDoseForm;
    private String maxSingleNTEDoseFormUnit;

    /**
     * Default Constructor.
     */
    public DoseRangeVersionVa() {
    }

    @Override
    public String getDoseLowUnits() {
        return doseLowUnits;
    }

    @Override
    public void setDoseLowUnits(String doseLowUnits) {
        this.doseLowUnits = doseLowUnits;
    }

    @Override
    public String getDoseHighUnits() {
        return doseHighUnits;
    }

    @Override
    public void setDoseHighUnits(String doseHighUnits) {
        this.doseHighUnits = doseHighUnits;
    }

    @Override
    public String getDoseFormLowUnits() {
        return doseFormLowUnits;
    }

    @Override
    public void setDoseFormLowUnits(String doseFormLowUnits) {
        this.doseFormLowUnits = doseFormLowUnits;
    }

    @Override
    public String getDoseFormHighUnits() {
        return doseFormHighUnits;
    }

    @Override
    public void setDoseFormHighUnits(String doseFormHighUnits) {
        this.doseFormHighUnits = doseFormHighUnits;
    }

    @Override
    public String getMaxSingleDoseUnits() {
        return maxSingleDoseUnits;
    }

    @Override
    public void setMaxSingleDoseUnits(String maxSingleDoseUnits) {
        this.maxSingleDoseUnits = maxSingleDoseUnits;
    }

    @Override
    public String getMaxSingleDoseFormUnits() {
        return maxSingleDoseFormUnits;
    }

    @Override
    public void setMaxSingleDoseFormUnits(String maxSingleDoseFormUnits) {
        this.maxSingleDoseFormUnits = maxSingleDoseFormUnits;
    }

    @Override
    public String getMaxDailyDoseUnits() {
        return maxDailyDoseUnits;
    }

    @Override
    public void setMaxDailyDoseUnits(String maxDailyDoseUnits) {
        this.maxDailyDoseUnits = maxDailyDoseUnits;
    }

    @Override
    public String getMaxDailyDoseFormUnits() {
        return maxDailyDoseFormUnits;
    }

    @Override
    public void setMaxDailyDoseFormUnits(String maxDailyDoseFormUnits) {
        this.maxDailyDoseFormUnits = maxDailyDoseFormUnits;
    }

    @Override
    public String getMaxLifeTimeDoseUnits() {
        return maxLifeTimeDoseUnits;
    }

    @Override
    public void setMaxLifeTimeDoseUnits(String maxLifeTimeDoseUnits) {
        this.maxLifeTimeDoseUnits = maxLifeTimeDoseUnits;
    }

    @Override
    public String getMaxLifeTimeDoseFormUnits() {
        return maxLifeTimeDoseFormUnits;
    }

    @Override
    public void setMaxLifeTimeDoseFormUnits(String maxLifeTimeDoseFormUnits) {
        this.maxLifeTimeDoseFormUnits = maxLifeTimeDoseFormUnits;
    }

    @Override
    public String getCrclThresholdUnits() {
        return crclThresholdUnits;
    }

    @Override
    public void setCrclThresholdUnits(String crclThresholdUnits) {
        this.crclThresholdUnits = crclThresholdUnits;
    }

    @Override
    public String getDoseRateLowUnits() {
        return doseRateLowUnits;
    }

    @Override
    public void setDoseRateLowUnits(String doseRateLowUnits) {
        this.doseRateLowUnits = doseRateLowUnits;
    }

    @Override
    public String getDoseRateHighUnits() {
        return doseRateHighUnits;
    }

    @Override
    public void setDoseRateHighUnits(String doseRateHighUnits) {
        this.doseRateHighUnits = doseRateHighUnits;
    }

    @Override
    public String getDoseFormRateLowUnits() {
        return doseFormRateLowUnits;
    }

    @Override
    public void setDoseFormRateLowUnits(String doseFormRateLowUnits) {
        this.doseFormRateLowUnits = doseFormRateLowUnits;
    }

    @Override
    public String getDoseFormRateHighUnits() {
        return doseFormRateHighUnits;
    }

    @Override
    public void setDoseFormRateHighUnits(String doseFormRateHighUnits) {
        this.doseFormRateHighUnits = doseFormRateHighUnits;
    }

    @Override
    public String getMaxSingleDoseRateUnits() {
        return maxSingleDoseRateUnits;
    }

    @Override
    public void setMaxSingleDoseRateUnits(String maxSingleDoseRateUnits) {
        this.maxSingleDoseRateUnits = maxSingleDoseRateUnits;
    }

    @Override
    public String getMaxSingleDoseFormRateUnits() {
        return maxSingleDoseFormRateUnits;
    }

    @Override
    public void setMaxSingleDoseFormRateUnits(String maxSingleDoseFormRateUnits) {
        this.maxSingleDoseFormRateUnits = maxSingleDoseFormRateUnits;
    }

    @Override
    public String getMaxDailyDoseRateUnits() {
        return maxDailyDoseRateUnits;
    }

    @Override
    public void setMaxDailyDoseRateUnits(String maxDailyDoseRateUnits) {
        this.maxDailyDoseRateUnits = maxDailyDoseRateUnits;
    }

    @Override
    public String getMaxDailyDoseFormRateUnits() {
        return maxDailyDoseFormRateUnits;
    }

    @Override
    public void setMaxDailyDoseFormRateUnits(String maxDailyDoseFormRateUnits) {
        this.maxDailyDoseFormRateUnits = maxDailyDoseFormRateUnits;
    }

    @Override
    public void setDoseLow(Double doseLow) {
        this.doseLow = convertToPrimitive(doseLow);
    }

    @Override
    public void setDoseRateLow(Double doseRateLow) {
        this.doseRateLow = convertToPrimitive(doseRateLow);
    }

    @Override
    public void setDoseRateHigh(Double doseRateHigh) {
        this.doseRateHigh = convertToPrimitive(doseRateHigh);
    }

    @Override
    public void setDoseFormRateLow(Double doseFormRateLow) {
        this.doseFormRateLow = convertToPrimitive(doseFormRateLow);
    }

    @Override
    public void setDoseFormRateHigh(Double doseFormRateHigh) {
        this.doseFormRateHigh = convertToPrimitive(doseFormRateHigh);
    }

    @Override
    public void setMaxSingleDoseRate(Double maxSingleDoseRate) {
        this.maxSingleDoseRate = convertToPrimitive(maxSingleDoseRate);
    }

    @Override
    public void setMaxSingleDoseFormRate(Double maxSingleDoseFormRate) {
        this.maxSingleDoseFormRate = convertToPrimitive(maxSingleDoseFormRate);
    }

    @Override
    public void setMaxDailyDoseRate(Double maxDailyDoseRate) {
        this.maxDailyDoseRate = convertToPrimitive(maxDailyDoseRate);
    }

    @Override
    public void setMaxDailyDoseFormRate(Double maxDailyDoseFormRate) {
        this.maxDailyDoseFormRate = convertToPrimitive(maxDailyDoseFormRate);
    }

    @Override
    public String getFdbdx() {
        return fdbdx;
    }

    @Override
    public void setFdbdx(String fdbdx) {
        this.fdbdx = fdbdx;
    }

    @Override
    public String getDxid() {
        return dxid;
    }

    @Override
    public void setDxid(String dxid) {
        this.dxid = dxid;
    }

    @Override
    public Long getCrclThreshold() {
        return crclThreshold;
    }

    @Override
    public void setCrclThreshold(Long crclThreshold) {
        this.crclThreshold = convertToPrimitive(crclThreshold);
    }

    @Override
    public void setLowEliminationHalfLife(Double lowEliminationHalfLife) {
        this.lowEliminationHalfLife = convertToPrimitive(lowEliminationHalfLife);
    }

    @Override
    public void setHighEliminationHalfLife(Double highEliminationHalfLife) {
        this.highEliminationHalfLife = convertToPrimitive(highEliminationHalfLife);
    }

    @Override
    public void setHalfLifeUnits(HalfLifeUnit halfLifeUnits) {
        this.halfLifeUnits = halfLifeUnits;
    }

    @Override
    public void setFrequencyLow(Double frequencyLow) {
        this.frequencyLow = convertToPrimitive(frequencyLow);
    }

    @Override
    public void setFrequencyHigh(Double frequencyHigh) {
        this.frequencyHigh = convertToPrimitive(frequencyHigh);
    }

    @Override
    public void setMaxDuration(Long maxDuration) {
        this.maxDuration = convertToPrimitive(maxDuration);
    }

    @Override
    public void setHepaticImpairmentIndicator(Long hepaticImpairmentIndicator) {
        this.hepaticImpairmentIndicator = convertToPrimitive(hepaticImpairmentIndicator);
    }

    @Override
    public void setRenalImpairmentIndicator(Long renalImpairmentIndicator) {
        this.renalImpairmentIndicator = convertToPrimitive(renalImpairmentIndicator);
    }

    @Override
    public void setWeightImpairmentIndicator(Long weightImpairmentIndicator) {
        this.weightImpairmentIndicator = convertToPrimitive(weightImpairmentIndicator);
    }

    @Override
    public void setBsaRequiredIndicator(Long requiredIndicator) {
        this.bsaRequiredIndicator = convertToPrimitive(requiredIndicator);
    }

    @Override
    public void setWarningCode(String warningCode) {
        this.warningCode = warningCode;
    }

    @Override
    public Double getDoseLow() {
        return doseLow;
    }

    @Override
    public Double getDoseFormLow() {
        return doseFormLow;
    }

    @Override
    public Double getDoseHigh() {
        return doseHigh;
    }

    @Override
    public Double getDoseFormHigh() {
        return doseFormHigh;
    }

    @Override
    public Double getMaxSingleDose() {
        return maxSingleDose;
    }

    @Override
    public Double getMaxSingleDoseForm() {
        return maxSingleDoseForm;
    }

    @Override
    public Double getMaxDailyDose() {
        return maxDailyDose;
    }

    @Override
    public Double getMaxDailyDoseForm() {
        return maxDailyDoseForm;
    }

    @Override
    public Double getMaxLifeTimeDose() {
        return maxLifeTimeDose;
    }

    @Override
    public Double getMaxLifeTimeDoseForm() {
        return maxLifeTimeDoseForm;
    }

    @Override
    public Double getLowEliminationHalfLife() {
        return lowEliminationHalfLife;
    }

    @Override
    public Double getHighEliminationHalfLife() {
        return highEliminationHalfLife;
    }

    @Override
    public HalfLifeUnit getHalfLifeUnits() {
        return halfLifeUnits;
    }

    @Override
    public Double getFrequencyLow() {
        return frequencyLow;
    }

    @Override
    public Double getFrequencyHigh() {
        return frequencyHigh;
    }

    @Override
    public Long getDurationLow() {
        return durationLow;
    }

    @Override
    public Long getDurationHigh() {
        return durationHigh;
    }

    @Override
    public Long getMaxDuration() {
        return maxDuration;
    }

    @Override
    public Long getHepaticImpairmentIndicator() {
        return hepaticImpairmentIndicator;
    }

    @Override
    public Long getRenalImpairmentIndicator() {
        return renalImpairmentIndicator;
    }

    @Override
    public Long getWeightImpairmentIndicator() {
        return weightImpairmentIndicator;
    }

    @Override
    public Long getBsaRequiredIndicator() {
        return bsaRequiredIndicator;
    }

    @Override
    public String getWarningCode() {
        return warningCode;
    }

    @Override
    public Double getDoseRateLow() {
        return doseRateLow;
    }

    @Override
    public Double getDoseRateHigh() {
        return doseRateHigh;
    }

    @Override
    public Double getDoseFormRateLow() {
        return doseFormRateLow;
    }

    @Override
    public Double getDoseFormRateHigh() {
        return doseFormRateHigh;
    }

    @Override
    public Double getMaxSingleDoseRate() {
        return maxSingleDoseRate;
    }

    @Override
    public Double getMaxSingleDoseFormRate() {
        return maxSingleDoseFormRate;
    }

    @Override
    public Double getMaxDailyDoseRate() {
        return maxDailyDoseRate;
    }

    @Override
    public Double getMaxDailyDoseFormRate() {
        return maxDailyDoseFormRate;
    }

    @Override
    public Long getAgeLowInDays() {
        return ageLowInDays;
    }

    @Override
    public void setAgeLowInDays(Long ageLowInDays) {
        this.ageLowInDays = convertToPrimitive(ageLowInDays);
    }

    @Override
    public Long getAgeHighInDays() {
        return ageHighInDays;
    }

    @Override
    public void setAgeHighInDays(Long ageHighInDays) {
        this.ageHighInDays = convertToPrimitive(ageHighInDays);
    }

    @Override
    public void setDoseFormLow(Double doseFormLow) {
        this.doseFormLow = convertToPrimitive(doseFormLow);
    }

    @Override
    public void setDoseHigh(Double doseHigh) {
        this.doseHigh = convertToPrimitive(doseHigh);
    }

    @Override
    public void setDoseFormHigh(Double doseFormHigh) {
        this.doseFormHigh = convertToPrimitive(doseFormHigh);
    }

    @Override
    public void setMaxSingleDose(Double maxSingleDose) {
        this.maxSingleDose = convertToPrimitive(maxSingleDose);
    }

    @Override
    public void setMaxSingleDoseForm(Double maxSingleDoseForm) {
        this.maxSingleDoseForm = convertToPrimitive(maxSingleDoseForm);
    }

    @Override
    public void setMaxDailyDose(Double maxDailyDose) {
        this.maxDailyDose = convertToPrimitive(maxDailyDose);
    }

    @Override
    public void setMaxDailyDoseForm(Double maxDailyDoseForm) {
        this.maxDailyDoseForm = convertToPrimitive(maxDailyDoseForm);
    }

    @Override
    public void setMaxLifeTimeDose(Double maxLifeTimeDose) {
        this.maxLifeTimeDose = convertToPrimitive(maxLifeTimeDose);
    }

    @Override
    public void setMaxLifeTimeDoseForm(Double maxLifeTimeDoseForm) {
        this.maxLifeTimeDoseForm = convertToPrimitive(maxLifeTimeDoseForm);
    }

    @Override
    public void setDurationLow(Long durationLow) {
        this.durationLow = convertToPrimitive(durationLow);
    }

    @Override
    public void setDurationHigh(Long durationHigh) {
        this.durationHigh = convertToPrimitive(durationHigh);
    }

    @Override
    public DoseRangeVA getVersionedObject() {
        return getDoseRangeVa();
    }

    /**
     * Sets the versioned object.
     * 
     * @param doseRangeVa
     *            the Dose Range VA to set
     */
    public void setVersionedObject(DoseRangeVA doseRangeVa) {
        this.setDoseRangeVa(doseRangeVa);
    }

    @Override
    public void copy(Version<DoseRange> other) {
        super.copy(other);
        DoseRangeVersionVa drVa = (DoseRangeVersionVa) other;
        setDoseRangeVa(drVa.getVersionedObject());

        ageHighInDays = drVa.getAgeHighInDays();
        ageLowInDays = drVa.getAgeLowInDays();
        setReferenceText(drVa.getReferenceText());
        conceptid = drVa.getConceptid();
        conceptType = drVa.getConceptType();
        doseRoute = drVa.getDoseRoute();
        doseType = drVa.getDoseType(); // dwd temp
        setReferenceText(drVa.getReferenceText());
        category = drVa.getCategory();
        fdbdx = drVa.getFdbdx();
        lowEliminationHalfLife = drVa.getLowEliminationHalfLife();
        highEliminationHalfLife = drVa.getHighEliminationHalfLife();
        frequencyLow = drVa.getFrequencyLow();
        frequencyHigh = drVa.getFrequencyHigh();
        durationLow = drVa.getDurationLow();
        durationHigh = drVa.getDurationHigh();
        maxDuration = drVa.getMaxDuration();
        renalImpairmentIndicator = drVa.getRenalImpairmentIndicator();
        weightImpairmentIndicator = drVa.getWeightImpairmentIndicator();
        bsaRequiredIndicator = drVa.getBsaRequiredIndicator();
        hepaticImpairmentIndicator = drVa.getHepaticImpairmentIndicator();
        warningCode = drVa.getWarningCode();
        doseLow = drVa.getDoseLow();
        doseRateLow = drVa.getDoseRateLow();
        doseRateHigh = drVa.getDoseRateHigh();
        doseFormRateLow = drVa.getDoseFormRateLow();
        doseFormRateHigh = drVa.getDoseFormRateHigh();
        maxSingleDoseRate = drVa.getMaxSingleDoseRate();
        maxSingleDoseFormRate = drVa.getMaxSingleDoseFormRate();
        maxDailyDoseRate = drVa.getMaxDailyDoseRate();
        maxDailyDoseFormRate = drVa.getMaxDailyDoseFormRate();
        dxid = drVa.getDxid();
        crclThreshold = drVa.getCrclThreshold();
        ageLowInDays = drVa.getAgeLowInDays();
        ageHighInDays = drVa.getAgeHighInDays();
        doseLowUnits = drVa.getDoseLowUnits();
        doseHighUnits = drVa.getDoseHighUnits();
        doseFormLowUnits = drVa.getDoseFormLowUnits();
        doseFormHighUnits = drVa.getDoseFormHighUnits();
        maxSingleDoseUnits = drVa.getMaxSingleDoseUnits();
        maxSingleDoseFormUnits = drVa.getMaxSingleDoseFormUnits();
        maxDailyDoseUnits = drVa.getMaxDailyDoseUnits();
        maxDailyDoseFormUnits = drVa.getMaxDailyDoseFormUnits();
        maxLifeTimeDoseUnits = drVa.getMaxLifeTimeDoseUnits();
        maxLifeTimeDoseFormUnits = drVa.getMaxLifeTimeDoseFormUnits();
        crclThresholdUnits = drVa.getCrclThresholdUnits();
        doseRateLowUnits = drVa.getDoseRateLowUnits();
        doseRateHighUnits = drVa.getDoseRateHighUnits();
        doseFormRateLowUnits = drVa.getDoseFormRateLowUnits();
        doseFormRateHighUnits = drVa.getDoseFormRateHighUnits();
        maxSingleDoseRateUnits = drVa.getMaxSingleDoseRateUnits();
        maxSingleDoseFormRateUnits = drVa.getMaxSingleDoseFormRateUnits();
        maxDailyDoseRateUnits = drVa.getMaxDailyDoseRateUnits();
        maxDailyDoseFormRateUnits = drVa.getMaxDailyDoseFormRateUnits();
        doseFormLow = drVa.getDoseFormLow();
        doseHigh = drVa.getDoseHigh();
        doseFormHigh = drVa.getDoseFormHigh();
        maxSingleDose = drVa.getMaxSingleDose();
        maxSingleDoseForm = drVa.getMaxSingleDoseForm();
        maxDailyDose = drVa.getMaxDailyDose();
        maxDailyDoseForm = drVa.getMaxDailyDoseForm();
        maxLifeTimeDose = drVa.getMaxLifeTimeDose();
        maxLifeTimeDoseForm = drVa.getMaxLifeTimeDoseForm();

        maxSingleNTEDose = drVa.getMaxSingleNTEDose();
        maxSingleNTEDoseUnit = drVa.getMaxSingleNTEDoseUnit();
        maxSingleNTEDoseForm = drVa.getMaxSingleNTEDoseForm();
        maxSingleNTEDoseFormUnit = drVa.getMaxSingleNTEDoseFormUnit();
        halfLifeUnits = drVa.getHalfLifeUnits();
    }

    @Override
    public Long getConceptid() {
        return conceptid;
    }

    @Override
    public void setConceptid(Long conceptid) {
        this.conceptid = conceptid;
    }

    @Override
    public Long getConceptType() {
        return conceptType;
    }

    @Override
    public void setConceptType(Long conceptType) {
        this.conceptType = conceptType;
    }

    @Override
    public DoseRoute getDoseRoute() {
        return doseRoute;
    }

    @Override
    public void setDoseRoute(DoseRoute doseRoute) {
        this.doseRoute = doseRoute;
    }

    @Override
    public DoseType getDoseType() {
        return doseType;
    }

    @Override
    public void setDoseType(DoseType doseType) {
        this.doseType = doseType;
    }

    @Override
    public String getCategory() {
        return category;
    }

    @Override
    public void setCategory(String category) {
        this.category = category;
    }

    /**
     * gets the concept discription.
     * 
     * @return a string
     */
    public String getConceptDescription() {
        return conceptDescription;
    }

    /**
     * Gets the doseRangeVa.
     * 
     * @return the doseRangeVa
     */
    public DoseRangeVA getDoseRangeVa() {
        return doseRangeVa;
    }

    /**
     * Sets the doseRangeVa.
     * 
     * @param doseRangeVa
     *            the doseRangeVa to set
     */
    public void setDoseRangeVa(DoseRangeVA doseRangeVa) {
        this.doseRangeVa = doseRangeVa;
    }

    /**
     * Gets the referenceText.
     * 
     * @return the referenceText
     */
    public String getReferenceText() {
        return referenceText;
    }

    /**
     * Sets the referenceText.
     * 
     * @param referenceText
     *            the referenceText to set
     */
    public void setReferenceText(String referenceText) {
        this.referenceText = referenceText;
    }

    @Override
    public Double getMaxSingleNTEDose() {
        return maxSingleNTEDose;
    }

    @Override
    public void setMaxSingleNTEDose(Double maxSingleNTEDose) {
        this.maxSingleNTEDose = convertToPrimitive(maxSingleNTEDose);
    }

    @Override
    public String getMaxSingleNTEDoseUnit() {
        return maxSingleNTEDoseUnit;
    }

    @Override
    public void setMaxSingleNTEDoseUnit(String maxSingleNTEDoseUnit) {
        this.maxSingleNTEDoseUnit = maxSingleNTEDoseUnit;
    }

    @Override
    public Double getMaxSingleNTEDoseForm() {
        return maxSingleNTEDoseForm;
    }

    @Override
    public void setMaxSingleNTEDoseForm(Double maxSingleNTEDoseForm) {
        this.maxSingleNTEDoseForm = convertToPrimitive(maxSingleNTEDoseForm);
    }

    @Override
    public String getMaxSingleNTEDoseFormUnit() {
        return maxSingleNTEDoseFormUnit;
    }

    @Override
    public void setMaxSingleNTEDoseFormUnit(String maxSingleNTEDoseFormUnit) {
        this.maxSingleNTEDoseFormUnit = maxSingleNTEDoseFormUnit;
    }

    /**
     * Checks to see if the value being passed in is not null before returning the primitive. This prevents
     * NullPointerExceptions when trying to set a primitive to an Object.
     * 
     * @param value
     *            the value
     * @return double
     */
    private double convertToPrimitive(Double value) {
        if (value != null) {
            return value.doubleValue();
        }
        return 0;
    }

    /**
     * Checks to see if the value being passed in is not null before returning the value. This prevents NullPointerExceptions
     * when trying to set a primitive to an Object.
     * 
     * @param value
     *            the value
     * @return long
     */
    private long convertToPrimitive(Long value) {
        if (value != null) {
            return value.longValue();
        }
        return 0L;
    }

    /**
     * Sets the conceptDescription.
     * 
     * @param conceptDescription
     *            the conceptDescription to set
     */
    public void setConceptDescription(String conceptDescription) {
        this.conceptDescription = conceptDescription;
    }
}
