package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.state.Status;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * Drug Drug Interaction Va.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class DrugDrugInteractionVa extends AbstractCustomizable<DrugDrugInteraction> implements
        Customizable<DrugDrugInteraction>, DrugDrugInteraction, Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 5810016637344314627L;

    private static final Long MAX_FDB_ID = 32000L;

    private Long id;
    private String requestor;
    private Long interactionId;
    private Long fdbId;
    private Long reverseFdbId;

    private DrugDrugInteractionExportManager exportManager;
    private List<Version<DrugDrugInteraction>> versionHistory;
    private Set<DrugPairVa> drugPairs;

    /**
     * DrugDrugInteractionVa Create.
     * 
     * @return DrugDrugInteractionVa
     */
    public static DrugDrugInteractionVa create() {
        return new DrugDrugInteractionVa(true);
    }

    /**
     * Constructor.
     * 
     */
    public DrugDrugInteractionVa() {
    }

    /**
     * Drug Drug Interaction Va ant compile.
     * 
     * @param initHistory
     *            - boolean
     */
    public DrugDrugInteractionVa(Boolean initHistory) {
        if (initHistory) {
            this.exportManager = new DrugDrugInteractionExportManager();
            this.exportManager.setDrugDrugInteractionVa(this);
            this.exportManager.setCurrentVersion(new DrugDrugInteractionVersionVa(this));
            this.exportManager.getCurrentVersion().setDate(new Date());
            versionHistory = new ArrayList<Version<DrugDrugInteraction>>();
            versionHistory.add(getCurrentVersion());
        }
    }

    /**
     * Get Id.
     * 
     * @return id
     */
    @Override
    public Long getId() {
        return id;
    }

    /**
     * Set Id.
     * 
     * @param id
     *            - long
     */
    @Override
    public void setId(Long id) {
        this.id = id;
    }

    /**
     * Get Interaction Id.
     * 
     * @return interactionid
     */
    @Override
    public Long getInteractionId() {
        return interactionId;
    }

    /**
     * Set InteractionId.
     * 
     * @param interactionId
     *            - Long
     */
    @Override
    public void setInteractionId(Long interactionId) {
        this.interactionId = interactionId;
    }

    /**
     * Get Description.
     * 
     * @return getCurrentVersion().getDescription()
     */
    @Override
    public String getDescription() {
        return getCurrentVersion().getDescription();
    }

    /**
     * Set Description.
     * 
     * @param description
     *            - String
     */
    @Override
    public void setDescription(String description) {
        getModifiableCurrentVersion().setDescription(description);
    }

    /**
     * Get FdbId.
     * 
     * @return fdbid
     */
    @Override
    public Long getFdbId() {
        return fdbId;
    }

    /**
     * Set FdbId.
     * 
     * @param fdbId
     *            - Long
     */
    @Override
    public void setFdbId(Long fdbId) {
        this.fdbId = fdbId;
    }

    /**
     * Reverse Description.
     * 
     * @return getCurrentVersion().getReverseDescription()
     */
    @Override
    public String getReverseDescription() {
        return getCurrentVersion().getReverseDescription();
    }

    /**
     * Get ReverseFdbId.
     * 
     * @return null || MAX_FDB_ID - fdbId
     */
    @Override
    public Long getReverseFdbId() {
        if (getFdbId() == null) {
            return null;
        }
        return reverseFdbId = MAX_FDB_ID - getFdbId();
    }

    /**
     * Sets the ReverseFdbId.
     * 
     * @param reverseFdbId
     *            the reverseFdbId to set
     */
    public void setReverseFdbId(Long reverseFdbId) {
        this.reverseFdbId = reverseFdbId;
    }

    /**
     * Get SeverityLevel. return getCurrentVersion.getSeverityLevel
     * 
     * @return SeverityLevel -the severity level
     */
    @Override
    public SeverityLevel getSeverityLevel() {
        return getCurrentVersion().getSeverityLevel();
    }

    /**
     * Set SeverityLevel.
     * 
     * @param severityLevel
     *            - SeverityLevel
     */
    @Override
    public void setSeverityLevel(SeverityLevel severityLevel) {
        getModifiableCurrentVersion().setSeverityLevel(severityLevel);
    }

    /**
     * Get ClinicalEffect1.
     * 
     * @return ClinicalEffect1
     */
    @Override
    public ClinicalEffect getClinicalEffect1() {
        return getCurrentVersion().getClinicalEffect1();
    }

    /**
     * Set ClinicalEffect 1.
     * 
     * @param clinicalEffect
     *            - ClinicalEffect
     */
    @Override
    public void setClinicalEffect1(ClinicalEffect clinicalEffect) {
        getModifiableCurrentVersion().setClinicalEffect1(clinicalEffect);
    }

    /**
     * Get Clinical Effect 2.
     * 
     * @return ClinicalEffect2
     * 
     */
    @Override
    public ClinicalEffect getClinicalEffect2() {
        return getCurrentVersion().getClinicalEffect2();
    }

    /**
     * Set Clinical Effect 2.
     * 
     * @param clinicalEffect
     *            - CliicalEffect
     */
    @Override
    public void setClinicalEffect2(ClinicalEffect clinicalEffect) {
        getModifiableCurrentVersion().setClinicalEffect2(clinicalEffect);
    }

    /**
     * Get MonographId.
     * 
     * @return monographId
     */
    @Override
    public Long getMonographId() {
        return getCurrentVersion().getMonographId();
    }

    /**
     * Set MonographId.
     * 
     * @param monographId
     *            - Long
     */
    @Override
    public void setMonographId(Long monographId) {
        getModifiableCurrentVersion().setMonographId(monographId);
    }

    /**
     * Get Requestor.
     * 
     * @return requestor
     */
    @Override
    public String getRequestor() {
        return this.requestor;
    }

    /**
     * Set Requestor.
     * 
     * @param requestor
     *            - String
     */
    @Override
    public void setRequestor(String requestor) {
        this.requestor = requestor;
    }

    /**
     * Get Exportable Version.
     * 
     * @return exportableVersion
     */
    @Override
    public DrugDrugInteractionVersionVa getExportableVersion() {
        return getExportManager().getExportableVersion();
    }

    /**
     * Set ExportableVersion.
     * 
     * @param v
     *            - Version<DrugDrugInteraction>
     */
    @Override
    public void setExportableVersion(Version<DrugDrugInteraction> v) {
        this.getExportManager().setExportableVersion((DrugDrugInteractionVersionVa) v);
    }

    /**
     * Get ExportedVersion.
     * 
     * @return exportedVersion
     */
    public DrugDrugInteractionVersionVa getExportedVersion() {
        return getExportManager().getExportedVersion();
    }

    /**
     * Set ExportedVersion.
     * 
     * @param v
     *            - Version<DrugDrugInteraction>
     */
    @Override
    public void setExportedVersion(Version<DrugDrugInteraction> v) {
        getExportManager().setExportedVersion((DrugDrugInteractionVersionVa) v);
    }

    /**
     * Get CurrentVersion.
     * 
     * @return currentVersion
     */
    @Override
    public DrugDrugInteractionVersionVa getCurrentVersion() {
        return getExportManager().getCurrentVersion();
    }

    /**
     * Set CurrentVersion.
     * 
     * @param v
     *            - Version<DrugDrugInteraction>
     */
    public void setCurrentVersion(Version<DrugDrugInteraction> v) {
        getExportManager().setCurrentVersion((DrugDrugInteractionVersionVa) v);
    }

    /**
     * Get VersionHIstory.
     * 
     * @return versionHistory
     */
    @Override
    public List<Version<DrugDrugInteraction>> getVersionHistory() {
        return versionHistory;
    }

    /**
     * Set Version History.
     * 
     * @param history
     *            - List<Version<drugDrugInteraction>>
     */
    @Override
    public void setVersionHistory(List<Version<DrugDrugInteraction>> history) {
        this.versionHistory = history;
    }

    /**
     * Get DrugPairs.
     * 
     * @return drugPairs
     */
    public Set<DrugPairVa> getDrugPairs() {
        return drugPairs;
    }

    /**
     * Set DrugPairs.
     * 
     * @param drugPairs
     *            - Set<DrugPairVa>
     */
    public void setDrugPairs(Set<DrugPairVa> drugPairs) {
        this.drugPairs = drugPairs;
    }

    /**
     * Get a version that can be modified. Once a version is persisted, It can no longer be modified (for audit purposes). If
     * the current version of this monograph is persistent, this method will create a copy of it, make it the current version
     * and return it.
     * 
     * @return the current non-persistent version
     */
    @Override
    protected DrugDrugInteractionVersionVa getModifiableCurrentVersion() {
        if (!getExportManager().getCurrentVersion().isModifiable()) {
            DrugDrugInteractionVersionVa v = newVersion();
            v.copy(getCurrentVersion());
            setCurrentVersion(v);
            setRolledBack(false);
        }
        return getExportManager().getCurrentVersion();
    }

    /**
     * New Version.
     * 
     * @return v
     */
    private DrugDrugInteractionVersionVa newVersion() {
        DrugDrugInteractionVersionVa v = new DrugDrugInteractionVersionVa();
        versionHistory.add(v);
        return v;
    }

    /**
     * Roll Back.
     * 
     */
    @Override
    public void rollBack() {
        if (!getExportManager().getCurrentVersion().isModifiable()) {
            DrugDrugInteractionVersionVa v = newVersion();
            setCurrentVersion(v);
        }
        getCurrentVersion().copy(getExportableVersion());
        setRolledBack(true);
    }

    /**
     * Get EdiNumber.
     * 
     * @return EdiNumber
     */
    @Override
    public ReferenceTextVa getEdiNumber() {
        return getCurrentVersion().getEdiNumber();
    }

    /**
     * Set EdiNumber.
     * 
     * @param ediNumber
     *            - ReferenceTextVa
     */
    @Override
    public void setEdiNumber(ReferenceTextVa ediNumber) {
        getModifiableCurrentVersion().setEdiNumber(ediNumber);
    }

    /**
     * Get EdiText.
     * 
     * @return EdiText.
     */
    @Override
    public String getEdiText() {
        return getCurrentVersion().getEdiText();
    }

    /**
     * SetEdi Text.
     * 
     * @param ediText
     *            - the edi text
     */
    @Override
    public void setEdiText(String ediText) {
        getModifiableCurrentVersion().setEdiText(ediText);
    }

    /**
     * Get DiFactNumber.
     * 
     * @return DiFactNumber
     */
    @Override
    public ReferenceTextVa getDiFactNumber() {
        return getCurrentVersion().getDiFactNumber();
    }

    /**
     * Set DiFactNumber.
     * 
     * @param diFactNumber
     *            - ReferenceTextVa
     */
    @Override
    public void setDiFactNumber(ReferenceTextVa diFactNumber) {
        getModifiableCurrentVersion().setDiFactNumber(diFactNumber);
    }

    /**
     * GetDiFactDocumentation.
     * 
     * @return DiFactDocumentation.
     */
    @Override
    public ReferenceTextVa getDiFactDocumentation() {
        return getCurrentVersion().getDiFactDocumentation();
    }

    /**
     * Set DiFactDocumentation.
     * 
     * @param diFactDocumentation
     *            - ReferenceTextVa
     */
    @Override
    public void setDiFactDocumentation(ReferenceTextVa diFactDocumentation) {
        getModifiableCurrentVersion().setDiFactDocumentation(diFactDocumentation);
    }

    /**
     * Get DiFactOnset.
     * 
     * @return DiFactOnset
     */
    @Override
    public ReferenceTextVa getDiFactOnset() {
        return getCurrentVersion().getDiFactOnset();
    }

    /**
     * Set DiFactonset.
     * 
     * @param diFactOnset
     *            - ReferenceTextVa
     */
    @Override
    public void setDiFactOnset(ReferenceTextVa diFactOnset) {
        getModifiableCurrentVersion().setDiFactOnset(diFactOnset);
    }

    /**
     * Get DiFactSeverity.
     * 
     * @return DiFactSeverity
     */
    @Override
    public ReferenceTextVa getDiFactSeverity() {
        return getCurrentVersion().getDiFactSeverity();
    }

    /**
     * Set DiFactSeverity.
     * 
     * @param diFactSeverity
     *            - ReferenceTextVa
     */
    @Override
    public void setDiFactSeverity(ReferenceTextVa diFactSeverity) {
        getModifiableCurrentVersion().setDiFactSeverity(diFactSeverity);
    }

    /**
     * Get DiFactText.
     * 
     * @return DiFactText
     */
    @Override
    public String getDiFactText() {
        return getCurrentVersion().getDiFactText();
    }

    /**
     * Set DiFactText.
     * 
     * @param diFactText
     *            - String
     */
    @Override
    public void setDiFactText(String diFactText) {
        getModifiableCurrentVersion().setDiFactText(diFactText);
    }

    /**
     * Get MicromedexOnset.
     * 
     * @return MicromedexOnset
     */
    @Override
    public ReferenceTextVa getMicromedexOnset() {
        return getCurrentVersion().getMicromedexOnset();
    }

    /**
     * Set MicromedexOnset.
     * 
     * @param micromedexOnset
     *            - ReferenceTextVa
     */
    @Override
    public void setMicromedexOnset(ReferenceTextVa micromedexOnset) {
        getModifiableCurrentVersion().setMicromedexOnset(micromedexOnset);
    }

    /**
     * Get MicromedexSeverity.
     * 
     * @return MicromedexSeverity
     */
    @Override
    public ReferenceTextVa getMicromedexSeverity() {
        return getCurrentVersion().getMicromedexSeverity();
    }

    /**
     * Set MicromedexSeverity.
     * 
     * @param micromedexSeverity
     *            ReferenceTextVa
     */
    @Override
    public void setMicromedexSeverity(ReferenceTextVa micromedexSeverity) {
        getModifiableCurrentVersion().setMicromedexSeverity(micromedexSeverity);
    }

    /**
     * Get MicromedexSubstantiation.
     * 
     * @return MicromedexSubstantiation
     */
    @Override
    public ReferenceTextVa getMicromedexSubstantiation() {
        return getCurrentVersion().getMicromedexSubstantiation();
    }

    /**
     * Set micromedexSubstantiation.
     * 
     * @param micromedexSubstantiation
     *            - referenceTextVa
     */
    @Override
    public void setMicromedexSubstantiation(ReferenceTextVa micromedexSubstantiation) {
        getModifiableCurrentVersion().setMicromedexSubstantiation(micromedexSubstantiation);
    }

    /**
     * Get Micromedex Text.
     * 
     * @return - Micromedex Text
     */
    @Override
    public String getMicromedexText() {
        return getCurrentVersion().getMicromedexText();
    }

    /**
     * Set Micromedex Text.
     * 
     * @param micromedexText
     *            - String
     */
    @Override
    public void setMicromedexText(String micromedexText) {
        getModifiableCurrentVersion().setMicromedexText(micromedexText);
    }

    /**
     * Get MedlineHit.
     * 
     * @return MedlineHit
     */
    @Override
    public ReferenceTextVa getMedlineHit() {
        return getCurrentVersion().getMedlineHit();
    }

    /**
     * Set MedLineHit.
     * 
     * @param medlineHit
     *            - ReferenceTextVa
     */
    @Override
    public void setMedlineHit(ReferenceTextVa medlineHit) {
        getModifiableCurrentVersion().setMedlineHit(medlineHit);
    }

    /**
     * Get MedlineText.
     * 
     * @return MedlineText.
     */
    @Override
    public String getMedlineText() {
        return getCurrentVersion().getMedlineText();
    }

    /**
     * MedlineText.
     * 
     * @param medlineText
     *            - string
     */
    @Override
    public void setMedlineText(String medlineText) {
        getModifiableCurrentVersion().setMedlineText(medlineText);
    }

    /**
     * PackageInsert.
     * 
     * @return PackageInsert
     */
    @Override
    public ReferenceTextVa getPackageInsert() {
        return getCurrentVersion().getPackageInsert();
    }

    /**
     * Set Package Insert.
     * 
     * @param packageInsert
     *            - ReferenceTextVa.
     */
    @Override
    public void setPackageInsert(ReferenceTextVa packageInsert) {
        getModifiableCurrentVersion().setPackageInsert(packageInsert);
    }

    /**
     * Get Package Insert text.
     * 
     * @return InsertText.
     */
    @Override
    public String getPackageInsertText() {
        return getCurrentVersion().getPackageInsertText();
    }

    /**
     * Set Package Text.
     * 
     * @param packageInsertText
     *            - String
     */
    @Override
    public void setPackageInsertText(String packageInsertText) {
        getModifiableCurrentVersion().setPackageInsertText(packageInsertText);
    }

    /**
     * PbmCriteria.
     * 
     * @return PbmCriteria
     */
    @Override
    public ReferenceTextVa getPbmCriteria() {
        return getCurrentVersion().getPbmCriteria();
    }

    /**
     * Set PbmCriteria.
     * 
     * @param pbmCriteria
     *            - ReferenceTextVa
     */
    @Override
    public void setPbmCriteria(ReferenceTextVa pbmCriteria) {
        getModifiableCurrentVersion().setPbmCriteria(pbmCriteria);
    }

    /**
     * Get PbmCriteriaText.
     * 
     * @return PbmCriteriaText
     */
    @Override
    public String getPbmCriteriaText() {
        return getCurrentVersion().getPbmCriteriaText();
    }

    /**
     * Set PbmCriteriaText.
     * 
     * @param pbmCriteriaText
     *            - String
     */
    @Override
    public void setPbmCriteriaText(String pbmCriteriaText) {
        getModifiableCurrentVersion().setPbmCriteriaText(pbmCriteriaText);
    }

    /**
     * Get AidsGuideLine.
     * 
     * @return AidsGuideline
     */
    @Override
    public ReferenceTextVa getAidsGuideline() {
        return getCurrentVersion().getAidsGuideline();
    }

    /**
     * SetAidesGuideLine ReferenceTextVa.
     * 
     * @param aidsGuideline
     *            - ReferenceTextVa
     */
    @Override
    public void setAidsGuideline(ReferenceTextVa aidsGuideline) {
        getModifiableCurrentVersion().setAidsGuideline(aidsGuideline);
    }

    /**
     * Get AidsGuideLineText.
     * 
     * @return AidsGuidelineText
     */
    @Override
    public String getAidsGuidelineText() {
        return getCurrentVersion().getAidsGuidelineText();
    }

    /**
     * Set AidsGuidlineText.
     * 
     * @param aidsGuidelineText
     *            - String
     */
    @Override
    public void setAidsGuidelineText(String aidsGuidelineText) {
        getModifiableCurrentVersion().setAidsGuidelineText(aidsGuidelineText);
    }

    /**
     * Get InteractionSource.
     * 
     * @return InteractionSource.
     */
    @Override
    public ReferenceTextVa getInteractionSource() {
        return getCurrentVersion().getInteractionSource();
    }

    /**
     * Set InteractionSource.
     * 
     * @param interactionSource
     *            - ReferenceTextVa
     */
    @Override
    public void setInteractionSource(ReferenceTextVa interactionSource) {
        getModifiableCurrentVersion().setInteractionSource(interactionSource);
    }

    /**
     * Get InteractionType.
     * 
     * @return InteractionType
     */
    @Override
    public ReferenceTextVa getInteractionType() {
        return getCurrentVersion().getInteractionType();
    }

    /**
     * Set Interaction Type.
     * 
     * @param interactionType
     *            - ReferenceTextVa
     */
    @Override
    public void setInteractionType(ReferenceTextVa interactionType) {
        getModifiableCurrentVersion().setInteractionType(interactionType);
    }

    /**
     * Get Level Of Evidence.
     * 
     * @return LevelOfEvidence
     */
    @Override
    public ReferenceTextVa getLevelOfEvidence() {
        return getCurrentVersion().getLevelOfEvidence();
    }

    /**
     * Set level of Evidence.
     * 
     * @param levelOfEvidence
     *            - ReferenceTextVa
     */
    @Override
    public void setLevelOfEvidence(ReferenceTextVa levelOfEvidence) {
        getModifiableCurrentVersion().setLevelOfEvidence(levelOfEvidence);
    }

    /**
     * Get Group Discussion.
     * 
     * @return GroupDiscussion.
     */
    @Override
    public String getGroupDiscussion() {
        return getCurrentVersion().getGroupDiscussion();
    }

    /**
     * Set Group Discussion.
     * 
     * @param groupDiscussion
     *            - String
     */
    @Override
    public void setGroupDiscussion(String groupDiscussion) {
        getModifiableCurrentVersion().setGroupDiscussion(groupDiscussion);
    }

    /**
     * A Drug-Drug Interaction can only be reviewed if all Drug Pairs associated with the DDI are in an Approved, Reviewed,
     * Rejected or Deleted state. But not all of them can be Rejected or Deleted.
     * 
     * @return boolean - true if can review
     */
    public boolean canReview() {
        boolean hasValidDrugPairs = false;

        // If there are no Drug Pairs, automatically set to false
        if (drugPairs.isEmpty()) {
            return false;
        }
        boolean flag = false;
        // if all dp in delete review, do not show the button.
        for (DrugPairVa drugPair : drugPairs) {
            final String statusName = drugPair.getStatus().getName();
            if (Status.STATUS_DELETE_REVIEWED.equals(statusName) || Status.STATUS_REJECTED.equals(statusName)
                    || Status.STATUS_DELETED.equals(statusName)) {
                flag = true;
            } else {
                flag = false;
                break;
            }
        }
        if (flag) {
            return false;// do not show the button
        }
        // old original code for loop
        for (DrugPairVa drugPair : drugPairs) {
            final String statusName = drugPair.getStatus().getName();
            if (Status.STATUS_APPROVED.equals(statusName) || Status.STATUS_REVIEWED.equals(statusName)
                    || Status.STATUS_DELETE_REVIEWED.equals(statusName)) {
                hasValidDrugPairs = true;
            } else if (!Status.STATUS_REJECTED.equals(statusName) && !Status.STATUS_DELETED.equals(statusName)) {
                return false; // do not show the button
            }
        }

        return hasValidDrugPairs;
    }

    /**
     * A Drug-Drug Interaction can only be approved if all Drug Pairs associated with the DDI are in an Approved, Rejected or
     * Deleted state. But not all of them can be Rejected or Deleted.
     * 
     * @return boolean - true if can approve
     */
    public boolean canApprove() {
        boolean hasValidDrugPairs = false;

        // If there are no Drug Pairs, automatically set to false
        if (drugPairs.isEmpty()) {
            return false;
        }

        for (DrugPairVa drugPair : drugPairs) {
            final String statusName = drugPair.getStatus().getName();
            if (Status.STATUS_APPROVED.equals(statusName)) {
                hasValidDrugPairs = true;
            } else if (!Status.STATUS_REJECTED.equals(statusName) && !Status.STATUS_DELETED.equals(statusName)) {
                return false;
            }
        }
        return hasValidDrugPairs;
    }

    /**
     * A Drug-Drug Interaction can only be deleted if all Drug Pairs associated with the DDI are in a Rejected or Deleted state.
     * 
     * @return boolean - true if can delete
     */
    public boolean canDelete() {
        // If there are no Drug Pairs, automatically set to true
        if (drugPairs.isEmpty()) {
            return true;
        }

        for (DrugPairVa drugPair : drugPairs) {
            final String statusName = drugPair.getStatus().getName();
            if (!Status.STATUS_REJECTED.equals(statusName) && !Status.STATUS_DELETED.equals(statusName)) {
                return false;
            }
        }
        return true;
    }

    /**
     * A Drug-Drug Interaction can only be deleted if all Drug Pairs associated with the DDI are in a Rejected or Deleted state.
     * 
     * @return boolean - true if can submit for delete.
     */
    public boolean canSubmitForDelete() {
        // If there are no Drug Pairs, automatically set to true
        if (drugPairs.isEmpty()) {
            return true;
        }

        for (DrugPairVa drugPair : drugPairs) {
            final String statusName = drugPair.getStatus().getName();
            if (!Status.STATUS_REJECTED.equals(statusName) && !Status.STATUS_DELETED.equals(statusName)
                    && !Status.STATUS_DELETE_REVIEWED.equals(statusName)) {
                return false;
            }
        }
        return true;
    }

    @Override
    protected ConceptXref createConceptXref() {
        return new DrugDrugInteractionXref();
    }

    /**
     * Gets the exportManager.
     * 
     * @return the exportManager
     */
    public DrugDrugInteractionExportManager getExportManager() {
        return exportManager;
    }

    /**
     * Sets the exportManager.
     * 
     * @param exportManager
     *            the exportManager to set
     */
    public void setExportManager(DrugDrugInteractionExportManager exportManager) {
        this.exportManager = exportManager;
    }

}
