package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;

import java.io.Serializable;
import java.util.List;
import java.util.Set;

/**
 * Drug Drug Interaction Va entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class DrugDrugInteractionVersionVa extends AbstractVersion<DrugDrugInteraction> implements
        Version<DrugDrugInteraction>, DrugDrugInteraction, Serializable, Cloneable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -2086594153511848029L;

    /** The versioned object. */

    private DrugDrugInteraction versionedObject;

    /** The severity level. */
    private SeverityLevel severityLevel;

    /** The clinical effect1. */
    private ClinicalEffect clinicalEffect1;

    /** The clinical effect2. */
    private ClinicalEffect clinicalEffect2;

    /** The monograph id. */
    private Long monographId;

    /** The description. */
    private String description;

    // Drug Reference attributes
    /** The edi number. */
    private ReferenceTextVa ediNumber;

    /** The edi text. */
    private String ediText;

    /** The di fact number. */
    private ReferenceTextVa diFactNumber;

    /** The di fact documentation. */
    private ReferenceTextVa diFactDocumentation;

    /** The di fact onset. */
    private ReferenceTextVa diFactOnset;

    /** The di fact severity. */
    private ReferenceTextVa diFactSeverity;

    /** The di fact text. */
    private String diFactText;

    /** The micromedex onset. */
    private ReferenceTextVa micromedexOnset;

    /** The micromedex severity. */
    private ReferenceTextVa micromedexSeverity;

    /** The micromedex substantiation. */
    private ReferenceTextVa micromedexSubstantiation;

    /** The micromedex text. */
    private String micromedexText;

    /** The medline hit. */
    private ReferenceTextVa medlineHit;

    /** The medline text. */
    private String medlineText;

    /** The package insert. */
    private ReferenceTextVa packageInsert;

    /** The package insert text. */
    private String packageInsertText;

    /** The pbm criteria. */
    private ReferenceTextVa pbmCriteria;

    /** The pbm criteria text. */
    private String pbmCriteriaText;

    /** The aids guideline. */
    private ReferenceTextVa aidsGuideline;

    /** The aids guideline text. */
    private String aidsGuidelineText;

    /** The interaction source. */
    private ReferenceTextVa interactionSource;

    /** The interaction type. */
    private ReferenceTextVa interactionType;

    /** The level of evidence. */
    private ReferenceTextVa levelOfEvidence;

    /** The group discussion. */
    private String groupDiscussion;
    /** Sting Description delimiter. */
    public static final String DESCRIPTION_DELIMITER = "/";

    /** Drug Drug Interaction Va Constructor. */
    public DrugDrugInteractionVersionVa() {
    }

    /**
     * DDI Constructor.
     * 
     * @param parent
     *            - DDIVA
     */
    public DrugDrugInteractionVersionVa(DrugDrugInteractionVa parent) {
        this.versionedObject = parent;
    }

    /**
     * Retrieves the Id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getId()
     * @return Long - the id
     */
    @Override
    public Long getId() {
        return versionedObject.getId();
    }

    /**
     * Sets the id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setId(java.lang.Long)
     * @param id
     *            - of type Long
     */
    @Override
    public void setId(Long id) {
        versionedObject.setId(id);
    }

    /**
     * Retrieves the interaction id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getInteractionId()
     * @return Long - the interaction id
     */
    @Override
    public Long getInteractionId() {
        return versionedObject.getInteractionId();
    }

    /**
     * Sets the Interaction Id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setInteractionId(java.lang.Long)
     * @param interactionId
     *            - of type Long
     */
    @Override
    public void setInteractionId(Long interactionId) {
        versionedObject.setInteractionId(interactionId);
    }

    /**
     * Retrieves the Description.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getDescription()
     * @return String - the description
     */
    @Override
    public String getDescription() {
        return description;
    }

    /**
     * Sets the Description.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setDescription(java.lang.String)
     * @param description
     *            - of type String
     */
    @Override
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Retrieves the Reverse Description.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getReverseDescription()
     * @return String the reverse description
     */
    @Override
    public String getReverseDescription() {
        final String[] descriptionParts = description.split(DESCRIPTION_DELIMITER);
        return descriptionParts[1] + DESCRIPTION_DELIMITER + descriptionParts[0];
    }

    /**
     * Retrieves the FDB Id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getFdbId()
     * @return Long - the fdb id
     */
    @Override
    public Long getFdbId() {
        return versionedObject.getFdbId();
    }

    /**
     * Retrieves the Reverse FDB Id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getReverseFdbId()
     * @return Long - the reverse FdbId.
     */
    @Override
    public Long getReverseFdbId() {
        return versionedObject.getReverseFdbId();
    }

    /**
     * Set FDB ID. Not set through a version
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setFdbId(java.lang.Long)
     * @param fdbId
     *            of type Long
     */
    @Override
    public void setFdbId(Long fdbId) {
        // Not settable through a version
    }

    /**
     * Retrieves the Severity Level.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getSeverityLevel()
     * @return SeverityLevel - the severity level
     */
    @Override
    public SeverityLevel getSeverityLevel() {
        return severityLevel;
    }

    /**
     * Sets the Severity Level.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setSeverityLevel(gov.va.med.pharmacy.pecs.domain.impl.lookup.
     *      SeverityLevel)
     * @param severityLevel
     *            of type SeverityLevel
     */
    @Override
    public void setSeverityLevel(SeverityLevel severityLevel) {
        this.severityLevel = severityLevel;
    }

    /**
     * Retrieves the Clinical Effects 1.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getClinicalEffect1()
     * @return ClinicalEffect - the clinical effects
     */
    @Override
    public ClinicalEffect getClinicalEffect1() {
        return clinicalEffect1;
    }

    /**
     * Sets the Clinical Effects.
     * 
     * @param clinicalEffect
     *            of type ClinicalEffect
     */
    @Override
    public void setClinicalEffect1(ClinicalEffect clinicalEffect) {
        this.clinicalEffect1 = clinicalEffect;
    }

    /**
     * Retrieve the ClinicalEffects2.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getClinicalEffect2()
     * @return ClinicalEffects - the clinical effects 2
     */
    @Override
    public ClinicalEffect getClinicalEffect2() {
        return clinicalEffect2;
    }

    /**
     * Sets the Clinical Effect 2.
     * 
     * @param clinicalEffect
     *            of type ClinicalEffect
     */
    @Override
    public void setClinicalEffect2(ClinicalEffect clinicalEffect) {
        this.clinicalEffect2 = clinicalEffect;
    }

    /**
     * Retrieves the Monograph Id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMonographId()
     * @return Long the monograph Id
     */
    @Override
    public Long getMonographId() {
        return monographId;
    }

    /**
     * Sets the Monograph Id.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setMonographId(java.lang.Long)
     * @param monographId
     *            of type Long
     */
    @Override
    public void setMonographId(Long monographId) {
        this.monographId = monographId;
    }

    /**
     * Retrieves the EDI number.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getEdiNumber()
     * @return ReferenceTextVa the edi number
     */
    @Override
    public ReferenceTextVa getEdiNumber() {
        return ediNumber;
    }

    /**
     * Sets the EDI number.
     * 
     * @param ediNumber
     *            - of type ReferenceTextVa
     */
    @Override
    public void setEdiNumber(ReferenceTextVa ediNumber) {
        this.ediNumber = ediNumber;
    }

    /**
     * Retrieves the Edi Text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getEdiText()
     * @return String - the edi text
     */
    @Override
    public String getEdiText() {
        return ediText;
    }

    /**
     * Sets the EDI text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setEdiText(java.lang.String)
     * @param ediText
     *            of type String
     */
    @Override
    public void setEdiText(String ediText) {
        this.ediText = ediText;
    }

    /**
     * Retrieves the Di Fact Number.
     * 
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getDiFactNumber()
     * @return ReferenceTextVa - the di fact number
     */
    @Override
    public ReferenceTextVa getDiFactNumber() {
        return diFactNumber;
    }

    /**
     * Sets the Di Fact Number.
     * 
     * @param diFactNumber
     *            of type ReferenceTextVa
     */
    @Override
    public void setDiFactNumber(ReferenceTextVa diFactNumber) {
        this.diFactNumber = diFactNumber;
    }

    /**
     * Retrieves the Di Fact Documentation.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getDiFactDocumentation()
     * @return ReferenceTextVa the di Fact Documentation
     */
    @Override
    public ReferenceTextVa getDiFactDocumentation() {
        return diFactDocumentation;
    }

    /**
     * Sets the DiFactDocumenation.
     * 
     * @param diFactDocumentation
     *            of type ReferenceTextVa
     */
    @Override
    public void setDiFactDocumentation(ReferenceTextVa diFactDocumentation) {
        this.diFactDocumentation = diFactDocumentation;
    }

    /**
     * Retrieves the DiFactOnset.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getDiFactOnset()
     * @return ReferenceTextVa - the diFactOnset
     */
    @Override
    public ReferenceTextVa getDiFactOnset() {
        return diFactOnset;
    }

    /**
     * Sets the DiFactOnset.
     * 
     * @param diFactOnset
     *            of type ReferenceTextVa
     */
    @Override
    public void setDiFactOnset(ReferenceTextVa diFactOnset) {
        this.diFactOnset = diFactOnset;
    }

    /**
     * Retrieves the DiFactSeverity.
     * 
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getDiFactSeverity()
     * @return ReferenceTextVa - the di fact severity
     * 
     */
    @Override
    public ReferenceTextVa getDiFactSeverity() {
        return diFactSeverity;
    }

    /**
     * Sets the DiFactSeverity.
     * 
     * 
     * @param diFactSeverity
     *            - of type ReferenceTextVa
     */
    @Override
    public void setDiFactSeverity(ReferenceTextVa diFactSeverity) {
        this.diFactSeverity = diFactSeverity;
    }

    /**
     * Retrieves the DiFactText.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getDiFactText()
     * 
     * @return String - the di fact text
     */
    @Override
    public String getDiFactText() {
        return diFactText;
    }

    /**
     * sets the DiFactText.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setDiFactText(java.lang.String)
     * @param diFactText
     *            - the di fact text
     */
    @Override
    public void setDiFactText(String diFactText) {
        this.diFactText = diFactText;
    }

    /**
     * Retrieves the MicromedexOnset.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMicromedexOnset()
     * @return ReferenceTextVa - the micromedex onset
     */
    @Override
    public ReferenceTextVa getMicromedexOnset() {
        return micromedexOnset;
    }

    /**
     * Sets the MicromedexOnset.
     * 
     * @param micromedexOnset
     *            - of type ReferenceTextVa
     */
    @Override
    public void setMicromedexOnset(ReferenceTextVa micromedexOnset) {
        this.micromedexOnset = micromedexOnset;
    }

    /**
     * Retrieves the Micromedex Severity.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMicromedexSeverity()
     * 
     * @return ReferenceTextVa - the micromedex severity
     */
    @Override
    public ReferenceTextVa getMicromedexSeverity() {
        return micromedexSeverity;
    }

    /**
     * Sets the Micromedex Severity.
     * 
     * @param micromedexSeverity
     *            - of type ReferenceTextVa
     */
    @Override
    public void setMicromedexSeverity(ReferenceTextVa micromedexSeverity) {
        this.micromedexSeverity = micromedexSeverity;
    }

    /**
     * Retrieves the Micromedex Substantiation.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMicromedexSubstantiation()
     * @return ReferenceTextVa - the micromedex substantiation
     */
    @Override
    public ReferenceTextVa getMicromedexSubstantiation() {
        return micromedexSubstantiation;
    }

    /**
     * Sets the Micromedex Substantiation.
     * 
     * @param micromedexSubstantiation
     *            of type ReferenceTextVa
     */
    @Override
    public void setMicromedexSubstantiation(ReferenceTextVa micromedexSubstantiation) {
        this.micromedexSubstantiation = micromedexSubstantiation;
    }

    /**
     * Retrieves the Micromedex text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMicromedexText()
     * @return String the micromedex text
     */
    @Override
    public String getMicromedexText() {
        return micromedexText;
    }

    /**
     * Sets the MicromedexText.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setMicromedexText(java.lang.String)
     * @param micromedexText
     *            of type String
     */
    @Override
    public void setMicromedexText(String micromedexText) {
        this.micromedexText = micromedexText;
    }

    /**
     * Retrieves the Medline Hit.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMedlineHit()
     * @return ReferenceTextVa - the medline hit
     */
    @Override
    public ReferenceTextVa getMedlineHit() {
        return medlineHit;
    }

    /**
     * Sets the Medline Hit.
     * 
     * @param medlineHit
     *            of type ReferenceTextVa
     */
    @Override
    public void setMedlineHit(ReferenceTextVa medlineHit) {
        this.medlineHit = medlineHit;
    }

    /**
     * Retrieves the MedlineText.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getMedlineText()
     * @return String the medline text
     */
    @Override
    public String getMedlineText() {
        return medlineText;
    }

    /**
     * Sets the Medline Text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setMedlineText(java.lang.String)
     * @param medlineText
     *            of type String
     */
    @Override
    public void setMedlineText(String medlineText) {
        this.medlineText = medlineText;
    }

    /**
     * Retrieves the Package Insert.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getPackageInsert()
     * @return ReferenceTextVa - the package Insert
     */
    @Override
    public ReferenceTextVa getPackageInsert() {
        return packageInsert;
    }

    /**
     * Sets the Package Insert.
     * 
     * @param packageInsert
     *            of type ReferenceTextVa
     */
    @Override
    public void setPackageInsert(ReferenceTextVa packageInsert) {
        this.packageInsert = packageInsert;
    }

    /**
     * Retrieves the Package Insert Text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getPackageInsertText()
     * @return String the package insert text
     */
    @Override
    public String getPackageInsertText() {
        return packageInsertText;
    }

    /**
     * Sets the Package Insert Text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setPackageInsertText(java.lang.String)
     * @param packageInsertText
     *            of type String
     */
    @Override
    public void setPackageInsertText(String packageInsertText) {
        this.packageInsertText = packageInsertText;
    }

    /**
     * Retrieves the PBM Criteria.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getPbmCriteria()
     * @return ReferenceTextVa the pbm criteria
     */
    @Override
    public ReferenceTextVa getPbmCriteria() {
        return pbmCriteria;
    }

    /**
     * Sets the PBM Criteria.
     * 
     * @param pbmCriteria
     *            of type ReferenceTextVa
     */
    @Override
    public void setPbmCriteria(ReferenceTextVa pbmCriteria) {
        this.pbmCriteria = pbmCriteria;
    }

    /**
     * Retrieves the PBM Criteria text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getPbmCriteriaText()
     * @return String the pbm criteria text
     */
    @Override
    public String getPbmCriteriaText() {
        return pbmCriteriaText;
    }

    /**
     * Sets the PBM Criteria Text.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setPbmCriteriaText(java.lang.String)
     * @param pbmCriteriaText
     *            of type String
     */
    @Override
    public void setPbmCriteriaText(String pbmCriteriaText) {
        this.pbmCriteriaText = pbmCriteriaText;
    }

    /**
     * Retrieves the Aids GuideLine.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getAidsGuideline()
     * @return ReferenceTextVa - the aids guideline
     */
    @Override
    public ReferenceTextVa getAidsGuideline() {
        return aidsGuideline;
    }

    /**
     * Set AidsGuideLine.
     * 
     * @param aidsGuideline
     *            the new aids guideline
     */
    @Override
    public void setAidsGuideline(ReferenceTextVa aidsGuideline) {
        this.aidsGuideline = aidsGuideline;
    }

    /**
     * Get AidsGuideLine.
     * 
     * @return aidsGuidelineText
     */
    @Override
    public String getAidsGuidelineText() {
        return aidsGuidelineText;
    }

    /**
     * Set AidsGuidLine.
     * 
     * @param aidsGuidelineText
     *            - String
     */
    @Override
    public void setAidsGuidelineText(String aidsGuidelineText) {
        this.aidsGuidelineText = aidsGuidelineText;
    }

    /**
     * Get InteractionSource.
     * 
     * @return interactionSource
     */
    @Override
    public ReferenceTextVa getInteractionSource() {
        return interactionSource;
    }

    /**
     * Set Interaction Source.
     * 
     * @param interactionSource
     *            - ReferenceTextVa
     */
    @Override
    public void setInteractionSource(ReferenceTextVa interactionSource) {
        this.interactionSource = interactionSource;
    }

    /**
     * Get InteractionType.
     * 
     * @return interactionType
     */
    @Override
    public ReferenceTextVa getInteractionType() {
        return interactionType;
    }

    /**
     * Set InteractionType.
     * 
     * @param interactionType
     *            - ReferenceTextVa
     */
    @Override
    public void setInteractionType(ReferenceTextVa interactionType) {
        this.interactionType = interactionType;
    }

    /**
     * Get LevelOfEvidence.
     * 
     * @return levelOfEvidence
     */
    @Override
    public ReferenceTextVa getLevelOfEvidence() {
        return levelOfEvidence;
    }

    /**
     * Set Level Of Evidence.
     * 
     * @param levelOfEvidence
     *            - ReferenceTextVa
     */
    @Override
    public void setLevelOfEvidence(ReferenceTextVa levelOfEvidence) {
        this.levelOfEvidence = levelOfEvidence;
    }

    /**
     * Get Group Discussion.
     * 
     * @return groupDiscussion
     */
    @Override
    public String getGroupDiscussion() {
        return groupDiscussion;
    }

    /**
     * Set Group Discussion.
     * 
     * @param groupDiscussion
     *            - String
     */
    @Override
    public void setGroupDiscussion(String groupDiscussion) {
        this.groupDiscussion = groupDiscussion;
    }

    /**
     * Get Version History.
     * 
     * @return VersionHistory
     * 
     */
    @Override
    public List<Version<DrugDrugInteraction>> getVersionHistory() {
        return versionedObject.getVersionHistory();
    }

    /**
     * Set VersionHistory.
     * 
     * @param history
     *            - List<Version<DrugDrugInteraction>>
     */
    @Override
    public void setVersionHistory(List<Version<DrugDrugInteraction>> history) {
        versionedObject.setVersionHistory(history);
    }

    /**
     * Get Version Object.
     * 
     * @return versionedObject.
     */
    @Override
    public DrugDrugInteraction getVersionedObject() {
        return versionedObject;
    }

    /**
     * Set VersionedObject.
     * 
     * @param drugDrugInteraction
     *            DDI
     */
    public void setVersionedObject(DrugDrugInteraction drugDrugInteraction) {
        this.versionedObject = drugDrugInteraction;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractVersion#copy(gov.va.med.pharmacy.pecs.domain.Version)
     */
    @Override
    public void copy(Version<DrugDrugInteraction> other) {
        super.copy(other);
        DrugDrugInteractionVersionVa ddivVa = (DrugDrugInteractionVersionVa) other;
        this.versionedObject = ddivVa.getVersionedObject();
        this.severityLevel = ddivVa.getSeverityLevel();
        this.clinicalEffect1 = ddivVa.getClinicalEffect1();
        this.clinicalEffect2 = ddivVa.getClinicalEffect2();
        this.aidsGuideline = ddivVa.getAidsGuideline();
        this.aidsGuidelineText = ddivVa.getAidsGuidelineText();
        this.description = ddivVa.getDescription();
        this.diFactDocumentation = ddivVa.getDiFactDocumentation();
        this.diFactNumber = ddivVa.getDiFactNumber();
        this.diFactOnset = ddivVa.getDiFactOnset();
        this.diFactSeverity = ddivVa.getDiFactSeverity();
        this.diFactText = ddivVa.getDiFactText();
        this.ediNumber = ddivVa.getEdiNumber();
        this.ediText = ddivVa.getEdiText();
        this.interactionSource = ddivVa.getInteractionSource();
        this.interactionType = ddivVa.getInteractionType();
        this.levelOfEvidence = ddivVa.getLevelOfEvidence();
        this.medlineHit = ddivVa.getMedlineHit();
        this.medlineText = ddivVa.getMedlineText();
        this.micromedexOnset = ddivVa.getMicromedexOnset();
        this.micromedexSeverity = ddivVa.getMicromedexSeverity();
        this.micromedexSubstantiation = ddivVa.getMicromedexSubstantiation();
        this.micromedexText = ddivVa.getMicromedexText();
        this.monographId = ddivVa.getMonographId();
        this.packageInsert = ddivVa.getPackageInsert();
        this.packageInsertText = ddivVa.getPackageInsertText();
        this.pbmCriteria = ddivVa.getPbmCriteria();
        this.pbmCriteriaText = ddivVa.getPbmCriteriaText();
        this.groupDiscussion = ddivVa.getGroupDiscussion();
    }

    /**
     * Do not set Sets the User Comment History on the version.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setUserCommentHistory(java.util.Set)
     * @param userCommentHistory
     *            of type Set<IUserComment>
     */
    @Override
    public void setUserCommentHistory(Set<UserComment> userCommentHistory) {
        // Do not set the user Comment history on the version

    }

    /**
     * Can Not Retrieves the User Comment History - always null.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getUserCommentHistory()
     * @return Set<IUserComment> the set of user comments in history
     */
    @Override
    public Set<UserComment> getUserCommentHistory() {
        // Do not retrieve the User Comment Histroy through the version.
        return getVersionedObject().getUserCommentHistory();
    }

}
