package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.UserComment;

import java.util.HashSet;
import java.util.Set;

/**
 * Domain object to hold Drug-Drug Interaction Cross Reference Information.
 * 
 * @author vhaisaduffg
 * 
 */
public final class DrugDrugInteractionXref implements ConceptXref {

    private Long fdbReferenceId;
    private Long interactionId;
    private Set<UserComment> userCommentHistory;

    /**
     * Sets the FDB Reference Identifier.
     * 
     * @return Long the fdb reference id
     */
    public Long getFdbReferenceId() {
        return fdbReferenceId;
    }

    /**
     * Sets the FDB Reference Identifier.
     * 
     * @param fdbReferenceId
     *            the fdb reference id
     */
    public void setFdbReferenceId(Long fdbReferenceId) {
        this.fdbReferenceId = fdbReferenceId;
    }

    /**
     * Returns the FDB Interaction Identifier.
     * 
     * @return Long the interaction id
     */
    public Long getInteractionId() {
        return interactionId;
    }

    /**
     * Sets the FDB Interaction Identifier the comments are associated.
     * 
     * @param interactionId
     *            the FDB Interaction Identifier
     */
    public void setInteractionId(Long interactionId) {
        this.interactionId = interactionId;
    }

    /**
     * Retrieves the User Comment History.
     * 
     * @return Set<IUserComment> the user comment history
     */
    public Set<UserComment> getUserCommentHistory() {
        if (userCommentHistory == null) {
            userCommentHistory = new HashSet<UserComment>();
        }
        return userCommentHistory;
    }

    /**
     * Sets the User Comment History..
     * 
     * @param userCommentHistory
     *            the user comment history
     */
    public void setUserCommentHistory(Set<UserComment> userCommentHistory) {
        this.userCommentHistory = userCommentHistory;
    }

    /**
     * Method addUserComment.
     * 
     * @param userComment
     *            of type IUserComment
     */
    public void addUserComment(UserComment userComment) {

        getUserCommentHistory().add(userComment);
        userComment.setFdbReference(this);
    }
}
