package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.DrugPair;
import gov.va.med.pharmacy.pecs.domain.RoutedGeneric;
import gov.va.med.pharmacy.pecs.domain.Version;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * DrugPair Va Entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class DrugPairVa extends AbstractCustomizable<DrugPair> implements Customizable<DrugPair>, DrugPair, Serializable {

    private static final long serialVersionUID = 6938454628015030487L;
    private Long id;
    private RoutedGeneric routedGeneric1;
    private RoutedGeneric routedGeneric2;
    private DrugDrugInteraction interaction;
    private DrugPairExportManager exportManager;
    private List<Version<DrugPair>> versionHistory;
    private Long reverseFdbId;

    /**
     * Constructor.
     * 
     */
    public DrugPairVa() {

    }

    /**
     * Full Constructor.
     * 
     * @param rtGen1
     *            - RoutedGeneric
     * @param rtGen2
     *            - RoutedGeneric
     * @param ddi
     *            - DrugDrugInteraction
     */
    private DrugPairVa(RoutedGeneric rtGen1, RoutedGeneric rtGen2, DrugDrugInteraction ddi) {
        routedGeneric1 = rtGen1;
        routedGeneric2 = rtGen2;
        interaction = ddi;
        exportManager = new DrugPairExportManager();
        exportManager.setDrugPairVa(this);
        setCurrentVersion(new DrugPairVersionVa());
        getCurrentVersion().setDate(new Date());
        getCurrentVersion().setVersionedObject(this);

        versionHistory = new ArrayList<Version<DrugPair>>();
        versionHistory.add(getCurrentVersion());
    }

    /**
     * Create Drug Pair.
     * 
     * @param rtGen1
     *            - RoutedGeneric
     * @param rtGen2
     *            - RoutedGeneric
     * @param ddi
     *            - DrugDrugInteraction
     * @return DrugPariVa
     */
    public static DrugPairVa createDrugPair(RoutedGeneric rtGen1, RoutedGeneric rtGen2, DrugDrugInteraction ddi) {
        return new DrugPairVa(rtGen1, rtGen2, ddi);
    }

    /**
     * Get Id.
     * 
     * @return id
     */

    public Long getId() {
        return id;
    }

    /**
     * Set Id.
     * 
     * @param id
     *            - Long
     */
    public void setId(Long id) {
        this.id = id;
    }

    /**
     * Get RoutedGeneric1.
     * 
     * @return routedGeneric1
     */
    @Override
    public RoutedGeneric getRoutedGeneric1() {
        return routedGeneric1;
    }

    /**
     * Set RoutedGeneric1.
     * 
     * @param routedGeneric
     *            the new routed generic1
     */
    public void setRoutedGeneric1(RoutedGeneric routedGeneric) {
        this.routedGeneric1 = routedGeneric;
    }

    /**
     * Get RoutedGeneric2.
     * 
     * @return RoutedGeneric
     */
    @Override
    public RoutedGeneric getRoutedGeneric2() {
        return routedGeneric2;
    }

    /**
     * Set RoutedGeneric 2.
     * 
     * @param routedGeneric
     *            the new routed generic2
     */
    public void setRoutedGeneric2(RoutedGeneric routedGeneric) {
        this.routedGeneric2 = routedGeneric;
    }

    /**
     * Get Interaction.
     * 
     * @return interaction
     */
    @Override
    public DrugDrugInteraction getInteraction() {
        return interaction;
    }

    /**
     * Set Interaction.
     * 
     * @param interaction
     *            the new interaction
     */
    public void setInteraction(DrugDrugInteraction interaction) {
        this.interaction = interaction;
    }

    /**
     * Medical references justifying the customization request.
     * 
     * @return the text
     */
    public String getReferenceText() {
        return getCurrentVersion().getReferenceText();
    }

    /**
     * Provide reference text.
     * 
     * @param referenceText
     *            the text
     */
    public void setReferenceText(String referenceText) {
        getModifiableCurrentVersion().setReferenceText(referenceText);
    }

    /**
     * Get ExportableVersion.
     * 
     * @return exportableVersion
     */
    @Override
    public Version<DrugPair> getExportableVersion() {
        return this.getExportManager().getExportableVersion();
    }

    /**
     * Set ExportableVersion.
     * 
     * @param v
     *            - Version
     */
    @Override
    public void setExportableVersion(Version<DrugPair> v) {
        this.getExportManager().setExportableVersion((DrugPairVersionVa) v);

    }

    /**
     * Get ExportedVersion.
     * 
     * @return the exported Drug Pair Version
     */
    public Version<DrugPair> getExportedVersion() {
        return this.getExportManager().getExportedVersion();
    }

    /**
     * Set ExportedVersion.
     * 
     * @param v
     *            DrugPairVersionVa
     */
    public void setExportedVersion(DrugPairVersionVa v) {
        this.getExportManager().setExportedVersion(v);
    }

    /**
     * Set ExportedVersion.
     * 
     * @param v
     *            DrugPairVersionVa
     */
    @Override
    public void setExportedVersion(Version<DrugPair> v) {
        this.getExportManager().setExportedVersion((DrugPairVersionVa) v);
    }

    @Override
    /**Get Current Version.
     * @return this.currentversion
     */
    public DrugPairVersionVa getCurrentVersion() {
        return this.getExportManager().getCurrentVersion();
    }

    /**
     * Set CurrentVersion.
     * 
     * @param v
     *            - DrugPairVersionVa
     */
    public void setCurrentVersion(DrugPairVersionVa v) {
        this.getExportManager().setCurrentVersion(v);
    }

    /**
     * Get a version that can be modified. Once a version is persisted, It can no longer be modified (for audit purposes). If
     * the current version of this monograph is persistent, this method will create a copy of it, make it the current version
     * and return it.
     * 
     * @return the current non-persistent version
     */
    @Override
    protected DrugPairVersionVa getModifiableCurrentVersion() {
        if (!getCurrentVersion().isModifiable()) {
            DrugPairVersionVa v = newVersion();
            v.copy(getCurrentVersion());
            setCurrentVersion(v);
            setRolledBack(false);
        }
        return getCurrentVersion();
    }

    /**
     * New Version.
     * 
     * @return v
     */
    private DrugPairVersionVa newVersion() {
        DrugPairVersionVa v = new DrugPairVersionVa();
        v.setVersionedObject(this);
        versionHistory.add(v);
        return v;
    }

    /**
     * Roll Back.
     * 
     */
    @Override
    public void rollBack() {
        if (!getCurrentVersion().isModifiable()) {
            DrugPairVersionVa v = newVersion();
            setCurrentVersion(v);
        }
        getCurrentVersion().copy(getExportableVersion());
        setRolledBack(true);
    }

    /**
     * Get Version History.
     * 
     * @return this.versionHistory
     */
    @Override
    public List<Version<DrugPair>> getVersionHistory() {
        return this.versionHistory;
    }

    /**
     * SetVersionHistory.
     * 
     * @param pecsMonographVersionVas
     *            the new version history
     */
    public void setVersionHistory(List<Version<DrugPair>> pecsMonographVersionVas) {
        this.versionHistory = pecsMonographVersionVas;
    }

    /**
     * Creates a new DrugPairXref object and returns it.
     * 
     * @return ConceptXref
     */
    @Override
    protected ConceptXref createConceptXref() {
        return new DrugPairXref();
    }

    /**
     * Gets the reverse FDB id.
     * 
     * @return the reverseFdbId
     */
    public Long getReverseFdbId() {
        return reverseFdbId;
    }

    /**
     * Sets the reverse FDB id.
     * 
     * @param reverseFdbId
     *            the reverseFdbId to set
     */
    public void setReverseFdbId(Long reverseFdbId) {
        this.reverseFdbId = reverseFdbId;
    }

    /**
     * Gets the exportManager.
     * 
     * @return the exportManager
     */
    public DrugPairExportManager getExportManager() {
        return exportManager;
    }

    /**
     * Sets the exportManager.
     * 
     * @param exportManager
     *            the exportManager to set
     */
    public void setExportManager(DrugPairExportManager exportManager) {
        this.exportManager = exportManager;
    }

}
