package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.DuplicateTherapy;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DuplicateAllowance;
import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * The Class DuplicateTherapyVA.
 * 
 * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy
 * 
 */
public final class DuplicateTherapyVa extends AbstractCustomizable<DuplicateTherapy> implements Customizable<DuplicateTherapy>,
        DuplicateTherapy, Serializable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1683588327337199249L;

    /** The Constant CATEGORY. */
    private static final String CATEGORY = "VA";

    private DuplicateTherapyExportManager exportManager;

    /** The version history. */
    private List<Version<DuplicateTherapy>> versionHistory = new ArrayList<Version<DuplicateTherapy>>(0);

    /** The id. */
    private Long id;

    /** The requestor. */
    private String requestor;

    /** The dtcid. */
    private Long dtcid;

    /** The category. */
    private String category;

    /**
     * Constructor method for DuplicateTherapyVa.
     * 
     * @param initialHistory
     *            - boolean - if true creates initial history
     */
    public DuplicateTherapyVa(boolean initialHistory) {
        super();
        if (initialHistory) {
            this.exportManager = new DuplicateTherapyExportManager();
            this.exportManager.setDuplicateTherapyVa(this);
            setCurrentVersion(new DuplicateTherapyVersionVa());
            getCurrentVersion().setDate(new Date());
            getCurrentVersion().setVersionedObject(this);
            versionHistory.add(getCurrentVersion());
        }
    }

    /**
     * Constructor Method for DuplicateTherapyVA which accepts no parameters.
     */
    public DuplicateTherapyVa() {
    }

    /**
     * Retrieves the Id.
     * 
     * @return Long - the Id
     */
    public Long getId() {
        return id;
    }

    /**
     * Sets the Id.
     * 
     * @param id
     *            - Type Long
     */
    @SuppressWarnings("unused")
    private void setId(Long id) {
        this.id = id;
    }

    /**
     * Retrieves the Duplicate Allowance off the current version.
     * 
     * @return Dup Allowance off Current Version.
     */
    @Override
    public DuplicateAllowance getCustomDupAllowance() {
        return getCurrentVersion().getCustomDupAllowance();
    }

    /**
     * Sets the Duplicate Allowance on Current Version.
     * 
     * @param allowance
     *            -Duplicate Allowance
     */
    @Override
    public void setCustomDupAllowance(DuplicateAllowance allowance) {
        getModifiableCurrentVersion().setCustomDupAllowance(allowance);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#getCustomDupAllowanceInd ()
     */
    @Override
    public Long getCustomDupAllowanceInd() {
        return getCurrentVersion().getCustomDupAllowanceInd();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#setCustomDupAllowanceInd (java.lang.Long)
     */
    @Override
    public void setCustomDupAllowanceInd(Long indicator) {
        getModifiableCurrentVersion().setCustomDupAllowanceInd(indicator);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#getCustomString()
     */
    @Override
    public String getCustomString() {
        return getCurrentVersion().getCustomString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#setCustomString(java .lang.String)
     */
    @Override
    public void setCustomString(String customString) {
        getModifiableCurrentVersion().setCustomString(customString);
    }

    /**
     * Retrieves the DTC Id.
     * 
     * First DataBank duplicate therapy class identifier
     * 
     * @return Long - the dtc id
     */
    public Long getDtcid() {
        return dtcid;
    }

    /**
     * Sets the DTC Id.
     * 
     * First DataBank duplicate therapy class identifier
     * 
     * @param dtcid
     *            - Type Long
     */
    public void setDtcid(Long dtcid) {
        this.dtcid = dtcid;
    }

    /**
     * Retrieves the category.
     * 
     * @return String - the category
     */
    @Override
    public String getCategory() {
        if (category == null) {
            category = CATEGORY;
        }
        return category;
    }

    /**
     * Sets the category.
     * 
     * @param category
     *            - a String
     */
    @Override
    public void setCategory(String category) {
        this.category = category;
    }

    /**
     * Retrieves the Reference Text. <BR>
     * <BR>
     * Medical references justifying the customization request.
     * 
     * @return the text
     */
    public String getReferenceText() {
        return getCurrentVersion().getReferenceText();
    }

    /**
     * Sets the reference text.
     * 
     * @param referenceText
     *            the text
     */
    public void setReferenceText(String referenceText) {
        getModifiableCurrentVersion().setReferenceText(referenceText);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractCustomizable#getRequestor()
     */
    @Override
    public String getRequestor() {
        return requestor;
    }

    /**
     * Sets the requester.
     * 
     * @param requestor
     *            of type String
     * 
     */
    @Override
    public void setRequestor(String requestor) {
        this.requestor = requestor;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getVersionHistory()
     */
    @Override
    public List<Version<DuplicateTherapy>> getVersionHistory() {
        return this.versionHistory;
    }

    /**
     * Sets a list of version history.
     * 
     * @param pecsDuplicateTherapyVersions
     *            - List of {@link Version}(s) of type {@link DuplicateTherapy}
     */
    public void setVersionHistory(List<Version<DuplicateTherapy>> pecsDuplicateTherapyVersions) {
        this.versionHistory = pecsDuplicateTherapyVersions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getExportableVersion()
     */
    @Override
    public DuplicateTherapyVersionVa getExportableVersion() {
        return this.getExportManager().getExportableVersion();
    }

    /**
     * Sets the category.
     * 
     * @param version
     *            - {@link Version} of type {@link DuplicateTherapy}
     */
    @Override
    public void setExportableVersion(Version<DuplicateTherapy> version) {
        this.getExportManager().setExportableVersion((DuplicateTherapyVersionVa) version);
    }

    /**
     * Retrieves the exported version.
     * 
     * @return DuplicateTherapyVersionVa - {@link DuplicateTherapyVersionVa}
     */
    public DuplicateTherapyVersionVa getExportedVersion() {
        return this.getExportManager().getExportedVersion();
    }

    /**
     * Sets the exported version.
     * 
     * @param version
     *            - {@link DuplicateTherapyVersionVa}
     */
    public void setExportedVersion(DuplicateTherapyVersionVa version) {
        this.getExportManager().setExportedVersion(version);
    }

    /**
     * Sets the exported Version.
     * 
     * @param version
     *            -
     */
    @Override
    public void setExportedVersion(Version<DuplicateTherapy> version) {
        this.getExportManager().setExportedVersion((DuplicateTherapyVersionVa) version);
    }

    /**
     * Retrieves the current version.
     * 
     * @return DuplicateTherapyVersionVa - {@link DuplicateTherapyVersionVa}
     */
    @Override
    public DuplicateTherapyVersionVa getCurrentVersion() {
        return this.getExportManager().getCurrentVersion();
    }

    /**
     * This method sets the current version.
     * 
     * @param version
     *            - {@link DuplicateTherapyVersionVa}
     */
    public void setCurrentVersion(DuplicateTherapyVersionVa version) {
        this.getExportManager().setCurrentVersion(version);
    }

    /**
     * Get a version that can be modified. Once a version is persisted, It can no longer be modified (for audit purposes). If
     * the current version of this monograph is persistent, this method will create a copy of it, make it the current version
     * and return it.
     * 
     * @return the current non-persistent version - {@link DuplicateTherapyVersionVa}
     */
    @Override
    protected DuplicateTherapyVersionVa getModifiableCurrentVersion() {
        if (!getCurrentVersion().isModifiable()) {
            DuplicateTherapyVersionVa version = newVersion();
            version.copy(getCurrentVersion());
            setCurrentVersion(version);
            setRolledBack(false);
        }
        return getCurrentVersion();
    }

    /**
     * This method creates a new version of the {@link DuplicateTherapyVersionVa} and adds it to version history.
     * 
     * @return {@link DuplicateTherapyVersionVa}
     */
    private DuplicateTherapyVersionVa newVersion() {
        DuplicateTherapyVersionVa version = new DuplicateTherapyVersionVa();
        versionHistory.add(version);
        return version;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#rollBack()
     */
    @Override
    public void rollBack() {
        if (!getCurrentVersion().isModifiable()) {
            DuplicateTherapyVersionVa version = newVersion();
            setCurrentVersion(version);
        }
        getCurrentVersion().copy(getExportableVersion());
        setExportableVersion();
        setRolledBack(true);
    }

    /**
     * This method creates a new Duplicate Therapy Va passing a true value to the class constructor.
     * 
     * @return DuplicateTherapyVa
     * @see {@link DuplicateTherapyVa}
     */
    public static DuplicateTherapyVa createDuplicateTherapy() {
        return new DuplicateTherapyVa(true);
    }

    /**
     * This method sets takes in a DuplicateTherapyDTO object and transfers multiple property values to this object.
     * 
     * @param fdb
     *            of type {@link DuplicateTherapyDTO}
     */
    public void setUpDtCat(DuplicateTherapyDTO fdb) {

        this.setCategory(fdb.getCategory());
        this.setCustomString(fdb.getCustomstring());
        // Method is no longer called possible removeable?? Need Review for clarifcation.
        // this.setCustomDupAllowance(fdb.getCustomdupallowance());
        this.setCustomDupAllowanceInd(fdb.getCustomdupallowind());

        this.setDtcid(fdb.getDtcid());
        this.setReferenceText(fdb.getReferenceText());
        this.setModifiedDate(new Date());
        this.setAssignedUser(fdb.getAssignedTo());
    }

    /**
     * Returns a textual representation of the class.
     * 
     * @return String
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "DuplicateTherapyVa [getId()=" + getId() + ", getCustomDupAllowance()=" + getCustomDupAllowance()
                + ", getCustomDupAllowanceInd()=" + getCustomDupAllowanceInd() + ", getCustomString()=" + getCustomString()
                + ", getDtcid()=" + getDtcid() + ", getCategory()=" + getCategory() + ", getReferenceText()="
                + getReferenceText() + ", getRequestor()=" + getRequestor() + ", getVersionHistory()=" + getVersionHistory()
                + ", getExportableVersion()=" + getExportableVersion() + ", getExportedVersion()=" + getExportedVersion()
                + ", getCurrentVersion()=" + getCurrentVersion() + ", getModifiableCurrentVersion()="
                + getModifiableCurrentVersion() + ", toString()=" + super.toString() + "]";
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractCustomizable#createConceptXref()
     */
    @Override
    protected ConceptXref createConceptXref() {

        return new DuplicateTherapyXref();
    }

    /**
     * Gets the exportManager.
     * 
     * @return the exportManager
     */
    public DuplicateTherapyExportManager getExportManager() {
        return exportManager;
    }

    /**
     * Sets the exportManager.
     * 
     * @param exportManager
     *            the exportManager to set
     */
    public void setExportManager(DuplicateTherapyExportManager exportManager) {
        this.exportManager = exportManager;
    }

}
