package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.DuplicateTherapy;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DuplicateAllowance;

import java.io.Serializable;
import java.util.Set;

/**
 * The Class Duplicate Therapy Version Va.
 * 
 * @see AbstractVersion
 * 
 */
public final class DuplicateTherapyVersionVa extends AbstractVersion<DuplicateTherapy> implements DuplicateTherapy,
        Version<DuplicateTherapy>, Serializable, Cloneable {

    private static final long serialVersionUID = 5372839643960083662L;
    private DuplicateAllowance customDupAllowance;
    private Long customDupAllowanceInd;
    private String customString;
    private String referenceText;
    private DuplicateTherapyVa duplicateTherapy;

    /**
     * Retrieves the Custom Duplicate Allowance.
     * 
     * @return DuplicateAllowance - custom duplicate allowance
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#getCustomDupAllowance()
     */
    @Override
    public DuplicateAllowance getCustomDupAllowance() {
        return customDupAllowance;
    }

    /**
     * Sets the Custom Duplicate Allowance.
     * 
     * @param allowance
     *            of type DuplicateAllowance
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#setCustomDupAllowance (java.lang.Long)
     */
    @Override
    public void setCustomDupAllowance(DuplicateAllowance allowance) {
        customDupAllowance = allowance;
    }

    /**
     * Retrieves the Custom Duplicate Allowance Indicator.
     * 
     * @return Long - custom duplicate allowance indicator
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#getCustomDupAllowanceInd ()
     */
    @Override
    public Long getCustomDupAllowanceInd() {
        return customDupAllowanceInd;
    }

    /**
     * Sets the Custom Duplicate Allowance Indicator.
     * 
     * @param indicator
     *            of type Long
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#setCustomDupAllowanceInd (java.lang.Long)
     */
    @Override
    public void setCustomDupAllowanceInd(Long indicator) {
        customDupAllowanceInd = indicator;
    }

    /**
     * Retrieves the Custom String.
     * 
     * @return String - custom string
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#getCustomString()
     */
    @Override
    public String getCustomString() {
        return customString;
    }

    /**
     * Sets the Custom String.
     * 
     * @param customString
     *            of type String
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#setCustomString(java .lang.String)
     */
    @Override
    public void setCustomString(String customString) {
        this.customString = customString;
    }

    /**
     * Retrieves the Versioned Object.
     * 
     * @return DuplicateTherapyVa
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractVersion#getVersionedObject ()
     */
    @Override
    public DuplicateTherapyVa getVersionedObject() {
        return duplicateTherapy;
    }

    /**
     * Sets the Versioned Object.
     * 
     * @param duplicateTherapyVa
     *            of type DuplicateTherapyVa
     */
    public void setVersionedObject(DuplicateTherapyVa duplicateTherapyVa) {
        this.duplicateTherapy = duplicateTherapyVa;
    }

    /**
     * Retrieves the Reference Text.
     * 
     * @return String - reference text
     */
    public String getReferenceText() {
        return referenceText;
    }

    /**
     * Sets the Reference Text.
     * 
     * @param referenceText
     *            of type String
     */
    public void setReferenceText(String referenceText) {
        this.referenceText = referenceText;
    }

    /*
     * BACKUP code public void copy(DuplicateTherapyVersionVa other) { super.copy(other); duplicateTherapy =
     * other.getVersionedObject(); dupAllowance = other.getCustomDupAllowance(); dupAllowanceInd =
     * other.getCustomDupAllowanceInd(); customString = other.getCustomString(); referenceText = other.getReferenceText(); }
     */
    /**
     * Copies the Version of type Duplicate Therapy.
     * 
     * @param version
     *            of type Version - typed as DuplicateTherapy
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractVersion#copy(gov.va.med .pharmacy.pecs.domain.Version)
     */
    @Override
    public void copy(Version<DuplicateTherapy> version) {

        if (version == null) {
            return;
        }
        // super.copy(version);
        setDate(version.getDate());
        setAssignedUser(version.getAssignedUser());
        setActionBy(version.getActionBy());
        setUser(version.getUser());
        setStatus(version.getStatus());

        DuplicateTherapyVersionVa duplicateTherayVersionVa = (DuplicateTherapyVersionVa) version;

        setDuplicateTherapy(duplicateTherayVersionVa.getVersionedObject());
        setCustomDupAllowance(duplicateTherayVersionVa.getCustomDupAllowance());
        setCustomDupAllowanceInd(duplicateTherayVersionVa.getCustomDupAllowanceInd());
        setCustomString(duplicateTherayVersionVa.getCustomString());
        setReferenceText(duplicateTherayVersionVa.getReferenceText());
    }

    /**
     * Clones the current object.
     * 
     * @return DuplicateTherapyVersionVa
     * @throws CloneNotSupportedException
     *             -
     * @see java.lang.Object#clone()
     */
    public DuplicateTherapyVersionVa clone() throws CloneNotSupportedException {
        DuplicateTherapyVersionVa clone = new DuplicateTherapyVersionVa();
        super.clone();
        clone.setAssignedUser(getAssignedUser());
        clone.setCustomDupAllowance(getCustomDupAllowance());
        clone.setCustomDupAllowanceInd(getCustomDupAllowanceInd());
        clone.setUser(getUser());
        return clone;
    }

    /**
     * Retrieves the Category.
     * 
     * @return String the Category
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#getCategory()
     */
    @Override
    public String getCategory() {
        return duplicateTherapy.getCategory();
    }

    // should we have a setter?
    /**
     * Sets the Category.
     * 
     * @param category
     *            of type String
     * @see gov.va.med.pharmacy.pecs.domain.DuplicateTherapy#setCategory(java.lang .String)
     */
    @Override
    public void setCategory(String category) {
        duplicateTherapy.setCategory(category);
    }

    /**
     * Retrieves the Duplicate Therapy VA.
     * 
     * @return DuplicateTherapyVa
     */
    public DuplicateTherapyVa getDuplicateTherapy() {
        return duplicateTherapy;
    }

    /**
     * Sets the Duplication therapy.
     * 
     * @param duplicateTherapy
     *            of type DuplicateTherapyVa
     */
    public void setDuplicateTherapy(DuplicateTherapyVa duplicateTherapy) {
        this.duplicateTherapy = duplicateTherapy;
    }

    /**
     * Do not set Sets the User Comment History on the version.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#setUserCommentHistory(java.util.Set)
     * @param userCommentHistory
     *            of type Set<IUserComment>
     */
    @Override
    public void setUserCommentHistory(Set<UserComment> userCommentHistory) {
        // Do not set the user Comment history on the version

    }

    /**
     * Can Not Retrieves the User Comment History - always null.
     * 
     * @see gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction#getUserCommentHistory()
     * @return Set<IUserComment> the set of user comments in history
     */
    @Override
    public Set<UserComment> getUserCommentHistory() {
        // Do not retrieve the User Comment Histroy through the version.
        return null;
    }

    /**
     * Returns a textual representation of the class.
     * 
     * @return String
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "DuplicateTherapyVersionVa [getCustomDupAllowance()=" + getCustomDupAllowance()
                + ", getCustomDupAllowanceInd()=" + getCustomDupAllowanceInd() + ", getCustomString()=" + getCustomString()
                + ", getVersionedObject()=" + getVersionedObject() + ", getReferenceText()=" + getReferenceText()
                + ", getCategory()=" + getCategory() + ", getDuplicateTherapy()=" + getDuplicateTherapy() + ", toString()="
                + super.toString() + "]";
    }

}
