package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.ComparisonReportConcept;

import java.sql.Blob;
import java.sql.Date;

/**
 * A domain object that holds an FDB Update Report.
 * 
 * @author vhaisaduffg
 * 
 */
public class FdbUpdateReport implements Comparable<FdbUpdateReport> {

    private Long id;
    private Date fdbUpdateDate;
    private ComparisonReportConcept concept;
    private Blob content;
    private String filename;

    /**
     * Returns the identifier for the report.
     * 
     * @return the id
     */
    public final Long getId() {
        return id;
    }

    /**
     * Sets the identifier for the report. This method is marked as private
     * since it should only be used by Hibernate.
     * 
     * @param id
     *            the id to set
     */
    @SuppressWarnings("unused")
    private final void setId(Long id) {
        this.id = id;
    }

    /**
     * Returns the update date for the FDB Incremental Update.
     * 
     * @return the fdbUpdateDate
     */
    public final Date getFdbUpdateDate() {
        return fdbUpdateDate;
    }

    /**
     * Sets the update date of the FDB Incremental Update.
     * 
     * @param fdbUpdateDate
     *            the fdbUpdateDate to set
     */
    public final void setFdbUpdateDate(Date fdbUpdateDate) {
        this.fdbUpdateDate = fdbUpdateDate;
    }

    /**
     * Returns the ComparisonReportConcept this object is associated with.
     * 
     * @return the concept
     */
    public ComparisonReportConcept getConcept() {
        return concept;
    }

    /**
     * Sets the ComparisonReportConcept this report is associated with.
     * 
     * @param concept
     *            the concept to set
     */
    public void setConcept(ComparisonReportConcept concept) {
        this.concept = concept;
    }

    /**
     * Returns the content of the report.
     * 
     * @return the content
     */
    public final Blob getContent() {
        return content;
    }

    /**
     * Sets the content of the report.
     * 
     * @param content
     *            the content to set
     */
    public final void setContent(Blob content) {
        this.content = content;
    }

    /**
     * Returns the file name that the report file should be saved as.
     * 
     * @return the filename
     */
    public String getFilename() {
        return filename;
    }

    /**
     * Sets the file name that the report file should be saved as.
     * 
     * @param filename
     *            the filename to set
     */
    public void setFilename(String filename) {
        this.filename = filename;
    }

    /**
     * Compares two FdbUpdateReport objects using the FdbUpdateDate as the
     * comparable value. This method returns the objects in descending order.
     * 
     * @param arg0
     *            FdbUpdateReport
     * @return int
     */
    @Override
    public int compareTo(FdbUpdateReport arg0) {
        if (getFdbUpdateDate().before(arg0.getFdbUpdateDate())) {
            return 1;
        }
        if (getFdbUpdateDate().after(arg0.getFdbUpdateDate())) {
            return -1;
        }
        return 0;
    }
}
