package gov.va.med.pharmacy.pecs.domain.impl.va;

/**
 * An interface to tell if a domain object has the PECS modifiable pattern implemented. When creating an object that implements
 * this interface, the modifiable attribute should be defaulted to true. Hibernate Event listeners in the
 * ModifiableEntityEventListener class will change the modifiable flag to false after a Hibernate Insert, Load or Update event.
 * 
 * @author vhaisaduffg
 * 
 */
public interface ModifiableEntity {

    /**
     * Checks if is modifiable.
     * 
     * @return true, if is modifiable
     */
    boolean isModifiable();

    /**
     * Sets the modifiable.
     * 
     * @param modifiable
     *            the new modifiable
     */
    void setModifiable(boolean modifiable);
}
