package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.MonographSection;
import gov.va.med.pharmacy.pecs.domain.MonographSection.SectionCode;

import java.util.HashMap;
import java.util.Map;

/**
 * ProfessionalDrugDrugMonographVa entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class ProfessionalDrugDrugMonographVa extends ConsumerDrugDrugMonographVa {
    private static final long serialVersionUID = -2490771573597811161L;

    private static HashMap<SectionCode, String> supportedSections = new HashMap<SectionCode, String>();
    static {
        supportedSections.put(SectionCode.TITLE, "MONOGRAPH TITLE:  ");
        supportedSections.put(SectionCode.MECHANISM_OF_ACTION, "MECHANISM OF ACTION:  ");
        supportedSections.put(SectionCode.DISCUSSION, "DISCUSSION:  ");
        supportedSections.put(SectionCode.CLINICAL_EFFECTS, "CLINICAL EFFECTS:  ");
        supportedSections.put(SectionCode.SEVERITY_LEVEL, "SEVERITY LEVEL:  ");
        supportedSections.put(SectionCode.PATIENT_MANAGEMENT, "PATIENT MANAGEMENT:  ");
        supportedSections.put(SectionCode.PREDISPOSING_FACTORS, "PREDISPOSING FACTORS:  ");
        supportedSections.put(SectionCode.REFERENCE, "REFERENCES:\n");
        supportedSections.put(SectionCode.DISCLAIMER, "DISCLAIMER:  ");
    }

    /**
     * Creates and returns a ProfessionalDrugDrugMonographVa object.
     * 
     * @return ProfessionalDrugDrugMonographVa
     */
    public static ProfessionalDrugDrugMonographVa createMonograph() {
        return new ProfessionalDrugDrugMonographVa(true);
    }

    /**
     * Basic Constructor.
     * 
     */
    public ProfessionalDrugDrugMonographVa() {
    }

    /**
     * Creates a ProfessionalDrugDrugMonograph.
     * 
     * @param initHistory
     *            - See initHistory
     */
    protected ProfessionalDrugDrugMonographVa(boolean initHistory) {
        super(initHistory);
    }

    /**
     * Get the mechanism of action for the corresponding interaction.
     * 
     * @return the discussion text
     */
    public String getMechanismOfAction() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.MECHANISM_OF_ACTION).getText();
    }

    /**
     * Set the mechanism of action text.
     * 
     * @param mechanism
     *            the discussion text
     */
    public void setMechanismOfAction(String mechanism) {
        setSectionText(MonographSection.SectionCode.MECHANISM_OF_ACTION, mechanism);
    }

    /**
     * Get the discussion.
     * 
     * @return the discussion text
     */
    public String getDiscussion() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.DISCUSSION).getText();
    }

    /**
     * Set the discussion.
     * 
     * @param discussion
     *            the discussion text
     */
    public void setDiscussion(String discussion) {
        setSectionText(MonographSection.SectionCode.DISCUSSION, discussion);
    }

    /**
     * Get the description of clinical effects.
     * 
     * @return the discussion text
     */
    public String getClinicalEffects() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.CLINICAL_EFFECTS).getText();
    }

    /**
     * Set the discussion.
     * 
     * @param effects
     *            the description of clinical effects
     */
    public void setClinicalEffects(String effects) {
        setSectionText(MonographSection.SectionCode.CLINICAL_EFFECTS, effects);
    }

    /**
     * Get the description of the severity level.
     * 
     * @return the severity level
     */
    public String getSeverityLevel() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.SEVERITY_LEVEL).getText();
    }

    /**
     * Set the severity level description.
     * 
     * @param severity
     *            the severity level description
     */
    public void setSeverityLevel(String severity) {
        setSectionText(MonographSection.SectionCode.SEVERITY_LEVEL, severity);
    }

    /**
     * Get the patient management guidance.
     * 
     * @return the patient management guidance
     */
    public String getPatientManagement() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.SEVERITY_LEVEL).getText();
    }

    /**
     * Set the patient management guidance.
     * 
     * @param management
     *            the patient management guidance
     */
    public void setPatientManagement(String management) {
        setSectionText(MonographSection.SectionCode.PATIENT_MANAGEMENT, management);
    }

    /**
     * Get the predisposing factors.
     * 
     * @return the predisposing factors
     */
    public String getPredisposingFactors() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.PREDISPOSING_FACTORS).getText();
    }

    /**
     * Set the predisposing factors.
     * 
     * @param factors
     *            the predisposing factors
     */
    public void setPredisposingFactors(String factors) {
        setSectionText(MonographSection.SectionCode.PREDISPOSING_FACTORS, factors);
    }

    /**
     * Get the literature references for this monograph.
     * 
     * @return the references
     */
    public String getReferences() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.REFERENCE).getText();
    }

    /**
     * Set the literature references for this monograph.
     * 
     * @param references
     *            the references
     */
    public void setReferences(String references) {
        setSectionText(MonographSection.SectionCode.REFERENCE, references);
    }

    /**
     * Get the disclaimer for this monograph.
     * 
     * @return the disclaimer
     */
    public String getDisclaimer() {
        return getCurrentVersion().getSection(MonographSection.SectionCode.DISCLAIMER).getText();
    }

    /**
     * Set the literature references for this monograph.
     * 
     * @param disclaimer
     *            a list of references (usually delimited by new lines)
     */
    public void setDisclaimer(String disclaimer) {
        setSectionText(MonographSection.SectionCode.DISCLAIMER, disclaimer);
    }

    @Override
    public String getVersionCode() {
        return "FDB-P";
    }

    @Override
    public Map<SectionCode, String> getSupportedSections() {
        return supportedSections;
    }
}
