package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.persistence.HibernateDAOImpl;

import java.util.Iterator;
import java.util.List;

import org.hibernate.FlushMode;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * ProfessionalDrugDrugMonographVa entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
@Repository
public class ProfessionalDrugDrugMonographVaDAO extends HibernateDAOImpl<ProfessionalDrugDrugMonographVa> {

    /**
     * Final String APPROVED_MONOGRAPHS_QUERY = "approvedMonographsQuery".
     */
    private final String APPROVED_MONOGRAPHS_QUERY = "approvedMonographsQuery";

    /** The APPROVE d_ monograph s_ quer y2. */
    private final String APPROVED_MONOGRAPHS_QUERY2 = "approvedMonographsQuery2";
    /**
     * Final String UNIQUE_MONOGRAPH_TITLE_QUERY = "uniqueMonographTitleQuery".
     */
    private final String UNIQUE_MONOGRAPH_TITLE_QUERY = "uniqueMonographTitleQuery";
    /**
     * Final String FIND_MONOGRAPH_BY_FDB_ID_QUERY = "findMonographByFDBIdQuery".
     */
    private final String FIND_MONOGRAPH_BY_FDB_ID_QUERY = "findMonographByFDBIdQuery";

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAOImpl#getPersistentClass()
     */
    @Override
    protected Class<ProfessionalDrugDrugMonographVa> getPersistentClass() {
        return ProfessionalDrugDrugMonographVa.class;
    }

    /**
     * Returns a list of containing all exportable approved monograph versions (only monograph id and title).
     * 
     * @return a list of monograph versions
     */
    @SuppressWarnings("unchecked")
    public List<Object[]/* MonographVersion */> getApprovedMonographs() {
        Query query = getSession().getNamedQuery(APPROVED_MONOGRAPHS_QUERY);
        return query.list();
    }

    /**
     * Monograph With Title.
     * 
     * @param title
     *            the title
     * @return exists/false
     */
    public boolean existsMonographWithTitle(String title) {
        if (title == null) {
            return false;
        }
        Query query = getSession().getNamedQuery(UNIQUE_MONOGRAPH_TITLE_QUERY);
        FlushMode originalFlushMode = getSession().getFlushMode();
        getSession().setFlushMode(FlushMode.COMMIT);
        query.setString("theTitle", title.toLowerCase());
        boolean exists = ((Long) query.iterate().next()) > 0;
        getSession().setFlushMode(originalFlushMode);
        return exists;
    }

    /**
     * find Monograph by FDB ID.
     * 
     * @param id
     *            long
     * @return Monograph or null
     */
    @SuppressWarnings("unchecked")
    public MonographVersion findMonographByFDBId(long id) {
        Query query = getSession().getNamedQuery(FIND_MONOGRAPH_BY_FDB_ID_QUERY);
        query.setLong("fdbId", id);
        Iterator<MonographVersion> iter = query.iterate();
        return iter.hasNext() ? iter.next() : null;
    }

}
