package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.authentication.kernel.VistaDivisionVO;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * The Class UserProfile.
 */
public final class UserProfile implements Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = -487987700291382832L;

    /** The role requestor. */
    public static final String ROLE_REQUESTOR = "PSS_CUSTOM_TABLES_REQUESTOR";

    /** The role approver. */
    public static final String ROLE_APPROVER = "PSS_CUSTOM_TABLES_APPROVER";

    /** The role admin. */
    public static final String ROLE_ADMIN = "PSS_CUSTOM_TABLES_ADMIN";

    /** The role rel manager. */
    public static final String ROLE_REL_MANAGER = "PSS_CUSTOM_TABLES_REL_MAN";

    /** The role sample role. */
    public static final String ROLE_SAMPLE_ROLE = "XUKAAJEE_SAMPLE_ROLE";

    /** The role auth kaajee user. */
    public static final String ROLE_AUTH_KAAJEE_USER = "AUTHENTICATED_KAAJEE_USER";

    /** The username. */
    private String username = null;

    /** The principal name. */
    private String principalName = null;

    /** The user vpid. */
    private String userVpid = null;

    /** The user duz. */
    private String userDuz = null;

    /** The user name01. */
    private String userName01 = null;

    /** The user name display. */
    private String userNameDisplay = null;

    /** The user last name. */
    private String userLastName = null;

    /** The user first name. */
    private String userFirstName = null;

    /** The user middle name. */
    private String userMiddleName = null;

    /** The user prefix. */
    private String userPrefix = null;

    /** The user suffix. */
    private String userSuffix = null;

    /** The user degree. */
    private String userDegree = null;

    /** The login station number. */
    private String loginStationNumber = null;

    /** The user parent administrative facility station number. */
    private String userParentAdministrativeFacilityStationNumber = null;

    /** The user parent computer system station number. */
    private String userParentComputerSystemStationNumber = null;

    /** The permitted divisions. */
    private Map<String, VistaDivisionVO> permittedDivisions = null;

    /** The cf divisions. */
    private Map<String, VistaDivisionVO> cfDivisions = null;

    /** The roles. */
    private Map<String, String> roles = null;

    /** The all roles. */
    private Map<String, String> allRoles = null;

    /**
     * Instantiates a new user profile.
     */
    public UserProfile() {
        roles = new HashMap<String, String>();
        allRoles = new HashMap<String, String>();

        allRoles.put(ROLE_ADMIN, ROLE_ADMIN);
        allRoles.put(ROLE_APPROVER, ROLE_APPROVER);
        allRoles.put(ROLE_REL_MANAGER, ROLE_REL_MANAGER);
        allRoles.put(ROLE_REQUESTOR, ROLE_REQUESTOR);
        allRoles.put(ROLE_AUTH_KAAJEE_USER, ROLE_AUTH_KAAJEE_USER);
        allRoles.put(ROLE_SAMPLE_ROLE, ROLE_SAMPLE_ROLE);
    }

    /**
     * Sets the role.
     * 
     * @param roleName
     *            the new role
     */
    public void setRole(String roleName) {
        if (roleName == null || roleName.equals(""))
            throw new IllegalArgumentException("Role name is null or cannot be determined");

        if (allRoles.containsKey(roleName))
            roles.put(roleName, roleName);
        else
            throw new IllegalArgumentException("Role name '" + roleName + "' does not match any known VistA security key");
    }

    /**
     * Gets the roles.
     * 
     * @return the roles
     */
    public Map<String, String> getRoles() {
        return roles;
    }

    /**
     * Checks if is in role.
     * 
     * @param roleName
     *            the role name
     * @return true, if is in role
     */
    public boolean isInRole(String roleName) {
        if (roleName != null && !roleName.equals("")) {
            if (roles.containsKey(roleName))
                return true;
        }

        return false;
    }

    /**
     * Checks if is admin.
     * 
     * @return true, if is admin
     */
    public boolean isAdmin() {
        if (roles.containsKey(ROLE_ADMIN))
            return true;

        return false;
    }

    /**
     * Checks if is approver.
     * 
     * @return true, if is approver
     */
    public boolean isApprover() {
        if (roles.containsKey(ROLE_APPROVER))
            return true;

        return false;
    }

    /**
     * Checks if is requestor.
     * 
     * @return true, if is requestor
     */
    public boolean isRequestor() {
        if (roles.containsKey(ROLE_REQUESTOR))
            return true;

        return false;
    }

    /**
     * Checks if is release manager.
     * 
     * @return true, if is release manager
     */
    public boolean isReleaseManager() {
        if (roles.containsKey(ROLE_REL_MANAGER))
            return true;

        return false;
    }

    /**
     * Checks if is auth kaajee user.
     * 
     * @return true, if is auth kaajee user
     */
    public boolean isAuthKAAJEEUser() {
        if (roles.containsKey(ROLE_AUTH_KAAJEE_USER))
            return true;

        return false;
    }

    /**
     * Gets the username.
     * 
     * @return the username
     */
    public String getUsername() {
        return username;
    }

    /**
     * Sets the username.
     * 
     * @param username
     *            the new username
     */
    public void setUsername(String username) {
        this.username = username;
    }

    /**
     * Gets the principal name.
     * 
     * @return the principal name
     */
    public String getPrincipalName() {
        return principalName;
    }

    /**
     * Sets the principal name.
     * 
     * @param principalName
     *            the new principal name
     */
    public void setPrincipalName(String principalName) {
        this.principalName = principalName;
    }

    /**
     * Gets the user vpid.
     * 
     * @return the user vpid
     */
    public String getUserVpid() {
        return userVpid;
    }

    /**
     * Sets the user vpid.
     * 
     * @param userVpid
     *            the new user vpid
     */
    public void setUserVpid(String userVpid) {
        this.userVpid = userVpid;
    }

    /**
     * Gets the user duz.
     * 
     * @return the user duz
     */
    public String getUserDuz() {
        return userDuz;
    }

    /**
     * Sets the user duz.
     * 
     * @param userDuz
     *            the new user duz
     */
    public void setUserDuz(String userDuz) {
        this.userDuz = userDuz;
    }

    /**
     * Gets the user name01.
     * 
     * @return the user name01
     */
    public String getUserName01() {
        return userName01;
    }

    /**
     * Sets the user name01.
     * 
     * @param userName01
     *            the new user name01
     */
    public void setUserName01(String userName01) {
        this.userName01 = userName01;
    }

    /**
     * Gets the user name display.
     * 
     * @return the user name display
     */
    public String getUserNameDisplay() {
        return userNameDisplay;
    }

    /**
     * Sets the user name display.
     * 
     * @param userNameDisplay
     *            the new user name display
     */
    public void setUserNameDisplay(String userNameDisplay) {
        this.userNameDisplay = userNameDisplay;
    }

    /**
     * Gets the user last name.
     * 
     * @return the user last name
     */
    public String getUserLastName() {
        return userLastName;
    }

    /**
     * Sets the user last name.
     * 
     * @param userLastName
     *            the new user last name
     */
    public void setUserLastName(String userLastName) {
        this.userLastName = userLastName;
    }

    /**
     * Gets the user first name.
     * 
     * @return the user first name
     */
    public String getUserFirstName() {
        return userFirstName;
    }

    /**
     * Sets the user first name.
     * 
     * @param userFirstName
     *            the new user first name
     */
    public void setUserFirstName(String userFirstName) {
        this.userFirstName = userFirstName;
    }

    /**
     * Gets the user middle name.
     * 
     * @return the user middle name
     */
    public String getUserMiddleName() {
        return userMiddleName;
    }

    /**
     * Sets the user middle name.
     * 
     * @param userMiddleName
     *            the new user middle name
     */
    public void setUserMiddleName(String userMiddleName) {
        this.userMiddleName = userMiddleName;
    }

    /**
     * Gets the user prefix.
     * 
     * @return the user prefix
     */
    public String getUserPrefix() {
        return userPrefix;
    }

    /**
     * Sets the user prefix.
     * 
     * @param userPrefix
     *            the new user prefix
     */
    public void setUserPrefix(String userPrefix) {
        this.userPrefix = userPrefix;
    }

    /**
     * Gets the user suffix.
     * 
     * @return the user suffix
     */
    public String getUserSuffix() {
        return userSuffix;
    }

    /**
     * Sets the user suffix.
     * 
     * @param userSuffix
     *            the new user suffix
     */
    public void setUserSuffix(String userSuffix) {
        this.userSuffix = userSuffix;
    }

    /**
     * Gets the user degree.
     * 
     * @return the user degree
     */
    public String getUserDegree() {
        return userDegree;
    }

    /**
     * Sets the user degree.
     * 
     * @param userDegree
     *            the new user degree
     */
    public void setUserDegree(String userDegree) {
        this.userDegree = userDegree;
    }

    /**
     * Gets the login station number.
     * 
     * @return the login station number
     */
    public String getLoginStationNumber() {
        return loginStationNumber;
    }

    /**
     * Sets the login station number.
     * 
     * @param loginStationNumber
     *            the new login station number
     */
    public void setLoginStationNumber(String loginStationNumber) {
        this.loginStationNumber = loginStationNumber;
    }

    /**
     * Gets the user parent administrative facility station number.
     * 
     * @return the user parent administrative facility station number
     */
    public String getUserParentAdministrativeFacilityStationNumber() {
        return userParentAdministrativeFacilityStationNumber;
    }

    /**
     * Sets the user parent administrative facility station number.
     * 
     * @param userParentAdministrativeFacilityStationNumber
     *            the new user parent administrative facility station number
     */
    public void setUserParentAdministrativeFacilityStationNumber(String userParentAdministrativeFacilityStationNumber) {
        this.userParentAdministrativeFacilityStationNumber = userParentAdministrativeFacilityStationNumber;
    }

    /**
     * Gets the user parent computer system station number.
     * 
     * @return the user parent computer system station number
     */
    public String getUserParentComputerSystemStationNumber() {
        return userParentComputerSystemStationNumber;
    }

    /**
     * Sets the user parent computer system station number.
     * 
     * @param userParentComputerSystemStationNumber
     *            the new user parent computer system station number
     */
    public void setUserParentComputerSystemStationNumber(String userParentComputerSystemStationNumber) {
        this.userParentComputerSystemStationNumber = userParentComputerSystemStationNumber;
    }

    /**
     * Gets the permitted divisions.
     * 
     * @return the permitted divisions
     */
    public Map<String, VistaDivisionVO> getPermittedDivisions() {
        return permittedDivisions;
    }

    /**
     * Sets the permitted divisions.
     * 
     * @param permittedDivisions
     *            the permitted divisions
     */
    public void setPermittedDivisions(Map<String, VistaDivisionVO> permittedDivisions) {
        this.permittedDivisions = permittedDivisions;
    }

    /**
     * Gets the cf divisions.
     * 
     * @return the cf divisions
     */
    public Map<String, VistaDivisionVO> getCfDivisions() {
        return cfDivisions;
    }

    /**
     * Sets the cf divisions.
     * 
     * @param cfDivisions
     *            the cf divisions
     */
    public void setCfDivisions(Map<String, VistaDivisionVO> cfDivisions) {
        this.cfDivisions = cfDivisions;
    }
}
