package gov.va.med.pharmacy.pecs.dto;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosing;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.metadata.ClassInfo;
import gov.va.med.pharmacy.pecs.metadata.FieldInfo;
import gov.va.med.pharmacy.pecs.util.LabelValue;
import gov.va.med.pharmacy.pecs.util.QueryResultLabel;
import gov.va.med.pharmacy.pecs.util.historyofchange.DoseRangeReportOrder;

import javax.validation.constraints.Digits;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * DoseRangeDTO entity.
 */
@ClassInfo(
        name = "Dose Range", description = "Dose Range", customPersistentClass = DoseRangeVersionVa.class,
        fdbPersistentClass = FdbDosing.class)
public final class DoseRangeDTO extends PecsDTO implements java.io.Serializable {
    private static final int DISPLAY_LINES_FIVE = 5;
    private static final long serialVersionUID = 8298980550777050473L;
    private static final Long DEFAULT_CONCEPT_TYPE = 6L;
    private static final String DEFAULT_FDBDX = "999";
    private static final String DEFAULT_DXID = "4892";
    private static final int LENGTH_ZERO = 0;
    private static final int LENGTH_THREE = 3;
    private static final int LENGTH_SIX = 6;
    private static final int LENGTH_TEN = 10;
    /**
     * String Representation of CONCEPTDESCRIPTION.
     */
    public static final String CONCEPTDESCRIPTION = "CONCEPTDESCRIPTION";
    /**
     * String Representation of REFERENCE_TEXT.
     */
    public static final String REFERENCE_TEXT = "REFERENCE_TEXT";
    @Digits(integer = LENGTH_TEN, fraction = LENGTH_ZERO)
    @NotNull()
    @Min(1)
    @FieldInfo(
            name = "AGEHIGHINDAYS", editable = true, customPersistentPath = "ageHighInDays", required = true, internal = false,
            fdbPersistentPath = "id.agehighindays", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.AGE_HIGH_IN_DAYS, reportHistoryCompare = true)
    private Long agehighindays;
    @Digits(integer = LENGTH_TEN, fraction = LENGTH_ZERO)
    @NotNull()    
    @FieldInfo(
            name = "AGELOWINDAYS", editable = true, customPersistentPath = "ageLowInDays", required = true, internal = false,
            fdbPersistentPath = "id.agelowindays", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.AGE_LOW_IN_DAYS, reportHistoryCompare = true)
    private Long agelowindays;
    @Digits(integer = LENGTH_SIX, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "BSAREQUIREDIND", editable = true, customPersistentPath = "bsaRequiredIndicator", required = false,
            internal = false, fdbPersistentPath = "bsarequiredind", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.BSA_REQUIRED_IND, reportHistoryCompare = true)
    private Long bsarequiredind;

    @FieldInfo(name = "CATEGORY", editable = false, customPersistentPath = "category", required = false, internal = false)
    private String category;
    @FieldInfo(
            name = "CONCEPTDESCRIPTION", displayLines = 2, editable = false, customPersistentPath = "conceptDescription",
            required = false, internal = false, fdbPersistentPath = "conceptDescription", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.CONCEPT_DESCRIPTION)
    private String conceptDescription;
    @FieldInfo(
            name = "CONCEPTID", editable = false, customPersistentPath = "conceptid", required = true, internal = false,
            fdbPersistentPath = "id.conceptid", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.CONCEPT_ID)
    private Long conceptid;
    @FieldInfo(
            name = "CONCEPTTYPE", editable = false, customPersistentPath = "conceptType", required = false, internal = false,
            fdbPersistentPath = "id.concepttype", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.CONCEPT_TYPE)
    private long concepttype = DEFAULT_CONCEPT_TYPE;
    @Digits(integer = LENGTH_SIX, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "CRCLTHRESHHOLD", editable = true, customPersistentPath = "crclThreshold", required = false,
            internal = false, fdbPersistentPath = "crclthreshhold", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.CRCL_THRESHHOLD, reportHistoryCompare = true)
    private Long crclthreshhold;
    @FieldInfo(
            name = "CRCLTHRESHHOLDUNITS", editable = true, lookup = true, customPersistentPath = "crclThresholdUnits",
            required = false, internal = false, fdbPersistentPath = "crclthreshholdunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.CRCL_THRESHHOLD_UNITS, reportHistoryCompare = true)
    private String crclthreshholdunits;
    @FieldInfo(
            name = "DOSEFORMLOW", editable = true, customPersistentPath = "doseFormLow", required = false, internal = false,
            fdbPersistentPath = "doseformlow", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_LOW,
            reportHistoryCompare = true)
    private Double doseformlow;
    @FieldInfo(
            name = "DOSEFORMLOWUNITS", editable = true, lookup = true, customPersistentPath = "doseFormLowUnits",
            required = false, internal = false, fdbPersistentPath = "doseformlowunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_LOW_UNITS, reportHistoryCompare = true)
    private String doseformlowunits;
    @FieldInfo(
            name = "DOSEFORMHIGH", editable = true, customPersistentPath = "doseFormHigh", required = false, internal = false,
            fdbPersistentPath = "doseformhigh", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_HIGH,
            reportHistoryCompare = true)
    private Double doseformhigh;
    @FieldInfo(
            name = "DOSEFORMHIGHUNITS", editable = true, lookup = true, customPersistentPath = "doseFormHighUnits",
            required = false, internal = false, fdbPersistentPath = "doseformhighunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_HIGH_UNITS, reportHistoryCompare = true)
    private String doseformhighunits;
    @FieldInfo(
            name = "DOSEFORMRATELOW", editable = true, customPersistentPath = "doseFormRateLow", required = false,
            internal = false, fdbPersistentPath = "doseformratelow", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_RATE_LOW, reportHistoryCompare = true)
    private Double doseformratelow;
    @FieldInfo(
            name = "DOSEFORMRATELOWUNITS", editable = true, lookup = true, customPersistentPath = "doseFormRateLowUnits",
            required = false, internal = false, fdbPersistentPath = "doseformratelowunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_RATE_LOW_UNITS, reportHistoryCompare = true)
    private String doseformratelowunits;
    @FieldInfo(
            name = "DOSEFORMRATEHIGH", editable = true, customPersistentPath = "doseFormRateHigh", required = false,
            internal = false, fdbPersistentPath = "doseformratehigh", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_RATE_HIGH, reportHistoryCompare = true)
    private Double doseformratehigh;
    @FieldInfo(
            name = "DOSEFORMRATEHIGHUNITS", editable = true, lookup = true, customPersistentPath = "doseFormRateHighUnits",
            required = false, internal = false, fdbPersistentPath = "doseformratehighunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_FORM_RATE_HIGH_UNITS, reportHistoryCompare = true)
    private String doseformratehighunits;
    @FieldInfo(
            name = "DOSELOW", editable = true, customPersistentPath = "doseLow", required = false, internal = false,
            fdbPersistentPath = "doselow", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DOSE_LOW,
            reportHistoryCompare = true)
    private Double doselow;
    @FieldInfo(
            name = "DOSELOWUNITS", editable = true, lookup = true, customPersistentPath = "doseLowUnits", required = false,
            internal = false, fdbPersistentPath = "doselowunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_LOW_UNITS, reportHistoryCompare = true)
    private String doselowunits;
    @FieldInfo(
            name = "DOSEHIGH", editable = true, customPersistentPath = "doseHigh", required = false, internal = false,
            fdbPersistentPath = "dosehigh", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DOSE_HIGH,
            reportHistoryCompare = true)
    private Double dosehigh;
    @FieldInfo(
            name = "DOSEHIGHUNITS", editable = true, lookup = true, customPersistentPath = "doseHighUnits", required = false,
            internal = false, fdbPersistentPath = "dosehighunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_HIGH_UNITS, reportHistoryCompare = true)
    private String dosehighunits;
    private long doseRangeId;
    @FieldInfo(
            name = "DOSERATELOWUNITS", editable = true, lookup = true, customPersistentPath = "doseRateLowUnits",
            required = false, internal = false, fdbPersistentPath = "doseratelowunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_RATE_LOW_UNITS, reportHistoryCompare = true)
    private String doseratelowunits;
    @FieldInfo(
            name = "DOSERATEHIGH", editable = true, customPersistentPath = "doseRateHigh", required = false, internal = false,
            fdbPersistentPath = "doseratehigh", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DOSE_RATE_HIGH,
            reportHistoryCompare = true)
    private Double doseratehigh;
    @FieldInfo(
            name = "DOSERATEHIGHUNITS", editable = true, lookup = true, customPersistentPath = "doseRateHighUnits",
            required = false, internal = false, fdbPersistentPath = "doseratehighunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_RATE_HIGH_UNITS, reportHistoryCompare = true)
    private String doseratehighunits;
    @FieldInfo(
            name = "DOSERATELOW", editable = true, customPersistentPath = "doseRateLow", required = false, internal = false,
            fdbPersistentPath = "doseratelow", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DOSE_RATE_LOW,
            reportHistoryCompare = true)
    private Double doseratelow;
    @FieldInfo(
            name = "DOSEROUTEDESC", editable = false, customPersistentPath = "doseRoute.name", required = false,
            internal = true, fdbPersistentPath = "id.doseroute.description1", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_ROUTE_DESC, reportHistoryCompare = true)
    private String doseRouteDesc = null;
    @FieldInfo(
            name = "DOSEROUTEID", editable = true, customPersistentPath = "doseRoute.code", required = true, internal = false,
            fdbPersistentPath = "id.doseroute.doserouteid", lookup = true, lookupType = DoseRoute.class, reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_ROUTE_ID, reportHistoryCompare = true)
    private String doserouteid;
    @FieldInfo(
            name = "DOSETYPEDESC", editable = false, customPersistentPath = "doseType.name", required = false, internal = true,
            fdbPersistentPath = "id.dosetype.description1", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_TYPE_DESC, reportHistoryCompare = true)
    private String doseTypeDesc = null;
    @FieldInfo(
            name = "DOSETYPEID", editable = true, customPersistentPath = "doseType.code", required = true, internal = false,
            fdbPersistentPath = "id.dosetype.dosetypeid", lookup = true, lookupType = DoseType.class, reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.DOSE_TYPE_ID, reportHistoryCompare = true)
    private String dosetypeid;

    @Digits(integer = LENGTH_TEN, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "DURATIONLOW", editable = true, customPersistentPath = "durationLow", required = false, internal = false,
            fdbPersistentPath = "durationlow", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DURATION_LOW,
            reportHistoryCompare = true)
    private Long durationlow;

    @Digits(integer = LENGTH_TEN, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "DURATIONHIGH", editable = true, customPersistentPath = "durationHigh", required = false, internal = false,
            fdbPersistentPath = "durationhigh", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DURATION_HIGH,
            reportHistoryCompare = true)
    private Long durationhigh;

    @FieldInfo(
            name = "DXID", editable = false, customPersistentPath = "dxid", required = false, internal = false,
            fdbPersistentPath = "dxid", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.DXID)
    private String dxid = DEFAULT_DXID;
    private long fdbCustomDosingVaId;
    private String fdbId;

    @FieldInfo(
            name = "FDBDX", editable = false, customPersistentPath = "fdbdx", required = false, internal = false,
            fdbPersistentPath = "id.fdbdx", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.FDBDX)
    private String fdbdx = DEFAULT_FDBDX;

    @FieldInfo(
            name = "FREQUENCYLOW", editable = true, customPersistentPath = "frequencyLow", required = false, internal = false,
            fdbPersistentPath = "frequencylow", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.FREQUENCY_LOW,
            reportHistoryCompare = true)
    private Double frequencylow;

    @FieldInfo(
            name = "FREQUENCYHIGH", editable = true, customPersistentPath = "frequencyHigh", required = false,
            internal = false, fdbPersistentPath = "frequencyhigh", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.FREQUENCY_HIGH, reportHistoryCompare = true)
    private Double frequencyhigh;

    @FieldInfo(
            name = "HIGHELIMINATIONHALFLIFE", editable = true, customPersistentPath = "highEliminationHalfLife",
            required = false, internal = false, fdbPersistentPath = "higheliminationhalflife", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.HIGH_ELIMINATION_HALF_LIFE, reportHistoryCompare = true)
    private Double higheliminationhalflife;

    @FieldInfo(
            name = "HALFLIFEUNITS", editable = true, lookup = true, customPersistentPath = "halfLifeUnits", required = false,
            internal = false, fdbPersistentPath = "halflifeunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.HALF_LIFE_UNITS, reportHistoryCompare = true)
    private String halflifeunits;

    @Digits(integer = LENGTH_SIX, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "HEPATICIMPAIRMENTIND", editable = true, customPersistentPath = "hepaticImpairmentIndicator",
            required = false, internal = false, fdbPersistentPath = "hepaticimpairmentind", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.HEPATIC_IMPAIRMENT_IND, reportHistoryCompare = true)
    private Long hepaticimpairmentind;

    @FieldInfo(
            name = "HITTYPE", hideFromDetailDisplay = true, fdbPersistentPath = "id.hittype", customPersistentPath = "",
            editable = false, required = false, internal = false)
    private Long hitType;

    @FieldInfo(
            name = "LOWELIMINATIONHALFLIFE", editable = true, customPersistentPath = "lowEliminationHalfLife",
            required = false, internal = false, fdbPersistentPath = "loweliminationhalflife", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.LOW_ELIMINATION_HALF_LIFE, reportHistoryCompare = true)
    private Double loweliminationhalflife;

    @FieldInfo(
            name = "MAXSINGLEDOSE", editable = true, customPersistentPath = "maxSingleDose", required = false,
            internal = false, fdbPersistentPath = "maxsingledose", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE, reportHistoryCompare = true)
    private Double maxsingledose;

    @FieldInfo(
            name = "MAXSINGLEDOSEUNITS", editable = true, lookup = true, customPersistentPath = "maxSingleDoseUnits",
            required = false, internal = false, fdbPersistentPath = "maxsingledoseunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_UNITS, reportHistoryCompare = true)
    private String maxsingledoseunits;

    @FieldInfo(
            name = "MAXSINGLEDOSEFORM", editable = true, customPersistentPath = "maxSingleDoseForm", required = false,
            internal = false, fdbPersistentPath = "maxsingledoseform", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_FORM, reportHistoryCompare = true)
    private Double maxsingledoseform;

    @FieldInfo(
            name = "MAXSINGLEDOSEFORMUNITS", editable = true, lookup = true, customPersistentPath = "maxSingleDoseFormUnits",
            required = false, internal = false, fdbPersistentPath = "maxsingledoseformunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_FORM_UNITS, reportHistoryCompare = true)
    private String maxsingledoseformunits;

    @FieldInfo(
            name = "MAXDAILYDOSE", editable = true, customPersistentPath = "maxDailyDose", required = false, internal = false,
            fdbPersistentPath = "maxdailydose", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE,
            reportHistoryCompare = true)
    private Double maxdailydose;

    @FieldInfo(
            name = "MAXDAILYDOSEUNITS", editable = true, lookup = true, customPersistentPath = "maxDailyDoseUnits",
            required = false, internal = false, fdbPersistentPath = "maxdailydoseunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_UNITS, reportHistoryCompare = true)
    private String maxdailydoseunits;

    @FieldInfo(
            name = "MAXDAILYDOSEFORM", editable = true, customPersistentPath = "maxDailyDoseForm", required = false,
            internal = false, fdbPersistentPath = "maxdailydoseform", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_FORM, reportHistoryCompare = true)
    private Double maxdailydoseform;

    @FieldInfo(
            name = "MAXDAILYDOSEFORMUNITS", editable = true, lookup = true, customPersistentPath = "maxDailyDoseFormUnits",
            required = false, internal = false, fdbPersistentPath = "maxdailydoseformunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_FORM_UNITS, reportHistoryCompare = true)
    private String maxdailydoseformunits;

    @Digits(integer = LENGTH_TEN, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "MAXDURATION", editable = true, customPersistentPath = "maxDuration", required = false, internal = false,
            fdbPersistentPath = "maxduration", reportHistory = true, reportHistoryOrder = DoseRangeReportOrder.MAX_DURATION,
            reportHistoryCompare = true)
    private Long maxduration;

    @FieldInfo(
            name = "MAXLIFETIMEDOSE", editable = true, customPersistentPath = "maxLifeTimeDose", required = false,
            internal = false, fdbPersistentPath = "maxlifetimedose", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_LIFETIME_DOSE, reportHistoryCompare = true)
    private Double maxlifetimedose;

    @FieldInfo(
            name = "MAXLIFETIMEDOSEUNITS", editable = true, lookup = true, customPersistentPath = "maxLifeTimeDoseUnits",
            required = false, internal = false, fdbPersistentPath = "maxlifetimedoseunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_LIFETIME_DOSE_UNITS, reportHistoryCompare = true)
    private String maxlifetimedoseunits;

    @FieldInfo(
            name = "MAXLIFETIMEDOSEFORM", editable = true, customPersistentPath = "maxLifeTimeDoseForm", required = false,
            internal = false, fdbPersistentPath = "maxlifetimedoseform", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_LIFETIME_DOSE_FORM, reportHistoryCompare = true)
    private Double maxlifetimedoseform;

    @FieldInfo(
            name = "MAXLIFETIMEDOSEFORMUNITS", editable = true, lookup = true,
            customPersistentPath = "maxLifeTimeDoseFormUnits", required = false, internal = false,
            fdbPersistentPath = "maxlifetimedoseformunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_LIFETIME_DOSE_FORM_UNITS, reportHistoryCompare = true)
    private String maxlifetimedoseformunits;

    @FieldInfo(
            name = "MAXSINGLEDOSERATE", editable = true, customPersistentPath = "maxSingleDoseRate", required = false,
            internal = false, fdbPersistentPath = "maxsingledoserate", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_RATE, reportHistoryCompare = true)
    private Double maxsingledoserate;

    @FieldInfo(
            name = "MAXSINGLEDOSERATEUNITS", editable = true, lookup = true, customPersistentPath = "maxSingleDoseRateUnits",
            required = false, internal = false, fdbPersistentPath = "maxsingledoserateunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_RATE_UNITS, reportHistoryCompare = true)
    private String maxsingledoserateunits;

    @FieldInfo(
            name = "MAXSINGLEDOSEFORMRATE", editable = true, customPersistentPath = "maxSingleDoseFormRate", required = false,
            internal = false, fdbPersistentPath = "maxsingledoseformrate", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_FORM_RATE, reportHistoryCompare = true)
    private Double maxsingledoseformrate;

    @FieldInfo(
            name = "MAXSINGLEDOSEFORMRATEUNITS", editable = true, lookup = true,
            customPersistentPath = "maxSingleDoseFormRateUnits", required = false, internal = false,
            fdbPersistentPath = "maxsingledoseformrateunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_DOSE_FORM_RATE_UNITS, reportHistoryCompare = true)
    private String maxsingledoseformrateunits;

    @FieldInfo(
            name = "MAXDAILYDOSERATE", editable = true, customPersistentPath = "maxDailyDoseRate", required = false,
            internal = false, fdbPersistentPath = "maxdailydoserate", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_RATE, reportHistoryCompare = true)
    private Double maxdailydoserate;

    @FieldInfo(
            name = "MAXDAILYDOSERATEUNITS", editable = true, lookup = true, customPersistentPath = "maxDailyDoseRateUnits",
            required = false, internal = false, fdbPersistentPath = "maxdailydoserateunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_RATE_UNITS, reportHistoryCompare = true)
    private String maxdailydoserateunits;

    @FieldInfo(
            name = "MAXDAILYDOSEFORMRATE", editable = true, customPersistentPath = "maxDailyDoseFormRate", required = false,
            internal = false, fdbPersistentPath = "maxdailydoseformrate", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_FORM_RATE, reportHistoryCompare = true)
    private Double maxdailydoseformrate;

    @FieldInfo(
            name = "MAXDAILYDOSEFORMRATEUNITS", editable = true, lookup = true,
            customPersistentPath = "maxDailyDoseFormRateUnits", required = false, internal = false,
            fdbPersistentPath = "maxdailydoseformrateunits", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_DAILY_DOSE_FORM_RATE_UNITS, reportHistoryCompare = true)
    private String maxdailydoseformrateunits;

    private boolean readOnlyRecord;
    private String recordId;
    @FieldInfo(
            name = "REFERENCE_TEXT", displayLines = DISPLAY_LINES_FIVE, internal = true, required = false, editable = true,
            lookup = false, customPersistentPath = "referenceText", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.REFERENCE_TEXT_REPORT_ORDER, reportHistoryCompare = true)
    private String referenceText;

    @Digits(integer = LENGTH_SIX, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "RENALIMPAIRMENTIND", editable = true, customPersistentPath = "renalImpairmentIndicator", required = false,
            internal = false, fdbPersistentPath = "renalimpairmentind", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.RENAL_IMPAIRMENT_IND, reportHistoryCompare = true)
    private Long renalimpairmentind;

    private String requestId;
    private boolean vaExists;
    private String warningcode;
    @Digits(integer = LENGTH_SIX, fraction = LENGTH_THREE)
    @FieldInfo(
            name = "WEIGHTREQUIREDIND", editable = true, customPersistentPath = "weightImpairmentIndicator", required = false,
            internal = false, fdbPersistentPath = "weightrequiredind", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.WEIGHT_REQUIRED_IND, reportHistoryCompare = true)
    private Long weightrequiredind;

    @FieldInfo(
            name = "MAXSINGLENTEDOSE", editable = true, customPersistentPath = "maxSingleNTEDose", required = false,
            internal = false, fdbPersistentPath = "maxSingleNTEDose", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_NTE_DOSE, reportHistoryCompare = true)
    private Double maxSingleNTEDose;

    @FieldInfo(
            name = "MAXSINGLENTEDOSEUNIT", editable = true, customPersistentPath = "maxSingleNTEDoseUnit", required = false,
            internal = false, fdbPersistentPath = "maxSingleNTEDoseUnit", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_NTE_DOSE_UNIT, reportHistoryCompare = true)
    private String maxSingleNTEDoseUnit;

    @FieldInfo(
            name = "MAXSINGLENTEDOSEFORM", editable = true, customPersistentPath = "maxSingleNTEDoseForm", required = false,
            internal = false, fdbPersistentPath = "maxSingleNTEDoseForm", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_NTE_DOSE_FORM, reportHistoryCompare = true)
    private Double maxSingleNTEDoseForm;

    @FieldInfo(
            name = "MAXSINGLENTEDOSEFORMUNIT", editable = true, customPersistentPath = "maxSingleNTEDoseFormUnit",
            required = false, internal = false, fdbPersistentPath = "maxSingleNTEDoseFormUnit", reportHistory = true,
            reportHistoryOrder = DoseRangeReportOrder.MAX_SINGLE_NTE_DOSE_FORM_UNIT, reportHistoryCompare = true)
    private String maxSingleNTEDoseFormUnit;

    /** default constructor. */
    public DoseRangeDTO() {

    }

    /**
     * DeepCopy FDB Custom Dosing Va.
     * 
     * @param source
     *            DoseRangeDTO
     */
    public void deepCopy(DoseRangeDTO source) {
        this.setReferenceText(source.getReferenceText());
        this.setCategory(source.getCategory());
        this.setConcepttype(source.getConcepttype());
        this.setConceptid(source.getConceptid());
        this.setConceptDescription(source.getConceptDescription());
        this.setAgelowindays(source.getAgelowindays());
        this.setAgehighindays(source.getAgehighindays());
        this.setDoserouteid(source.getDoserouteid());
        this.setDosetypeid(source.getDosetypeid());
        this.setFdbdx(source.getFdbdx());
        this.setDoselow(source.getDoselow());
        this.setDoselowunits(source.getDoselowunits());
        this.setDosehigh(source.getDosehigh());
        this.setDosehighunits(source.getDosehighunits());
        this.setDoseformlow(source.getDoseformlow());
        this.setDoseformlowunits(source.getDoseformlowunits());
        this.setDoseformhigh(source.getDoseformhigh());
        this.setDoseformhighunits(source.getDoseformhighunits());
        this.setMaxsingledose(source.getMaxsingledose());
        this.setMaxsingledoseunits(source.getMaxsingledoseunits());
        this.setMaxsingledoseform(source.getMaxsingledoseform());
        this.setMaxsingledoseformunits(source.getMaxsingledoseformunits());
        this.setMaxdailydose(source.getMaxdailydose());
        this.setMaxdailydoseunits(source.getMaxdailydoseunits());
        this.setMaxdailydoseform(source.getMaxdailydoseform());
        this.setMaxdailydoseformunits(source.getMaxdailydoseformunits());
        this.setMaxlifetimedose(source.getMaxlifetimedose());
        this.setMaxlifetimedoseunits(source.getMaxlifetimedoseformunits());
        this.setMaxlifetimedoseform(source.getMaxlifetimedoseform());
        this.setMaxlifetimedoseformunits(source.getMaxlifetimedoseunits());
        this.setLoweliminationhalflife(source.getLoweliminationhalflife());
        this.setHigheliminationhalflife(source.getHigheliminationhalflife());
        this.setHalflifeunits(source.getHalflifeunits());
        this.setFrequencylow(source.getFrequencylow());
        this.setFrequencyhigh(source.getFrequencyhigh());
        this.setDurationlow(source.getDurationlow());
        this.setDurationhigh(source.getDurationhigh());
        this.setMaxduration(source.getMaxduration());
        this.setHepaticimpairmentind(source.getHepaticimpairmentind());
        this.setRenalimpairmentind(source.getRenalimpairmentind());
        this.setCrclthreshhold(source.getCrclthreshhold());
        this.setCrclthreshholdunits(source.getCrclthreshholdunits());
        this.setWeightrequiredind(source.getWeightrequiredind());
        this.setBsarequiredind(source.getBsarequiredind());
        this.setDoseratelow(source.getDoseratelow());
        this.setDoseratelowunits(source.getDoseratelowunits());
        this.setDoseratehigh(source.getDoseratehigh());
        this.setDoseratehighunits(source.getDoseratehighunits());
        this.setDoseformratelow(source.getDoseformratelow());
        this.setDoseformratelowunits(source.getDoseformratelowunits());
        this.setDoseformratehigh(source.getDoseformratehigh());
        this.setDoseformratehighunits(source.getDoseformratehighunits());
        this.setMaxsingledoserate(source.getMaxsingledoserate());
        this.setMaxsingledoserateunits(source.getMaxsingledoserateunits());
        this.setMaxsingledoseformrate(source.getMaxsingledoseformrate());
        this.setMaxsingledoseformrateunits(source.getMaxsingledoseformrateunits());
        this.setMaxdailydoserate(source.getMaxdailydoserate());
        this.setMaxdailydoserateunits(source.getMaxdailydoserateunits());
        this.setMaxdailydoseformrate(source.getMaxdailydoseformrate());
        this.setMaxdailydoseformrateunits(source.getMaxdailydoseformrateunits());
        this.setDxid(source.getDxid());
        this.setFdbId(source.getFdbId());
        this.setActionDate(source.getActionDate());
        this.setActionReason(source.getActionReason());
        this.setActionBy(source.getActionBy());
        this.setAssignedTo(source.getAssignedTo());
        this.setRequestId(source.getRequestId());
        this.setRequestor(source.getRequestor());
        this.setDoseRouteDesc(source.getDoseRouteDesc());
        this.setDoseTypeDesc(source.getDoseTypeDesc());
    }

    /**
     * Generate Record ID.
     * 
     * @return the Current Record of DoseRangeDTO
     */
    public String generateRecordId() {
        return (isCurrent() ? "C" : "H") + getFdbCustomDosingVaId();
    }

    /**
     * Get Record Id.
     * 
     * @return recordId
     */
    public String getRecordId() {
        return recordId;
    }

    /**
     * Set RecordId.
     * 
     * @param recordId
     *            -String
     */
    public void setRecordId(String recordId) {
        this.recordId = recordId;
    }

    /**
     * Get DoseRouteDesec.
     * 
     * @return doseRouteDesc
     */
    public String getDoseRouteDesc() {
        return doseRouteDesc;
    }

    /**
     * Set DoseRouteDesc.
     * 
     * @param doseRouteDesc
     *            String
     */
    public void setDoseRouteDesc(String doseRouteDesc) {
        this.doseRouteDesc = doseRouteDesc;
    }

    /**
     * Get DoseTypeDesc.
     * 
     * @return doseTypesDesc
     */
    public String getDoseTypeDesc() {
        return doseTypeDesc;
    }

    /**
     * Set Dose Type Description.
     * 
     * @param doseTypeDesc
     *            - String
     */
    public void setDoseTypeDesc(String doseTypeDesc) {
        this.doseTypeDesc = doseTypeDesc;
    }

    /**
     * Get Warning Code.
     * 
     * @return String warning code
     */
    public String getWarningcode() {
        return warningcode;
    }

    /**
     * Set Warning Code.
     * 
     * @param warningcode
     *            - String
     */
    public void setWarningcode(String warningcode) {
        this.warningcode = warningcode;
    }

    /**
     * Get Dose Range Id.
     * 
     * @return doseRangeId
     */
    public long getDoseRangeId() {
        return doseRangeId;
    }

    /**
     * Set Dose Range Id.
     * 
     * @param doseRangeId
     *            - Long
     */
    public void setDoseRangeId(long doseRangeId) {
        this.doseRangeId = doseRangeId;
    }

    /**
     * Get Fdb Custom Dosing Va Id.
     * 
     * @return fdbCustomDosingVaId
     */
    public long getFdbCustomDosingVaId() {
        return fdbCustomDosingVaId;
    }

    /**
     * Set Fdb Custom Dosing Va Id.
     * 
     * @param fdbCustomDosingVaId
     *            - Long
     */
    public void setFdbCustomDosingVaId(long fdbCustomDosingVaId) {
        this.fdbCustomDosingVaId = fdbCustomDosingVaId;
    }

    /**
     * Get FdbId.
     * 
     * @return fdbId
     */
    public String getFdbId() {
        return fdbId;
    }

    /**
     * Set FdbId.
     * 
     * @param fdbId
     *            - String
     */
    public void setFdbId(String fdbId) {
        this.fdbId = fdbId;
    }

    /**
     * Boolean is Read Only Record.
     * 
     * @return readOnlyRecord
     */
    public boolean isReadOnlyRecord() {
        return readOnlyRecord;
    }

    /**
     * Set ReadOnlyRecord.
     * 
     * @param readOnlyRecord
     *            boolean
     */
    public void setReadOnlyRecord(boolean readOnlyRecord) {
        this.readOnlyRecord = readOnlyRecord;
    }

    /**
     * Get RequestId.
     * 
     * @return - requestId
     */
    public String getRequestId() {
        return requestId;
    }

    /**
     * Set RequestId.
     * 
     * @param requestId
     *            - String
     */
    public void setRequestId(String requestId) {
        this.requestId = requestId;
    }

    /**
     * Get Reference Text.
     * 
     * @return referenceText
     */
    public String getReferenceText() {
        return referenceText;
    }

    /**
     * Set ReferenceText.
     * 
     * @param referenceText
     *            - String
     */
    public void setReferenceText(String referenceText) {
        this.referenceText = referenceText;
    }

    /**
     * Get Category.
     * 
     * @return category
     */
    public String getCategory() {
        return category;
    }

    /**
     * Set Category.
     * 
     * @param category
     *            - String
     */
    public void setCategory(String category) {
        this.category = category;
    }

    /**
     * Get Concepttype.
     * 
     * @return concepttype
     */
    public long getConcepttype() {
        return concepttype;
    }

    /**
     * Set Concept Type.
     * 
     * @param concepttype
     *            - long
     */
    public void setConcepttype(long concepttype) {
        this.concepttype = concepttype;
    }

    /**
     * Get ConceptId.
     * 
     * @return conceptid
     */
    public Long getConceptid() {
        return conceptid;
    }

    /**
     * Set Concept ID.
     * 
     * @param conceptid
     *            - Long
     */
    public void setConceptid(Long conceptid) {
        this.conceptid = conceptid;
    }

    /**
     * Get Concept Description.
     * 
     * @return the conceptDescription
     */
    public String getConceptDescription() {
        return conceptDescription;
    }

    /**
     * Set concept Description.
     * 
     * @param conceptDescription
     *            the conceptDescription to set
     */
    public void setConceptDescription(String conceptDescription) {
        this.conceptDescription = conceptDescription;
    }

    /**
     * Get Age Low in Days.
     * 
     * @return agelowindays
     */
    public Long getAgelowindays() {
        return agelowindays;
    }

    /**
     * Set Age Low in Days.
     * 
     * @param agelowindays
     *            - Long
     */
    public void setAgelowindays(Long agelowindays) {
        this.agelowindays = agelowindays;
    }

    /**
     * Get Age High in Days.
     * 
     * @return agehighindays
     */
    public Long getAgehighindays() {
        return agehighindays;
    }

    /**
     * Set Age High In Days.
     * 
     * @param agehighindays
     *            - Long
     */
    public void setAgehighindays(Long agehighindays) {
        this.agehighindays = agehighindays;
    }

    /**
     * Get Dose routed Id.
     * 
     * @return doserouteid
     */
    public String getDoserouteid() {
        return doserouteid;
    }

    /**
     * Set Dose Route Id.
     * 
     * @param doserouteid
     *            - String
     */
    public void setDoserouteid(String doserouteid) {
        this.doserouteid = doserouteid;
    }

    /**
     * Get DoseType Id.
     * 
     * @return - dosetypeid
     */
    public String getDosetypeid() {
        return dosetypeid;
    }

    /**
     * Set Dose Type Id.
     * 
     * @param dosetypeid
     *            - String
     */
    public void setDosetypeid(String dosetypeid) {
        this.dosetypeid = dosetypeid;
    }

    /**
     * Get Fdb dx.
     * 
     * @return fdbdx
     */
    public String getFdbdx() {
        return fdbdx;
    }

    /**
     * Set Fdbdx.
     * 
     * @param fdbdx
     *            - String
     */
    public void setFdbdx(String fdbdx) {
        this.fdbdx = fdbdx;
    }

    /**
     * Get Dose Low.
     * 
     * @return doselow
     */
    public Double getDoselow() {
        return doselow;
    }

    /**
     * Set Dose Low.
     * 
     * @param doselow
     *            - Double
     */
    public void setDoselow(Double doselow) {
        this.doselow = doselow;
    }

    /**
     * Get Dose Low Units.
     * 
     * @return doselowunits
     */
    public String getDoselowunits() {
        return doselowunits;
    }

    /**
     * Set Dose Low Units.
     * 
     * @param doselowunits
     *            - String
     */
    public void setDoselowunits(String doselowunits) {
        this.doselowunits = doselowunits;
    }

    /**
     * Get Dose High.
     * 
     * @return dosehigh
     */
    public Double getDosehigh() {
        return dosehigh;
    }

    /**
     * Set Does High.
     * 
     * @param dosehigh
     *            - Double
     */
    public void setDosehigh(Double dosehigh) {
        this.dosehigh = dosehigh;
    }

    /**
     * Get Dose High Units.
     * 
     * @return dosehighunits
     */
    public String getDosehighunits() {
        return dosehighunits;
    }

    /**
     * Set Dose High Units.
     * 
     * @param dosehighunits
     *            - String
     */
    public void setDosehighunits(String dosehighunits) {
        this.dosehighunits = dosehighunits;
    }

    /**
     * Get Dose Form Low.
     * 
     * @return doseformlow
     */
    public Double getDoseformlow() {
        return doseformlow;
    }

    /**
     * Set Dose Form Low.
     * 
     * @param doseformlow
     *            - Double
     */
    public void setDoseformlow(Double doseformlow) {
        this.doseformlow = doseformlow;
    }

    /**
     * Get Dose Form Low Units.
     * 
     * @return doseformlowunits
     */
    public String getDoseformlowunits() {
        return doseformlowunits;
    }

    /**
     * Set Dose Form Low Units.
     * 
     * @param doseformlowunits
     */
    public void setDoseformlowunits(String doseformlowunits) {
        this.doseformlowunits = doseformlowunits;
    }

    /**
     * Get Dose Form High.
     * 
     * @return doseformhigh
     */
    public Double getDoseformhigh() {
        return doseformhigh;
    }

    /**
     * Set Dose Form High.
     * 
     * @param doseformhigh
     *            - Double
     */
    public void setDoseformhigh(Double doseformhigh) {
        this.doseformhigh = doseformhigh;
    }

    /**
     * Get Dose form High Units.
     * 
     * @return doseformhighunits
     */
    public String getDoseformhighunits() {
        return doseformhighunits;
    }

    /**
     * Set Dose Form high Units.
     * 
     * @param doseformhighunits
     *            - String
     */
    public void setDoseformhighunits(String doseformhighunits) {
        this.doseformhighunits = doseformhighunits;
    }

    /**
     * Get Max single does.
     * 
     * @return maxsingledose
     */
    public Double getMaxsingledose() {
        return maxsingledose;
    }

    /**
     * Set Max single Dose.
     * 
     * @param maxsingledose
     *            Double
     */
    public void setMaxsingledose(Double maxsingledose) {
        this.maxsingledose = maxsingledose;
    }

    /**
     * Get Max single Dose unit.
     * 
     * @return maxsingledoseunits
     */
    public String getMaxsingledoseunits() {
        return maxsingledoseunits;
    }

    /**
     * Set Max Single Dose Units.
     * 
     * @param maxsingledoseunits
     *            - String
     */
    public void setMaxsingledoseunits(String maxsingledoseunits) {
        this.maxsingledoseunits = maxsingledoseunits;
    }

    /**
     * Get Max Single Dose Form.
     * 
     * @return maxsingledoseform
     */
    public Double getMaxsingledoseform() {
        return maxsingledoseform;
    }

    /**
     * Set Max Single Dose Form.
     * 
     * @param maxsingledoseform
     *            - Double
     */
    public void setMaxsingledoseform(Double maxsingledoseform) {
        this.maxsingledoseform = maxsingledoseform;
    }

    /**
     * Get Max Single Dose Form Units.
     * 
     * @return maxsingledoseformunits
     */
    public String getMaxsingledoseformunits() {
        return maxsingledoseformunits;
    }

    /**
     * Set Max Single Dose Form Units.
     * 
     * @param maxsingledoseformunits
     *            - String
     */
    public void setMaxsingledoseformunits(String maxsingledoseformunits) {
        this.maxsingledoseformunits = maxsingledoseformunits;
    }

    /**
     * Get Max Daily Dose.
     * 
     * @return maxdailydose
     */
    public Double getMaxdailydose() {
        return maxdailydose;
    }

    /**
     * Set Max Daily Dose.
     * 
     * @param maxdailydose
     *            - Double
     */
    public void setMaxdailydose(Double maxdailydose) {
        this.maxdailydose = maxdailydose;
    }

    /**
     * Get Max Daily Dose Units.
     * 
     * @return maxdailydoseunits
     */
    public String getMaxdailydoseunits() {
        return maxdailydoseunits;
    }

    /**
     * Set Max Daily Dose Units.
     * 
     * @param maxdailydoseunits
     *            - String
     */
    public void setMaxdailydoseunits(String maxdailydoseunits) {
        this.maxdailydoseunits = maxdailydoseunits;
    }

    /**
     * Get Max Daily Dose Form.
     * 
     * @return maxdailydoseform
     */
    public Double getMaxdailydoseform() {
        return maxdailydoseform;
    }

    /**
     * Set Max Daily Dose Form.
     * 
     * @param maxdailydoseform
     *            - Double
     */
    public void setMaxdailydoseform(Double maxdailydoseform) {
        this.maxdailydoseform = maxdailydoseform;
    }

    /**
     * Get Max Daily Dose Form Units.
     * 
     * @return maxdailydoseformunits
     */
    public String getMaxdailydoseformunits() {
        return maxdailydoseformunits;
    }

    /**
     * Set Max Daily Dose Form Units.
     * 
     * @param maxdailydoseformunits
     *            - String
     */
    public void setMaxdailydoseformunits(String maxdailydoseformunits) {
        this.maxdailydoseformunits = maxdailydoseformunits;
    }

    /**
     * Get Max Life Time Dose.
     * 
     * @return maxlifetimedose
     */
    public Double getMaxlifetimedose() {
        return maxlifetimedose;
    }

    /**
     * Set Max Life Time Dose.
     * 
     * @param maxlifetimedose
     *            - Double
     */
    public void setMaxlifetimedose(Double maxlifetimedose) {
        this.maxlifetimedose = maxlifetimedose;
    }

    /**
     * Get max Life Time Dose Units.
     * 
     * @return maxlifetimedoseunits
     */
    public String getMaxlifetimedoseunits() {
        return maxlifetimedoseunits;
    }

    /**
     * Set Max Life Time Dose Units.
     * 
     * @param maxlifetimedoseunits
     *            - String
     */
    public void setMaxlifetimedoseunits(String maxlifetimedoseunits) {
        this.maxlifetimedoseunits = maxlifetimedoseunits;
    }

    /**
     * Get Max Life Time Dose Form.
     * 
     * @return maxlifetimedoseform
     */
    public Double getMaxlifetimedoseform() {
        return maxlifetimedoseform;
    }

    /**
     * Set Max Life Time Dose Form.
     * 
     * @param maxlifetimedoseform
     *            - Double
     */
    public void setMaxlifetimedoseform(Double maxlifetimedoseform) {
        this.maxlifetimedoseform = maxlifetimedoseform;
    }

    /**
     * Get Max Life Time Dose Form Units.
     * 
     * @return maxlifetimedoseformunits
     */
    public String getMaxlifetimedoseformunits() {
        return maxlifetimedoseformunits;
    }

    /**
     * Set Max Life Time Dose Form Units.
     * 
     * @param maxlifetimedoseformunits
     *            - String
     */
    public void setMaxlifetimedoseformunits(String maxlifetimedoseformunits) {
        this.maxlifetimedoseformunits = maxlifetimedoseformunits;
    }

    /**
     * Get Low elimination half life.
     * 
     * @return loweliminationhalflife
     */
    public Double getLoweliminationhalflife() {
        return loweliminationhalflife;
    }

    /**
     * Set Low Elimination Half Life.
     * 
     * @param loweliminationhalflife
     *            - Double
     */
    public void setLoweliminationhalflife(Double loweliminationhalflife) {
        this.loweliminationhalflife = loweliminationhalflife;
    }

    /**
     * Get High Elimination Half Life.
     * 
     * @return higheleminationhalflife
     */
    public Double getHigheliminationhalflife() {
        return higheliminationhalflife;
    }

    /**
     * Set High Elimination Half Life.
     * 
     * @param higheliminationhalflife
     *            - double
     */
    public void setHigheliminationhalflife(Double higheliminationhalflife) {
        this.higheliminationhalflife = higheliminationhalflife;
    }

    /**
     * Get Half Life Units.
     * 
     * @return halflifeunits.
     */
    public String getHalflifeunits() {
        return halflifeunits;
    }

    /**
     * Set Half Life Units.
     * 
     * @param halflifeunits
     *            - String
     */
    public void setHalflifeunits(String halflifeunits) {
        this.halflifeunits = halflifeunits;
    }

    /**
     * Get Frequency Low.
     * 
     * @return frequencylow
     */
    public Double getFrequencylow() {
        return frequencylow;
    }

    /**
     * Set Frequency Low.
     * 
     * @param frequencylow
     *            -Double
     */
    public void setFrequencylow(Double frequencylow) {
        this.frequencylow = frequencylow;
    }

    /**
     * Get Frequency High.
     * 
     * @return frequencyhigh
     */
    public Double getFrequencyhigh() {
        return frequencyhigh;
    }

    /**
     * Set Frequency High.
     * 
     * @param frequencyhigh
     *            - Double
     */
    public void setFrequencyhigh(Double frequencyhigh) {
        this.frequencyhigh = frequencyhigh;
    }

    /**
     * Get Duration Low.
     * 
     * @return durationlow
     */
    public Long getDurationlow() {
        return durationlow;
    }

    /**
     * Set Duration Low.
     * 
     * @param durationlow
     *            - Long
     */
    public void setDurationlow(Long durationlow) {
        this.durationlow = durationlow;
    }

    /**
     * Get Duration High.
     * 
     * @return durationhigh
     */
    public Long getDurationhigh() {
        return durationhigh;
    }

    /**
     * Set Duration High.
     * 
     * @param durationhigh
     *            - Long
     */
    public void setDurationhigh(Long durationhigh) {
        this.durationhigh = durationhigh;
    }

    /**
     * Get Max Duration.
     * 
     * @return maxduration
     */
    public Long getMaxduration() {
        return maxduration;
    }

    /**
     * Set Max Duration.
     * 
     * @param maxduration
     *            - Long
     */
    public void setMaxduration(Long maxduration) {
        this.maxduration = maxduration;
    }

    /**
     * Get Hepaticimpairmentind.
     * 
     * @return hepaticimpairmentind
     */
    public Long getHepaticimpairmentind() {
        return hepaticimpairmentind;
    }

    /**
     * Set Hepaticimpairmentind.
     * 
     * @param hepaticimpairmentind
     *            - Long
     */
    public void setHepaticimpairmentind(Long hepaticimpairmentind) {
        this.hepaticimpairmentind = hepaticimpairmentind;
    }

    /**
     * Get Renalimpairmentind.
     * 
     * @return renalimpairmentind
     */
    public Long getRenalimpairmentind() {
        return renalimpairmentind;
    }

    /**
     * Set Renalimparmentind.
     * 
     * @param renalimpairmentind
     *            - Long
     */
    public void setRenalimpairmentind(Long renalimpairmentind) {
        this.renalimpairmentind = renalimpairmentind;
    }

    /**
     * Get Crcl threshold.
     * 
     * @return crclthreshhold
     */
    public Long getCrclthreshhold() {
        return crclthreshhold;
    }

    /**
     * Set Crclthreshhold.
     * 
     * @param crclthreshhold
     *            - Long
     */
    public void setCrclthreshhold(Long crclthreshhold) {
        this.crclthreshhold = crclthreshhold;
    }

    /**
     * Get Crcl threshold Units.
     * 
     * @return crclthreshholdunits
     */
    public String getCrclthreshholdunits() {
        return crclthreshholdunits;
    }

    /**
     * Set Crclthreshholdunits.
     * 
     * @param crclthreshholdunits
     *            - String
     */
    public void setCrclthreshholdunits(String crclthreshholdunits) {
        this.crclthreshholdunits = crclthreshholdunits;
    }

    /**
     * Get Weight Required ind.
     * 
     * @return weightrequiredind
     */
    public Long getWeightrequiredind() {
        return weightrequiredind;
    }

    /**
     * Set Weight Required ind.
     * 
     * @param weightrequiredind
     *            - Long
     */
    public void setWeightrequiredind(Long weightrequiredind) {
        this.weightrequiredind = weightrequiredind;
    }

    /**
     * Get Bsarequaridind.
     * 
     * @return bsarequiredind
     */
    public Long getBsarequiredind() {
        return bsarequiredind;
    }

    /**
     * Set Bs are Required ind.
     * 
     * @param bsarequiredind
     *            Long
     */
    public void setBsarequiredind(Long bsarequiredind) {
        this.bsarequiredind = bsarequiredind;
    }

    /**
     * Get Dose Rate Low.
     * 
     * @return doseratelow
     */
    public Double getDoseratelow() {
        return doseratelow;
    }

    /**
     * Set Dose Rate Low.
     * 
     * @param doseratelow
     *            - Double
     */
    public void setDoseratelow(Double doseratelow) {
        this.doseratelow = doseratelow;
    }

    /**
     * Get Dose Rate Low Units.
     * 
     * @return doseratelowunits
     */
    public String getDoseratelowunits() {
        return doseratelowunits;
    }

    /**
     * Set Dose Rate Low Units.
     * 
     * @param doseratelowunits
     *            - String
     */
    public void setDoseratelowunits(String doseratelowunits) {
        this.doseratelowunits = doseratelowunits;
    }

    /**
     * Get Dose Rate High.
     * 
     * @return doseratehigh
     */
    public Double getDoseratehigh() {
        return doseratehigh;
    }

    /**
     * Set Dose Rate Hight.
     * 
     * @param doseratehigh
     *            double
     */
    public void setDoseratehigh(Double doseratehigh) {
        this.doseratehigh = doseratehigh;
    }

    /**
     * Get Dose Rate High Units.
     * 
     * @return doseratehighunits
     */
    public String getDoseratehighunits() {
        return doseratehighunits;
    }

    /**
     * Set Dose Rate High Units.
     * 
     * @param doseratehighunits
     *            - String
     */
    public void setDoseratehighunits(String doseratehighunits) {
        this.doseratehighunits = doseratehighunits;
    }

    /**
     * Get Dose Form Rate Low.
     * 
     * @return doseformratelow
     */
    public Double getDoseformratelow() {
        return doseformratelow;
    }

    /**
     * Set Dose Form Rate Low.
     * 
     * @param doseformratelow
     *            - Double
     */
    public void setDoseformratelow(Double doseformratelow) {
        this.doseformratelow = doseformratelow;
    }

    /**
     * Get Dose Form Rate Low Units.
     * 
     * @return doseformratelowunits
     */
    public String getDoseformratelowunits() {
        return doseformratelowunits;
    }

    /**
     * Set Dose Form Rate Low Units.
     * 
     * @param doseformratelowunits
     *            - String
     */
    public void setDoseformratelowunits(String doseformratelowunits) {
        this.doseformratelowunits = doseformratelowunits;
    }

    /**
     * Get Dose Form Rate High.
     * 
     * @return doseformratehigh
     */
    public Double getDoseformratehigh() {
        return doseformratehigh;
    }

    /**
     * Set Dose Form Rate High.
     * 
     * @param doseformratehigh
     *            - Double
     */
    public void setDoseformratehigh(Double doseformratehigh) {
        this.doseformratehigh = doseformratehigh;
    }

    /**
     * Get Dose Form Rate High Units.
     * 
     * @return doseformratehighunits
     */
    public String getDoseformratehighunits() {
        return doseformratehighunits;
    }

    /**
     * Set Dose Form Rate High Units.
     * 
     * @param doseformratehighunits
     *            - String
     */
    public void setDoseformratehighunits(String doseformratehighunits) {
        this.doseformratehighunits = doseformratehighunits;
    }

    /**
     * Get Max Single Dose Rate.
     * 
     * @return maxsingledoserate
     */
    public Double getMaxsingledoserate() {
        return maxsingledoserate;
    }

    /**
     * Set Max Single Dose Rate.
     * 
     * @param maxsingledoserate
     *            - Double
     */
    public void setMaxsingledoserate(Double maxsingledoserate) {
        this.maxsingledoserate = maxsingledoserate;
    }

    /**
     * Get Max Single Dose Rate Units.
     * 
     * @return maxsingledoserateunits
     */
    public String getMaxsingledoserateunits() {
        return maxsingledoserateunits;
    }

    /**
     * Set maxsingledoserateunits.
     * 
     * @param maxsingledoserateunits
     *            - String
     */
    public void setMaxsingledoserateunits(String maxsingledoserateunits) {
        this.maxsingledoserateunits = maxsingledoserateunits;
    }

    /**
     * Get Max Single Dose Form rate.
     * 
     * @return maxsingledoseformrate
     */
    public Double getMaxsingledoseformrate() {
        return maxsingledoseformrate;
    }

    /**
     * Set max sing dose form rate.
     * 
     * @param maxsingledoseformrate
     *            - Double
     */
    public void setMaxsingledoseformrate(Double maxsingledoseformrate) {
        this.maxsingledoseformrate = maxsingledoseformrate;
    }

    /**
     * Get Max Single Dose Form Rate Units.
     * 
     * @return maxsingledoseformrateunits
     */
    public String getMaxsingledoseformrateunits() {
        return maxsingledoseformrateunits;
    }

    /**
     * Set Max Single Dose Form rate Unit.
     * 
     * @param maxsingledoseformrateunits
     *            - String
     */
    public void setMaxsingledoseformrateunits(String maxsingledoseformrateunits) {
        this.maxsingledoseformrateunits = maxsingledoseformrateunits;
    }

    /**
     * Get Max Daily Dose Rate.
     * 
     * @return maxdailydoserate.
     */
    public Double getMaxdailydoserate() {
        return maxdailydoserate;
    }

    /**
     * Set Max Daily Dose Rate.
     * 
     * @param maxdailydoserate
     *            - Double
     */
    public void setMaxdailydoserate(Double maxdailydoserate) {
        this.maxdailydoserate = maxdailydoserate;
    }

    /**
     * Get Max Daily Dose Rate Units.
     * 
     * @return maxdailydosrateunits
     */
    public String getMaxdailydoserateunits() {
        return maxdailydoserateunits;
    }

    /**
     * Set Max Daily Dose Rate Units.
     * 
     * @param maxdailydoserateunits
     *            - String
     */
    public void setMaxdailydoserateunits(String maxdailydoserateunits) {
        this.maxdailydoserateunits = maxdailydoserateunits;
    }

    /**
     * Get Max Daily Dose Form Rate.
     * 
     * @return maxdailydoseformrate
     */
    public Double getMaxdailydoseformrate() {
        return maxdailydoseformrate;
    }

    /**
     * Set Max Daily Dose Form Rate.
     * 
     * @param maxdailydoseformrate
     *            - Double
     */
    public void setMaxdailydoseformrate(Double maxdailydoseformrate) {
        this.maxdailydoseformrate = maxdailydoseformrate;
    }

    /**
     * Get Max Daily Dose Format.
     * 
     * @return max daily dose form rate units.
     */
    public String getMaxdailydoseformrateunits() {
        return maxdailydoseformrateunits;
    }

    /**
     * Set Max Daily Dose Format.
     * 
     * @param maxdailydoseformrateunits
     *            - String
     */
    public void setMaxdailydoseformrateunits(String maxdailydoseformrateunits) {
        this.maxdailydoseformrateunits = maxdailydoseformrateunits;
    }

    /**
     * Get DXID.
     * 
     * @return dxid
     */
    public String getDxid() {
        return dxid;
    }

    /**
     * Set DXID.
     * 
     * @param dxid
     *            String
     */
    public void setDxid(String dxid) {
        this.dxid = dxid;
    }

    /**
     * Get Hit Type.
     * 
     * @return hitType
     */
    public Long getHitType() {
        return hitType;
    }

    /**
     * Set HitType.
     * 
     * @param hittype
     *            - Long
     */
    public void setHitType(Long hittype) {
        this.hitType = hittype;
    }

    /**
     * Gets the max single nte dose.
     * 
     * @return the max single nte dose
     */
    public Double getMaxSingleNTEDose() {
        return maxSingleNTEDose;
    }

    /**
     * Sets the max single nte dose.
     * 
     * @param maxSingleNTEDose
     *            the new max single nte dose
     */
    public void setMaxSingleNTEDose(Double maxSingleNTEDose) {
        this.maxSingleNTEDose = maxSingleNTEDose;
    }

    /**
     * Gets the max single nte dose unit.
     * 
     * @return the max single nte dose unit
     */
    public String getMaxSingleNTEDoseUnit() {
        return maxSingleNTEDoseUnit;
    }

    /**
     * Sets the max single nte dose unit.
     * 
     * @param maxSingleNTEDoseUnit
     *            the new max single nte dose unit
     */
    public void setMaxSingleNTEDoseUnit(String maxSingleNTEDoseUnit) {
        this.maxSingleNTEDoseUnit = maxSingleNTEDoseUnit;
    }

    /**
     * Gets the max single nte dose form.
     * 
     * @return the max single nte dose form
     */
    public Double getMaxSingleNTEDoseForm() {
        return maxSingleNTEDoseForm;
    }

    /**
     * Sets the max single nte dose form.
     * 
     * @param maxSingleNTEDoseForm
     *            the new max single nte dose form
     */
    public void setMaxSingleNTEDoseForm(Double maxSingleNTEDoseForm) {
        this.maxSingleNTEDoseForm = maxSingleNTEDoseForm;
    }

    /**
     * Gets the max single nte dose form unit.
     * 
     * @return the max single nte dose form unit
     */
    public String getMaxSingleNTEDoseFormUnit() {
        return maxSingleNTEDoseFormUnit;
    }

    /**
     * Sets the max single nte dose form unit.
     * 
     * @param maxSingleNTEDoseFormUnit the new max single nte dose form unit
     */
    public void setMaxSingleNTEDoseFormUnit(String maxSingleNTEDoseFormUnit) {
        this.maxSingleNTEDoseFormUnit = maxSingleNTEDoseFormUnit;
    }

    /**
     * Get Selected Label Value.
     * 
     * @return LabelValue
     */
    @Override
    public LabelValue getSelect() {
        LabelValue lv = new LabelValue();
        StringBuffer sb = new StringBuffer();

        if (getFdbCustomDosingVaId() > 0) {
            sb.append("/ct/pecs/doseRangeCustomization/VA/");
            sb.append(getFdbCustomDosingVaId());
            if (isCurrent()) {
                lv.setLabel(QueryResultLabel.ACTIVE);
            } else {
                lv.setLabel(QueryResultLabel.HISTORICAL);
            }
        } else {
            sb.append("/ct/pecs/doseRangeCustomization/FDB/");
            sb.append(getConceptid() + "|" + getConcepttype() + "|" + getAgelowindays() + "|" + getAgehighindays() + "|"
                    + getDoserouteid() + "|" + getDosetypeid() + "|" + getFdbdx() + "|" + getHitType());
            lv.setLabel(QueryResultLabel.OPEN);
        }
        lv.setValue(sb.toString());
        return lv;

    }

    /**
     * isVaExists.
     * 
     * @return vaExists
     */
    public boolean isVaExists() {
        return vaExists;
    }

    /**
     * Set VaExist.
     * 
     * @param vaExists boolean
     */
    public void setVaExists(boolean vaExists) {
        this.vaExists = vaExists;
    }
}