package gov.va.med.pharmacy.pecs.dto;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdiminteraction;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.domain.impl.va.DdiminteractionReferenceVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.metadata.ClassInfo;
import gov.va.med.pharmacy.pecs.metadata.CompositeField;
import gov.va.med.pharmacy.pecs.metadata.FieldInfo;
import gov.va.med.pharmacy.pecs.util.LabelValue;
import gov.va.med.pharmacy.pecs.util.QueryResultLabel;
import gov.va.med.pharmacy.pecs.util.historyofchange.DrugDrugInteractionReportOrder;

/**
 * DrugDrugInteractionDTO entity.
 */
@ClassInfo(
        name = "Drug-Drug Interaction", description = "Drug-Drug Interaction",
        customPersistentClass = DrugDrugInteractionVersionVa.class, fdbPersistentClass = FdbDdiminteraction.class)
public class DrugDrugInteractionDTO extends PecsDTO {

    private static final long serialVersionUID = -2692205115588732918L;
    /** String FDB_ID. */
    public static final String FDB_ID = "FDB_ID";
    /** String MonographId. */
    public static final String MONOGRAPHID = "MONOGRAPHID";
    /** String SeverityLevelCode. */
    public static final String SEVERITYLEVELCODE = "SEVERITYLEVELCODE";
    /** String Severity Interaction ID. */
    public static final String INTERACTIONID = "INTERACTIONID";
    /** String Description 1. */
    public static final String DESCRIPTION1 = "DESCRIPTION1";
    /** String Clinical Effect Code 1. */
    public static final String CLINICALEFFECTCODE1 = "CLINICALEFFECTCODE1";
    /** String Clinical effect code 2. */
    public static final String CLINICALEFFECTCODE2 = "CLINICALEFFECTCODE2";

    private long ddiminteractionVaId;

    @CompositeField
    private DdiminteractionReferenceVa ddiminteractionReferenceVa;
    @FieldInfo(
            name = "INTERACTIONID", editable = false, customPersistentPath = "versionedObject.interactionId", required = false,
            fdbPersistentPath = "interactionid", reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.INTERACTIONID_REPORT_ORDER)
    private long interactionid;
    private long seqno;
    @FieldInfo(
            name = "DESCRIPTION1", editable = true, customPersistentPath = "description", required = true,
            fdbPersistentPath = "description1", reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.DESCRIPTION1_REPORT_ORDER, reportHistoryCompare = true)
    private String description1;
    private String description2;
    private String description3;
    private String description4;
    private String description5;
    private String description6;
    /** The Constant REVERSE_FDB_ID. */
    public static final String REVERSE_FDB_ID = "REVERSE_FDB_ID";

    @FieldInfo(
            name = "SEVERITYLEVELCODE", editable = true, customPersistentPath = "severityLevel.code", required = true,
            fdbPersistentPath = "severitylevelcode", lookup = true, lookupType = SeverityLevel.class, reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.SEVERITYLEVELCODE_REPORT_ORDER, reportHistoryCompare = true)
    private String severitylevelcode;
    @FieldInfo(
            name = MONOGRAPHID, customPersistentPath = "monographId", fdbPersistentPath = "monographid", reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.MONOGRAPHID_REPORT_ORDER, reportHistoryCompare = true)
    private Long monographid;
    private String uicategory1;
    private String uicategory2;
    private String uicategory3;
    private String uicategory4;
    private String uicategory5;
    private String uicategory6;
    @FieldInfo(
            name = "CLINICALEFFECTCODE1", editable = true, customPersistentPath = "clinicalEffect1.code", required = true,
            fdbPersistentPath = "clinicaleffectcode1", lookup = true, lookupType = ClinicalEffect.class, reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.CLINICALEFFECTCODE1_REPORT_ORDER, reportHistoryCompare = true)
    private String clinicaleffectcode1;
    @FieldInfo(
            name = CLINICALEFFECTCODE2, customPersistentPath = "clinicalEffect2.code",
            fdbPersistentPath = "clinicaleffectcode2", reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.CLINICALEFFECTCODE2_REPORT_ORDER, reportHistoryCompare = true)
    private String clinicaleffectcode2;
    @FieldInfo(
            name = FDB_ID, editable = false, customPersistentPath = "versionedObject.fdbId", reportHistory = true,
            reportHistoryOrder = DrugDrugInteractionReportOrder.FDB_ID_REPORT_ORDER)
    private long fdbId;

    @FieldInfo(
            name = "DOCHUMANTRIALIND", hideFromDetailDisplay = true, fdbPersistentPath = "dochumantrialind",
            customPersistentPath = "", editable = false, required = false, internal = false)
    private long docHumanTrialInd;
    @FieldInfo(
            name = "DOCRVWARTICLEIND", hideFromDetailDisplay = true, fdbPersistentPath = "docrvwarticleind",
            customPersistentPath = "", editable = false, required = false, internal = false)
    private long docRvwArticleInd;
    @FieldInfo(
            name = "DOCMTGABSTRACTIND", hideFromDetailDisplay = true, fdbPersistentPath = "docmtgabstractind",
            customPersistentPath = "", editable = false, required = false, internal = false)
    private long docMtgAbstractInd;
    @FieldInfo(
            name = "EDIPAGEREF", hideFromDetailDisplay = true, fdbPersistentPath = "edipageref", customPersistentPath = "",
            editable = false, required = false, internal = false)
    private String ediPageRef;
    @FieldInfo(
            name = "DOCANIMALSTUDYIND", hideFromDetailDisplay = true, fdbPersistentPath = "docanimalstudyind",
            customPersistentPath = "", editable = false, required = false, internal = false)
    private long docAnimalStudyInd;
    @FieldInfo(
            name = "DOCMFGIND", hideFromDetailDisplay = true, fdbPersistentPath = "docmfgind", customPersistentPath = "",
            editable = false, required = false, internal = false)
    private long docMfgInd;
    @FieldInfo(
            name = "DOCCASERPTIND", hideFromDetailDisplay = true, fdbPersistentPath = "doccaserptind",
            customPersistentPath = "", editable = false, required = false, internal = false)
    private long docCaseRptInd;

    /** The reverse fdb id. */
    @FieldInfo(
            name = REVERSE_FDB_ID, internal = true, editable = false, customPersistentPath = "versionedObject.reverseFdbId",
            fdbPersistentPath = "")
    private long reverseFdbId;

    /**
     * Retrieves the reverse Fdb Id.
     * 
     * @return the reverseFdbId
     */
    public final long getReverseFdbId() {
        return reverseFdbId;
    }

    /**
     * Set's the Reverse FdbId.
     * 
     * @param reverseFdbId
     *            the reverseFdbId to set
     */
    public final void setReverseFdbId(long reverseFdbId) {
        this.reverseFdbId = reverseFdbId;
    }

    private boolean readOnly = false;
    private boolean duplicate = false;

    /** default constructor. */
    public DrugDrugInteractionDTO() {
    }

    /**
     * Get Select Label Value.
     * 
     * @return lv
     */
    @Override
    public final LabelValue getSelect() {
        StringBuffer sb = new StringBuffer();
        LabelValue lv = new LabelValue();
        sb.append("/ct/pecs/drugdrug/interaction/");
        // If the FdbCustomDdimVaId equal the FDB Id, assume this is a FDB Drug
        // Pair.
        if (getRequestor() == null) {
            sb.append("FDB/");
            sb.append(getFdbId());
            lv.setLabel(QueryResultLabel.OPEN);
        } else {
            sb.append("VA/");
            sb.append(getDdiminteractionVaId());
            if (isCurrent()) {
                lv.setLabel(QueryResultLabel.ACTIVE);
            } else {
                lv.setLabel(QueryResultLabel.HISTORICAL);
            }
        }
        lv.setValue(sb.toString());
        return lv;
    }

    /**
     * Getter DdiminteractionVaId.
     * 
     * @return ddiminteractionVaId
     */
    public final Long getDdiminteractionVaId() {
        return ddiminteractionVaId;
    }

    /**
     * Setter DdiminteractionVaId.
     * 
     * @param ddiminteractionVaId
     *            - Long
     * */
    public final void setDdiminteractionVaId(Long ddiminteractionVaId) {
        this.ddiminteractionVaId = ddiminteractionVaId;
    }

    /**
     * Getter for DdiminteractionReferenceVa.
     * 
     * @return ddiminteractionReferenceVa
     * */
    public final DdiminteractionReferenceVa getDdiminteractionReferenceVa() {
        return ddiminteractionReferenceVa;
    }

    /**
     * SetDbiminteractionReferenceVa.
     * 
     * @param ddiminteractionReferenceVa
     *            - DdiminteractionReferenceVa
     * */
    public final void setDdiminteractionReferenceVa(DdiminteractionReferenceVa ddiminteractionReferenceVa) {
        this.ddiminteractionReferenceVa = ddiminteractionReferenceVa;
    }

    /**
     * Getter for Interactionid.
     * 
     * @return interactionid
     * */
    public final long getInteractionid() {
        return interactionid;
    }

    /**
     * Setter for Interation Id.
     * 
     * @param interactionid
     *            - long
     * */
    public final void setInteractionid(long interactionid) {
        this.interactionid = interactionid;
    }

    /**
     * Getter Seqno.
     * 
     * @return seqno
     * */
    public final long getSeqno() {
        return seqno;
    }

    /**
     * Setter Seqno.
     * 
     * @param seqno
     *            - long
     * */
    public final void setSeqno(long seqno) {
        this.seqno = seqno;
    }

    /**
     * Getter Description1.
     * 
     * @return desctiption1
     * */
    public final String getDescription1() {
        return description1;
    }

    /**
     * Setter Description 1.
     * 
     * @param description1
     *            - String
     * */
    public final void setDescription1(String description1) {
        this.description1 = description1;
    }

    /**
     * Getter Description2.
     * 
     * @return description2
     * 
     * */
    public final String getDescription2() {
        return description2;
    }

    /**
     * Setter Description2.
     * 
     * @param description2
     *            - String
     * */
    public final void setDescription2(String description2) {
        this.description2 = description2;
    }

    /**
     * Getter Description3.
     * 
     * @return description3
     * 
     * */
    public final String getDescription3() {
        return description3;
    }

    /**
     * Setter Description 3.
     * 
     * @param description3
     *            - String
     * */
    public final void setDescription3(String description3) {
        this.description3 = description3;
    }

    /**
     * Getter Description 4.
     * 
     * @return description4
     * 
     * */
    public final String getDescription4() {
        return description4;
    }

    /**
     * Setter Description 4.
     * 
     * @param description4
     *            - String
     * */
    public final void setDescription4(String description4) {
        this.description4 = description4;
    }

    /**
     * Getter Description5.
     * 
     * @return description5
     * 
     * */
    public final String getDescription5() {
        return description5;
    }

    /**
     * Setter Description5.
     * 
     * @param description5
     *            - String
     * 
     * */
    public final void setDescription5(String description5) {
        this.description5 = description5;
    }

    /**
     * Getter Description 6.
     * 
     * @return description6
     * 
     * 
     * */
    public final String getDescription6() {
        return description6;
    }

    /**
     * Setter Description6.
     * 
     * @param description6
     *            -String
     * */
    public final void setDescription6(String description6) {
        this.description6 = description6;
    }

    /**
     * Getter Severity Level Code.
     * 
     * @return severitylevelcode
     * */
    public final String getSeveritylevelcode() {
        return severitylevelcode;
    }

    /**
     * Setter Severity Level Code.
     * 
     * @param severitylevelcode
     *            - String
     * */
    public final void setSeveritylevelcode(String severitylevelcode) {
        this.severitylevelcode = severitylevelcode;
    }

    /*
     * public String getSeveritylevelForRpt() { return severitylevelForRpt; }
     * 
     * public void setSeveritylevelForRpt(String severitylevelForRpt) { this.severitylevelForRpt = severitylevelForRpt; }
     */

    /**
     * Getter Monographid.
     * 
     * @return monographid
     * 
     * */
    public final Long getMonographid() {
        return monographid;
    }

    /**
     * Setter Monographid.
     * 
     * @param monographid
     *            - Long
     */
    public final void setMonographid(Long monographid) {
        this.monographid = monographid;
    }

    /**
     * Getter Uicategory1.
     * 
     * @return uicategory1
     */
    public final String getUicategory1() {
        return uicategory1;
    }

    /**
     * Setter Uicategory1.
     * 
     * @param uicategory1
     *            - String
     */
    public final void setUicategory1(String uicategory1) {
        this.uicategory1 = uicategory1;
    }

    /**
     * Getter Uicategory2.
     * 
     * @return uicategory2
     */
    public final String getUicategory2() {
        return uicategory2;
    }

    /**
     * Setter Uicategory2.
     * 
     * @param uicategory2
     *            - String
     * 
     * */
    public final void setUicategory2(String uicategory2) {
        this.uicategory2 = uicategory2;
    }

    /**
     * Getter Uicategory3.
     * 
     * @return uicategory3
     * */
    public final String getUicategory3() {
        return uicategory3;
    }

    /**
     * Setter Uicategory3.
     * 
     * @param uicategory3
     *            - String
     * */
    public final void setUicategory3(String uicategory3) {
        this.uicategory3 = uicategory3;
    }

    /**
     * Getter Uicatery4.
     * 
     * @return uicategory4
     * */
    public final String getUicategory4() {
        return uicategory4;
    }

    /**
     * Setter Uicategory4.
     * 
     * @param uicategory4
     *            - String
     * 
     * */
    public final void setUicategory4(String uicategory4) {
        this.uicategory4 = uicategory4;
    }

    /**
     * Getter Uicategory5.
     * 
     * 
     * @return uicategory5
     */
    public final String getUicategory5() {
        return uicategory5;
    }

    /**
     * Setter Uicategory5.
     * 
     * @param uicategory5
     *            - String
     */
    public final void setUicategory5(String uicategory5) {
        this.uicategory5 = uicategory5;
    }

    /**
     * Getter Uicategory6.
     * 
     * @return uicategory6
     */
    public final String getUicategory6() {
        return uicategory6;
    }

    /**
     * Setter Uicategory6.
     * 
     * @param uicategory6
     *            - String
     */
    public final void setUicategory6(String uicategory6) {
        this.uicategory6 = uicategory6;
    }

    /**
     * Getter ClinicaleffectCode1.
     * 
     * @return ClinicaleffectCode1
     */
    public final String getClinicaleffectcode1() {
        return clinicaleffectcode1;
    }

    /**
     * Setter Clinicaleffectcode1.
     * 
     * @param clinicaleffectcode1
     *            - String
     */
    public final void setClinicaleffectcode1(String clinicaleffectcode1) {
        this.clinicaleffectcode1 = clinicaleffectcode1;
    }

    /**
     * Getter Clinicaleffectcode2.
     * 
     * @return clinicaleffectcode2
     */
    public final String getClinicaleffectcode2() {
        return clinicaleffectcode2;
    }

    /**
     * Setter Clinicaleffectcode2.
     * 
     * @param clinicaleffectcode2
     *            - String
     */
    public final void setClinicaleffectcode2(String clinicaleffectcode2) {
        this.clinicaleffectcode2 = clinicaleffectcode2;
    }

    /**
     * Getter FdbId.
     * 
     * @return fdbId
     */
    public final long getFdbId() {
        return fdbId;
    }

    /**
     * Setter FdbId.
     * 
     * @param fdbId
     *            - Long
     */
    public final void setFdbId(long fdbId) {
        this.fdbId = fdbId;
    }

    /**
     * Getter DocHumanTrialInd.
     * 
     * @return docHumanTialInd.
     */
    public final long getDocHumanTrialInd() {
        return docHumanTrialInd;
    }

    /**
     * Setter DocHumanTrialInd.
     * 
     * @param docHumanTrialInd
     *            - Long
     */
    public final void setDocHumanTrialInd(long docHumanTrialInd) {
        this.docHumanTrialInd = docHumanTrialInd;
    }

    /**
     * Getter DocRvwArticleInd.
     * 
     * @return docRvwArticleInd
     */
    public final long getDocRvwArticleInd() {
        return docRvwArticleInd;
    }

    /**
     * Setter DocRvwArticleInd.
     * 
     * @param docRvwArticleInd
     *            - Long
     */
    public final void setDocRvwArticleInd(long docRvwArticleInd) {
        this.docRvwArticleInd = docRvwArticleInd;
    }

    /**
     * Getter DocMtgAbstractInd.
     * 
     * @return docMtgAbstractInd
     */
    public final long getDocMtgAbstractInd() {
        return docMtgAbstractInd;
    }

    /**
     * Setter DocMtgAbstractInd.
     * 
     * @param docMtgAbstractInd
     *            - Long
     */
    public final void setDocMtgAbstractInd(long docMtgAbstractInd) {
        this.docMtgAbstractInd = docMtgAbstractInd;
    }

    /**
     * Getter EdiPageRef.
     * 
     * @return ediPageRef
     */
    public final String getEdiPageRef() {
        return ediPageRef;
    }

    /**
     * Set EdiPageRef.
     * 
     * @param ediPageRef
     *            -String
     */
    public final void setEdiPageRef(String ediPageRef) {
        this.ediPageRef = ediPageRef;
    }

    /**
     * Getter DocAnimalStudyInd.
     * 
     * @return docAnimalStudyInd
     */
    public final long getDocAnimalStudyInd() {
        return docAnimalStudyInd;
    }

    /**
     * Setter DocAnimalStudyInd.
     * 
     * @param docAnimalStudyInd
     *            - long
     */
    public final void setDocAnimalStudyInd(long docAnimalStudyInd) {
        this.docAnimalStudyInd = docAnimalStudyInd;
    }

    /**
     * Getter DocMfgInd.
     * 
     * @return docMfgInd
     */
    public final long getDocMfgInd() {
        return docMfgInd;
    }

    /**
     * Setter DocMfgInd.
     * 
     * @param docMfgInd
     *            long
     */
    public final void setDocMfgInd(long docMfgInd) {
        this.docMfgInd = docMfgInd;
    }

    /**
     * Getter DocCaseRptInd.
     * 
     * @return docCaseRptInd
     */
    public final long getDocCaseRptInd() {
        return docCaseRptInd;
    }

    /**
     * Setter DocCaseRptInd.
     * 
     * @param docCaseRptInd
     *            - long
     */
    public final void setDocCaseRptInd(long docCaseRptInd) {
        this.docCaseRptInd = docCaseRptInd;
    }

    /**
     * Getter Read Only.
     * 
     * @return readOnly.
     */
    public final boolean isReadOnly() {
        return readOnly;
    }

    /**
     * Setter ReadOnly.
     * 
     * @param readOnly
     *            - boolean
     */
    public final void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;
    }

    @Override
    public final String toString() {
        return "[VA ddi " + getInteractionid() + "]";
    }

    /**
     * Generate RecordId.
     * 
     * @return generateRecordId.
     */
    public final String generateRecordId() {
        return (isCurrent() ? "C" : "H") + getDdiminteractionVaId();
    }

    /**
     * Getter isDuplicate.
     * 
     * @return duplicate
     */
    public final boolean isDuplicate() {
        return duplicate;
    }

    /**
     * Setter Duplicate.
     * 
     * @param duplicate
     *            - boolean
     */
    public final void setDuplicate(boolean duplicate) {
        this.duplicate = duplicate;
    }

    /**
     * isDrugPairButtonViewable.
     * 
     * @return boolean
     */
    public final boolean isDrugPairButtonViewable() {
        if (getActionUser() != null) {
            if (getActionUser().isRequestor() || getActionUser().isApprover()) {
                return true;
            }
        }
        return false;
    }
}
