package gov.va.med.pharmacy.pecs.dto;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdimdruglink;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVersionVa;
import gov.va.med.pharmacy.pecs.metadata.ClassInfo;
import gov.va.med.pharmacy.pecs.metadata.FieldInfo;
import gov.va.med.pharmacy.pecs.util.LabelValue;
import gov.va.med.pharmacy.pecs.util.QueryResultLabel;

import java.util.Date;

/**
 * Drug pair DTO.
 */
@ClassInfo(
        name = "Drug Pair", description = "Drug pair", customPersistentClass = DrugPairVersionVa.class,
        fdbPersistentClass = FdbDdimdruglink.class)
public final class DrugPairDTO extends PecsDTO {

    private static final int DISPLAY_FIVE = 5;

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 6191621014722840266L;

    /** The Constant REFERENCE_TEXT. */
    public static final String REFERENCE_TEXT = "REFERENCE_TEXT";

    /** The Constant RTGENID1. */
    public static final String RTGENID1 = "RTGENID1";

    /** The Constant RTGENID2. */
    public static final String RTGENID2 = "RTGENID2";

    /** The Constant INTERACTIONID. */
    public static final String INTERACTIONID = "INTERACTIONID";

    /** The Constant FDB_ID. */
    public static final String FDB_ID = "FDB_ID";

    /** The Constant RTGENID1_DESC. */
    public static final String RTGENID1_DESC = "RTGENID1_DESC";

    /** The Constant RTGENID2_DESC. */
    public static final String RTGENID2_DESC = "RTGENID2_DESC";

    /** The Constant INTERACTIONID_DESC. */
    public static final String INTERACTIONID_DESC = "INTERACTIONID_DESC";

    /** The Constant SEVERITYLEVELCODE. */
    public static final String SEVERITYLEVELCODE = "SEVERITYLEVELCODE";

    /** The Constant SEVERITYLEVELCODE_DESC. */
    public static final String SEVERITYLEVELCODE_DESC = "SEVERITYLEVELCODE_DESC";
    /** The Constant REVERSE_FDB_ID. */
    public static final String REVERSE_FDB_ID = "REVERSE_FDB_ID";

    /** The read only record. */
    private boolean readOnlyRecord;

    /** The fdb custom ddim va id. */
    private Long fdbCustomDdimVaId;

    /** The reference text. */
    @FieldInfo(name = REFERENCE_TEXT, internal = true, displayLines = DISPLAY_FIVE, customPersistentPath = "referenceText")
    private String referenceText;

    /** The rtgenid1. */
    @FieldInfo(
            name = RTGENID1, internal = false, required = true, lookup = true, lookupMethod = "getGenericRoutedDrugs",
            customPersistentPath = "versionedObject.routedGeneric1.rtGenId", fdbPersistentPath = "id.rtgenid1")
    private Long rtgenid1;

    /** The rtgenid2. */
    @FieldInfo(
            name = RTGENID2, internal = false, required = true, lookup = true, lookupMethod = "getGenericRoutedDrugs",
            customPersistentPath = "versionedObject.routedGeneric2.rtGenId", fdbPersistentPath = "id.rtgenid2")
    private Long rtgenid2;

    /** The interactionid. */
    @FieldInfo(
            name = INTERACTIONID, internal = false, required = true, lookup = true, lookupMethod = "getCurrentStagingData",
            customPersistentPath = "versionedObject.interaction.interactionId", fdbPersistentPath = "id.interactionid")
    private Long interactionid;

    /** The uicategory1. */
    private String uicategory1;

    /** The uicategory2. */
    private String uicategory2;

    /** The uicategory3. */
    private String uicategory3;

    /** The uicategory4. */
    private String uicategory4;

    /** The uicategory5. */
    private String uicategory5;

    /** The uicategory6. */
    @FieldInfo(name = "UICATEGORY6", internal = false, editable = false, customPersistentPath = "", fdbPersistentPath = "")
    private String uicategory6;

    /** The fdb id. */
    @FieldInfo(name = FDB_ID, internal = true, editable = false, customPersistentPath = "versionedObject.interaction.fdbId")
    private Long fdbId;

    /** The rtgenid1 desc. */
    @FieldInfo(
            name = RTGENID1_DESC, internal = true, editable = false,
            customPersistentPath = "versionedObject.routedGeneric1.description", fdbPersistentPath = "rtgenid1Desc")
    // change when FDB DP gets properly mapped
    private String rtgenid1Desc = null;

    /** The rtgenid2 desc. */
    @FieldInfo(
            name = RTGENID2_DESC, internal = true, editable = false,
            customPersistentPath = "versionedObject.routedGeneric2.description", fdbPersistentPath = "rtgenid2Desc")
    // change when FDB DP gets properly mapped
    private String rtgenid2Desc = null;

    /** The interactionid desc. */
    @FieldInfo(
            name = INTERACTIONID_DESC, internal = true, editable = false,
            customPersistentPath = "versionedObject.interaction.exportManager.currentVersion.description",
            fdbPersistentPath = "interactionidDesc")
    // change when FDB DP gets properly mapped
    private String interactionidDesc = null;

    /** The severitylevelcode. */
    @FieldInfo(
            name = SEVERITYLEVELCODE, editable = false,
            customPersistentPath = "versionedObject.interaction.exportManager.currentVersion.severityLevel.code",
            fdbPersistentPath = "severitylevelcode")
    // change when FDB DP gets properly mapped
    private String severitylevelcode = null;

    /** The severitylevelcode desc. */
    @FieldInfo(
            name = SEVERITYLEVELCODE_DESC, internal = true, editable = false,
            customPersistentPath = "versionedObject.interaction.exportManager.currentVersion.severityLevel.description",
            fdbPersistentPath = "severitylevelcodeDesc")
    // change when FDB DP get properly mapped
    private String severitylevelcodeDesc = null;
    /** The reverse FDB Id. */
    @FieldInfo(
            name = REVERSE_FDB_ID, internal = true, editable = false,
            customPersistentPath = "versionedObject.interaction.reverseFdbId", fdbPersistentPath = "")
    private Long reverseFdbId;

    /**
     * Retrieves the reverse Fdb Id.
     * 
     * @return the reverseFdbId
     */
    public final Long getReverseFdbId() {
        return reverseFdbId;
    }

    /**
     * Set's the Reverse FdbId.
     * 
     * @param reverseFdbId
     *            the reverseFdbId to set
     */
    public final void setReverseFdbId(Long reverseFdbId) {
        this.reverseFdbId = reverseFdbId;
    }

    /** The export date */
    @FieldInfo(
            name = EXPORT_DATE, reportHistory = true, reportHistoryOrder = EXPORT_DATE_REPORT_ORDER, editable = false,
            customPersistentPath = "exportDate", nonReportable = true, internal = true)
    private Date exportDate;

    /**
     * Retrives the Export Date.
     * 
     * @return the exportDate
     */
    public Date getExportDate() {
        return exportDate;
    }

    /**
     * Set the Export Date in the DTO.
     * 
     * @param exportDate
     *            the exportDate to set
     */
    public void setExportDate(Date exportDate) {
        this.exportDate = exportDate;
    }

    /** The drug pair id. */
    private Long drugPairId;

    /** default constructor. */
    public DrugPairDTO() {
    }

    /**
     * Get FdbCustomDdimVaId.
     * 
     * @return fdbCustomDdimVaId
     */
    public Long getFdbCustomDdimVaId() {
        return this.fdbCustomDdimVaId;
    }

    /**
     * Set FdbCustomDdimVaId.
     * 
     * @param fdbCustomDdimVaId
     *            - Long
     */
    public void setFdbCustomDdimVaId(Long fdbCustomDdimVaId) {
        this.fdbCustomDdimVaId = fdbCustomDdimVaId;
    }

    /**
     * Get ReferenceText.
     * 
     * @return referenctText
     */
    public String getReferenceText() {
        return this.referenceText;
    }

    /**
     * Set ReferenceText.
     * 
     * @param referenceText
     *            - String
     */
    public void setReferenceText(String referenceText) {
        this.referenceText = referenceText;
    }

    /**
     * Get Rtgenid1.
     * 
     * @return rtgenid1
     */
    public Long getRtgenid1() {
        return this.rtgenid1;
    }

    /**
     * Set Rtgenid1.
     * 
     * @param rtgenid1
     *            - Long
     */
    public void setRtgenid1(Long rtgenid1) {
        this.rtgenid1 = rtgenid1;
    }

    /**
     * Get Rtgenid2.
     * 
     * @return rtgenid2
     */
    public Long getRtgenid2() {
        return this.rtgenid2;
    }

    /**
     * Set Rtgenid2.
     * 
     * @param rtgenid2
     *            - Long
     */
    public void setRtgenid2(Long rtgenid2) {
        this.rtgenid2 = rtgenid2;
    }

    /**
     * Get Interaction Id.
     * 
     * @return interactionid
     */
    public Long getInteractionid() {
        return this.interactionid;
    }

    /**
     * Set Interactionid.
     * 
     * @param interactionid
     *            - Long
     */
    public void setInteractionid(Long interactionid) {
        this.interactionid = interactionid;
    }

    /**
     * Get Uicategory1.
     * 
     * @return uicategory1
     */
    public String getUicategory1() {
        return this.uicategory1;
    }

    /**
     * Set Uicategory1.
     * 
     * @param uicategory1
     *            - String
     */
    public void setUicategory1(String uicategory1) {
        this.uicategory1 = uicategory1;
    }

    /**
     * Get Uicategory2.
     * 
     * @return uicategory2
     */
    public String getUicategory2() {
        return this.uicategory2;
    }

    /**
     * Set Uicategory2.
     * 
     * @param uicategory2
     *            - String
     */
    public void setUicategory2(String uicategory2) {
        this.uicategory2 = uicategory2;
    }

    /**
     * Get Uicategory3.
     * 
     * @return uicategory3
     */
    public String getUicategory3() {
        return this.uicategory3;
    }

    /**
     * Set Uicategory3.
     * 
     * @param uicategory3
     *            - String
     */
    public void setUicategory3(String uicategory3) {
        this.uicategory3 = uicategory3;
    }

    /**
     * Get Uicategory4.
     * 
     * @return uicategory4
     */
    public String getUicategory4() {
        return this.uicategory4;
    }

    /**
     * Set Uicategory4.
     * 
     * @param uicategory4
     *            - String
     */
    public void setUicategory4(String uicategory4) {
        this.uicategory4 = uicategory4;
    }

    /**
     * Get Uicategory5.
     * 
     * @return uicategory5
     */
    public String getUicategory5() {
        return this.uicategory5;
    }

    /**
     * Set Uicategory5.
     * 
     * @param uicategory5
     *            - String
     */
    public void setUicategory5(String uicategory5) {
        this.uicategory5 = uicategory5;
    }

    /**
     * Get Uicategory6.
     * 
     * @return uicategory6
     */
    public String getUicategory6() {
        return this.uicategory6;
    }

    /**
     * Set Uicategory6.
     * 
     * @param uicategory6
     *            - String
     */
    public void setUicategory6(String uicategory6) {
        this.uicategory6 = uicategory6;
    }

    /**
     * Get FDBID.
     * 
     * @return fdbId
     */
    public Long getFdbId() {
        return this.fdbId;
    }

    /**
     * Set FdbID.
     * 
     * @param fdbId
     *            - Long
     */
    public void setFdbId(Long fdbId) {
        this.fdbId = fdbId;
    }

    /**
     * Get Rtgenid1Desc.
     * 
     * @return rtgenid1Desc
     */
    public String getRtgenid1Desc() {
        return rtgenid1Desc;
    }

    /**
     * Set Rtgenid1Desc.
     * 
     * @param rtgenid1Desc
     *            - String
     */
    public void setRtgenid1Desc(String rtgenid1Desc) {
        this.rtgenid1Desc = rtgenid1Desc;
    }

    /**
     * Get Rtgenid2Desc.
     * 
     * @return rtgenid2Desc
     */
    public String getRtgenid2Desc() {
        return rtgenid2Desc;
    }

    /**
     * Set Rtgenid2Desc.
     * 
     * @param rtgenid2Desc
     *            - String
     */
    public void setRtgenid2Desc(String rtgenid2Desc) {
        this.rtgenid2Desc = rtgenid2Desc;
    }

    /**
     * Get Interaction Id Desc.
     * 
     * @return interactionidDesc
     */
    public String getInteractionidDesc() {
        return interactionidDesc;
    }

    /**
     * Set Interaction Id Desc.
     * 
     * @param interactionidDesc
     *            - String
     */
    public void setInteractionidDesc(String interactionidDesc) {
        this.interactionidDesc = interactionidDesc;
    }

    /**
     * Get Severity Level Code.
     * 
     * @return severitylevelcode
     */
    public String getSeveritylevelcode() {
        return severitylevelcode;
    }

    /**
     * Set Severity Level Code.
     * 
     * @param severitylevelcode
     *            - String
     */
    public void setSeveritylevelcode(String severitylevelcode) {
        this.severitylevelcode = severitylevelcode;
    }

    /**
     * Get SeverityLevel Code Desc.
     * 
     * @return severitylevelcodeDesc
     */
    public String getSeveritylevelcodeDesc() {
        return severitylevelcodeDesc;
    }

    /**
     * Set SeverityLevelCodeDesc.
     * 
     * @param severitylevelcodeDesc
     *            - String
     */
    public void setSeveritylevelcodeDesc(String severitylevelcodeDesc) {
        this.severitylevelcodeDesc = severitylevelcodeDesc;
    }

    /**
     * Deprecated generate Record.
     * 
     * @return the string
     */
    @Deprecated
    public String generateRecordId() {
        return (isCurrent() ? "C" : "H") + getFdbCustomDdimVaId();
    }

    /**
     * Set DrugPairId.
     * 
     * @param drugPairId
     *            - Long
     */
    public void setDrugPairId(Long drugPairId) {
        this.drugPairId = drugPairId;
    }

    /**
     * Get DrugPairId.
     * 
     * @return drugPairId
     */
    public Long getDrugPairId() {
        return drugPairId;
    }

    /**
     * Boolean Check to see if it is readable.
     * 
     * @return readOnlyRecord.
     */
    public boolean isReadOnlyRecord() {
        return readOnlyRecord;
    }

    /**
     * Set ReadOnlyRecord.
     * 
     * @param readOnlyRecord
     *            - boolean
     */
    public void setReadOnlyRecord(boolean readOnlyRecord) {
        this.readOnlyRecord = readOnlyRecord;
    }

    /**
     * Returns a LabelValue object with the URL and label to display for the Anchor in a results grid.
     * 
     * @return the select
     */
    @Override
    public LabelValue getSelect() {
        StringBuffer sb = new StringBuffer();
        LabelValue lv = new LabelValue();
        sb.append("/ct/pecs/drugPairDetail/");
        // If the FdbCustomDdimVaId equal the FDB Id, assume this is a FDB Drug
        // Pair.

        if (getFdbCustomDdimVaId() == getFdbId()) {
            sb.append("FDB/");
            sb.append(getInteractionid() + "|" + getRtgenid1() + "|" + getRtgenid2());
            lv.setLabel(QueryResultLabel.OPEN);
        } else {
            sb.append("VA/");
            sb.append(getFdbCustomDdimVaId());
            if (isCurrent()) {
                lv.setLabel(QueryResultLabel.ACTIVE);
            } else {
                lv.setLabel(QueryResultLabel.HISTORICAL);
            }
        }
        lv.setValue(sb.toString());
        return lv;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public final int hashCode() {
        final int prime = 31;
        final int bits = 32;
        int result = 1;
        result = prime * result + (int) (rtgenid1 ^ (rtgenid1 >>> bits));
        result = prime * result + (int) (rtgenid2 ^ (rtgenid2 >>> bits));
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public final boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        DrugPairDTO other = (DrugPairDTO) obj;
        if (rtgenid1 != null) {
            if (!rtgenid1.equals(other.rtgenid1))
                return false;
        }
        if (rtgenid2 != null) {
            if (!rtgenid2.equals(other.rtgenid2))
                return false;
        }
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "[VA drug pair (" + getRtgenid1() + ", " + getRtgenid2() + ") for ddi " + getInteractionid()
                + " with Severity Level Code " + getSeveritylevelcode() + "]";
    }
}
