/**
 * 
 */
package gov.va.med.pharmacy.pecs.dto;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbMonographDdim;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbMonographDdimId;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbMonographDdimLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographXref;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.ClassInfo;
import gov.va.med.pharmacy.pecs.metadata.FieldInfo;
import gov.va.med.pharmacy.pecs.util.LabelValue;
import gov.va.med.pharmacy.pecs.util.QueryResultLabel;
import gov.va.med.pharmacy.pecs.util.historyofchange.ProfessionalMonographReportOrder;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

/**
 * MonographDTO entity.
 * 
 * @author susarlan
 * 
 */
@ClassInfo(
        name = "Professional Monograph", description = "Professional Monograph",
        customPersistentClass = MonographVersion.class, fdbPersistentClass = FdbMonographDdimLite.class)
public final class MonographDTO extends PecsDTO implements java.io.Serializable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -8121228204688142223L;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(MonographDTO.class);

    /** The Constant PROFESSIONAL_VERSION_CODE. */
    private static final String PROFESSIONAL_VERSION_CODE = "FDB-P";

    /** The va monograph disclaimer. */
    private static String VA_MONOGRAPH_DISCLAIMER =
            "The information contained in this monograph is intended to supplement the knowledge of "
                    + "physicians, pharmacists, and other healthcare professionals regarding drug therapy problems "
                    + "and patient counseling information."
                    + "This information is advisory only and is not intended to replace "
                    + "sound clinical judgment in the delivery of healthcare services.\n";

    /** The Constant MONOGRAPH_TITLE_CODE. */
    private static final String MONOGRAPH_TITLE_CODE = "T";

    /** The Constant MECHANISM_OF_ACTION_CODE. */
    private static final String MECHANISM_OF_ACTION_CODE = "A";

    /** The Constant DISCUSSION_CODE. */
    private static final String DISCUSSION_CODE = "D";

    /** The Constant CLINICAL_EFFECTS_CODE. */
    private static final String CLINICAL_EFFECTS_CODE = "E";

    /** The Constant SEVERITY_LEVEL_CODE. */
    private static final String SEVERITY_LEVEL_CODE = "L";

    /** The Constant PATIENT_MANAGEMENT_CODE. */
    private static final String PATIENT_MANAGEMENT_CODE = "M";

    /** The Constant PREDISPOSING_FACTORS_CODE. */
    private static final String PREDISPOSING_FACTORS_CODE = "P";

    /** The Constant REFERENCE_CODE. */
    private static final String REFERENCE_CODE = "R";

    /** The Constant DISCLAIMER_CODE. */
    private static final String DISCLAIMER_CODE = "Z";

    /** The Constant MONOGRAPH_TITLE_LABEL. */
    private static final String MONOGRAPH_TITLE_LABEL = "MONOGRAPH TITLE:  ";

    /** The Constant MECHANISM_OF_ACTION_LABEL. */
    private static final String MECHANISM_OF_ACTION_LABEL = "MECHANISM OF ACTION:  ";

    /** The Constant DISCUSSION_LABEL. */
    private static final String DISCUSSION_LABEL = "DISCUSSION:  ";

    /** The Constant CLINICAL_EFFECTS_LABEL. */
    private static final String CLINICAL_EFFECTS_LABEL = "CLINICAL EFFECTS:  ";

    /** The Constant SEVERITY_LEVEL_LABEL. */
    private static final String SEVERITY_LEVEL_LABEL = "SEVERITY LEVEL:  ";

    /** The Constant PATIENT_MANAGEMENT_LABEL. */
    private static final String PATIENT_MANAGEMENT_LABEL = "PATIENT MANAGEMENT:  ";

    /** The Constant PREDISPOSING_FACTORS_LABEL. */
    private static final String PREDISPOSING_FACTORS_LABEL = "PREDISPOSING FACTORS:  ";

    /** The Constant REFERENCE_LABEL. */
    private static final String REFERENCE_LABEL = "REFERENCES:\n";

    /** The Constant DISCLAIMER_LABEL. */
    private static final String DISCLAIMER_LABEL = "DISCLAIMER:  ";

    /**
     * Constant String VA_MONOGRAPH_ID.
     * 
     */
    public static final String VA_MONOGRAPH_ID = "VA_MONOGRAPH_ID";

    /**
     * Constant String FDB_ID.
     * 
     */
    public static final String FDB_ID = "FDB_ID";

    /** The Constant LINENUMBERS5. */
    private static final int LINENUMBERS5 = 5;

    /** The Constant LINENUMBERS10. */
    private static final int LINENUMBERS10 = 10;
    /**
     * A static variable to the Reference Text field.
     */
    public static final String REFERENCE_TEXT = "REFERENCE_TEXT";

    /** The fdb custom monograph va id. */
    private long fdbCustomMonographVaId;

    /** The reference text. */
    @FieldInfo(
            name = "REFERENCE_TEXT", editable = true, customPersistentPath = "referenceText", required = false,
            internal = true, displayLines = LINENUMBERS5, reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.REFERENCE_TEXT_HISTORY_REPORT_ORDER,
            reportHistoryCompare = true)
    private String referenceText;

    /** The languageid. */
    @FieldInfo(
            name = "LANGUAGEID", editable = false, customPersistentPath = "versionedObject.languageId", required = false,
            internal = false, fdbPersistentPath = "id.languageid")
    private long languageid = 1;

    /** The versioncode. */
    private String versioncode;

    /** The categorycode. */
    private String categorycode;

    /** The monographid. */
    @FieldInfo(
            name = "VA_MONOGRAPH_ID", editable = false, customPersistentPath = "versionedObject.id",
            fdbPersistentPath = "id.monographid", required = false, internal = false, reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.VA_MONOGRPAH_ID_REPORT_ORDER, reportHistoryCompare = false)
    private long monographid;

    /** The fdb id. */
    @FieldInfo(
            name = "FDB_ID", editable = false, customPersistentPath = "versionedObject.customizedMonographId",
            required = false, internal = true, reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.FDB_ID_REPORT_ORDER, reportHistoryCompare = false)
    private long fdbId;

    /** The monograph title. */
    @FieldInfo(
            name = "TITLE", editable = true, customPersistentPath = "title", fdbPersistentPath = "linetext", required = true,
            internal = false, displayLines = 2, reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.TITLE_REPORT_ORDER, reportHistoryCompare = true)
    private String monographTitle;

    /** The monograph display title. */
    private String monographDisplayTitle;

    /** The mechanism of action. */
    @FieldInfo(
            name = "MECHANISM_OF_ACTION", editable = true, customPersistentPath = "", required = false, internal = false,
            displayLines = 2, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.MECHANISM_OF_ACTION_REPORT_ORDER, reportHistoryCompare = true)
    private String mechanismOfAction;

    /** The discussion. */
    @FieldInfo(
            name = "DISCUSSION", editable = true, customPersistentPath = "", required = false, internal = false,
            displayLines = LINENUMBERS5, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.DISCUSSION_REPORT_ORDER, reportHistoryCompare = true)
    private String discussion;

    /** The clinical effects. */
    @FieldInfo(
            name = "CLINICAL_EFFECTS", editable = true, customPersistentPath = "clinicalEffects", required = true,
            internal = false, displayLines = LINENUMBERS5, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.CLINICAL_EFFECTS_REPORT_ORDER, reportHistoryCompare = true)
    private String clinicalEffects;

    /** The severity level. */
    @FieldInfo(
            name = "SEVERITY_LEVEL", editable = true, customPersistentPath = "severityLevel", required = true,
            internal = false, displayLines = LINENUMBERS5, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.SEVERITY_LEVEL_REPORT_ORDER, reportHistoryCompare = true)
    private String severityLevel;

    /** The patient management. */
    @FieldInfo(
            name = "PATIENT_MANAGEMENT", editable = true, customPersistentPath = "", required = false, internal = false,
            displayLines = LINENUMBERS5, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.PATIENT_MANAGEMENT_REPORT_ORDER, reportHistoryCompare = true)
    private String patientManagement;

    /** The predisposing factors. */
    @FieldInfo(
            name = "PREDISPOSING_FACTORS", editable = true, customPersistentPath = "", required = false, internal = false,
            displayLines = LINENUMBERS5, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.PREDISPOSING_FACTORS_REPORT_ORDER,
            reportHistoryCompare = true)
    private String predisposingFactors;

    /** The reference. */
    @FieldInfo(
            name = "REFERENCE", editable = true, customPersistentPath = "", required = false, internal = false,
            displayLines = LINENUMBERS5, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.REFERENCE_REPORT_ORDER, reportHistoryCompare = true)
    private String reference;

    /** The disclaimer. */
    @FieldInfo(
            name = "DISCLAIMER", editable = false, customPersistentPath = "", required = false, internal = false,
            displayLines = LINENUMBERS10, fdbPersistentPath = "", reportHistory = true,
            reportHistoryOrder = ProfessionalMonographReportOrder.DISCLAIMER_REPORT_ORDER, reportHistoryCompare = true)
    private String disclaimer;
    /**
     * Protected Boolean Object.
     * 
     */
    private boolean readOnlyRecord;

    /** The record id. */
    private String recordId;

    /** Basic Constructor. */
    public MonographDTO() {
    }

    /**
     * Checks to see if the FDB Monograph is null or new status.
     * 
     * @param fdbMonographs
     *            - List<FdbMonographDdim>
     * @return null or Monograph VA
     */
    public static MonographDTO createFromFdb(List<FdbMonographDdim> fdbMonographs) {

        if (fdbMonographs == null || fdbMonographs.size() == 0)
            return null;

        FdbMonographDdimId monoId = null;
        MonographXref xref = null;
        Map<String, ArrayList<FdbMonographDdim>> monographSections = new HashMap<String, ArrayList<FdbMonographDdim>>();
        Map<Long, String> sectionSequences = new HashMap<Long, String>();
        for (FdbMonographDdim fdbMonograph : fdbMonographs) {

            FdbMonographDdimId currId = fdbMonograph.getId();
            if (monoId == null) {
                monoId = currId;
            } else {
                if (!currId.getMonographid().equals(monoId.getMonographid())) {
                    LOGGER.error("Sepcified fdb monograph list has differing ids. Looking for '" + monoId.getMonographid()
                            + "' found '" + currId.getMonographid() + "'");
                    throw new CustomizationServiceException("Specified fdb monogrpah list has different ids");
                }
            }

            String sectCode = fdbMonograph.getSectioncode();
            if (monographSections.containsKey(sectCode)) {
                ArrayList<FdbMonographDdim> sectionData = monographSections.get(sectCode);
                int sequenceNumber = currId.getSequencenumber().intValue();
                int size = sectionData.size();
                boolean valAdded = false;
                for (int i = 0; i < size; i++) {
                    FdbMonographDdim listMonograph = sectionData.get(i);
                    if (sequenceNumber <= listMonograph.getId().getSequencenumber().intValue()) {
                        sectionData.add(i, fdbMonograph);
                        valAdded = true;
                        break;
                    }
                }
                if (!valAdded) {
                    sectionData.add(fdbMonograph);
                }
                monographSections.put(sectCode, sectionData);
            } else {
                ArrayList<FdbMonographDdim> sectionData = new ArrayList<FdbMonographDdim>();
                sectionData.add(fdbMonograph);
                monographSections.put(sectCode, sectionData);
                sectionSequences.put(fdbMonograph.getId().getSequencenumber(), sectCode);

            }
            if (xref == null) {
                xref = (MonographXref) fdbMonograph.getConceptXref();
            }
        }

        MonographDTO mVa = createMonographDTOFromFdbDdim(monoId, monographSections);
        mVa.setConceptXref(xref);
        return mVa;
    }

    /**
     * Creates the Monograph Va from FDBDdim.
     * 
     * @param monoId
     *            FDBMONOGRAPHDdimID
     * @param monographSections
     *            Map<String,ArrayList<FdbMonographDdim>>
     * @return Monograph Va
     */
    public static MonographDTO createMonographDTOFromFdbDdim(FdbMonographDdimId monoId,
            Map<String, ArrayList<FdbMonographDdim>> monographSections) {
        MonographDTO mVa = new MonographDTO();
        mVa.setMonographid(monoId.getMonographid());
        mVa.setLanguageid(monoId.getLanguageid());
        mVa.setVersioncode(monoId.getVersioncode());

        Set<String> monographKeySet = monographSections.keySet();
        for (String currSection : monographKeySet) {
            String linetext = new String("");
            ArrayList<FdbMonographDdim> sectionText = monographSections.get(currSection);
            for (FdbMonographDdim fdbMonograph : sectionText) {
                if (fdbMonograph.getFormatcode().equals("1")) {
                    if (REFERENCE_CODE.equals(currSection)) {
                        linetext = linetext + "\n" + fdbMonograph.getLinetext();
                    } else
                        linetext = linetext + fdbMonograph.getLinetext();
                } else if (fdbMonograph.getFormatcode().equals("2")) {
                    if (REFERENCE_CODE.equals(currSection)) {
                        linetext = linetext + "\n\n" + fdbMonograph.getLinetext();
                    } else
                        linetext = linetext + "\n" + fdbMonograph.getLinetext();
                } else {
                    linetext = linetext + fdbMonograph.getLinetext();
                }
            }
            populateSections(mVa, currSection, linetext);
        }
        mVa.stripLabels();
        mVa.setRecordId(new Long(mVa.getMonographid()).toString() + "|" + mVa.getMonographDisplayTitle());

        return mVa;
    }

    /**
     * Populate the Sections of the Monograph.
     * 
     * @param mVa
     *            - MonographDTO
     * @param currSection
     *            - String
     * @param ltxt
     *            - String
     */
    // Is Referenced in workspace cannot remove
    private static void populateSections(MonographDTO mVa, String currSection, String ltxt) {
        if (MONOGRAPH_TITLE_CODE.equals(currSection)) {
            mVa.setMonographTitle(ltxt);
            mVa.setMonographDisplayTitle(ltxt + " - " + new Long(mVa.getMonographid()).toString());
        } else if (MECHANISM_OF_ACTION_CODE.equals(currSection)) {
            mVa.setMechanismOfAction(ltxt);
        } else if (DISCUSSION_CODE.equals(currSection)) {
            mVa.setDiscussion(ltxt);
        } else if (CLINICAL_EFFECTS_CODE.equals(currSection)) {
            mVa.setClinicalEffects(ltxt);
        } else if (SEVERITY_LEVEL_CODE.equals(currSection)) {
            mVa.setSeverityLevel(ltxt);
        } else if (PATIENT_MANAGEMENT_CODE.equals(currSection)) {
            mVa.setPatientManagement(ltxt);
        } else if (PREDISPOSING_FACTORS_CODE.equals(currSection)) {
            mVa.setPredisposingFactors(ltxt);
        } else if (REFERENCE_CODE.equals(currSection)) {
            mVa.setReference(ltxt);
        } else if (DISCLAIMER_CODE.equals(currSection)) {
            mVa.setDisclaimer(ltxt);
        }
    }

    /**
     * * Strip the labels from the individual sections for operation in the custom table.
     */
    private void stripLabels() {
        stripMonographTitle();

        stripMonographDisplayTitle();

        stripMechanimsOfActionLabel();

        stripDiscussionLabel();

        stripClinicalEffectsLabel();

        stripSeverityLevelLabel();

        stripPatientManagementLabel();

        stripPredisposingFactorsLabel();

        stripReferenceLabel();

        stripDisclaimerLabel();
    }

    /**
     * Strip disclaimer label.
     */
    private void stripDisclaimerLabel() {
        if (disclaimer != null && disclaimer.startsWith(DISCLAIMER_LABEL)) {
            disclaimer = disclaimer.substring(DISCLAIMER_LABEL.length());
        }
    }

    /**
     * Strip reference label.
     */
    private void stripReferenceLabel() {
        if (reference != null && reference.startsWith("\n" + REFERENCE_LABEL)) {

            String refLabel = "\n" + REFERENCE_LABEL;
            reference = reference.substring(refLabel.length());
        }
    }

    /**
     * Strip predisposing factors label.
     */
    private void stripPredisposingFactorsLabel() {
        if (predisposingFactors != null && predisposingFactors.startsWith(PREDISPOSING_FACTORS_LABEL)) {
            predisposingFactors = predisposingFactors.substring(PREDISPOSING_FACTORS_LABEL.length());
        }
    }

    /**
     * Strip patient management label.
     */
    private void stripPatientManagementLabel() {
        if (patientManagement != null && patientManagement.startsWith(PATIENT_MANAGEMENT_LABEL)) {
            patientManagement = patientManagement.substring(PATIENT_MANAGEMENT_LABEL.length());
        }
    }

    /**
     * Strip severity level label.
     */
    private void stripSeverityLevelLabel() {
        if (severityLevel != null && severityLevel.startsWith(SEVERITY_LEVEL_LABEL)) {
            severityLevel = severityLevel.substring(SEVERITY_LEVEL_LABEL.length());
        }
    }

    /**
     * Strip clinical effects label.
     */
    private void stripClinicalEffectsLabel() {
        if (clinicalEffects != null && clinicalEffects.startsWith(CLINICAL_EFFECTS_LABEL)) {
            clinicalEffects = clinicalEffects.substring(CLINICAL_EFFECTS_LABEL.length());
        }
    }

    /**
     * Strip discussion label.
     */
    private void stripDiscussionLabel() {
        if (discussion != null && discussion.startsWith(DISCUSSION_LABEL)) {
            discussion = discussion.substring(DISCUSSION_LABEL.length());
        }
    }

    /**
     * Strip mechanims of action label.
     */
    private void stripMechanimsOfActionLabel() {
        if (mechanismOfAction != null && mechanismOfAction.startsWith(MECHANISM_OF_ACTION_LABEL)) {
            mechanismOfAction = mechanismOfAction.substring(MECHANISM_OF_ACTION_LABEL.length());
        }
    }

    /**
     * Strip monograph display title.
     */
    private void stripMonographDisplayTitle() {
        if (monographDisplayTitle != null && monographDisplayTitle.startsWith(MONOGRAPH_TITLE_LABEL)) {
            monographDisplayTitle = monographDisplayTitle.substring(MONOGRAPH_TITLE_LABEL.length());
        }
    }

    /**
     * Strip monograph title.
     */
    private void stripMonographTitle() {
        if (monographTitle != null && monographTitle.startsWith(MONOGRAPH_TITLE_LABEL)) {
            monographTitle = monographTitle.substring(MONOGRAPH_TITLE_LABEL.length());
        }
    }

    /**
     * Getter for FDB Custom Monograph VA Id.
     * 
     * @return fdbCustomMonographVaId
     */
    public long getFdbCustomMonographVaId() {
        return fdbCustomMonographVaId;
    }

    /**
     * Setter for FdbCustomMonographVaID takes one parmater type Long.
     * 
     * @param fdbCustomMonographVaId
     *            - long
     */
    public void setFdbCustomMonographVaId(long fdbCustomMonographVaId) {
        this.fdbCustomMonographVaId = fdbCustomMonographVaId;
    }

    /**
     * Getter for ReferenceText.
     * 
     * @return referenceText
     */
    public String getReferenceText() {
        return referenceText;
    }

    /**
     * Setter for ReferenceText takes one parameter type String.
     * 
     * @param referenceText
     *            - String
     */
    public void setReferenceText(String referenceText) {
        this.referenceText = referenceText;
    }

    /**
     * Getter for Language ID.
     * 
     * @return languageid
     */
    public long getLanguageid() {
        return languageid;
    }

    /**
     * Setter for language Id takes one parameter type long.
     * 
     * @param languageid
     *            - long
     */
    public void setLanguageid(long languageid) {
        this.languageid = languageid;
    }

    /**
     * Getter for Version Code.
     * 
     * @return versioncode
     */
    public String getVersioncode() {
        return versioncode;
    }

    /**
     * Setter for Version Code takes one parameter type String.
     * 
     * @param versioncode
     *            - String
     */
    public void setVersioncode(String versioncode) {
        this.versioncode = versioncode;
    }

    /**
     * Getter for Category Code.
     * 
     * @return categorycode
     */
    public String getCategorycode() {
        return categorycode;
    }

    /**
     * Setter for CategoryCode takes one parameter type String.
     * 
     * @param categorycode
     *            - String
     */
    public void setCategorycode(String categorycode) {
        this.categorycode = categorycode;
    }

    /**
     * Getter for VA Monograph ID.
     * 
     * @return monographid
     */
    public long getMonographid() {
        return monographid;
    }

    /**
     * Setter for VA Monograph ID takes one parameter type long.
     * 
     * @param monographid
     *            - long
     */
    public void setMonographid(long monographid) {
        this.monographid = monographid;
    }

    /**
     * Getter for FdbId.
     * 
     * @return fdbId
     */
    public long getFdbId() {
        return fdbId;
    }

    /**
     * Setter FdbId takes one parameter type Long.
     * 
     * @param fdbId
     *            - long
     */
    public void setFdbId(long fdbId) {
        this.fdbId = fdbId;
    }

    /**
     * Getter MonographTitle.
     * 
     * @return monographTitle
     */
    public String getMonographTitle() {
        return monographTitle;
    }

    /**
     * Getter for MonographTitleWithLabel.
     * 
     * @return MONOGRAPH_TITLE_LABEL + monographTitle
     */
    public String getMonographTitleWithLabel() {
        return MONOGRAPH_TITLE_LABEL + monographTitle;
    }

    /**
     * Setter for MonographTitle takes one parameter type String.
     * 
     * @param monographTitle
     *            - String
     */
    public void setMonographTitle(String monographTitle) {
        this.monographTitle = monographTitle;
    }

    /**
     * Getter for MonographDisplayTitle.
     * 
     * @return monographDisplayTitle
     */
    public String getMonographDisplayTitle() {
        return monographDisplayTitle;
    }

    /**
     * Set MonographDisplayTitle takes one parameter type String.
     * 
     * @param monographDisplayTitle
     *            - String
     */
    public void setMonographDisplayTitle(String monographDisplayTitle) {
        this.monographDisplayTitle = monographDisplayTitle;
    }

    /**
     * Getter MechanismOfAction.
     * 
     * @return mechanismOfAction
     */
    public String getMechanismOfAction() {
        return mechanismOfAction;
    }

    /**
     * Setter for MechanismOfAction takes one parameter type String.
     * 
     * @param mechanismOfAction
     *            - String
     */
    public void setMechanismOfAction(String mechanismOfAction) {
        this.mechanismOfAction = mechanismOfAction;
    }

    /**
     * Getter for Discussion.
     * 
     * @return discussion
     */
    public String getDiscussion() {
        return discussion;
    }

    /**
     * Setter Discussion takes one parameter String.
     * 
     * @param discussion
     *            - String
     */
    public void setDiscussion(String discussion) {
        this.discussion = discussion;
    }

    /**
     * Getter ClinicalEffects.
     * 
     * @return clinicalEffects
     */
    public String getClinicalEffects() {
        return clinicalEffects;
    }

    /**
     * Setter ClinicalEffects takes String parameter.
     * 
     * @param clinicalEffects
     *            - string
     */
    public void setClinicalEffects(String clinicalEffects) {
        this.clinicalEffects = clinicalEffects;
    }

    /**
     * Getter SeverityLevel.
     * 
     * @return severityLevel
     */
    public String getSeverityLevel() {
        return severityLevel;
    }

    /**
     * Setter for Severity Level takes String parameter.
     * 
     * @param severityLevel
     *            - String
     */
    public void setSeverityLevel(String severityLevel) {
        this.severityLevel = severityLevel;
    }

    /**
     * Gets the patient management.
     * 
     * @return the patient management
     */
    public String getPatientManagement() {
        return patientManagement;
    }

    /**
     * Setter for PatientManagement takes parameter String.
     * 
     * @param patientManagement
     *            - String
     */
    public void setPatientManagement(String patientManagement) {
        this.patientManagement = patientManagement;
    }

    /**
     * Getter for PredisposingFactors.
     * 
     * @return predisposingFactos
     */
    public String getPredisposingFactors() {
        return predisposingFactors;
    }

    /**
     * Setter for PredisposingFactors.
     * 
     * @param predisposingFactors
     *            - String
     */
    public void setPredisposingFactors(String predisposingFactors) {
        this.predisposingFactors = predisposingFactors;
    }

    /**
     * Getter for Reference.
     * 
     * @return reference
     */
    public String getReference() {
        return reference;
    }

    /**
     * Setter for Reference.
     * 
     * @param reference
     *            - String
     */
    public void setReference(String reference) {
        this.reference = reference;
    }

    /**
     * Getter for Disclaimer.
     * 
     * @return disclaimer
     */
    public String getDisclaimer() {
        if (disclaimer == null || disclaimer.length() <= 0)
            setDisclaimer(VA_MONOGRAPH_DISCLAIMER);
        return disclaimer;
    }

    /**
     * Setter for Disclaimer takes a String parameter.
     * 
     * @param disclaimer
     *            - String
     */
    public void setDisclaimer(String disclaimer) {
        this.disclaimer = disclaimer;
    }

    /**
     * Getter for RecordId.
     * 
     * @return recordId
     */
    public String getRecordId() {
        return recordId;
    }

    /**
     * Setter for Record ID.
     * 
     * @param recordId
     *            -String
     */
    public void setRecordId(String recordId) {
        this.recordId = recordId;
    }

    /**
     * Getter generateRecordId.
     * 
     * @return Long.toString(getFdbCustomMonographVaId()
     */
    public String generateRecordId() {
        return Long.toString(getFdbCustomMonographVaId());
    }

    /**
     * Getter for boolean isReadOnlyRecord.
     * 
     * @return readOnlyRecord
     */
    public boolean isReadOnlyRecord() {
        return readOnlyRecord;
    }

    /**
     * Set ReadOnlyRecord takes parameter boolean.
     * 
     * @param readOnlyRecord
     *            - boolean
     */
    public void setReadOnlyRecord(boolean readOnlyRecord) {
        this.readOnlyRecord = readOnlyRecord;
    }

    /**
     * Getter VA MonographId.
     * 
     * @return the vaMonographId
     */
    public static String getVaMonographId() {
        return VA_MONOGRAPH_ID;
    }

    /**
     * Getter for Fdb_Id.
     * 
     * @return FDB_ID
     */
    public static String getfirstDataBankId() {
        return FDB_ID;
    }

    /**
     * To String to print out for MonographDTO.
     * 
     * @return s
     */
    @Override
    public String toString() {
        String s =
                "{fdbCustomMonographVaId = " + fdbCustomMonographVaId + ", referenceText = " + referenceText
                        + ", languageid = " + languageid + ", versioncode = " + versioncode + ", categorycode = "
                        + categorycode + ", monographid = " + monographid + ", actionReason = " + getActionReason()
                        + ", actionBy = " + getActionBy() + ", requestor = " + getRequestor() + ", assignedTo = "
                        + getAssignedTo() + ", fdbId = " + fdbId + ", monographTitle = " + monographTitle
                        + ", mechanismOfAction = " + mechanismOfAction + ", discussion = " + discussion
                        + ", clinicalEffects = " + clinicalEffects + ", severityLevel = " + severityLevel
                        + ", patientManagement = " + patientManagement + ", predisposingFactors = " + predisposingFactors
                        + ", reference = " + reference + ", disclaimer = " + disclaimer;

        if (getActionDate() != null) {
            s = s + ", actionDate = " + getActionDate().toString();
        } else {
            s = s + ", actionDate = null";
        }

        s = s + "}";

        return s;
    }

    /**
     * Displays Monograph Title.
     * 
     * @param custom
     *            - boolean
     */
    public void setDisplayTitle(boolean custom) {
        String displayTitle = getMonographTitle() + " - " + getMonographid();
        if (custom) {
            displayTitle = displayTitle + " (custom)";
        }
        setMonographDisplayTitle(displayTitle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.ct.domain.staging.PecsDTO#getSelect()
     */
    @Override
    public LabelValue getSelect() {
        StringBuffer sb = new StringBuffer();
        LabelValue lv = new LabelValue();
        sb.append("/ct/pecs/monograph/professional/");
        if (getRequestor() == null) {
            sb.append("FDB/");
            // sb.append(getFdbId());
            sb.append(getMonographid());
            lv.setLabel(QueryResultLabel.OPEN);
        } else {
            sb.append("VA/");
            // sb.append(getMonographid());
            sb.append(getFdbCustomMonographVaId());
            if (isCurrent()) {
                lv.setLabel(QueryResultLabel.ACTIVE);
            } else {
                lv.setLabel(QueryResultLabel.HISTORICAL);
            }
        }
        lv.setValue(sb.toString());
        return lv;
    }

    /**
     * Returns Professional Version Code.
     * 
     * @return PROFESSIONAL_VERSION_CODE
     */
    public static String getProfessionalVersionCode() {
        return PROFESSIONAL_VERSION_CODE;
    }

}
