package gov.va.med.pharmacy.pecs.dto;

import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.metadata.FieldInfo;
import gov.va.med.pharmacy.pecs.util.LabelValue;
import gov.va.med.pharmacy.pecs.util.PecsDateFormat;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.Set;

import org.apache.commons.lang.time.DateFormatUtils;

/**
 * PECS DTO ENTITY.
 * 
 * @author DOMAINOVERSD
 * 
 */
public abstract class PecsDTO implements Serializable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 3862273267637521819L;

    /** String Representation of ACTION_DATE. */
    public static final String ACTION_DATE = "ACTION_DATE";

    /** String Representation of EXPORT_DATE. */
    public static final String EXPORT_DATE = "EXPORT_DATE";

    /** String Representation ACTION_REASON. */
    public static final String ACTION_REASON = "ACTION_REASON";

    /** String Representation of ACTION_BY. */
    public static final String ACTION_BY = "ACTION_BY";

    /** String Representation of ASSIGNED_TO. */
    public static final String ASSIGNED_TO = "ASSIGNED_TO";

    /** String Representation of REQUESTO. */
    public static final String REQUESTOR = "REQUESTOR";

    /** String Representation of ACTION_STATUS. */
    public static final String ACTION_STATUS = "ACTION_STATUS";

    /** String Representation of CURRENT_ACTION_REASON. */
    public static final String CURRENT_ACTION_REASON = "CURRENT_ACTION_REASON";

    /** The Constant USER_COMMENT_HISTORY. */
    public static final String USER_COMMENT_HISTORY = "USER_COMMENT_HISTORY";

    /** The metadata name for the export date display string. */
    public static final String EXPORT_DATE_DISPLAY_STRING = "EXPORT_DATE_DISPLAY_STRING";

    /** The metadata name for the action date display string. */
    public static final String ACTION_DATE_DISPLAY_STRING = "ACTION_DATE_DISPLAY_STRING";

    /** The Constant ACTION_DATE_REPORT_ORDER. */
    protected static final int ACTION_DATE_REPORT_ORDER = 105;

    /** The Constant EXPORT_DATE_REPORT_OREDER. */
    protected static final int EXPORT_DATE_REPORT_ORDER = 106;

    /** The Constant ACTION_BY_REPORT_ORDER. */
    protected static final int ACTION_BY_REPORT_ORDER = 107;

    /** The Constant ASSIGNED_TO_REPORT_ORDER. */
    protected static final int ASSIGNED_TO_REPORT_ORDER = 108;

    /** The Constant CURRENT_ACTION_REASON_REPORT_ORDER. */
    protected static final int CURRENT_ACTION_REASON_REPORT_ORDER = 111;

    /** The Constant REQUESTOR_REPORT_ORDER. */
    protected static final int REQUESTOR_REPORT_ORDER = 109;

    /** The export date. */
    @FieldInfo(
            name = EXPORT_DATE, reportHistory = true, reportHistoryOrder = EXPORT_DATE_REPORT_ORDER, editable = false,
            internal = true, customPersistentPath = "exportDate", nonQueriable = true, hideFromDetailDisplay = true)
    private Date exportDate;

    /** The action date. */
    @FieldInfo(
            name = ACTION_DATE, editable = false, internal = true, customPersistentPath = "date", reportHistory = true,
            reportHistoryOrder = ACTION_DATE_REPORT_ORDER, nonQueriable = true, hideFromDetailDisplay = true)
    private Date actionDate;

    /** The action reason. */
    @FieldInfo(
            name = ACTION_REASON, displayLines = 5, editable = false, internal = true, customPersistentPath = "commentHistory",
            nonQueriable = true)
    private String actionReason;

    /** The action by. */
    @FieldInfo(
            name = ACTION_BY, editable = false, internal = true, customPersistentPath = "actionBy", reportHistory = true,
            reportHistoryOrder = ACTION_BY_REPORT_ORDER)
    private String actionBy;

    /** The assigned to. */
    @FieldInfo(
            name = ASSIGNED_TO, internal = true, customPersistentPath = "assignedUser", reportHistory = true,
            reportHistoryOrder = ASSIGNED_TO_REPORT_ORDER)
    private String assignedTo;

    /** The current action reason. */
    @FieldInfo(
            name = CURRENT_ACTION_REASON, displayLines = 5, internal = true, customPersistentPath = "comment",
            reportHistory = true, reportHistoryOrder = CURRENT_ACTION_REASON_REPORT_ORDER)
    private String currentActionReason = null;

    /** The status. */
    @FieldInfo(
            name = ACTION_STATUS, editable = false, internal = true, customPersistentPath = "status.name",
            reportHistory = true, reportHistoryOrder = 1)
    private String status;

    /** The requestor. */
    @FieldInfo(
            name = REQUESTOR, editable = false, internal = true, customPersistentPath = "versionedObject.requestor",
            reportHistory = true, reportHistoryOrder = REQUESTOR_REPORT_ORDER)
    private String requestor;

    /** The is current. */
    private boolean isCurrent = true;

    /** The action user. */
    private UserProfile actionUser;

    /** The allowable actions. */
    private Collection<String> allowableActions;

    /** The master record id. */
    private long masterRecordId;

    /** The user comment history. */
    @FieldInfo(
            name = USER_COMMENT_HISTORY, editable = false, internal = true, customPersistentPath = "userCommentHistory",
            fdbPersistentPath = "userCommentHistory", reportHistory = false, nonQueriable = true, nonReportable = true)
    private Set<UserComment> userCommentHistory;

    /** Contains the formatted Export Date. */
    @FieldInfo(
            name = EXPORT_DATE_DISPLAY_STRING, editable = false, internal = true, customPersistentPath = "exportDate",
            reportHistory = false, nonQueriable = false, nonReportable = true, hideFromDetailDisplay = false)
    private String exportDateDisplayString;

    /** Contains the formatted Action Date.    */
    @FieldInfo(
            name = ACTION_DATE_DISPLAY_STRING, editable = false, internal = true, customPersistentPath = "date",
            reportHistory = false, nonQueriable = false, nonReportable = true, hideFromDetailDisplay = false)
    private String actionDateDisplayString;

    /** The concept xref. */
    private ConceptXref conceptXref;

    /**
     * Get Action Date.
     * 
     * @return actionDate
     */
    public Date getActionDate() {
        return this.actionDate;
    }

    /**
     * Set ActionDate.
     * 
     * @param actionDate
     *            - Date
     */
    public void setActionDate(Date actionDate) {
        this.actionDate = actionDate;
    }

    /**
     * Get the action date of the the record formatted in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>.
     * 
     * @return the actionDate represented in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>
     */
    public String getActionDateDisplayString() {
        if (getActionDate() == null) {
            return null;
        }
        return DateFormatUtils.format(getActionDate(), PecsDateFormat.PECS_DATE_TIME_FORMAT,
                PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE);
    }

    // /**
    // * Unimplemented setter. The setter is needed to satisfy some JavaBean requirement of the framework, but since it is a
    // * read-only field the implementation just throws an exception.
    // *
    // * @param actionDate
    // * the actionDate to set
    // * @throws Exception
    // */
    // public void setActionDateDisplayString(String actionDate) throws Exception {
    // throw new Exception("Setter not implemented - this is a read-only field.");
    // }

    /**
     * Get ActionReason.
     * 
     * @return actionReason
     */
    public String getActionReason() {
        return this.actionReason;
    }

    /**
     * Set ActionReason.
     * 
     * @param actionReason
     *            - String
     */
    public void setActionReason(String actionReason) {
        this.actionReason = actionReason;
    }

    /**
     * Get ActionBy.
     * 
     * @return actionBy
     */
    public String getActionBy() {
        return this.actionBy;
    }

    /**
     * Set ActionBy.
     * 
     * @param actionBy
     *            - String
     */
    public void setActionBy(String actionBy) {
        this.actionBy = actionBy;
    }

    /**
     * Get AssignedTo.
     * 
     * @return assignedTo
     */
    public String getAssignedTo() {
        return this.assignedTo;
    }

    /**
     * Set AssignedTo.
     * 
     * @param assignedTo
     *            - String
     */
    public void setAssignedTo(String assignedTo) {
        this.assignedTo = assignedTo;
    }

    /**
     * Get Requester.
     * 
     * @return requester
     */
    public String getRequestor() {
        return this.requestor;
    }

    /**
     * Set Requestor.
     * 
     * @param requestor
     *            - String
     */
    public void setRequestor(String requestor) {
        this.requestor = requestor;
    }

    /**
     * Get CurrentActionReason.
     * 
     * @return currentActionReason
     */
    public String getCurrentActionReason() {
        return this.currentActionReason;
    }

    /**
     * Set CurrentActionReason.
     * 
     * @param currentActionReason
     *            - String
     */
    public void setCurrentActionReason(String currentActionReason) {
        this.currentActionReason = currentActionReason;
    }

    /**
     * Set Status.
     * 
     * @param status
     *            - String
     */
    public void setStatus(String status) {
        this.status = status;
    }

    /**
     * Get Status.
     * 
     * @return status.
     */
    public String getStatus() {
        return status;
    }

    /**
     * Set Current.
     * 
     * @param isCurrent
     *            - boolean
     */
    public void setCurrent(boolean isCurrent) {
        this.isCurrent = isCurrent;
    }

    /**
     * Is Current.
     * 
     * @return isCurrent
     */
    public boolean isCurrent() {
        return isCurrent;
    }

    /**
     * Get ActionUser.
     * 
     * @return actionUser
     */
    public UserProfile getActionUser() {
        return actionUser;
    }

    /**
     * Set ActionUser.
     * 
     * @param actionUser
     *            - UserProfile
     */
    public void setActionUser(UserProfile actionUser) {
        this.actionUser = actionUser;
        setActionBy(actionUser.getUsername());
    }

    /**
     * Set AllowableActions.
     * 
     * @param actions
     *            - SET<STRING>
     */
    public void setAllowableActions(Set<String> actions) {
        this.allowableActions = actions;
    }

    /**
     * Set AllowableActions.
     * 
     * @return allowableActions
     */
    public Collection<String> getAllowableActions() {
        return allowableActions;
    }

    /**
     * Get Abstract LabelValue.
     * 
     * @return - select
     */
    public abstract LabelValue getSelect();

    /**
     * GetMaster Record.
     * 
     * @return the masterRecordId
     */
    public long getMasterRecordId() {
        return masterRecordId;
    }

    /**
     * Set MasterRecord Id.
     * 
     * @param masterRecordId
     *            the masterRecordId to set
     */
    public void setMasterRecordId(long masterRecordId) {
        this.masterRecordId = masterRecordId;
    }

    /**
     * 
     * Gets the user comment history.
     * 
     * @return the user comment history
     */

    public Set<UserComment> getUserCommentHistory() {

        if (getConceptXref() != null && getConceptXref().getUserCommentHistory() != null) {

            return getConceptXref().getUserCommentHistory();

        }

        return userCommentHistory;

    }

    /**
     * 
     * Sets the user comment history.
     * 
     * 
     * 
     * @param userCommentHistory
     * 
     *            the new user comment history
     */

    public void setUserCommentHistory(Set<UserComment> userCommentHistory) {

        if (this.getConceptXref() != null) {

            getConceptXref().setUserCommentHistory(userCommentHistory);

        }

    }

    /** The list of comments. */
    private String listOfComments = "";

    /**
     * Get the list of Comments.
     * 
     * @return the listOfComments
     */
    public String getListOfComments() {
        if (listOfComments.equalsIgnoreCase("")) {
            Set<UserComment> comments = this.getUserCommentHistory();

            if (comments == null || comments.isEmpty()) {
                listOfComments = "";
            } else {
                StringBuffer sb = new StringBuffer();
                for (UserComment userComment : comments) {
                    sb.append(userComment);
                }
                listOfComments = sb.toString();
            }
        }
        return listOfComments;
    }

    /**
     * Set List of Comments.
     * 
     * @param listOfComments
     *            the listOfComments to set
     */
    public void setListOfComments(String listOfComments) {
        this.listOfComments = listOfComments;
    }

    /**
     * Gets the conceptXref.
     * 
     * @return the conceptXref
     */
    public ConceptXref getConceptXref() {
        return conceptXref;
    }

    /**
     * Sets the conceptXref.
     * 
     * @param conceptXref
     *            the conceptXref to set
     */
    public void setConceptXref(ConceptXref conceptXref) {
        this.conceptXref = conceptXref;
    }

    /**
     * Get the export date of the the record.
     * 
     * @return the exportDate
     */
    public Date getExportDate() {
        return exportDate;
    }

    /**
     * Set the export date of the record.
     * 
     * @param exportDate
     *            the exportDate to set
     */
    public void setExportDate(Date exportDate) {
        this.exportDate = exportDate;
    }

    /**
     * Get the export date of the the record formatted in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>.
     * 
     * @return the exportDate represented in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>
     */
    public String getExportDateDisplayString() {
        if (getExportDate() == null) {
            return null;
        }
        return DateFormatUtils.format(getExportDate(), PecsDateFormat.PECS_DATE_TIME_FORMAT,
                PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE);
    }

    // /**
    // * Unimplemented setter. The setter is needed to satisfy some JavaBean requirement of the framework, but since it is a
    // * read-only field the implementation just throws an exception.
    // *
    // * @param exportDate
    // * the exportDate to set
    // * @throws Exception
    // */
    // public void setExportDateDisplayString(String exportDate) throws Exception {
    // throw new Exception("Setter not implemented - this is a read-only field.");
    // }
}
