package gov.va.med.pharmacy.pecs.exception;

/**
 * An exception to indicate that an error has occurred while FTP'ing a file to another server. The information included in the
 * exception is:
 * 
 * The name of the target server. The directory the file was to be placed in. The user name being used to perform the FTP.
 * 
 * @author vhaisaduffg
 * 
 */
public class FTPException extends RuntimeException {

    /**
     * 
     */
    private static final long serialVersionUID = -2504174427396833088L;
    private String server;
    private String directoryName;
    private String userName;

    /**
     * FTP Exception.
     * 
     */
    public FTPException() {
        super();
    }

    /**
     * FTP Exception.
     * 
     * @param arg0
     *            - String
     */
    public FTPException(String arg0) {
        super(arg0);
    }

    /**
     * FTPException.
     * 
     * @param cause
     *            Throwable
     */
    public FTPException(Throwable cause) {
        super(cause);
    }

    /**
     * FTPException.
     * 
     * @param message
     *            String
     * @param cause
     *            Throwable
     */
    public FTPException(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * FTPException.
     * 
     * @param message
     *            - String
     * @param cause
     *            - Throwable
     * @param server
     *            - String
     */
    public FTPException(String message, Throwable cause, String server) {
        super(message, cause);
        setServer(server);
    }

    /**
     * Get Server.
     * 
     * @return server
     */
    public String getServer() {
        return server;
    }

    /**
     * Set Server.
     * 
     * @param server
     *            - String
     */
    public void setServer(String server) {
        this.server = server;
    }

    /**
     * Get Directory Name.
     * 
     * @return dirctory name
     */
    public String getDirectoryName() {
        return directoryName;
    }

    /**
     * Set Directory Name.
     * 
     * @param directoryName
     *            - String
     */
    public void setDirectoryName(String directoryName) {
        this.directoryName = directoryName;
    }

    /**
     * Get User Name.
     * 
     * @return userName
     */
    public String getUserName() {
        return userName;
    }

    /**
     * Set User name.
     * 
     * @param userName
     *            - String
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }
}
