package gov.va.med.pharmacy.pecs.exception;

import java.util.Collections;
import java.util.List;

/**
 * Validation Exception Entity.
 * 
 */
public final class ValidationException extends CustomizationServiceException {
    /**
     * 
     */
    private static final long serialVersionUID = 6345863577181437112L;
    private List<String> messages;

    /**
     * Validation Exception.
     * 
     */
    public ValidationException() {
        super();
    }

    /**
     * Validation Exception.
     * 
     * @param msg
     *            - String
     * @param t
     *            - Throwable
     */
    public ValidationException(String msg, Throwable t) {
        super(msg, t);
    }

    /**
     * Validation Exception.
     * 
     * @param msg
     *            - String
     */
    public ValidationException(String msg) {
        super(msg);
    }

    /**
     * Validation Exception.
     * 
     * @param t
     *            - throwable
     */
    public ValidationException(Throwable t) {
        super(t);
    }

    /**
     * Validation Exception.
     * 
     * @param messages
     *            - List
     */
    public ValidationException(List<String> messages) {
        super();
        this.messages = messages;
    }

    /**
     * Get Messages.
     * 
     * @return Collections.singletonList(getMessage())
     */
    public List<String> getMessages() {
        if (messages != null) {
            return messages;
        } else {
            return Collections.singletonList(getMessage());
        }
    }

    @Override
    public String getMessage() {
        if (messages != null) {
            return formatMessages(messages);
        } else {
            return super.getMessage();
        }
    }

    /**
     * Format Message.
     * 
     * @param messageList
     *            - List<String>
     * @return buf.toString.
     */
    private String formatMessages(List<String> messageList) {
        StringBuffer buf = new StringBuffer();
        for (String m : messageList) {
            if (buf.length() > 0) {
                buf.append("\r\n");
            }
            buf.append(m);
        }
        return buf.toString();
    }

}
