package gov.va.med.pharmacy.pecs.fdbreports;

import java.io.Serializable;

/**
 * FdbColumnUpdate Class Entity Implements Serializable.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class FdbColumnUpdate implements Serializable {

    /**
     * String value for a double.
     */
    public static final String DOUBLE = "DOUBLE";
    /**
     * String constant for a small int.
     */
    public static final String SMALLINT = "SMALLINT";
    /**
     * String constant for a integer.
     */
    public static final String INTEGER = "INTEGER";
    /**
     * String constant for a varchar.
     */
    public static final String VARCHAR = "VARCHAR";
    private static final long serialVersionUID = -8585294512909076544L;
    private FdbTableColumn column;
    private String currentValue;
    private String previousValue;

    /**
     * Get Current Value.
     * 
     * @return currentValue
     */
    public String getCurrentValue() {
        return currentValue;
    }

    /**
     * Set Current Value.
     * 
     * @param val
     *            String
     */
    public void setCurrentValue(String val) {
        if (val != null && FdbColumnType.VARCHAR.equals(getColumn().getDataType())) {
            // get rid of single quotes
            StringBuffer buf = new StringBuffer();
            boolean sawSingleQuote = false;
            for (int i = 0; i < val.length(); i++) {
                char c = val.charAt(i);
                if (c == '\'' && !sawSingleQuote) {
                    // first single quote
                    sawSingleQuote = true;
                } else {
                    // non-single quote character or second single quote
                    sawSingleQuote = false;
                    buf.append(c);
                }
            }
            currentValue = buf.toString();
        } else {
            currentValue = val;
        }
    }

    /**
     * Get Current Value As Object.
     * 
     * @return null/parsed Long
     */
    public Object getCurrentValueAsObject() {
        if (currentValue == null) {
            return null;
        }
        FdbColumnType dataType = getColumn().getDataType();

        switch (getColumn().getDataType()) {
        case DOUBLE:
            return Double.parseDouble(currentValue);
        case SMALLINT:
            return Long.parseLong(currentValue);
        case INTEGER:
            return Long.parseLong(currentValue);
        case VARCHAR:
            return currentValue;
        default:
            String msg = "Unknown data type " + dataType + " for column " + getColumn().getName();
            throw new IllegalArgumentException(msg);

        }

    }

    /**
     * Get Previous Value.
     * 
     * @return previous Value
     */
    public String getPreviousValue() {
        return previousValue;
    }

    /**
     * Set Previous Value.
     * 
     * @param previousValue
     *            - String
     */
    public void setPreviousValue(String previousValue) {
        this.previousValue = previousValue;
    }

    /**
     * Set Column.
     * 
     * @param column
     *            - FdbTableColumn
     */
    public void setColumn(FdbTableColumn column) {
        this.column = column;
    }

    /**
     * Get Column.
     * 
     * @return FdbTableColumn
     */
    public FdbTableColumn getColumn() {
        return column;
    }

    /**
     * Gets the isChanged.
     * 
     * @return the isChanged
     */
    public boolean isChanged() {
        if (previousValue != null) {
            return !previousValue.equals(currentValue);
        }
        if (currentValue != null) {
            return !currentValue.equals(previousValue);
        }
        return false;
    }

    @Override
    public String toString() {
        return "[" + column.getName() + ": " + previousValue + " -> " + currentValue + "]";
    }

}
