package gov.va.med.pharmacy.pecs.fdbreports;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * FdbUpdateRecord implements {@link Serializable}.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class FdbUpdateRecord implements Serializable {
    private static final long serialVersionUID = 1L;
    /**
     * Static Final String REC_TYPE_ADD = "A".
     */
    public static final String REC_TYPE_ADD = "A";
    /**
     * Static Final String REC_TYPE_CHANGE = "C".
     */
    public static final String REC_TYPE_CHANGE = "C";

    /**
     * Final String REC_TYPE_DELETE = "D".
     * 
     */
    public static final String REC_TYPE_DELETE = "D";
    private String type;
    private FdbTableMetadata table;
    private List<FdbColumnUpdate> values;

    /**
     * Get Type.
     * 
     * @return type
     */
    public String getType() {
        return type;
    }

    /**
     * Set Type.
     * 
     * @param type
     *            the type of action
     */
    public void setType(String type) {
        this.type = type;
    }

    /**
     * List FDB Column Update.
     * 
     * @return a list of update records
     */
    public List<FdbColumnUpdate> getValues() {
        return values;
    }

    /**
     * Get Value.
     * 
     * @param name
     *            v
     * @return FdbbColumn Update Value.
     */
    public FdbColumnUpdate getValue(String name) {
        for (FdbColumnUpdate v : getValues()) {
            if (v.getColumn().getName().equalsIgnoreCase(name)) {
                return v;
            }
        }
        return null;
    }

    /**
     * Get Value of Fdb Column Update.
     * 
     * @param col
     *            the FDB table column
     * @return FdbColumn Update or Null
     */
    public FdbColumnUpdate getValue(FdbTableColumn col) {
        for (FdbColumnUpdate v : getValues()) {
            if (v.getColumn().equals(col)) {
                return v;
            }
        }
        return null;
    }

    /**
     * Set Values.
     * 
     * @param values
     *            List<FdbColumnUpdate>
     */
    public void setValues(List<FdbColumnUpdate> values) {
        this.values = values;
    }

    /**
     * Add Value.
     * 
     * @param v
     *            FdbColumnUpdate
     */
    public void addValue(FdbColumnUpdate v) {
        values.add(v);
    }

    /**
     * Set Table.
     * 
     * @param table
     *            FdbTableMetadata.
     */
    public void setTable(FdbTableMetadata table) {
        this.table = table;
    }

    /**
     * FdbTableMetadata.
     * 
     * @return table
     */
    public FdbTableMetadata getTable() {
        return table;
    }

    /**
     * Get Key Values.
     * 
     * @return Array List Res
     */
    public List<FdbColumnUpdate> getKeyValues() {
        ArrayList<FdbColumnUpdate> res = new ArrayList<FdbColumnUpdate>();
        for (FdbColumnUpdate col : values) {
            if (col.getColumn().isKey()) {
                res.add(col);
            }
        }
        return res;
    }

    @Override
    public String toString() {
        return type + " | " + table.getTableName() + " | " + values;
    }
}
