package gov.va.med.pharmacy.pecs.metadata;

import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;
import gov.va.med.pharmacy.pecs.dto.MonographDTO;

import java.util.HashMap;
import java.util.Map;

/**
 * Enumeration Concept.
 * 
 * @author vhaisaduffg
 */
public enum Concept {
    /**
     * Class Import parameter deceleration.
     * 
     */
    DRUG_DRUG_INTERACTION(DrugDrugInteractionDTO.class), DRUG_PAIR(DrugPairDTO.class), PROFESSIONAL_MONOGRAPH(
            MonographDTO.class),
    /**
     * Class Import Parameter Deceleration.
     * 
     */
    DUPLICATE_THERAPY(DuplicateTherapyDTO.class), DOSE_RANGE(DoseRangeDTO.class);

    private Class<?> clazz;

    /**
     * Concept Clazz.
     * 
     * @param clazz
     *            Class<?>
     */
    private Concept(Class<?> clazz) {
        this.clazz = clazz;
    }

    /**
     * Get Concept Class.
     * 
     * @return this.clazz
     */
    public Class<?> getConceptClass() {
        return this.clazz;
    }

    private static final Map<String, Concept> STRING_TO_ENUM = new HashMap<String, Concept>();

    static {
        for (Concept concept : values()) {
            STRING_TO_ENUM.put(concept.toString(), concept);
        }

    }

    /**
     * Concept String Name.
     * 
     * @param conceptName
     *            String
     * @return string To Enum
     */
    public static Concept fromString(String conceptName) {
        return STRING_TO_ENUM.get(conceptName);
    }
}
