package gov.va.med.pharmacy.pecs.metadata;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * 
 * This class will take metadata from the database and put into a string annotation.
 * 
 * @author VHAISACARUSR
 * 
 */
public final class MetadataCreator {
    private static final String FDB_QUERY = "select CONFIG_METHOD_NAME  from config_column_va "
            + "where config_table_name = 'FDB_MONOGRAPH_DDIM' and CONFIG_COLUMN_NAME = ?";
    private static final String PECS_QUERY =
            "select * from config_column_va where config_table_name = 'PECS_MONOGRAPH_VERSION_VA'";

    /**
     * Default private constructor.
     */
    private MetadataCreator() {
    }

    /**
     * Main method to run the program.
     * 
     * @param args
     *            no args needed
     * @throws SQLException
     *             if sql goes bad
     * @throws ClassNotFoundException
     *             if class not found
     */
    public static void main(String[] args) throws SQLException, ClassNotFoundException {

        Connection connection = null;
        String driverName = "oracle.jdbc.driver.OracleDriver";
        Class.forName(driverName);

        // Create a connection to the database
        String serverName = "127.0.0.1";
        String portNumber = "1521";
        String sid = "dev";
        String url = "jdbc:oracle:thin:@" + serverName + ":" + portNumber + ":" + sid;
        String username = "system";
        String password = "system_1";

        connection = DriverManager.getConnection(url, username, password);

        PreparedStatement statement = connection.prepareStatement(PECS_QUERY);
        ResultSet rs = statement.executeQuery();
        while (rs.next()) {
            PreparedStatement statement2 = connection.prepareStatement(FDB_QUERY);
            String cName = rs.getString("CONFIG_COLUMN_NAME");
            statement2.setString(1, cName);

            ResultSet rs2 = statement2.executeQuery();

            String output =
                    "@FieldInfo(name = \"" + cName + "\", editable = " + convertToBool(rs.getString("COLUMN_IS_EDITABLE"))
                            + ",  customPersistentPath = \"" + rs.getString("CONFIG_METHOD_NAME") + "\"" + ", required = "
                            + convertToBool(rs.getString("COLUMN_IS_REQUIRED")) + ", internal = "
                            + convertToBool(rs.getString("COLUMN_IS_INTERNAL"));
            int rows = rs.getInt("COLUMN_DISPLAY_ROWS");
            if (rows != 1 && rows != 0) {
                output = output + ", displayLines=" + rows;
            }
            while (rs2.next()) {

                output = output + ", fdbPersistentPath = \"" + rs2.getString(1) + "\"";
            }
            output = output + ")";
            // System .out.println(output);
        }
    }

    /**
     * converts a string of Y or N to a String of true or false;
     * 
     * @param s
     *            string to convert
     * @return either true or false
     */
    private static String convertToBool(String s) {
        if ("Y".equalsIgnoreCase(s)) {
            return "true";
        }
        return "false";
    }
}
