package gov.va.med.pharmacy.pecs.metadata;

/**
 * Field meta data that can be changed by users. This needs to be a persistent class.
 */
public final class UserModifiableFieldMetadata {

    /** The field id. */
    private long fieldId;

    /** The concept name. */
    private String conceptName;

    /** The field name. */
    private String fieldName;

    /** The include in query results. */
    private boolean includeInQueryResults;

    /** The include in reports. */
    private boolean includeInReports;

    /** The include in detail display. */
    private boolean includeInDetailDisplay;

    /** The display order. */
    private int displayOrder;

    /** The display name. */
    private String displayName;

    /**
     * Gets the field id.
     * 
     * @return the field id
     */
    public long getFieldId() {
        return fieldId;
    }

    /**
     * Sets the field id.
     * 
     * @param id
     *            the new field id
     */
    public void setFieldId(long id) {
        this.fieldId = id;
    }

    /**
     * Gets the field name.
     * 
     * @return the field name
     */
    public String getFieldName() {
        return fieldName;
    }

    /**
     * Sets the field name.
     * 
     * @param name
     *            the new field name
     */
    public void setFieldName(String name) {
        this.fieldName = name;
    }

    /**
     * Checks if is include in query results.
     * 
     * @return true, if is include in query results
     */
    public boolean isIncludeInQueryResults() {
        return includeInQueryResults;
    }

    /**
     * Sets the include in query results.
     * 
     * @param includeInQueryResults
     *            the new include in query results
     */
    public void setIncludeInQueryResults(boolean includeInQueryResults) {
        this.includeInQueryResults = includeInQueryResults;
    }

    /**
     * Checks if is include in reports.
     * 
     * @return true, if is include in reports
     */
    public boolean isIncludeInReports() {
        return includeInReports;
    }

    /**
     * Sets the include in reports.
     * 
     * @param includeInReports
     *            the new include in reports
     */
    public void setIncludeInReports(boolean includeInReports) {
        this.includeInReports = includeInReports;
    }

    /**
     * Checks if is include in detail display.
     * 
     * @return true, if is include in detail display
     */
    public boolean isIncludeInDetailDisplay() {
        return includeInDetailDisplay;
    }

    /**
     * Sets the include in detail display.
     * 
     * @param includeInDetailDisplay
     *            the new include in detail display
     */
    public void setIncludeInDetailDisplay(boolean includeInDetailDisplay) {
        this.includeInDetailDisplay = includeInDetailDisplay;
    }

    /**
     * Gets the display order.
     * 
     * @return the display order
     */
    public int getDisplayOrder() {
        return displayOrder;
    }

    /**
     * Sets the display order.
     * 
     * @param displayOrder
     *            the new display order
     */
    public void setDisplayOrder(int displayOrder) {
        this.displayOrder = displayOrder;
    }

    /**
     * Sets the concept name.
     * 
     * @param conceptName
     *            the new concept name
     */
    public void setConceptName(String conceptName) {
        this.conceptName = conceptName;
    }

    /**
     * Gets the concept name.
     * 
     * @return the concept name
     */
    public String getConceptName() {
        return conceptName;
    }

    /**
     * Gets the display name.
     * 
     * @return the display name
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * Sets the display name.
     * 
     * @param displayName
     *            the new display name
     */
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((conceptName == null) ? 0 : conceptName.hashCode());
        result = prime * result + ((displayName == null) ? 0 : displayName.hashCode());
        result = prime * result + displayOrder;
        result = prime * result + (int) (fieldId ^ (fieldId >>> 32));
        result = prime * result + ((fieldName == null) ? 0 : fieldName.hashCode());
        result = prime * result + (includeInDetailDisplay ? 1231 : 1237);
        result = prime * result + (includeInQueryResults ? 1231 : 1237);
        result = prime * result + (includeInReports ? 1231 : 1237);
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        UserModifiableFieldMetadata other = (UserModifiableFieldMetadata) obj;
        if (conceptName == null) {
            if (other.conceptName != null)
                return false;
        } else if (!conceptName.equals(other.conceptName))
            return false;
        if (displayName == null) {
            if (other.displayName != null)
                return false;
        } else if (!displayName.equals(other.displayName))
            return false;
        if (displayOrder != other.displayOrder)
            return false;
        if (fieldId != other.fieldId)
            return false;
        if (fieldName == null) {
            if (other.fieldName != null)
                return false;
        } else if (!fieldName.equals(other.fieldName))
            return false;
        if (includeInDetailDisplay != other.includeInDetailDisplay)
            return false;
        if (includeInQueryResults != other.includeInQueryResults)
            return false;
        if (includeInReports != other.includeInReports)
            return false;
        return true;
    }

}
