package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographXref;

import java.util.List;

/**
 * The Interface ConceptXrefDAO.
 */
public interface ConceptXrefDAO extends HibernateDAO<ConceptXref> {

    /**
     * Find dd ix ref by fdb id.
     * 
     * @param fdbId
     *            the fdb id
     * @return the drug drug interaction xref
     */
    DrugDrugInteractionXref findDDIxRefByFDBId(long fdbId);

    /**
     * Findd tx ref by fdb id.
     * 
     * @param fdbId
     *            the fdb id
     * @return the concept xref
     */
    ConceptXref findDTxRefByFDBId(long fdbId);

    /**
     * Find monograph x ref by fdb id.
     * 
     * @param fdbid
     *            the fdbid
     * @return the monograph xref
     */
    MonographXref findMonographxRefByFDBId(MonographKey fdbid);

    /**
     * Finds a Dose Range cross reference by the Fdb Dosing Id.
     * 
     * @param fdbId
     *            an instance of FdbDosingId
     * @return the dose range cross reference
     */
    DoseRangeXref findDoseRangexRefByFDBId(DoseRangeKey fdbId);

    /**
     * Finds the Custom Dose Ranges created from a FDB Record.
     * 
     * @param key
     *            The dose range key
     * @return a list of custom records
     */
    List<DoseRangeVersionVa> findCustomByXref(DoseRangeKey key);

}
