package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographXref;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * This is a test.
 * 
 * @author vhaisaduffg
 * 
 */
@Repository
public final class ConceptXrefDAOImpl extends HibernateDAOImpl<ConceptXref> implements ConceptXrefDAO {
    private static final Log CHILDLOGGER = LogFactory.getLog(ConceptXrefDAOImpl.class);
    private static final String DDI_XREF_QUERY = "findXRefByFDBId";
    private static final String DT_XREF_QUERY = "findDTXRefByFDBId";
    private static final String MONONGRAPH_XREF_QUERY = "findMonographXRefByFDBId";
    private static final String DOSE_RANGE_XREF_QUERY = "findDoseRangeXRefByFDBId";
    private static final String DOSE_RANGE_BY_XREF = "findDoseRangeByxRef";

    @Override
    protected Class<ConceptXref> getPersistentClass() {
        return ConceptXref.class;
    }

    @Override
    @SuppressWarnings("unchecked")
    public DrugDrugInteractionXref findDDIxRefByFDBId(long fdbId) {
        ConceptXref xref = null;

        Query query = getSession().getNamedQuery(DDI_XREF_QUERY);
        query.setLong("interactionId", fdbId);
        List<ConceptXref> xrefs = query.list();
        if (xrefs.size() == 1) {
            xref = xrefs.get(0);
        } else {
            CHILDLOGGER.warn("more than one object exists for dose range key " + fdbId);
        }
        return (DrugDrugInteractionXref) xref;
    }

    @Override
    @SuppressWarnings("unchecked")
    public ConceptXref findDTxRefByFDBId(long fdbId) {
        ConceptXref xref = null;

        Query query = getSession().getNamedQuery(DT_XREF_QUERY);
        query.setLong("dtcId", fdbId);
        List<ConceptXref> xrefs = query.list();
        if (xrefs.size() == 1) {
            xref = xrefs.get(0);
        } else if (xrefs.size() > 1) {
            CHILDLOGGER.warn("more than one object exists for duplicate therapy key " + fdbId);
        } else {
            CHILDLOGGER.warn("No xref found for duplicate therapy id: " + fdbId);
        }
        return xref;
    }

    @SuppressWarnings("unchecked")
    @Override
    public MonographXref findMonographxRefByFDBId(MonographKey fdbId) {
        ConceptXref xref = null;
        Query query = getSession().getNamedQuery(MONONGRAPH_XREF_QUERY);
        query.setLong("monographId", fdbId.getMonographId());
        query.setLong("languageId", fdbId.getLanguageId());
        query.setLong("sequenceNumber", fdbId.getSequenceNumber());
        query.setString("versionCode", fdbId.getVersionCode());
        List<ConceptXref> xrefs = query.list();
        if (xrefs.size() == 1) {
            xref = xrefs.get(0);
        }
        return (MonographXref) xref;

    }

    @SuppressWarnings("unchecked")
    @Override
    public DoseRangeXref findDoseRangexRefByFDBId(DoseRangeKey fdbId) {
        ConceptXref xref = null;

        Query query = getSession().getNamedQuery(DOSE_RANGE_XREF_QUERY);
        query.setLong("ageHighInDays", fdbId.getAgeHighInDays());
        query.setLong("ageLowInDays", fdbId.getAgeLowInDays());
        query.setLong("conceptId", fdbId.getConceptId());
        query.setLong("conceptType", fdbId.getConceptType());
        query.setLong("hitType", fdbId.getHitType());
        query.setString("doseRouteId", fdbId.getDoseRouteId());
        query.setString("doseTypeId", fdbId.getDoseTypeId());
        query.setString("fdbdx", fdbId.getFdbdx());
        List<ConceptXref> xrefs = query.list();
        if (xrefs.size() == 1) {
            xref = xrefs.get(0);
        } else if (xrefs.size() > 1) {
            CHILDLOGGER.warn("more than one object exists for dose range key " + fdbId);
        } else {
            CHILDLOGGER.warn("No xref found for dose range key " + fdbId);
        }
        return (DoseRangeXref) xref;

    }

    @SuppressWarnings("unchecked")
    @Override
    public List<DoseRangeVersionVa> findCustomByXref(DoseRangeKey key) {
        ConceptXref doseRangeXref = findDoseRangexRefByFDBId(key);
        if (doseRangeXref == null) {
            return new ArrayList<DoseRangeVersionVa>();
        }
        Query query = getSession().getNamedQuery(DOSE_RANGE_BY_XREF);
        query.setProperties(doseRangeXref);
        return query.list();

    }

}
