package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;

import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * DoseRangeVersionDAO extends HibernateDAOImpl.
 * 
 * @author DOMAINOVERSD
 * 
 */
@Repository
public class DoseRangeVersionVaDAO extends HibernateDAOImpl<DoseRangeVersionVa> {

    /** The Constant FIND_BY_CONCEPT_ROUTE_TYPE. */
    private final static String FIND_BY_CONCEPT_ROUTE_TYPE = "findCustomDosingByConceptRouteType";

    /** The Constant FIND_BY_ID. */
    private final static String FIND_BY_ID = "findCustomDosingById";

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAOImpl#getPersistentClass()
     */
    @Override
    protected Class<DoseRangeVersionVa> getPersistentClass() {
        return DoseRangeVersionVa.class;
    }

    /**
     * List find.
     * 
     * @param conceptId
     *            - Long
     * @param doseRouteCode
     *            - String
     * @param doseTypeCode
     *            - String
     * @return the list
     */
    @SuppressWarnings("unchecked")
    public List<DoseRangeVersionVa> find(Long conceptId, String doseRouteCode, String doseTypeCode) {
        Query query = getSession().getNamedQuery(FIND_BY_CONCEPT_ROUTE_TYPE);
        query.setParameter("conceptId", conceptId);
        query.setParameter("doseRouteCode", doseRouteCode);
        query.setParameter("doseTypeCode", doseTypeCode);
        return query.list();
    }

    /**
     * Find.
     * 
     * @param conceptId
     *            the concept id
     * @param conceptType
     *            the concept type
     * @param ageLowInDays
     *            the age low in days
     * @param ageHighInDays
     *            the age high in days
     * @param doseRouteId
     *            the dose route id
     * @param doseTypeId
     *            the dose type id
     * @param fdbdx
     *            the fdbdx
     * @return the list
     */
    @SuppressWarnings("unchecked")
    public List<DoseRangeVersionVa> find(Long conceptId, Long conceptType, Long ageLowInDays, Long ageHighInDays,
            String doseRouteId, String doseTypeId, String fdbdx) {
        Query query = getSession().getNamedQuery(FIND_BY_ID);
        query.setParameter("conceptType", conceptType);
        query.setParameter("ageLowInDays", ageLowInDays);
        query.setParameter("ageHighInDays", ageHighInDays);
        query.setParameter("conceptId", conceptId);
        query.setParameter("doseRouteCode", doseRouteId);
        query.setParameter("doseTypeCode", doseTypeId);
        query.setParameter("fdbdx", fdbdx);
        return query.list();
    }

}
