package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVersionVa;

import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * The class DuplicateTherapyVersionVADAO provides methods to find data related to Duplicate Therapy. <br>
 * <br>
 * Hibernate data access for @link{DuplicateTherapyVersionVA} objects.
 * 
 * @see HibernateDAOImpl
 * 
 */
@Repository
public class DuplicateTherapyVersionVaDAO extends HibernateDAOImpl<DuplicateTherapyVersionVa> {
    private static final String FIND_BY_DTCID_CS_CDA = "findByDtcid_CS_CDA";
    private static final String FIND_BY_DTCID = "findByDtcid";

    @Override
    protected final Class<DuplicateTherapyVersionVa> getPersistentClass() {
        return DuplicateTherapyVersionVa.class;
    }

    /**
     * This method finds a list of Duplicate Therapy Version VA objects by dtcid, custom string, custom dup allowance. <br>
     * <br>
     * 
     * @see List<{@link DuplicateTherapyVersionVa}>
     * 
     * @param dtcid
     *            of type Long
     * @param customString
     *            of type String
     * @param customDupAllowance
     *            of type Long
     * 
     * @return List of type <DuplicateTherapyVersionVa>
     */
    @SuppressWarnings("unchecked")
    public final List<DuplicateTherapyVersionVa> find(Long dtcid, String customString, Long customDupAllowance) {
        Query query = getSession().getNamedQuery(FIND_BY_DTCID_CS_CDA);
        query.setParameter("dtcid", dtcid);
        query.setParameter("customString", customString);
        query.setLong("customDupAllowance", customDupAllowance);
        return query.list();
    }

    /**
     * This method finds a list of Duplicate Therapy Version VA objects by dtcid.
     * 
     * {@link DuplicateTherapyVersionVa}
     * 
     * @param dtcid
     *            - Long
     * @return List<DuplicateTherapyVersionVa>
     */
    @SuppressWarnings("unchecked")
    public final List<DuplicateTherapyVersionVa> find(Long dtcid) {
        Query query = getSession().getNamedQuery(FIND_BY_DTCID);
        query.setParameter("dtcid", dtcid);
        return query.list();
    }
}
