package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.va.FdbUpdateReport;

import java.sql.Date;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * Used to persist and retrieve FdbUpdateReport objects to the database.
 * 
 * @author vhaisaduffg
 * 
 */
@Repository
public class FdbUpdateReportDao extends HibernateDAOImpl<FdbUpdateReport> {

    private static final String CLEANUP_QUERY = "cleanupFdbUpdateReport";
    private static final Long REPORT_RETENTION_PERIOD = new Long(1000L * 60L * 60L * 24L * 7L * 8L);
    private static final Logger LOGGER = Logger.getLogger(FdbUpdateReportDao.class);

    /**
     * Return the FdbUpdateReport class as the persistent class that is acted
     * upon.
     * 
     * @return FdbUpdateReport.class
     */
    @Override
    protected Class<FdbUpdateReport> getPersistentClass() {
        return FdbUpdateReport.class;
    }

    /**
     * Deletes reports older than retentionDate from the system.
     * 
     * @param retentionDate
     *            the last date that a report will be kept
     * @return int
     */
    public int cleanup(Date retentionDate) {
        Query query = getSession().getNamedQuery(CLEANUP_QUERY);
        query.setDate("retentionDate", new Date(System.currentTimeMillis() - REPORT_RETENTION_PERIOD));
        LOGGER.debug(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>before removing from list date of removal:"
                + new Date(System.currentTimeMillis() - REPORT_RETENTION_PERIOD));
        int i = query.executeUpdate();
        if (i > 0) {
            LOGGER.debug(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>removed from list");
        } else {
            LOGGER.debug(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>nothing removed from list");
        }

        return i;
    }
}
