package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.customupdate.RecordCounts;

import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * GenericDAO extends HibernateDAOImpl.
 * 
 */
@Repository
public final class GenericDAO extends HibernateDAOImpl<Object> {

    @Override
    protected Class<Object> getPersistentClass() {
        return Object.class;
    }

    /**
     * Find custom update counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the record counts
     */
    public RecordCounts findCustomUpdateCounts(String conceptClassName) {
        RecordCounts recordCounts = new RecordCounts();
        recordCounts.setAddCount(findCustomUpdateAddCounts(conceptClassName));
        recordCounts.setChangeCount(findCustomUpdateChangeCounts(conceptClassName));
        recordCounts.setDeleteCount(findCustomUpdateDeleteCounts(conceptClassName));
        return recordCounts;
    }

    /**
     * Find custom update counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the record counts
     */
    public RecordCounts findCustomUpdateCountsDDIStrings(String conceptClassName) {
        RecordCounts recordCounts = new RecordCounts();
        recordCounts.setAddCount(findDrugDrugInteractionStringsCustomUpdateAddCounts(conceptClassName));
        recordCounts.setDeleteCount(findDrugDrugInteractionStringsCustomUpdateDeleteCounts(conceptClassName));
        return recordCounts;
    }

    /**
     * Find custom update add records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findCustomUpdateAddRecords(String conceptClassName) {
        String addHql = getCustomUpdateAddHQL(conceptClassName);
        Query results = getSession().createQuery(addHql);
        return results.list();
    }

    /**
     * Find custom update change records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findCustomUpdateChangeRecords(String conceptClassName) {
        String changeHql = getCustomUpdateChangeHQL(conceptClassName);
        Query results = getSession().createQuery(changeHql);
        return results.list();
    }

    /**
     * Find custom update delete records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findCustomUpdateDeleteRecords(String conceptClassName) {
        String deleteHql = getCustomUpdateDeleteHQL(conceptClassName);
        Query results = getSession().createQuery(deleteHql);
        return results.list();
    }

    /**
     * Find custom update delete records ddiStrings.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugDrugInteractionStringsCustomUpdateDeleteRecords(String conceptClassName) {
        String deleteHql = getCustomUpdateDeleteDrugDrugInteractionStringsHQL(conceptClassName);
        Query results = getSession().createQuery(deleteHql);
        return results.list();
    }

    /**
     * Find custom update add counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    private Long findCustomUpdateAddCounts(String conceptClassName) {

        String addHql = getCountPrefixHQL() + getCustomUpdateAddHQL(conceptClassName);

        Query results = getSession().createQuery(addHql);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);

    }

    /**
     * Find custom update delete counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    private Long findCustomUpdateDeleteCounts(String conceptClassName) {

        /** The Constant DELETE_HQL. */
        String deleteHQL = getCountPrefixHQL() + getCustomUpdateDeleteHQL(conceptClassName);
        Query results = getSession().createQuery(deleteHQL);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);
    }

    /**
     * Find custom update change counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    private Long findCustomUpdateChangeCounts(String conceptClassName) {
        String changeHQL = getCountPrefixHQL() + getCustomUpdateChangeHQL(conceptClassName);
        Query results = getSession().createQuery(changeHQL);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);
    }

    /**
     * Find custom update full counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    public Long findCustomUpdateFullCounts(String conceptClassName) {

        String FULL_HQL = getCountPrefixHQL() + getCustomUpdateFullHQL(conceptClassName);
        Query results = getSession().createQuery(FULL_HQL);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);
    }

    /**
     * Find custom update full counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    public Long findDrugDrugInteractionStringsCustomUpdateFullCounts(String conceptClassName) {

        String FULL_HQL = getCountPrefixHQL() + getCustomUpdateDrugDrugInteractionStringsFullHQL(conceptClassName);
        Query results = getSession().createQuery(FULL_HQL);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);
    }

    /**
     * Find drug drug interaction strings custom update add counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    public Long findDrugDrugInteractionStringsCustomUpdateAddCounts(String conceptClassName) {

        String ADD_HQL = getCountPrefixHQL() + getCustomUpdateAddDrugDrugInteractionStringsHQL(conceptClassName);
        Query results = getSession().createQuery(ADD_HQL);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);
    }

    /**
     * Find drug drug interaction strings custom update delete counts.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the long
     */
    public Long findDrugDrugInteractionStringsCustomUpdateDeleteCounts(String conceptClassName) {

        String DELETE_HQL = getCountPrefixHQL() + getCustomUpdateDeleteDrugDrugInteractionStringsHQL(conceptClassName);
        Query results = getSession().createQuery(DELETE_HQL);
        if (results.iterate().hasNext()) {
            return (Long) results.iterate().next();
        }
        return new Long(0);
    }

    /**
     * Find custom update records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findCustomUpdateRecords(String conceptClassName) {
        String FULL_HQL = getCustomUpdateFullHQL(conceptClassName);
        Query results = getSession().createQuery(FULL_HQL);
        return results.list();
    }

    /**
     * Find custom update records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugDrugInteractionStringsCustomUpdateFullRecords(String conceptClassName) {
        String FULL_HQL = getCustomUpdateDrugDrugInteractionStringsFullHQL(conceptClassName);
        Query results = getSession().createQuery(FULL_HQL);
        return results.list();
    }

    /**
     * Gets the custom update full hql.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the custom update full hql
     */
    private String getCustomUpdateFullHQL(String conceptClassName) {
        return "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " m "
                + "where m.exportManager.exportableVersion.status.code = 'Approved'";
    }

    /**
     * Specialized get custom update add hql for ddiStrings..
     * 
     * @param conceptClassName
     *            String
     * @return the custom update full hql
     * 
     * */
    private String getCustomUpdateDrugDrugInteractionStringsFullHQL(String conceptClassName) {
        return "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " d "
                + "where d.exportManager.exportableVersion.status.code = 'Approved' and (d.fdbId is not null and d.fdbId > 0)";
    }

    /**
     * Gets the custom update add hql.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the custom update add hql
     */
    private String getCustomUpdateAddHQL(String conceptClassName) {
        return "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " m " + "where m in (" + "select m1 "
                + "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " m1 "
                + "where m1.exportManager.exportableVersion.status.code = 'Approved' "
                + "  and m1.exportManager.exportedVersion.status.code = 'Deleted')" + "or m in (" + "select m2 "
                + "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " m2 "
                + "where m2.exportManager.exportableVersion.status.code = 'Approved' "
                + "  and m2.exportManager.exportedVersion is null)";

    }

    /**
     * Specialized get custom update add hql for ddiStrings.
     * 
     * @param conceptClassName
     *            String
     * @return custom update delete hql.
     * */
    private String getCustomUpdateAddDrugDrugInteractionStringsHQL(String conceptClassName) {
        return "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " d " + "where d in (" + "select d1 "
                + "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " d1 "
                + "where d1.exportManager.exportableVersion.status.code = 'Approved' "
                + "  and d1.exportManager.exportedVersion.status.code = 'Deleted' "
                + "  and (d1.fdbId is not null and d1.fdbId > 0)) " + "or d in (" + "select d2 "
                + "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " d2 "
                + "where d2.exportManager.exportableVersion.status.code = 'Approved' "
                + "  and d2.exportManager.exportedVersion is null" + "  and (d2.fdbId is not null and d2.fdbId > 0))";

    }

    /**
     * Specialized get custom update delete hql for ddiStrings.
     * 
     * @param conceptClassName
     *            String
     * @return custom update delete hql.
     * 
     * */
    private String getCustomUpdateDeleteDrugDrugInteractionStringsHQL(String conceptClassName) {
        return "from gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa d "
                + "where d.exportManager.exportableVersion.status.code = 'Deleted' "
                + "  and d.exportManager.exportedVersion.status.code = 'Approved' "
                + "  and (d.fdbId is not null and d.fdbId > 0)";
    }

    /**
     * Gets the custom update delete hql.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the custom update delete hql
     */
    private String getCustomUpdateDeleteHQL(String conceptClassName) {

        return "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " m "
                + "where m.exportManager.exportableVersion.status.code = 'Deleted' "
                + "  and m.exportManager.exportedVersion.status.code = 'Approved'";
    }

    /**
     * Gets the custom update change hql.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the custom update change hql
     */
    private String getCustomUpdateChangeHQL(String conceptClassName) {

        return "from gov.va.med.pharmacy.pecs.domain.impl.va." + conceptClassName + " m "
                + "where m.exportManager.exportableVersion.status.code = 'Approved' "
                + "  and m.exportManager.exportedVersion.status.code = 'Approved' "
                + "  and m.exportManager.exportableVersion != m.exportManager.exportedVersion";
    }

    /**
     * Find custom update add records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugPairsCustomUpdateAddRecords(String conceptClassName) {
        String addHql = getDrugPairPrefixHQL() + getCustomUpdateAddHQL(conceptClassName);
        Query results = getSession().createQuery(addHql);
        return results.list();
    }

    /**
     * Find custom update add records ddiStrings.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugDrugInteractionStringsCustomUpdateAddRecords(String conceptClassName) {
        String addHql = getCustomUpdateAddDrugDrugInteractionStringsHQL(conceptClassName);
        Query results = getSession().createQuery(addHql);
        return results.list();
    }

    /**
     * Find custom update change records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugPairsCustomUpdateChangeRecords(String conceptClassName) {
        String changeHql = getDrugPairPrefixHQL() + getCustomUpdateChangeHQL(conceptClassName);
        Query results = getSession().createQuery(changeHql);
        return results.list();
    }

    /**
     * Find custom update delete records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugPairsCustomUpdateDeleteRecords(String conceptClassName) {
        String deleteHql = getDrugPairPrefixHQL() + getCustomUpdateDeleteHQL(conceptClassName);
        Query results = getSession().createQuery(deleteHql);
        return results.list();
    }

    /**
     * Find custom update records.
     * 
     * @param conceptClassName
     *            the concept class name
     * @return the list
     */
    public List<?> findDrugPairsCustomUpdateRecords(String conceptClassName) {
        String FULL_HQL = getDrugPairPrefixHQL() + getCustomUpdateFullHQL(conceptClassName);
        Query results = getSession().createQuery(FULL_HQL);
        return results.list();
    }

    /**
     * Gets the count prefix hql.
     * 
     * @return the count prefix hql
     */
    private String getCountPrefixHQL() {
        return "select count(*) ";
    }

    /**
     * Gets the drug pair prefix hql.
     * 
     * @return the drug pair prefix hql
     */
    private String getDrugPairPrefixHQL() {
        return "select m.routedGeneric1.rtGenId, m.routedGeneric2.rtGenId, m.interaction.interactionId ";
    }

}
