package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.Page;

import java.io.Serializable;
import java.util.List;

import org.hibernate.Session;

/**
 * HibernateDAO entity interface.
 * 
 * @author DOMAINOVERSD
 * 
 * @param <T>
 */
public interface HibernateDAO<T> {

    /**
     * Session Get Session.
     * 
     * @return session
     */
    public Session getSession();

    /**
     * findByQueryByPage.
     * 
     * @param query
     *            - String
     * @param page
     *            - int
     * @return Query
     */
    Page findByQueryByPage(String query, int page);

    /**
     * List Find All.
     * 
     * @param className
     *            CLass<X>
     * 
     * @param <X>
     *            CLass
     * @return class
     */
    <X> List<X> findAll(Class<X> className);

    /**
     * findByQuery.
     * 
     * @param query
     *            - String
     * @param clz
     *            Class<X>
     * @param <X>
     *            Class
     * @return List<X>
     */
    <X> List<X> findByQuery(String query, Class<X> clz);

    /**
     * Find All.
     * 
     * @return List<T>
     */
    List<T> findAll();

    /**
     * fidnById.
     * 
     * @param id
     *            Serializable
     * @return found T
     */
    T findById(Serializable id);

    /**
     * T merge.
     * 
     * @param detachedInstance
     *            - T
     * @return merge T
     */
    T merge(T detachedInstance);

    /**
     * Object instance save or update.
     * 
     * @param instance
     *            - Object
     */
    void saveOrUpdate(Object instance);

    /**
     * Serializable Save.
     * 
     * @param transientInstance
     *            - Object
     * @return transientInstance.
     */
    Serializable save(Object transientInstance);

    /**
     * delete persistent Instance.
     * 
     * @param persistentInstance
     *            - Object
     */
    void delete(Object persistentInstance);

    /**
     * Results Counter.
     * 
     * @param hqlQuery
     *            - String
     * @return resultCount
     */
    long resultCount(String hqlQuery);

    /**
     * List<?> findByQuery.
     * 
     * @param query
     *            ConceptQuery
     * @param custom
     *            boolean
     * @return Query
     */
    List<?> findByQuery(ConceptQuery query, boolean custom);

    /**
     * Update.
     * 
     * @param detachedInstance
     *            the detached instance
     */
    void update(Object detachedInstance);
}
