package gov.va.med.pharmacy.pecs.persistence.event.hibernate;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosing;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;

import javax.annotation.Resource;

import org.hibernate.event.PostLoadEvent;

/**
 * Listens for Post Load events from Hibernate and processes the object just loaded if needed.
 * 
 * @author vhaisaduffg
 * 
 */
public class PostLoadEventListener extends AbstractEventListener implements org.hibernate.event.PostLoadEventListener {

    private static final long serialVersionUID = -43454879807645228L;
    @Resource
    private DrugConceptDao drugConceptDao;

    /**
     * Called by Hibernate when an object is loaded.
     * 
     * @param event
     *            PostLoadEvent
     */
    @Override
    public final void onPostLoad(PostLoadEvent event) {
        processEntity(event.getEntity());
        processDoseRange(event.getEntity());

    }

    /**
     * Processes the object if the entity is a DoseRangeVA or FdbDosing object.
     * 
     * @param entity
     *            Object
     */
    private void processDoseRange(Object entity) {
        if (entity instanceof DoseRangeVersionVa) {
            process((DoseRangeVersionVa) entity);
        } else if (entity instanceof FdbDosing) {
            process((FdbDosing) entity);
        }
    }

    /**
     * Retrieves the Concept Description associated with a DoseRangeVA object and populates the description in the object.
     * 
     * @param entity
     *            DoseRangeVa
     */
    private void process(DoseRangeVersionVa entity) {
        entity.setConceptDescription(drugConceptDao.getDrugConceptDescription(entity.getConceptid(), entity.getConceptType()));
    }

    /**
     * Retrieves the Concept Description associated with a FdbDosing object and populates the description in the object.
     * 
     * @param entity
     *            FdbDosing entity
     */
    private void process(FdbDosing entity) {
        entity.setConceptDescription(drugConceptDao.getDrugConceptDescription(entity.getConceptId(), entity.getConceptType()));
    }

    /**
     * Sets the DrugConceptDao.
     * 
     * @param dao
     *            DrugConceptDao
     */
    public final void setDrugConceptDao(DrugConceptDao dao) {
        drugConceptDao = dao;
    }

    /**
     * Checks to make sure all injectable beans have been injected.
     * 
     * @throws Exception
     *             if all injectable beans have not been injected
     */
    public final void afterPropertiesSet() throws Exception {
        if (drugConceptDao == null) {
            throw new Exception("DrugConceptDao has not been injected in to DoseRangePostLoadEventListener.");
        }
    }
}
