package gov.va.med.pharmacy.pecs.query;

import java.util.Collections;
import java.util.List;

/**
 * Modifies the query condition to ensure that only current versions are included.
 * 
 * @author vhaisaalbua
 */
public final class CurrentRecordsVisitor implements QueryConditionVisitor {

    /** The crt records cond. */
    private final QueryCondition CRT_RECORDS_COND = new HibernateCondition(
            "versionId = versionedObject.exportManager.currentVersion.versionId");

    /** The result. */
    private QueryCondition result;

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#getResult()
     */
    @Override
    public QueryCondition getResult() {
        return result;
    }

    /**
     * List of Query Conditions - visitCondition.
     * 
     * @param cond
     *            QueryCondition.
     * @return empty List.
     */
    private List<QueryCondition> visitCondition(QueryCondition cond) {
        result =
                ConjunctionCondition.createConjunctionCondition(Conjunction.AND, CRT_RECORDS_COND, new ParenthesisCondition(
                        cond));
        return Collections.emptyList();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitAndConjunctionCondition(gov.va.med.pharmacy.pecs.query.
     * AndConjunctionCondition)
     */
    @Override
    public List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        return visitCondition(cond);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitFieldCondition(gov.va.med.pharmacy.pecs.query.FieldCondition)
     */
    @Override
    public List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        return visitCondition(cond);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitOrConjunctionCondition(gov.va.med.pharmacy.pecs.query.
     * OrConjunctionCondition)
     */
    @Override
    public List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        return visitCondition(cond);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitHibernateCondition(gov.va.med.pharmacy.pecs.query.
     * HibernateCondition)
     */
    @Override
    public List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        return visitCondition(cond);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitParenthesisCondition(gov.va.med.pharmacy.pecs.query.
     * ParenthesisCondition)
     */
    @Override
    public List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        return visitCondition(cond.getNested());
    }

}
