package gov.va.med.pharmacy.pecs.query;


import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Dojo Grid Support.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class DoJoGridSuport {

    /** The pattern. */
    private static Pattern pattern = Pattern.compile("(\\[([\\\\/])@(\\[[^\\[\\]]+\\]]))?\\[(\\d*):(\\d+)\\]");

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(DoJoGridSuport.class);

    /** The json. */
    private final String json;

    /** The sort column. */
    private String sortColumn;

    /** The sort asc. */
    private boolean sortAsc;

    /** The start row. */
    private int startRow;

    /** The end row. */
    private int endRow;

    /** The parsed. */
    private boolean parsed;

    /**
     * DoJo Grid Support.
     * 
     * @param query
     *            - String
     */
    public DoJoGridSuport(String query) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("Processing JSON query: " + query);
        }
        List<String> parts = splitJsonString(query);
        json = parts.get(0);

        if (parts.size() > 1) {
            Matcher matcher = pattern.matcher(parts.get(1));
            if (matcher.find()) {
                if (matcher.group(1) != null) {
                    if ("/".equals(matcher.group(2))) {
                        sortAsc = true;
                    } else {
                        sortAsc = false;
                    }
                    //@formatter:off
                    sortColumn = matcher.group(3).substring(2, matcher.group(3).length() - 3);
                }
                startRow = "".equals(matcher.group(4)) ? 0 : Integer.valueOf(matcher.group(4));
                endRow = Integer.valueOf(matcher.group(5));
                //@formatter:on
            } else {
                LOGGER.error("Couldn't parse extra query information: " + parts.get(1));
            }
            parsed = true;
        } else {
            LOGGER.error("Couldn't parse query information: " + query);
        }
    }

    /**
     * Looks for and splits a JSON String using a delimiter of "]/[". This delimiter is being used to account for paging
     * information being sent in an AJAX request. This will take in to account a field that contains a slash. If the delimiter
     * is not found, the entire String is returned.
     * 
     * @param jsonString
     *            - String
     * @return retrun the new String
     */
    private List<String> splitJsonString(String jsonString) {
        List<String> retString = new ArrayList<String>();
        final String delim = "]/[";

        int delimIndex = jsonString.indexOf(delim);
        if (delimIndex == -1) {
            retString.add(jsonString);
        } else {
            retString.add(jsonString.substring(0, ++delimIndex));
            retString.add(jsonString.substring(++delimIndex));
        }
        return retString;
    }

    /**
     * Get Json.
     * 
     * @return json
     */
    public String getJson() {
        return json;
    }

    /**
     * Get String Sort Column.
     * 
     * @return sortColumn
     */
    public String getSortColumn() {
        return sortColumn;
    }

    /**
     * Sort Asc.
     * 
     * @return sortAsc
     */
    public boolean sortAsc() {
        return sortAsc;
    }

    /**
     * Gets the start row.
     * 
     * @return the start row
     */
    public int getStartRow() {
        return startRow;
    }

    /**
     * Get EndRow.
     * 
     * @return endRow
     */
    public int getEndRow() {
        return endRow;
    }

    /**
     * isParsed.
     * 
     * @return parsed
     */
    public boolean isParsed() {
        return parsed;
    }

    /**
     * hasSorting.
     * 
     * @return sortColumn when not null
     */
    public boolean hasSorting() {
        return sortColumn != null;
    }

    /**
     * Get Page Number.
     * 
     * @return the Row / Default Size
     */
    public int getPageNumber() {
        return (getStartRow() + 1) / Page.DEFAULT_SIZE + 1;
    }
}
