package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import java.util.List;

/**
 * FieldCondition implements QueryCondition.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class FieldCondition implements QueryCondition {
    private ConceptField field;
    private final ConditionOperator operator;
    private Object value;

    /**
     * FieldCondition.
     * 
     * @param f
     *            -ConceptField
     * @param op
     *            -ConditionOperator
     * @param val
     *            - Object
     */
    public FieldCondition(ConceptField f, ConditionOperator op, Object val) {
        field = f;
        operator = op;
        value = val;
    }

    @Override
    public FieldCondition copy() {
        FieldCondition dolly = new FieldCondition(field, operator, value);
        return dolly;
    }

    @Override
    public StringBuffer toHQL(boolean custom) {
        StringBuffer res = new StringBuffer();
        if (String.class.equals(field.getType())) {
            res.append("lower(");
        }
        res.append(field.getPersistentPath(custom));
        if (String.class.equals(field.getType())) {
            res.append(")");
        }
        res.append(" ").append(operator.getSymbol()).append(" ");
        res.append(operator.formatHQLValue(value, field));

        return res;
    }

    @Override
    public StringBuffer toXML() {
        StringBuffer res = new StringBuffer();
        res.append(openElement(CONDITION));

        res.append(openElement(FIELD));
        res.append(field.getName());
        res.append(closeElement(FIELD));

        res.append(openElement(OPERATOR));
        res.append(operator.getName());
        res.append(closeElement(OPERATOR));

        res.append(openElement(VALUE));
        res.append(operator.formatXMLValue(value, field));
        res.append(closeElement(VALUE));

        res.append(closeElement(CONDITION));
        return res;
    }

    @Override
    public StringBuffer toText() {
        StringBuffer res = new StringBuffer();
        res.append(field.getDisplayName());
        res.append(" ").append(operator.getSymbol()).append(" ");
        res.append(operator.formatXMLValue(value, field));
        return res;
    }

    /**
     * OpenElement.
     * 
     * @param name
     *            String
     * @return < name >
     */
    private String openElement(String name) {
        return "<" + name + ">";
    }

    /**
     * closeElement.
     * 
     * @param name
     *            - String
     * @return < name >
     */
    private String closeElement(String name) {
        return "</" + name + ">";
    }

    @Override
    public QueryCondition combine(QueryCondition cond, Conjunction conj) {
        return ConjunctionCondition.createConjunctionCondition(conj, this, cond);
    }

    @Override
    public List<QueryCondition> accept(QueryConditionVisitor visitor) {
        return visitor.visitFieldCondition(this);
    }

    /**
     * Get ConceptField.
     * 
     * @return field
     */
    public ConceptField getField() {
        return field;
    }

    /**
     * Set ConceptField.
     * 
     * @param field
     *            ConceptField
     * 
     */
    public void setField(ConceptField field) {
        this.field = field;
    }

    /**
     * Get Condition Operator.
     * 
     * @return operator
     */
    public ConditionOperator getOperator() {
        return operator;
    }

    /**
     * Get Object Value.
     * 
     * @return value
     */
    public Object getValue() {
        return value;
    }

    /**
     * Set Object Value.
     * 
     * @param value
     *            Object
     */
    public void setValue(Object value) {
        this.value = value;
    }

    @Override
    public String toString() {
        return toText().toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((field == null) ? 0 : field.hashCode());
        result = prime * result + ((operator == null) ? 0 : operator.hashCode());
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o instanceof FieldCondition) {
            FieldCondition other = (FieldCondition) o;
            return field.equals(other.field) && operator == other.operator
                    && (value == null ? other.value == null : value.equals(other.value));
        }
        return false;
    }
}
