package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import java.util.ArrayList;
import java.util.List;

/**
 * InOperator extends AnyTypeOperator.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class InOperator extends AnyTypeOperator {
    @Override
    public String getName() {
        return IN;
    }

    @Override
    public String getSymbol() {
        return "IN";
    }

    @Override
    public Object parseValue(String val, ConceptField field) {
        // this assumes val is a list of comma-separated values
        // of the appropriate type for the given field; individual
        // field values cannot contain commas
        List<Object> list = new ArrayList<Object>();
        for (String s : val.split(",")) {
            s = s.trim();
            // special case
            if ("null".equalsIgnoreCase(s)) {
                list.add(null);
            } else {
                list.add(super.parseValue(s, field));
            }
        }
        return list;
    }

    @SuppressWarnings("unchecked")
    @Override
    public String formatHQLValue(Object val, ConceptField field) {
        // the value should be a list
        List<Object> list = (List<Object>) val;
        StringBuffer buf = new StringBuffer();
        for (Object v : list) {
            if (buf.length() > 0) {
                buf.append(',');
            } else {
                buf.append('(');
            }
            buf.append(super.formatHQLValue(v, field));
        }
        buf.append(')');

        return buf.toString();
    }

    @SuppressWarnings("unchecked")
    @Override
    public String formatXMLValue(Object val, ConceptField field) {
        // the value should be a list
        List<Object> list = (List<Object>) val;
        StringBuffer buf = new StringBuffer();
        for (Object v : list) {
            if (buf.length() > 0) {
                buf.append(',');
            }
            buf.append(super.formatXMLValue(v, field));
        }

        return buf.toString();
    }
}
