package gov.va.med.pharmacy.pecs.query;

import java.util.Collections;
import java.util.List;

/**
 * A visitor that modifies a drug pair query so that it becomes insensitive to the order of drugs in a drug pair.
 * 
 * @author vhaisaalbua
 */
public final class MonographVisitor implements QueryConditionVisitor {
    private final QueryCondition TITLE_SECTION_COND = new HibernateCondition("sectioncode = 'T'");
    private QueryCondition result;

    @Override
    public QueryCondition getResult() {
        return result;
    }

    /**
     * Visit Condition is a List of Query Conditions.
     * 
     * @param cond
     *            QueryCondition.
     * @return Collecion.emptyList
     */
    private List<QueryCondition> visitCondition(QueryCondition cond) {
        result =
                ConjunctionCondition.createConjunctionCondition(Conjunction.AND, TITLE_SECTION_COND, new ParenthesisCondition(
                        cond));
        return Collections.emptyList();
    }

    @Override
    public List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        return visitCondition(cond);
    }

    @Override
    public List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        return visitCondition(cond);
    }

    @Override
    public List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        return visitCondition(cond);
    }

    @Override
    public List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        return visitCondition(cond);
    }

    @Override
    public List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        return visitCondition(cond.getNested());
    }

}
