package gov.va.med.pharmacy.pecs.query;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * OperatorFactory entity.
 * 
 * @author DOMAINOVERSD.
 * 
 */
public final class OperatorFactory {
    private static OperatorFactory instance = new OperatorFactory();
    private final Map<String, ConditionOperator> mappings;

    /**
     * OperatorFactory Constructor.
     * 
     */
    private OperatorFactory() {
        List<ConditionOperator> operators =
                Arrays.asList(new ConditionOperator[] { new EqualOperator(), new NotEqualOperator(), new GreaterThanOperator(),
                        new GreaterThanOrEqualOperator(), new LessThanOperator(), new LessThanOrEqualOperator(),
                        new ContainsOperator(), new InOperator(), new BeginsWithOperator(), new EndsWithOperator() });
        mappings = new HashMap<String, ConditionOperator>();
        for (ConditionOperator op : operators) {
            mappings.put(op.getName(), op);
        }
    }

    /**
     * Operator Factory Get Instance.
     * 
     * @return instance
     */
    public static OperatorFactory getInstance() {
        return instance;
    }

    /**
     * Get Operator.
     * 
     * @param name
     *            - String
     * @return mappings of object name
     */
    public ConditionOperator getOperator(String name) {
        return mappings.get(name);
    }

}
