package gov.va.med.pharmacy.pecs.query;

import java.util.List;

/**
 * Parenthesis Condition entity implements QueryCondition.
 * 
 * @author DOMAINOVERSD.
 * 
 */
public final class ParenthesisCondition implements QueryCondition {
    private final QueryCondition nested;

    /**
     * ParenthesisCondition Constructor.
     * 
     * @param cond
     *            QueryCondition.
     */
    public ParenthesisCondition(QueryCondition cond) {
        nested = cond;
    }

    @Override
    public List<QueryCondition> accept(QueryConditionVisitor visitor) {
        return visitor.visitParenthesisCondition(this);
    }

    @Override
    public QueryCondition combine(QueryCondition cond, Conjunction conj) {
        return ConjunctionCondition.createConjunctionCondition(conj, this, cond);
    }

    @Override
    public QueryCondition copy() {
        return new ParenthesisCondition(nested.copy());
    }

    /**
     * QueryCondition Get Nested Object.
     * 
     * @return nested
     */
    public QueryCondition getNested() {
        return nested;
    }

    @Override
    public StringBuffer toHQL(boolean custom) {
        StringBuffer res = new StringBuffer("(");
        res.append(nested.toHQL(custom)).append(")");
        return res;
    }

    @Override
    public StringBuffer toText() {
        StringBuffer res = new StringBuffer("(");
        res.append(nested.toText()).append(")");
        return res;
    }

    @Override
    public StringBuffer toXML() {
        return nested.toXML();
    }

}
