package gov.va.med.pharmacy.pecs.query;

import java.util.List;

/**
 * A visitor that builds a string representation of this query.
 * 
 * @author vhaisaalbua
 */
public abstract class StringSerializationVisitor implements QueryConditionVisitor {
    private QueryCondition result;
    private boolean firstPass = true;

    @Override
    public QueryCondition getResult() {
        return result;
    }

    /**
     * Is First Pass.
     * 
     * @param cond
     *            -QueryCondition
     * @return boolean T/F
     */
    private boolean isFirstPass(QueryCondition cond) {
        if (firstPass) {
            firstPass = false;
            // remember original query condition;
            // this is what we want getResult() to return
            // (so that this visitor has no effect of the query state)
            result = cond;
            return true;
        }
        return false;
    }

    @Override
    public List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        return visitConjunctionCondition(cond, Conjunction.AND);
    }

    @Override
    public List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        return visitConjunctionCondition(cond, Conjunction.OR);
    }

    /**
     * Visit Conjunction Condition.
     * 
     * @param cond
     *            -ConjunctionCondition
     * @param conj
     *            -Conjunction
     * @return doVisitConjunctionCondition(cond, conj)
     */
    protected List<QueryCondition> visitConjunctionCondition(ConjunctionCondition cond, Conjunction conj) {
        isFirstPass(cond);
        return doVisitConjunctionCondition(cond, conj);
    }

    @Override
    public List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        isFirstPass(cond);
        return doVisitFieldCondition(cond);
    }

    @Override
    public List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        isFirstPass(cond);
        return doVisitHibernateCondition(cond);
    }

    /**
     * List Do Visit Conjunction Condition.
     * 
     * @param cond
     *            - ConjunctionCondition
     * @param conj
     *            - Condition
     * @return - abstract
     */
    protected abstract List<QueryCondition> doVisitConjunctionCondition(ConjunctionCondition cond, Conjunction conj);

    /**
     * List<QueryCondition> Do Visit Field Condition.
     * 
     * @param cond
     *            FieldCondition
     * @return abstract list
     */
    protected abstract List<QueryCondition> doVisitFieldCondition(FieldCondition cond);

    /**
     * List<QueryCondition> Do Visity Hobernate Condition.
     * 
     * @param cond
     *            - HibernateCondition
     * @return abstract List
     */
    protected abstract List<QueryCondition> doVisitHibernateCondition(HibernateCondition cond);

    // Force subclasses to implement
    @Override
    public abstract String toString();

}
