package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * XmlConceptQueryFactory Entity. Implements Concept Query Factory
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class XmlConceptQueryFactory implements ConceptQueryFactory {
    private static final Logger LOGGER = Logger.getLogger(XmlConceptQueryFactory.class);

    @SuppressWarnings("unchecked")
    @Override
    public ConceptQuery createQuery(ConceptMetadata meta, String xml) {
        try {
            Document document = DocumentHelper.parseText(xml);
            Element root = document.getRootElement();
            List<Element> conditionEls = root.elements();
            if (conditionEls.isEmpty()) {
                return new ConceptQuery(meta);
            }

            Iterator<Element> iter = conditionEls.iterator();
            QueryCondition condition = parseConditionEl(meta, iter.next());
            ConceptQuery query = new ConceptQuery(meta, condition);

            for (; iter.hasNext();) {
                Conjunction conj = parseConjunctionEl(iter.next());
                QueryCondition cond = parseConditionEl(meta, iter.next());
                query.addCondition(cond, conj);
            }

            return query;
        } catch (Exception e) {
            String msg = "Error parsing JSON data";
            LOGGER.error(msg, e);
            throw new RuntimeException(msg, e);
        }
    }

    /**
     * Query Condition.
     * 
     * @param meta
     *            -ConceptMetadata
     * @param conditionEl
     *            -Element
     * @return FieldCondition
     */
    private QueryCondition parseConditionEl(ConceptMetadata meta, final Element conditionEl) {
        String fieldName = conditionEl.elementTextTrim(QueryCondition.FIELD);
        ConceptField field = meta.getField(fieldName);
        if (field == null) {
            throw new IllegalArgumentException("Unknown field " + fieldName);
        }
        ConditionOperator operator =
                OperatorFactory.getInstance().getOperator(conditionEl.elementTextTrim(QueryCondition.OPERATOR));
        String value = conditionEl.elementTextTrim(QueryCondition.VALUE);

        return new FieldCondition(field, operator, operator.parseValue(value, field));
    }

    /**
     * Parse Conjunction.
     * 
     * @param elem
     *            -String
     * @return Conjunction of C
     */
    private static Conjunction parseConjunctionEl(final Element elem) {
        String c = elem.getTextTrim();
        return Conjunction.valueOf(c);
    }

}
