package gov.va.med.pharmacy.pecs.rules;

import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

/**
 * An abstraction representing business rules governing data consistency (the
 * name for this interface is probably too generic). The only rules that can be
 * represented are the ones that are concerned with data for an individual
 * object (constraints like uniqueness of a fields across all customizations
 * cannot be checked).
 */

public interface BusinessRule {
    /**
     * Ensure that the changes made to an object to this method are consistent
     * with this rule.
     * 
     * @param crt
     *            the data entered by a user
     * @param prev
     *            the corresponding data before the user modifications
     * @return an indication about the result of the check
     */
    ValidationResult validate(Object crt, Object prev);

    /**
     * Configure the rule with the metadata needed to inspect the objects that
     * will be validated at run-time.
     * 
     * @param meta
     *            the concept metadata
     */
    void setMetadata(ConceptMetadata meta);
}
