package gov.va.med.pharmacy.pecs.rules;

import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

/**
 * A trigger that checks conditions necessary for evaluating business rules.
 */
public interface BusinessRuleTrigger {
    /**
     * Decides whether the conditions are met for a business rule to fire.
     * 
     * @param crt
     *            the data entered by a user
     * @param prev
     *            the corresponding data before the user modifications
     * @param action
     *            the action the user is attempting to perform
     * @return true if conditions are met for the business rule evaluation,
     *         false otherwise
     */
    boolean shouldFire(Object crt, Object prev, String action);

    /**
     * Configure the trigger with the metadata needed to inspect the objects at
     * run-time.
     * 
     * @param meta
     *            the concept metadata
     */
    void setMetadata(ConceptMetadata meta);
}
