package gov.va.med.pharmacy.pecs.rules;

import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.util.List;

/**
 * An attempt to encapsulate the enforcement of various business rules for a concept. Business rules are registered along with
 * triggers that verify when they should be checked. At run time the caller will present object to be validated for the
 * application of a specific action.
 */
public interface BusinessRulesValidation {
    /**
     * Set up the business rules.
     * 
     * @param rule
     *            the business rule that needs to be validated
     * @param trigger
     *            the trigger on which the business rule validation fires
     */
    void registerRule(BusinessRule rule, BusinessRuleTrigger trigger);

    /**
     * Invoke run-time business rule validation.
     * 
     * @param crt
     *            the data entered by a user
     * @param prev
     *            the corresponding data before the user modifications
     * @param action
     *            the action the user is attempting to perform
     * @return a list of validation results
     */
    List<ValidationResult> validate(Object crt, Object prev, String action);

    /**
     * The metadata for the fields of associated concept.
     * 
     * @return the metadata
     */
    ConceptMetadata getMetadata();
}
