package gov.va.med.pharmacy.pecs.rules;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import org.apache.log4j.Logger;

/**
 * A rule that verifies that no non-internal fields were changed.
 */
public final class InternalFieldsRule extends AbstractBusinessRuleImpl {
    private static final Logger LOGGER = Logger.getLogger(InternalFieldsRule.class);

    /** InteranlFieldsRule Basic Constructor. */
    public InternalFieldsRule() {
    }

    /**
     * InternalFieldsRule Constructor.
     * 
     * @param msg
     *            - String
     */
    public InternalFieldsRule(String msg) {
        setDefaultMessage(msg);
    }

    @Override
    public ValidationResult validate(Object crt, Object prev) {
        ValidationResult res = new ValidationResult(this);
        for (ConceptField f : getMeta().getFields()) {
            if (!f.isInternal()) {
                Object prevVal = f.getValue(prev);
                Object crtVal = f.getValue(crt);
                if (!equal(prevVal, crtVal)) {
                    LOGGER.info("Field " + f.getName() + " was modified in object " + crt);
                    if (res.getMessages().isEmpty() && getDefaultMessage() != null) {
                        res.addMessage(getDefaultMessage());
                    }
                    res.addMessage(f.getDisplayName() + " field cannot be changed for this action");
                }
            }
        }

        return res;
    }

}
