package gov.va.med.pharmacy.pecs.rules;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

/**
 * Checks whether a field matches a given regex. A supplied error message will be included in case of failure, to make it
 * understandable by users.
 */
public class RegexRule extends AbstractBusinessRuleImpl {
    private static final Logger LOGGER = Logger.getLogger(RegexRule.class);
    private final String fieldName;
    private final Pattern pattern;
    private final String errorMessage;

    /**
     * RegexRule Constructor.
     * 
     * @param fieldName
     *            - String
     * @param regex
     *            - String
     * @param errorMessage
     *            - String
     */
    public RegexRule(String fieldName, String regex, String errorMessage) {
        this.fieldName = fieldName;
        pattern = Pattern.compile(regex);
        this.errorMessage = errorMessage;
    }

    //@formatter:off
    @Override
    public ValidationResult validate(Object crt, Object prev) {
        ValidationResult res = new ValidationResult(this);
        ConceptField f = getMeta().getField(fieldName);
        String val = String.valueOf(f.getValue(crt));
        Matcher matcher = pattern.matcher(val);
        if (!matcher.matches()) {
            LOGGER.info("Field " + fieldName + " of " + crt + " (value " + val + ") did not match '" + pattern.pattern() + "'");
            res.addMessage(f.getDisplayName() + " field is invalid: " + errorMessage);
        }

        return res;
    }
    //@formatter:on
}
