package gov.va.med.pharmacy.pecs.rules;

import java.util.ArrayList;
import java.util.List;

/**
 * Records the result of a business rule evaluation.
 */
public final class ValidationResult {
    private final ArrayList<String> messages = new ArrayList<String>();
    private final BusinessRule businessRule;

    /**
     * ValidationResult.
     * 
     * @param rule
     *            - BusinessRule
     */
    public ValidationResult(BusinessRule rule) {
        businessRule = rule;
    }

    /**
     * Get BusinessRule.
     * 
     * @return businessRule
     */
    public BusinessRule getBusinessRule() {
        return businessRule;
    }

    /**
     * The messages returned by the business rule. In general, an empty list means no violations found.
     * 
     * @return a (possibly empty) list of messages
     */
    public List<String> getMessages() {
        return messages;
    }

    /**
     * Add a new message to this result. That usually provides details about an issue that was encountered during the evaluation
     * of a business rule.
     * 
     * @param message
     *            the (error) message
     */
    public void addMessage(String message) {
        messages.add(message);
    }

}
