/**
 * 
 */
package gov.va.med.pharmacy.pecs.rules.validation;

import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.ReferenceTextFieldRule;
import gov.va.med.pharmacy.pecs.rules.RegexRule;

/**
 * 
 * Encapsulates the business rules for Duplicate Therapy.
 * 
 * @author vhaisacarusr
 */
public final class DuplicateTherapyBusinessRule {
    /**
     * private constructor to make sure that the class is not instantiated
     */
    private DuplicateTherapyBusinessRule() {
    }

    /**
     * Validates the Duplicate Therapy object.
     * 
     * @param meta
     *            the concept metadata
     * @return the validator
     */
    public static final BusinessRulesValidation validate(ConceptMetadata meta) {
        // all required fields must be present
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        validator.registerRule(new RegexRule(DuplicateTherapyDTO.DESCRIPTION, ".{0,255}", "it can have at most 255 characters"),
                new AlwaysFireTrigger());
        validator.registerRule(new ReferenceTextFieldRule(DuplicateTherapyDTO.REFERENCE_TEXT), new AlwaysFireTrigger());

        return validator;
    }
}
