package gov.va.med.pharmacy.pecs.state;

import gov.va.med.pharmacy.pecs.domain.Customizable;

import java.util.Set;

import org.apache.commons.scxml.SCXMLExpressionException;
import org.apache.commons.scxml.model.ModelException;

/**IPecsStateMachine.
 * 
 * @author DOMAINOVERSD
 *
 */
public interface IPecsStateMachine {

    /**
     * Execute a state transition for the given action. If no transition fires
     * for this event (ignoring conditions), this will is considered an error
     * (attempt to perform an invalid action).
     * 
     * @param action
     *            The action name.
     * @return the id of the new current state of the machine.
     * @throws ModelException - Exception Thrown
     * @throws SCXMLExpressionException - Exception Thrown
     */
    public abstract/* Status */String applyAction(final String action) throws ModelException, SCXMLExpressionException;

    /**Get Current Status.
     * 
     * @return CurrentStatus
     * @throws SCXMLExpressionException - Throws Exception
     */
    public abstract/* Status */String getCurrentStatus() throws SCXMLExpressionException;

    /**
     * The set of actions that would cause a transition to fire, given the state
     * machine's current state.
     * 
     * @return the set of actions
     * @throws SCXMLExpressionException - Java Exception
     */
    public abstract Set<String> getAllowableActions() throws SCXMLExpressionException;

    /**
     * Set the context in which this state machine operates. That consists of
     * the object that's being acted upon and the user who performs the action.
     * 
     * @param concept
     *            the object whose state is managed by this state machine
     * @param user
     *            the login of the user processing the object
     */
    // use the UserProfile instead of the user name
    // so we can access their role
    public abstract void setContext(Customizable<?> concept, String user);

}
