package gov.va.med.pharmacy.pecs.state;

import java.net.URL;
import java.util.List;

import org.apache.commons.scxml.env.SimpleErrorHandler;
import org.apache.commons.scxml.io.SCXMLParser;
import org.apache.commons.scxml.model.SCXML;
import org.xml.sax.ErrorHandler;

/**
 * PecsStateMachineFactory Entity Implements StatusTranslator.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class PecsStateMachineFactory implements StatusTranslator {

    private final SCXML stateMachine;
    private List<Status> statusList;
    private final StatusDAO dao;

    /**
     * Pecs State Machine Factory Constructor.
     * 
     * @param urlString
     *            - String
     * @param dao
     *            - StatusDAO
     * @throws Exception
     *             - throws
     */
    public PecsStateMachineFactory(String urlString, StatusDAO dao) throws Exception {
        URL url = PecsStateMachine.class.getClassLoader().getResource(urlString);
        ErrorHandler errHandler = new SimpleErrorHandler();
        stateMachine = SCXMLParser.parse(url, errHandler);
        this.dao = dao;
    }

    /**
     * Get Status List.
     * 
     * @return - List
     */
    private List<Status> getStatusList() {
        if (statusList == null) {
            statusList = dao.findAll();
        }
        return statusList;
    }

    /**
     * PecsStateMachine Get State Machine.
     * 
     * @return stateMachine
     */
    public PecsStateMachine getStateMachine() {
        return new PecsStateMachine(stateMachine, this);
    }

    @Override
    public Status getStatusByCode(String code) {
        for (Status s : getStatusList()) {
            if (s.getCode().equalsIgnoreCase(code)) {
                return s;
            }
        }
        return null;
    }

    @Override
    public Status getStatusById(int statusId) {
        for (Status s : getStatusList()) {
            if (s.getStatusId() == statusId) {
                return s;
            }
        }
        return null;
    }

    @Override
    public String translate(int statusId) {
        Status s = getStatusById(statusId);
        return s == null ? null : s.getCode();
    }

    @Override
    public int translate(String code) {
        Status s = getStatusByCode(code);
        return s == null ? 0 : s.getStatusId();
    }
}
