package gov.va.med.pharmacy.pecs.state;

import gov.va.med.pharmacy.pecs.persistence.HibernateDAOImpl;

import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * A DAO class for accessing Status domain objects.
 * 
 * @author vhaisaduffg
 * 
 */
@Repository
public final class StatusDAO extends HibernateDAOImpl<Status> {

    private static final String FIND_BY_CODE = "findByCode";

    /**
     * Returns the persistent class this DAO uses.
     * 
     * @return Class<Status>
     */
    @Override
    protected Class<Status> getPersistentClass() {
        return Status.class;
    }

    /**
     * Finds a Status by a particular code.
     * 
     * @param code
     *            the code of the Status to find
     * @return Status a null value is returned if no Status is found that matches the code
     */
    @SuppressWarnings("unchecked")
    public Status findByCode(String code) {
        Query query = getSession().getNamedQuery(FIND_BY_CODE);
        query.setParameter("code", code);
        query.setCacheable(true);

        List<Status> statusList = query.list();
        if (statusList.size() == 1) {
            return statusList.get(0);
        }
        return null;
    }
}
