package gov.va.med.pharmacy.pecs.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * A sub-class of the java.text.SimpleDateFormat that sets the standard format for all Date/Times displayed in the PECS system.
 * 
 * @author vhaisaduffg
 * 
 */
public class PecsDateFormat extends SimpleDateFormat {

    private static final long serialVersionUID = 664630934746293088L;
    private static final String PECS_DATE_FORMAT = "yyyy-MM-dd";
    private static final String PECS_TIME_FORMAT = "HH:mm:ss";
    public static final String PECS_DATE_TIME_FORMAT = PECS_DATE_FORMAT + " " + PECS_TIME_FORMAT;
    public static final TimeZone PECS_DEFAULT_DISPLAY_TIMEZONE = TimeZone.getTimeZone("US/Eastern");

    /**
     * Pecs Date Format.
     * 
     */
    public PecsDateFormat() {
        this.applyPattern(PECS_DATE_TIME_FORMAT);
    }

    /**
     * Format Date.
     * 
     * @param date
     *            - Date
     * @return formattedDate
     */
    public final String formatDate(Date date) {
        String formattedDate;
        this.applyPattern(PECS_DATE_FORMAT);
        formattedDate = format(date);
        this.applyPattern(PECS_DATE_TIME_FORMAT);
        return formattedDate;
    }

    /**
     * Format Time.
     * 
     * @param date
     *            - Date
     * @return formattedTime
     */
    public final String formatTime(Date date) {
        String formattedTime;
        this.applyPattern(PECS_TIME_FORMAT);
        formattedTime = format(date);
        this.applyPattern(PECS_DATE_TIME_FORMAT);
        return formattedTime;
    }
}
