package gov.va.med.pharmacy.pecs.util.config;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;

/**
 * PecsEncrypter Data Entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class PECSEncrypter {

    /** Pecs Encrypted. */
    private PECSEncrypter() {

    }

    //@formatter:off 
    private static byte[] keyBytes = new byte[] { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c,
            0x0d, 0x0e, 0x0f };
  //@formatter:on
    private static String algorithm = "AES";
    private static SecretKeySpec skSpec;
    private static Cipher cipher;

    /**
     * Set Up Encrypted.
     * 
     * @throws Exception
     *             - Exception
     */
    private static void setUp() throws Exception {
        skSpec = new SecretKeySpec(keyBytes, algorithm);
        cipher = Cipher.getInstance("AES/ECB/NoPadding");
    }

    /**
     * Decryption Method.
     * 
     * @param encryptedString
     *            Sting
     * @return String(decryptedText
     * @throws Exception
     *             - Exception
     */
    public static String decrypt(String encryptedString) throws Exception {
        setUp();
        byte[] decodedString = Base64.decodeBase64(encryptedString);
        byte[] decryptedText = new byte[decodedString.length];
        cipher.init(Cipher.DECRYPT_MODE, skSpec);
        cipher.update(decodedString, 0, decodedString.length, decryptedText, 0);
        cipher.doFinal(decodedString);
        return new String(decryptedText).trim();
    }

    /**
     * Ecryption.
     * 
     * @param decryptedString
     *            - String
     * @return String Encrypted
     * @throws Exception
     *             - Exception
     */
    public static String encrypt(String decryptedString) throws Exception {
        setUp();
        String paddedString = padString(decryptedString);
        //@formatter:off
        int blocks = paddedString.length() / 16;
        if ((paddedString.length() % 16) > 0) {
            ++blocks;
        }
        byte[] encryptedText = new byte[blocks * 16];
        //@formatter:on
        cipher.init(Cipher.ENCRYPT_MODE, skSpec);
        cipher.update(paddedString.getBytes(), 0, paddedString.length(), encryptedText, 0);
        cipher.doFinal(paddedString.getBytes());
        byte[] base64Encoded = Base64.encodeBase64(encryptedText);
        return new String(base64Encoded);
    }

    /**
     * String Pad String as Required.
     * 
     * @param unpaddedString
     *            - String
     * @return sb.toString()
     */
    private static String padString(String unpaddedString) {
        StringBuffer sb = new StringBuffer();
        sb.append(unpaddedString);
        //@formatter:off
        int paddingLength = 16 - (sb.length() % 16);
        //@formatter:on
        for (int i = 0; i < paddingLength; i++) {
            sb.append(' ');
        }
        return sb.toString();
    }
}
