package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.customupdate.CtVersionVa;
import gov.va.med.pharmacy.pecs.customupdate.CustomUpdateFileVa;
import gov.va.med.pharmacy.pecs.customupdate.FdbVersion;

import java.util.Date;
import java.util.List;

/**
 * Interface definition for a CustomUpdateService.
 * 
 * @author vhaisaduffg
 * 
 */
public interface CustomUpdateService {

    /** String for Export file dir property. */
    static final String EXPORT_FILE_DIR_PROPERTY = "export.file.dir";
    /** String for Export File Server Property. */
    static final String EXPORT_FILE_SERVER_PROPERTY = "export.file.server";
    /** String for export user name property. */
    static final String EXPORT_USER_NAME_PROPERTY = "export.user.name";
    /** String for export password property. */
    static final String EXPORT_PASSWORD_PROPERTY = "export.password";

    /**
     * Returns the current FdbVersion object.
     * 
     * @return FdbVersion
     */
    FdbVersion getFdbVersion();

    /**
     * * Retrieve update file based on version If version is 0 create a new update file for current data and return the newly
     * created file Return a zip file that contains all files needed for executing the FDB Updater tool.
     * 
     * @param filepath
     *            : The file path that will be used for generating the file
     * @param createdBy
     *            : User named
     */
    void generateCustomUpdateFiles(String filepath, String createdBy);

    /**
     * Retrieves and returns the CustomUpdateFileVa object that matches the passed in version key.
     * 
     * @param ctVersionKey
     *            the primary key of the CustomUpdateFileVa object to retrieve
     * @return CustomUpdateFileVa
     */
    CustomUpdateFileVa findUpdateFile(Long ctVersionKey);

    /**
     * Returns a list of Years that we have custom update files.
     * 
     * @return the list
     */
    List<Date> findAllYears();

    /**
     * Returns the List of Months base off of the Year Requested.
     * 
     * @param year
     *            the year
     * @return the list
     */
    List<Date> findAllMonthsOfUpdatesByYears(String year);

    /**
     * Returns List of Custom Update Files sorted by Year and Month.
     * 
     * @param year
     *            the year
     * @param month
     *            the month
     * @return the custom update file by month andyear
     */
    List<CtVersionVa> getCustomUpdateFileByMonthAndyear(String year, String month);

    /**
     * Returns a list of all CtVersionVa objects.
     * 
     * @return List<CtVersionVa>
     */
    List<CtVersionVa> findAllCtVersionVas();
}
