package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosing;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;

import java.util.List;


/**
 * The Interface DoseRangeService.
 */
public interface DoseRangeService extends PecsService<DoseRangeDTO> {

    /**
     * Retrieves the FDB Data.
     * 
     * @param key
     *            the dose range key
     * @return DoseRangeDTOinformation
     */
    DoseRangeDTO getFdbData(DoseRangeKey key);

    /**
     * Gets the fdb dosing.
     * 
     * @param key
     *            the key
     * @return the fdb dosing
     */
    FdbDosing getFdbDosing(DoseRangeKey key);

    /**
     * Retrieves the Staging Data.
     * 
     * @param conceptId
     *            the concept id
     * @param doseRouteId
     *            the dose route id
     * @param doseTypeId
     *            the dose type id
     * @return List of DoseRangeDTO(s) information
     */
    List<DoseRangeDTO> getStagingData(Long conceptId, String doseRouteId, String doseTypeId);

    /**
     * Retrieves the Staging Data.
     * 
     * @param conceptId
     *            the concept Id
     * @param conceptType
     *            the concept type
     * @param ageLowInDays
     *            the low age in days
     * @param ageHighInDays
     *            the high age in days
     * @param doseRouteId
     *            the dose route id
     * @param doseTypeId
     *            the dose type id
     * @param fdbdx
     *            the fdb dx
     * @return DoseRangeDTO information
     */
    DoseRangeDTO getStagingData(Long conceptId, Long conceptType, Long ageLowInDays, Long ageHighInDays, String doseRouteId,
            String doseTypeId, String fdbdx);

    /**
     * Retrieves the Staging Data by Version Id.
     * 
     * @param versionId
     *            the version id
     * @return the DoseRangeDTO information
     */
    DoseRangeDTO getStagingDataByVersionId(Long versionId);

    /**
     * Retrieves the Current Staging Data by Version Id.
     * 
     * @param versionId
     *            the version id
     * @return DoseRangeDTO
     */
    DoseRangeDTO getCurrentStagingDataByVersionId(Long versionId);

    /**
     * Retrieves the Concept Description.
     * 
     * @param conceptId
     *            the id of the concept
     * @param conceptType
     *            the type of the concept
     * @return String the Concept Description
     */
    String getConceptDescription(Long conceptId, Long conceptType);

    /**
     * Retrieves the History Report data.
     * 
     * @param versionId
     *            the record version id
     * @return List of type DoseRangeDTO
     */
    List<DoseRangeDTO> getVersionHistoryReportData(Long versionId);

    /**
     * Finds the Custom Dose Ranges created from a FDB Record.
     * 
     * @param key
     *            The dose range key
     * @return a list of custom records
     */
    List<DoseRangeDTO> findCustomByXref(DoseRangeKey key);

}
