package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;

import java.util.List;

/**
 * This is an API that retrieves Custom VA objects from the database. In most cases this will return a DTO.
 * 
 * @author vhaisacarusr
 * 
 */
public interface DrugDrugInteractionService extends PecsService<DrugDrugInteractionDTO> {
    /**
     * Retrieves the custom staging data for the supplied record ID as a DTO.
     * 
     * @param identifier
     *            the record ID
     * @return the record for the supplied ID or <code>null</code> if there is no record.
     */
    DrugDrugInteractionDTO getStagingData(Long identifier);

    /**
     * Retrieves all of the custom records associated with an FDB record as DTO's.
     * 
     * @param fdbId
     *            the ID of the FDB record
     * @return a list records for the supplied ID. Never <code>null</code>
     */
    List<DrugDrugInteractionDTO> getStagingByFdbId(Long fdbId);

    /**
     * Retrieves the current version of the staging record for the supplied ID as a DTO.
     * 
     * @param interactionId
     *            the interaction ID of the record to be retrieved
     * @return the record for the supplied ID or <code>null</code> if there is no record.
     */
    DrugDrugInteractionDTO getStagingByInteractionId(Long interactionId);

    /**
     * Retrieves the record for the interaction by the interaction ID.
     * 
     * @param interactionId
     *            the interaction ID for the record.
     * @return a <code>DrugDrugInteractionVa</code> this is not a DTO
     */
    DrugDrugInteractionVa getDDIByInteractionId(Long interactionId);

    /**
     * Retrieves the FDB Drug Drug Interaction for the given identifier and returns the data in a DrugDrugInteractionDTO object.
     * 
     * @param id
     *            the FDB ID
     * @return a DrugDrugInteractionDTO
     */
    DrugDrugInteractionDTO findFdbById(Long id);

    /**
     * This method is simply for development purposes. It should be replaced once a way to obtain all Interaction Descriptions
     * is developed.
     * 
     * @return a list of interaction descriptions
     */
    @SuppressWarnings("rawtypes")
    List getInteractionDescriptions();

    /**
     * Find report history data.
     * 
     * @param versionId
     *            the version id
     * @return the list
     */
    List<DrugDrugInteractionDTO> findReportHistoryData(Long versionId);

    /**
     * Find all the Interaction's for Interaction Id.
     * 
     * @param interactionId
     * @return List of interactions.
     */
    List<DrugDrugInteractionDTO> getStagingListByInteractionId(Long interactionId);
}
