package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;

import java.util.List;

/**
 * Interface for the Duplicate Therapy Service.
 * 
 */
public interface DuplicateTherapyService extends PecsService<DuplicateTherapyDTO> {

    /**
     * Retrieves the FDB Data by DTCID.
     * 
     * @param dtcid
     *            of Type Long
     * @return DuplicateTherapyDTO
     */
    DuplicateTherapyDTO getFdbDataByDtcid(Long dtcid);

    /**
     * Retrieves the Staging Data by Version Id.
     * 
     * @param versionId
     *            of Type Long
     * @return DuplicateTherapyDTO
     */
    DuplicateTherapyDTO getStagingDataByVersionId(Long versionId);

    /**
     * Retrieves the Staging Data.
     * 
     * @param dtcId
     *            of Type Long
     * @param customString
     *            of Type String
     * @param customDupAllowance
     *            of Type Long
     * @return List of DuplicateTherapyDTO
     */
    List<DuplicateTherapyDTO> getStagingData(Long dtcId, String customString, Long customDupAllowance);

    /**
     * Retrieves the Staging Data.
     * 
     * @param dtcId
     *            of Type Long
     * @return List of DuplicateTherapyDTO
     */
    List<DuplicateTherapyDTO> getStagingData(Long dtcId);

    /**
     * Retrieves the history data for a Duplicate Therapy for inclusion on the history report.
     * 
     * @param versionId
     *            the version id of the Duplicate Therapy Version being viewed
     * @return List<DuplicateTherapyDTO>
     */
    List<DuplicateTherapyDTO> getVersionHistoryReportData(Long versionId);

}
