package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.impl.va.MonographKey;
import gov.va.med.pharmacy.pecs.dto.MonographDTO;

import java.util.List;

/**
 * The Interface MonographService.
 */
public interface MonographService extends PecsService<MonographDTO> {

    /**
     * Exists monograph with title.
     * 
     * @param title
     *            the title
     * @return true, if successful
     */
    boolean existsMonographWithTitle(String title);

    /**
     * Gets the all monographs.
     * 
     * @return the all monographs
     */
    List<MonographDTO> getAllMonographs();

    /**
     * Gets the all active monographs.
     * 
     * @return the all active monographs
     */
    List<MonographDTO> getAllActiveMonographs();

    /**
     * Find monograph by fdb id.
     * 
     * @param id
     *            the id
     * @return the monograph va
     */
    MonographDTO findMonographByFDBId(long id);

    /**
     * Find monograph by version id.
     * 
     * @param id
     *            the id
     * @return the monograph va
     */
    MonographDTO findMonographByVersionId(long id);

    /**
     * Find monograph by id.
     * 
     * @param id
     *            the id
     * @return the monograph va
     */
    MonographDTO findMonographById(long id);

    /**
     * Find fdb monograph by id.
     * 
     * @param id
     *            the id
     * @return the monograph va
     */
    MonographDTO findFdbMonographById(long id);

    /**
     * Gets the version codes.
     * 
     * @return the version codes
     */
    List<String> getVersionCodes();

    /**
     * Retrieves the history data for a Professional Monograph for inclusion on the history report.
     * 
     * @param versionId
     *            the version id of the Duplicate Therapy Version being viewed
     * @return List<MonographDTO>
     */
    List<MonographDTO> getVersionHistoryReportData(Long versionId);

    /**
     * Populate monograph id.
     * 
     * @param source
     *            the source
     * @return the monograph key
     */
    MonographKey populateMonographId(MonographDTO source);

}
