package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.lock.ConceptRecordId;
import gov.va.med.pharmacy.pecs.lock.Lock;

/**
 * This inteface will handle all of the locking of concepts.
 * 
 * @author vhaisacarusr
 * 
 */
public interface RecordLockingService {
    /**
     * Returns true if the record is currently locked false otherwise.
     * 
     * @param conceptRecordId
     *            the record id to check.
     * @return true if the record is currently locked false otherwise
     */
    boolean isLocked(ConceptRecordId conceptRecordId);

    /**
     * Gets the persisted locking information.
     * 
     * @param conceptRecordId
     *            the record id used to create the lock
     * @return lock the record locking information.
     * @see gov.va.med.pharmacy.pecs.service.RecordLockingService#getLockingInformation()
     */
    Lock getLockingInformation(ConceptRecordId conceptRecordId);

    /**
     * Persists a lock object to the database.
     * 
     * @param lock
     *            the lock to persist.
     * @see gov.va.med.pharmacy.pecs.service.RecordLockingService#lockRecord(gov.va.med.pharmacy.pecs.lock.Lock)
     */
    void lockRecord(Lock lock);

    /**
     * Deletes the lock object from the database.
     * 
     * @param lock
     *            the lock to delete
     * @see gov.va.med.pharmacy.pecs.service.RecordLockingService#deleteLock(gov.va.med.pharmacy.pecs.lock.Lock)
     */
    public void deleteLock(Lock lock);

    /**
     * Delete all locks for user.
     * 
     * @param username
     *            the username
     */
    public void deleteAllLocksForUser(String username);
}
