package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.impl.va.SavedQueryVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.util.List;

/**
 * The Interface SavedQueryService.
 */
public interface SavedQueryService {

    /**
     * Save.
     * 
     * @param profile
     *            the profile
     * @param concept
     *            the concept
     * @param query
     *            the query
     * @param queryName
     *            the query name
     */
    public void save(UserProfile profile, String concept, String query, String queryName);

    // /**
    // * Create a new saved query record. The savedQuery object fields are populated when
    // * a custom query is created and executed at the presentation tier. The id of the user
    // * who created and executed this query is also persisted. If a savedQuery with the same
    // * query string and user id already exists in the database, that record is returned and
    // * a new record is not created.
    // *
    // * @param savedQuery saved query model object
    // * @return SavedQueryVa persisted model object, having it's primary record key set.
    // * @throws CustomizationServiceException
    // */
    // @Deprecated
    // public SavedQueryVa createSavedQuery(SavedQueryVa savedQuery, boolean enabled)
    // throws CustomizationServiceException;
    //
    // /**
    // * Modify an existing savedQueryVa record. The savedQuery object is a valid, in-process
    // * record. According to business rules, the only object field allowed to be updated is the
    // * create date field. A validation is performed to check that all fields of this object match
    // * what is currently persisted. If any field(s), except create date to not match, a new savedQuery
    // * record is created and returned. The savedQuery SQL string is not executed.
    // *
    // * @param savedQuery an in-process, existing savedQuery object.
    // * @return SavedQueryVa a persisted model object with an updated create date field.
    // * @throws CustomizationServiceException
    // */
    // @Deprecated
    // public SavedQueryVa modifySavedQuery(SavedQueryVa savedQuery)
    // throws CustomizationServiceException;
    //
    //
    // /**
    // * Delete (remove row from table) the ExecutedQuery record associated with the savedQuery
    // * object's ID key. Since other ExecuteQuery records may have a foreign key reference to the
    // * indicated savedQuery, only the ExecuteQuery record associated with the userId and the savedQuery
    // * will be removed.
    // *
    // * @param userId The id of the user to delete the query for.
    // * @param savedQuery the associated query to delete.
    // * @throws CustomizationServiceException
    // */
    // @Deprecated
    // public void deleteSavedQuery(String userId, SavedQueryVa savedQuery)
    // throws CustomizationServiceException;

    /**
     * Delete saved query.
     * 
     * @param profile
     *            the profile
     * @param queryId
     *            the query id
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public void deleteSavedQuery(UserProfile profile, Long queryId) throws CustomizationServiceException;

    /**
     * Return a saved query object that is identified by the id parameter. If the model object cannot be found, null is
     * returned.
     * 
     * @param Id
     *            the database table record id of the SavedQueryVa object.
     * @return SavedQueryVa record that is identified by the id parameter
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public SavedQueryVa findSavedQueryById(Long Id) throws CustomizationServiceException;

    // /**
    // * Return a List of saved query objects that are identified by the userId that created
    // * the records. If no saved queries can be found, null is returned.
    // *
    // * @param userId the id of the user that executed the queries
    // * @return List of SavedQueryVa objects that were executed by the user associated with the
    // * user id parameter.
    // * @throws CustomizationServiceException
    // */
    // public List<SavedQueryVa> findSavedQueryByUserId(String userId)
    // throws CustomizationServiceException;

    // /**
    // * Return a List of saved queries by table name. SavedQueryVa records are executed against
    // * one table. This method provides search capability, returning all saved queries that have
    // * been executed against a specified table regardless of the user that executed the query.
    // *
    // * @param tableName a tablename of a database table in the customization staging database schema.
    // * @return List of SavedQueryVa objects.
    // * @throws CustomizationServiceException
    // */
    // public List<SavedQueryVa> findSavedQueryByTable(String tableName)
    // throws CustomizationServiceException;

    /**
     * Returns a list of saved queries for the specified table that was not created by the user passed in. Some constraints on
     * the saved queries returned are: the query must be enabled and the query must have been created in the last 30 days.
     * 
     * @param tableName
     *            the table name
     * @param user
     *            the user
     * @return the list
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public List<SavedQueryVa> findSavedQueryForOthersByTable(String tableName, String user)
            throws CustomizationServiceException;

    /**
     * Return a List of saved queries by user id and table name. SavedQueryVa records are executed against one table. This
     * method provides search capability, returning all saved queries that have been executed by a specified user and a
     * specified table.
     * 
     * @param userId
     *            the id of the user that executed the queries
     * @param tableName
     *            a tablename of a database table in the customization staging database schema.
     * @return List of SavedQueryVa objects.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public List<SavedQueryVa> findSavedQueryByUserIdAndTable(String userId, String tableName)
            throws CustomizationServiceException;

    // /**
    // * Return a List of executed queries by user id and table name. SavedQueryVa records are executed against
    // * one table. This method provides search capability, returning all executed queries that have
    // * been executed by a specified user and a specified table.
    // *
    // * @param userId the id of the user that executed the queries
    // * @param tableName a tablename of a database table in the customization staging database schema.
    // * @return List of SavedQueryVa objects.
    // * @throws CustomizationServiceException
    // */
    // public List<ExecutedQueryVa> findExecutedQueryByUserIdAndTable(String userId, String tableName)
    // throws CustomizationServiceException;

    // /**
    // * Retrieve a List of saved query objects that have been assigned a name given by the queryName
    // * parameter. The query name is not a unique field, so many possible queries may be returned.
    // *
    // * @param queryName the name assigned by the UI user to a saved query
    // * @return List of all saved queries that match the queryName parameter.
    // * @throws CustomizationServiceException
    // */
    // public List<SavedQueryVa> findSavedQueryByName(String queryName)
    // throws CustomizationServiceException;

    // /**
    // * Execute the HQL search string associated with the query parameter. The HQL can query
    // * against the custom staging database schema, or the FDB schema. The object type in the List returned
    // * is defined by the class parameter in the Query constructor.
    // *
    // * @param query object representing the HQL to be executed against the customization staging or FDB
    // * databases.
    // * @return List of objects whose types are defined in the query parameter.
    // * @throws CustomizationServiceException
    // */
    // @Deprecated
    // public List executeDynamicQuery(Query query)
    // throws CustomizationServiceException;
    //
    //
    // /**
    // * Execute the HQL search string associated with the query parameter. The HQL can query
    // * against the custom Staging database schema. The object type in the List returned
    // * is defined by the class parameter in the Query constructor.
    // *
    // * @param hqlQuery representing the HQL to be executed against the customization staging or FDB
    // * databases.
    // * @return List of objects whose types are defined in the query parameter.
    // * @throws CustomizationServiceException
    // */
    // @Deprecated
    // public List executeHQLQueryStaging(String hqlQuery)
    // throws CustomizationServiceException;
    //
    //
    // /**
    // * Execute the HQL search string associated with the query parameter. The HQL can query
    // * against the FDB database schema. The object type in the List returned
    // * is defined by the class parameter in the Query constructor.
    // *
    // * @param hqlQuery representing the HQL to be executed against the customization staging or FDB
    // * databases.
    // * @return List of objects whose types are defined in the query parameter.
    // * @throws CustomizationServiceException
    // */
    // @Deprecated
    // public List executeHQLQueryFDB(String hqlQuery)
    // throws CustomizationServiceException;
}
