package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.impl.va.CtUsersVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

/**
 * The Interface UserService.
 */
public interface UserService {

    /**
     * This method receives the KAAJEE LoginUserInfoVO authentication object and creates the custom tables user profile object,
     * including the determination and setting of the user role(s).
     * 
     * @param request
     *            HTTP session request object.
     * @return Custom tables user profile object.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public UserProfile authenticate(HttpServletRequest request) throws CustomizationServiceException;

    /**
     * This method loads a List of the users that have been entered in the CT_USERS_VA table. These users are in the APPROVER
     * role in the VistA system for the Custom Tables application.
     * 
     * @return List<CtUsersVa> of users that are entered in the CT_USERS_VA table.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public List<CtUsersVa> findApprovers() throws CustomizationServiceException;

    /**
     * This method creates a new record into the CT_USERS_VA table, given information entered into the GUI. NOTE: there are
     * currently NO validation checks (except trim and upper case), as to the validity of the information entered against the
     * VistA system. This is due to the unavailability of an API via KAAJEE to perform validation checks.
     * 
     * @param ctUsersVa
     *            The model object containing information received from the GUI.
     * @return CtUsersVa object that has had record ID and date information populated.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public CtUsersVa createRecord(CtUsersVa ctUsersVa) throws CustomizationServiceException;

    /**
     * This method performs a data update of an existing record with information that has been received from the GUI.
     * 
     * @param ctUsersVa
     *            The model object containing information received from the GUI.
     * @return CtUsersVa object that has had record ID and date information populated and updated.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public CtUsersVa modifyRecord(CtUsersVa ctUsersVa) throws CustomizationServiceException;

    /**
     * This method is a convenience method for batch updates of CtUsersVa records for changes received from the GUI.
     * 
     * @param records
     *            List of model objects to update.
     * @return List<CtUsersVa> containing the updated records.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    public List<CtUsersVa> modifyRecords(List<CtUsersVa> records) throws CustomizationServiceException;

}
