package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.ComparisonReportConcept;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateParser;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.service.impl.TerminologyServiceImpl;

import java.io.IOException;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;

/**
 * Abstract class that holds the common services and methods.
 * 
 * @author VHAISACARUSR
 * 
 */
public abstract class AbstractComparisonReportProcessor implements InitializingBean {
    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(AbstractComparisonReportProcessor.class);

    /** The comparison report utility. */
    @Resource
    private ComparisonReportUtility comparisonReportUtility;

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /**
     * Saves a Report.
     * 
     * @param helper
     *            The report helper
     * @param fileNameToSave
     *            the name the file will be saved as
     * @param reportKey
     *            the key to the terminology service for ComparisonReportConcept
     */
    protected final void saveReport(ExcelUpdateReportHelper helper, String fileNameToSave, String reportKey) {
        TerminologyServiceImpl terminologyServiceImpl = (TerminologyServiceImpl) getTerminologyService();
        ComparisonReportConcept concept =
                (ComparisonReportConcept) terminologyServiceImpl.getByCode(ComparisonReportConcept.class, reportKey);
        try {
            getComparisonReportUtility().saveReport(helper, fileNameToSave, concept);
        } catch (IOException e) {
            LOGGER.error("Error in writing file " + fileNameToSave + "for concept " + concept.getName(), e);
        } catch(RuntimeException re) {
            LOGGER.error("A RuntimeException has occured while trying to save the Comparison Report for the concept " 
                    + concept.getName(), re);
        }

    }

    /**
     * This method does all of the work for the processing of the report.
     * 
     * @param parser
     *            the FDB Update Parser
     * @param fileDate
     *            the file date
     */
    public abstract void processComparisonReport(FdbUpdateParser parser);

    /**
     * Gets the comparisonReportUtility.
     * 
     * @return the comparisonReportUtility
     */
    public final ComparisonReportUtility getComparisonReportUtility() {
        return comparisonReportUtility;
    }

    /**
     * Sets the comparisonReportUtility.
     * 
     * @param comparisonReportUtility
     *            the comparisonReportUtility to set
     */
    public final void setComparisonReportUtility(ComparisonReportUtility comparisonReportUtility) {
        this.comparisonReportUtility = comparisonReportUtility;
    }

    /**
     * Gets the terminologyService.
     * 
     * @return the terminologyService
     */
    public final TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminologyService.
     * 
     * @param terminologyService
     *            the terminologyService to set
     */
    public final void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // TODO Auto-generated method stub

    }

}
