package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdiminteraction;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.MonographDTO;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.fdbreports.FdbColumnUpdate;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;

/**
 * The Class DDIReportGenerator.
 */
public final class DDIReportGenerator implements FdbUpdateReportGenerator {

    /** The Constant HEADER_COLUMNS. */
    private static final List<String> HEADER_COLUMNS = Arrays.asList(new String[] { "Action Status", "Action Date",
            "DATUP will delete", "VA Interaction ID", "FDB Interaction ID", "Interaction Description", "Monograph ID",
            "Severity Level", "Clinical Effect 1", "Clinical Effect 2", "Drug Pairs" });

    /** The aug. */
    private final FdbUpdateDataAugmentation aug;

    /** The sorted updated array list. */
    private List<SortedDrugPair> sortedUpdatedArrayList;

    /** The interaction id. */
    private Long interactionID = 0L;

    /** The Constant COLUMN_VALUE_1. */
    private static final int COLUMN_VALUE_1 = 1;

    /** The Constant COLUMN_VALUE_2. */
    private static final int COLUMN_VALUE_2 = 2;

    /** The Constant COLUMN_VALUE_3. */
    private static final int COLUMN_VALUE_3 = 3;

    /** The Constant COLUMN_VALUE_4. */
    private static final int COLUMN_VALUE_4 = 4;

    /** The Constant COLUMN_VALUE_5. */
    private static final int COLUMN_VALUE_5 = 5;

    /** The Constant COLUMN_VALUE_6. */
    private static final int COLUMN_VALUE_6 = 6;

    /** The Constant COLUMN_VALUE_7. */
    private static final int COLUMN_VALUE_7 = 7;

    /** The Constant COLUMN_VALUE_8. */
    private static final int COLUMN_VALUE_8 = 8;

    /** The Constant COLUMN_VALUE_9. */
    private static final int COLUMN_VALUE_9 = 9;

    /** The Constant COLUMN_VALUE_10. */
    private static final int COLUMN_VALUE_10 = 10;

    /** The Constant COLUMN_VALUE_11. */
    private static final int COLUMN_VALUE_11 = 11;

    /** The Constant INTERACTION_ID_COL_KEY. */
    private static final String INTERACTION_ID_COL_KEY = "interactionid";

    /** The Constant DESCRIPTION_COL_KEY. */
    private static final String DESCRIPTION_COL_KEY = "description1";

    /** The Constant SEVERITY_LEVEL_COL_KEY. */
    private static final String SEVERITY_LEVEL_COL_KEY = "severitylevelcode";

    /** The Constant CLINICAL_EFFECT1_COL_KEY. */
    private static final String CLINICAL_EFFECT1_COL_KEY = "clinicaleffectcode1";

    /** The Constant CLINICAL_EFFECT2_COL_KEY. */
    private static final String CLINICAL_EFFECT2_COL_KEY = "clinicaleffectcode2";

    /** The Constant MONOGRAPH_COL_KEY. */
    private static final String MONOGRAPH_COL_KEY = "monographid";

    /** The severity list. */
    private List<SeverityLevel> severityList;

    /** The clinical list. */
    private List<ClinicalEffect> clinicalList;

    /** The mono list. */
    private List<MonographDTO> monoList;

    /**
     * Instantiates a new dDI report generator.
     * 
     * @param aug
     *            the aug
     */
    public DDIReportGenerator(FdbUpdateDataAugmentation aug) {
        this.aug = aug;
    }

    /**
     * Instantiates a new dDI report generator.
     * 
     * @param aug
     *            the aug
     * @param sortedUpdatedArrayList
     *            the sorted updated array list
     */
    public DDIReportGenerator(FdbUpdateDataAugmentation aug, List<SortedDrugPair> sortedUpdatedArrayList) {
        this.aug = aug;
        this.sortedUpdatedArrayList = sortedUpdatedArrayList;
    }

    /**
     * The main method for generating a DDI Comparison report.
     * 
     * @param fileDate
     *            the file date
     * @param severityLevels
     *            a list of Severity Levels
     * @param clinicalEffects
     *            a list of Clinical Effects
     * @param monographDescs
     *            a list of Monograph Descriptions
     * @return ExcelUpdateReportHelper
     * @throws Exception
     *             if an error occurs
     */
    public final ExcelUpdateReportHelper generateReport(String fileDate, List<SeverityLevel> severityLevels,
            List<ClinicalEffect> clinicalEffects, List<MonographDTO> monographDescs) throws Exception {
        FdbUpdateRecord updatedRecord;

        setMonoList(monographDescs);
        setSeverityList(severityLevels);
        setClinicalList(clinicalEffects);

        ExcelUpdateReportHelper helper = new ExcelUpdateReportHelper();
        helper.createWorkbook();
        helper.createWorksheet("DDI-DP FDB Comparison Report");// shows on the tab

        int row = helper.createHeader(HEADER_COLUMNS, fileDate);

        int recordTypeCounterA = 0;
        int recordTypeCounterC = 0;
        int recordTypeCounterD = 0;
        while ((updatedRecord = aug.nextRecord()) != null) {
            if (updatedRecord.getType().equals("A")) {
                // ignore the record
                continue;
            }

            aug.populateCorrespondingFdbValues(updatedRecord);
            if (!shouldReport(updatedRecord)) {
                // ignore and continue
                continue;
            }
            recordTypeCounterA = recordTypeCounterA + 1;
            if (updatedRecord.getType().equals("D")) {
                recordTypeCounterD = recordTypeCounterD + 1;
            }

            row = printCustom(aug, updatedRecord, helper, row);

            printAfterUpdate(aug, updatedRecord, helper, ++row);

            if (checkForDrugPairChanges(interactionID)) {
                helper.addCell(row, COLUMN_VALUE_11, "See FDB Interaction ID " + interactionID + "-DP");// FDB Interaction
            }

            printBeforeUpdate(aug, updatedRecord, helper, ++row);

            if (updatedRecord.getType().equals("C")) {
                recordTypeCounterC = recordTypeCounterC + 1;
            }
            helper.createSeparatorRow(++row, HEADER_COLUMNS.size());
        }// end while

        if ((recordTypeCounterA == 0) || ((recordTypeCounterC == 0) && (recordTypeCounterD == 0))) {
            helper.createNoDataFoundRow(++row);
        }
        return helper;
    }

    /**
     * Compares before and after update values and highlights in Red values that have changed. This method only prints After
     * Update values.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param vals
     *            the vals
     */
    private void printRecordAfterUpdate(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate vals) {

        if (vals.getColumn().getName().equals("description1")) {
            processCell(helper, row, vals, COLUMN_VALUE_6, true);
        } else if (vals.getColumn().getName().equals("monographid")) {
            monographIdHelperAddCell(helper, row, vals, true);
        } else if (vals.getColumn().getName().equals("severitylevelcode")) {
            processCellNumeric(helper, row, vals, COLUMN_VALUE_8, true);
        } else if (vals.getColumn().getName().equals("clinicaleffectcode1")) {
            processClinicalEffectCode(helper, row, vals, COLUMN_VALUE_9, true);
        } else if (vals.getColumn().getName().equals("clinicaleffectcode2")) {
            processClinicalEffectCode(helper, row, vals, COLUMN_VALUE_10, true);
        }
    }

    /**
     * Compares before and after update values and highlights in Red values that have changed. This method only prints Before
     * Update values.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param values
     *            the values
     */
    private void printRecordBeforeUpdate(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate values) {
        if (values.getColumn().getName().equals(DESCRIPTION_COL_KEY)) {
            processCell(helper, row, values, COLUMN_VALUE_6, false);
        } else if (values.getColumn().getName().equals(MONOGRAPH_COL_KEY)) {
            monographIdHelperAddCell(helper, row, values, false);
        } else if (values.getColumn().getName().equals(SEVERITY_LEVEL_COL_KEY)) {
            processCellNumeric(helper, row, values, COLUMN_VALUE_8, false);
        } else if (values.getColumn().getName().equals(CLINICAL_EFFECT1_COL_KEY)) {
            processClinicalEffectCode(helper, row, values, COLUMN_VALUE_9, false);
        }
        if (values.getColumn().getName().equals(CLINICAL_EFFECT2_COL_KEY)) {
            processClinicalEffectCode(helper, row, values, COLUMN_VALUE_10, false);
        }
    }

    /**
     * Record type d gather previous values.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param values
     *            the values
     */
    private void recordTypeDGatherPreviousValues(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate values) {
        if (values.getColumn().getName().equals(DESCRIPTION_COL_KEY)) {
            helper.addCell(row, COLUMN_VALUE_6, values.getPreviousValue());// description1
        } else if (values.getColumn().getName().equals(MONOGRAPH_COL_KEY)) {
            helper.addCell(row, COLUMN_VALUE_7, getMonographDesc(values.getPreviousValue()));
        } else if (values.getColumn().getName().equals(SEVERITY_LEVEL_COL_KEY)) {
            helper.addCell(row, COLUMN_VALUE_8, new Long(values.getPreviousValue()));// severity
        } else if (values.getColumn().getName().equals(CLINICAL_EFFECT1_COL_KEY)) {
            helper.addCell(row, COLUMN_VALUE_9, getClinicalEffectDesc(values.getPreviousValue()));
        } else if (values.getColumn().getName().equals(CLINICAL_EFFECT2_COL_KEY)) {
            if (values.getPreviousValue() == null) {
                values.setPreviousValue("");
            }
            helper.addCell(row, COLUMN_VALUE_10, getClinicalEffectDesc(values.getPreviousValue()));
        }
    }

    /**
     * Clinical effect code2 helper add cell.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param vals
     *            the vals
     * @param column
     *            the column number of the cell
     * @param afterUpdate
     *            the boolean for after update
     */
    private void processClinicalEffectCode(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate vals, Integer column,
            boolean afterUpdate) {
        if (vals.getCurrentValue() == null) {
            vals.setCurrentValue("");
        }
        if (vals.getPreviousValue() == null) {
            vals.setPreviousValue("");
        }
        String code = (afterUpdate ? vals.getCurrentValue() : vals.getPreviousValue());
        if (vals.getCurrentValue().equals(vals.getPreviousValue())) {
            helper.addCell(row, column, getClinicalEffectDesc(code));
        } else {
            helper.addUpdatedCell(row, column, getClinicalEffectDesc(code));
        }
    }

    /**
     * Monograph id helper add cell.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param vals
     *            the vals
     * @param afterUpdate
     *            the boolean for after update
     * @return true, if successful
     */
    private boolean monographIdHelperAddCell(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate vals,
            boolean afterUpdate) {
        String currVal = vals.getCurrentValue().trim();
        String prevVal = vals.getPreviousValue().trim();
        String desc = getMonographDesc((afterUpdate ? currVal : prevVal));
        if (currVal.equals(prevVal)) {
            // monographid
            helper.addCell(row, COLUMN_VALUE_7, desc);
            return true;
        } else {
            helper.addUpdatedCell(row, COLUMN_VALUE_7, desc);

        }
        return false;
    }

    /**
     * Process cell.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param values
     *            the values
     * @param columnNumber
     *            the column number
     * @param afterUpdate
     *            the after update
     * @return true, if successful
     */
    private boolean processCell(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate values, int columnNumber,
            boolean afterUpdate) {
        String currentValue = values.getCurrentValue().trim();
        String previousValue = values.getPreviousValue().trim();
        String value = (afterUpdate ? currentValue : previousValue);
        if (currentValue.equals(previousValue)) {
            helper.addCell(row, columnNumber, value);
            return true;
        } else {
            helper.addUpdatedCell(row, columnNumber, value);
        }
        return false;
    }

    /**
     * Process cell numeric.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row number
     * @param values
     *            the values
     * @param column
     *            the column number
     * @param afterUpdate
     *            true to print the current value, false to print the previous value
     * @return true, if successful
     */
    private boolean processCellNumeric(ExcelUpdateReportHelper helper, Integer row, FdbColumnUpdate values, int column,
            boolean afterUpdate) {
        Long value = new Long((afterUpdate ? values.getCurrentValue() : values.getPreviousValue()));
        if (values.getCurrentValue().equals(values.getPreviousValue())) {
            helper.addCell(row, column, value);
            return true;
        } else {
            helper.addUpdatedCell(row, column, value);
        }
        return false;
    }

    /**
     * Ddi va helper add cell.
     * 
     * @param helper
     *            the helper
     * @param row
     *            the row
     * @param dto
     *            the dto
     */
    private void addHelperCell(ExcelUpdateReportHelper helper, Integer row, DrugDrugInteractionDTO dto) {
        helper.addCell(row, COLUMN_VALUE_1, dto.getStatus());// action status
        helper.addCell(row, COLUMN_VALUE_2, dto.getActionDate()); // action date
        helper.addCell(row, COLUMN_VALUE_4, dto.getInteractionid());// VA
        helper.addCell(row, COLUMN_VALUE_5, dto.getFdbId());// FDB interaction id
        helper.addCell(row, COLUMN_VALUE_6, dto.getDescription1());// description
        helper.addCell(row, COLUMN_VALUE_7, getMonographDesc(StringUtils.EMPTY + dto.getMonographid()));
        helper.addCell(row, COLUMN_VALUE_8, new Long(dto.getSeveritylevelcode()));// severity
        // clinicalEffects
        helper.addCell(row, COLUMN_VALUE_9, getClinicalEffectDesc(dto.getClinicaleffectcode1()));
        helper.addCell(row, COLUMN_VALUE_10, getClinicalEffectDesc(dto.getClinicaleffectcode2()));
    }

    /**
     * Save workbook.
     * 
     * @param directory
     *            the directory
     * @param fileName
     *            the file name
     * @param workbook
     *            the workbook
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    public final void saveWorkbook(String directory, String fileName, Workbook workbook) throws IOException {
        FileOutputStream out = new FileOutputStream(new File(directory + "\\" + fileName));
        workbook.write(out);
        out.flush();
    }

    /**
     * Gets the clinical effect desc.
     * 
     * @param code
     *            the code
     * @return the clinical effect desc
     */
    private String getClinicalEffectDesc(String code) {
        String desc = null;
        for (ClinicalEffect clinicalEffect : getClinicalList()) {
            String value = clinicalEffect.getCode();
            if (value.equals(code)) {
                desc = clinicalEffect.getDescription().trim();
            }
        }
        if (desc == null) {
            return "";
        } else {
            return desc;
        }
    }

    /**
     * Gets the monograph desc.
     * 
     * @param identifier
     *            the Monograph Identifier the code
     * @return the monograph desc
     */
    private String getMonographDesc(String identifier) {
        String desc = null;
        for (MonographDTO monographDTO : getMonoList()) {
            String value = StringUtils.EMPTY + monographDTO.getMonographid();
            if (value.equals(identifier)) {
                desc = monographDTO.getMonographDisplayTitle();
            }
        }
        if (desc == null) {
            return "";
        } else {
            return desc;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.fdbreports.impl.FdbUpdateReportGenerator#generateReport(java.lang.String)
     */
    @Override
    public ExcelUpdateReportHelper generateReport(String fileDt) throws Exception {
        return null;
    }

    /**
     * Checks to see if the FdbUpdateRecord is a reverse DDI or not. If it is a reverse, this method returns true.
     * 
     * @param updatedRecord
     *            FdbUpdateRecord
     * @return true if the record not be reported
     */
    private boolean filterOutReverse(FdbUpdateRecord updatedRecord) {
        FdbColumnUpdate interactionIdColumn = updatedRecord.getValue("interactionid");
        if (interactionIdColumn != null) {
            Long interactionId = new Long(interactionIdColumn.getCurrentValue());
            if (interactionId <= FdbDdiminteraction.MAX_DDI_FORWARD_INTERACTION_ID) {
                return false;
            }
        }
        return true;
    }

    /**
     * Checks the fields in the FdbUpdateRecord to see if any data has been changed.
     * 
     * @param updatedRecord
     *            FdbUpdateRecord
     * @return returns false if the record should not be reported
     */
    private boolean shouldReport(FdbUpdateRecord updatedRecord) {
        boolean descChanged = false, severityLevelChanged = false, clinicalEffect1Changed = false, clinicalEffect2Changed =
                false;

        FdbColumnUpdate column;

        if (!filterOutReverse(updatedRecord)) {
            column = updatedRecord.getValue(DESCRIPTION_COL_KEY);
            descChanged = shouldReport(column);
            column = updatedRecord.getValue(SEVERITY_LEVEL_COL_KEY);
            severityLevelChanged = shouldReport(column);
            column = updatedRecord.getValue(CLINICAL_EFFECT1_COL_KEY);
            clinicalEffect1Changed = shouldReport(column);
            column = updatedRecord.getValue(CLINICAL_EFFECT2_COL_KEY);
            clinicalEffect2Changed = shouldReport(column);
        }

        return descChanged | severityLevelChanged | clinicalEffect1Changed | clinicalEffect2Changed;
    }

    /**
     * Compares the current value against the previous value in the FdbColumnUpdate object.
     * 
     * @param column
     *            FdbColumnUpdate
     * @return true if the current and previous values are not equal.
     */
    private boolean shouldReport(FdbColumnUpdate column) {
        if (column.getCurrentValue() == null) {
            return false;   // If the Current Value is a null, then that value is not being changed.
        }
        return !column.getCurrentValue().equals(column.getPreviousValue());
    }

    /**
     * Loops through the Drug Pair list checking to see if a Drug Pair change has been sent during the same update. If so, then
     * return true.
     * 
     * @param interactionId
     *            the Interaction Identifier
     * @return true if an associated Drug Pair change is found
     */
    private boolean checkForDrugPairChanges(Long interactionId) {
        for (SortedDrugPair drugPair : sortedUpdatedArrayList) {
            if (interactionID.equals(drugPair.getInteractionid())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Finds the Custom DDI for the associated FDB DDI and includes it on the report.
     * 
     * @param augmentation
     *            FdbUpdateDataAugmentation
     * @param updatedRecord
     *            FdbUpdateRecord
     * @param helper
     *            ExcelUpdateReportHelper
     * @param row
     *            int
     * @return int the ending row number in case more than one Custom is printed
     */
    private int printCustom(FdbUpdateDataAugmentation augmentation, FdbUpdateRecord updatedRecord,
            ExcelUpdateReportHelper helper, Integer row) {
        @SuppressWarnings("unchecked")
        List<DrugDrugInteractionDTO> dtos =
                (List<DrugDrugInteractionDTO>) augmentation.findCorrespondingVaObjects(updatedRecord);
        if (dtos.size() > 0) {
            for (DrugDrugInteractionDTO dto : dtos) {
                helper.createVaCustomRow(++row);
                addHelperCell(helper, row, dto);
            }
        } else {
            helper.createNotCustomizedRow(++row);
        }
        return row;
    }

    /**
     * Prints the Latest FDB values as they will be after they are applied to the database.
     * 
     * @param augmentation
     *            FdbUpdateDataAugmentation
     * @param updatedRecord
     *            FdbUpdateRecord
     * @param helper
     *            ExcelUpdateReportHelper
     * @param row
     *            int
     */
    private void printAfterUpdate(FdbUpdateDataAugmentation augmentation, FdbUpdateRecord updatedRecord,
            ExcelUpdateReportHelper helper, Integer row) {
        List<FdbColumnUpdate> listOfValues = updatedRecord.getValues();
        helper.createLatestFdbRow(row);

        for (FdbColumnUpdate vals : listOfValues) {
            helper.addCell(row, COLUMN_VALUE_3, (updatedRecord.getType().equals("D") ? "Yes" : ""));// DATUP WILL DELETE
            if (vals.getColumn().getName().equals(INTERACTION_ID_COL_KEY)) {
                helper.addCell(row, COLUMN_VALUE_5, new Long(vals.getCurrentValue()));// interactionid
                interactionID = new Long(vals.getCurrentValue());
            }

            if (!updatedRecord.getType().equals("D")) {
                printRecordAfterUpdate(helper, row, vals);
            }
        } // end for for text file data
    }

    /**
     * Prints the Current FDB record.
     * 
     * @param augmentation
     *            FdbUpdateDataAugmentation
     * @param updatedRecord
     *            FdbUpdateRecord
     * @param helper
     *            ExcelUpdateReportHelper
     * @param row
     *            int
     */
    private void printBeforeUpdate(FdbUpdateDataAugmentation augmentation, FdbUpdateRecord updatedRecord,
            ExcelUpdateReportHelper helper, Integer row) {

        List<FdbColumnUpdate> listOfValues = updatedRecord.getValues();
        helper.createPreviousFdbRow(row);// this is from
        for (FdbColumnUpdate values : listOfValues) {
            if (values.getColumn().getName().equals("interactionid")) {
                helper.addCell(row, COLUMN_VALUE_5, new Long(values.getPreviousValue()));// interactionid
            }
            if (updatedRecord.getType().equals("D")) {
                recordTypeDGatherPreviousValues(helper, row, values);
            } else {
                printRecordBeforeUpdate(helper, row, values);
            }
        }// end for
    }

    /**
     * Returns the Severity Level list.
     * 
     * @return List
     */
    public List<SeverityLevel> getSeverityList() {
        return severityList;
    }

    /**
     * Sets the Severity Level list.
     * 
     * @param severityList
     *            List
     */
    public void setSeverityList(List<SeverityLevel> severityList) {
        this.severityList = severityList;
    }

    /**
     * Returns the Clinical Effect List.
     * 
     * @return List
     */
    public List<ClinicalEffect> getClinicalList() {
        return clinicalList;
    }

    /**
     * Sets the Clinical Effect List.
     * 
     * @param clinicalList
     *            List
     */
    public void setClinicalList(List<ClinicalEffect> clinicalList) {
        this.clinicalList = clinicalList;
    }

    /**
     * Returns the Monograph Description list.
     * 
     * @return List
     */
    public List<MonographDTO> getMonoList() {
        return monoList;
    }

    /**
     * Sets the Monograph Description list.
     * 
     * @param monoList
     *            List
     */
    public void setMonoList(List<MonographDTO> monoList) {
        this.monoList = monoList;
    }
}
