/**
 * 
 */
package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Performs the FDB comparison report data augmentation for the Dose Range concept.
 * 
 * @author VHAISACARUSR
 * 
 */
@Component
public final class DRFdbUpdateDataAgumentaion extends FdbUpdateDataAugmentation {

    private DoseRangeService doseRangeService;

    /**
     * The constructor takes a service and passes the DAO to the super class.
     * 
     * @param doseRangeService
     *            the dose range service
     */
    @Autowired
    public DRFdbUpdateDataAgumentaion(DoseRangeService doseRangeService) {
        super(doseRangeService.getDao(false));
        this.doseRangeService = doseRangeService;
    }

    @Override
    public final List<DoseRangeDTO> findCorrespondingVaObjects(FdbUpdateRecord record) {
        DoseRangeKey key = populateDoseRangeKey(record);
        return doseRangeService.findCustomByXref(key);
    }

    /**
     * @param record
     *            the record to extract the key from
     * @return the Key object for the record
     */
    private DoseRangeKey populateDoseRangeKey(FdbUpdateRecord record) {
        Long conceptId = (Long) record.getValue("conceptId").getCurrentValueAsObject();
        Long conceptType = (Long) record.getValue("conceptType").getCurrentValueAsObject();
        Long ageLowInDays = (Long) record.getValue("ageLowInDays").getCurrentValueAsObject();
        Long ageHighInDays = (Long) record.getValue("ageHighInDays").getCurrentValueAsObject();
        String doseRouteId = (String) record.getValue("doseRouteId").getCurrentValueAsObject();
        String doseTypeId = (String) record.getValue("doseTypeId").getCurrentValueAsObject();
        String fdbdx = (String) record.getValue("fdbdx").getCurrentValueAsObject();
        Long hitType = (Long) record.getValue("hitType").getCurrentValueAsObject();

        DoseRangeKey key = new DoseRangeKey();
        key.setAgeHighInDays(ageHighInDays);
        key.setAgeLowInDays(ageLowInDays);
        key.setConceptId(conceptId);
        key.setConceptType(conceptType);
        key.setDoseRouteId(doseRouteId);
        key.setDoseTypeId(doseTypeId);
        key.setFdbdx(fdbdx);
        key.setHitType(hitType);
        return key;
    }

    /**
     * Gets the doseRangeService.
     * 
     * @return the doseRangeService
     */
    public DoseRangeService getDoseRangeService() {
        return doseRangeService;
    }

    /**
     * Sets the doseRangeService.
     * 
     * @param doseRangeService
     *            the doseRangeService to set
     */
    public void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }

}
