package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.fdbreports.FdbColumnUpdate;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFRow;

/**
 * The Class DTReportGenerator.
 */
public final class DTReportGenerator implements FdbUpdateReportGenerator {

    /** The aug. */
    private final DTFdbUpdateDataAugmentation aug;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(DTReportGenerator.class);

    /**
     * Instantiates a new dT report generator.
     * 
     * @param aug
     *            the aug
     */
    public DTReportGenerator(DTFdbUpdateDataAugmentation aug) {
        this.aug = aug;
    }

    /**
     * Generate report.
     * 
     * @param fileDate
     *            the file date
     * @return the excel update report helper
     * @throws Exception
     *             the exception
     */
    @Override
    public ExcelUpdateReportHelper generateReport(String fileDate) throws Exception {
        FdbUpdateRecord record;
        LOGGER.debug("generating report for Duplicate Therapy");

        ExcelUpdateReportHelper helper = new ExcelUpdateReportHelper();
        helper.createWorkbook();
        helper.createWorksheet("DT FDB Comparison Report");// shows on the tab in excel
        int row = helper.createHeader(DTReportColumn.headers, fileDate);

        int recordTypeACounter = 0;
        int recordTypeCCounter = 0;
        int recordTypeDCounter = 0;
        while ((record = aug.nextRecord()) != null) {
            if (record.getType().equals("A")) {
                // ignore the record
                continue;
            }
            recordTypeACounter = recordTypeACounter + 1;
            if (record.getType().equals("D")) {
                recordTypeDCounter = recordTypeDCounter + 1;
            }
            XSSFRow vaRowCreated = null;
            XSSFRow latestRowCreated = null;
            XSSFRow prevRowCreated = null;
            // this is VA staging database data
            List<DuplicateTherapyDTO> dtos = aug.findCorrespondingVaObjects(record);
            if (dtos.size() <= 0) {
                vaRowCreated = helper.createNotCustomizedRow(++row);
            }
            for (DuplicateTherapyDTO dto : dtos) {
                vaRowCreated = helper.createVaCustomRow(++row);
                helper.addCell(row, DTReportColumn.ACTION_STATUS.index(), dto.getStatus());// action status
                helper.addCell(row, DTReportColumn.ACITON_DATE.index(), dto.getActionDate()); // action date
                // helper.addCell(row, 3, "");//DATUP WILL DELETE field remain empty
                helper.addCell(row, DTReportColumn.DTCID.index(), dto.getDtcid());// dtcid
                helper.addCell(row, DTReportColumn.DUP_ALLOWANCE.index(), dto.getCustomdupallowance());// dup allowance
                helper.addCell(row, DTReportColumn.DESCRIPTION.index(), dto.getCustomstring());// description

            }
            aug.populateCorrespondingFdbValues(record);
            List<FdbColumnUpdate> mylstvalues = record.getValues();

            latestRowCreated = helper.createLatestFdbRow(++row); // coming from text file
            boolean dupSame = false;
            boolean descriptionSame = false;
            for (FdbColumnUpdate values : mylstvalues) {
                if (record.getType().equals("D")) {
                    helper.addCell(row, DTReportColumn.DATUP_DELETE.index(), "Yes");// DATUP WILL DELETE
                } else {
                    helper.addCell(row, DTReportColumn.DATUP_DELETE.index(), "");// DATUP WILL NOT DELETE
                }

                if (DTReportColumn.DTCID.updateKey().equals(values.getColumn().getName())) {
                    helper.addCell(row, DTReportColumn.DTCID.index(), new Long(values.getCurrentValue()));// dtcid
                }
                if ("D".equals(record.getType())) {
                    break;
                } else {
                    if (DTReportColumn.DUP_ALLOWANCE.updateKey().equals(values.getColumn().getName())) {
                        if (values.getCurrentValue().equals(values.getPreviousValue())) {
                            helper.addCell(row, DTReportColumn.DUP_ALLOWANCE.index(), new Long(values.getCurrentValue()));
                            dupSame = true;
                        } else {
                            //@formatter:off
                            helper.addUpdatedCell(row, DTReportColumn.DUP_ALLOWANCE.index(), 
                                    new Long(values.getCurrentValue()));
                            //@formatter:on
                        }
                    }
                }
                if ("D".equals(record.getType())) {
                    break;
                } else {
                    if (DTReportColumn.DESCRIPTION.updateKey().equals(values.getColumn().getName())) {
                        String currentValue = values.getCurrentValue().trim();
                        String previousValue = values.getPreviousValue().trim();
                        if (currentValue.equals(previousValue)) {
                            helper.addCell(row, DTReportColumn.DESCRIPTION.index(), currentValue);// description
                            descriptionSame = true;
                        } else {
                            helper.addUpdatedCell(row, DTReportColumn.DESCRIPTION.index(), currentValue);// description
                        }
                    }
                }
            } // end for for text file data

            // this is for FDBdatabase values
            prevRowCreated = helper.createPreviousFdbRow(++row);// this is from FDB db query
            for (FdbColumnUpdate vals : mylstvalues) {
                // // row 3 --- block from fdb database --- considered previous value
                if (DTReportColumn.DTCID.updateKey().equals(vals.getColumn().getName())) {
                    helper.addCell(row, DTReportColumn.DTCID.index(), new Long(vals.getPreviousValue()));// dtcid
                }
                if ("D".equals(record.getType())) {
                    if (DTReportColumn.DUP_ALLOWANCE.updateKey().equals(vals.getColumn().getName())) {
                        helper.addCell(row, DTReportColumn.DUP_ALLOWANCE.index(), new Long(vals.getPreviousValue()));
                    }
                } else {
                    if (DTReportColumn.DUP_ALLOWANCE.updateKey().equals(vals.getColumn().getName())) {
                        if (vals.getCurrentValue().equals(vals.getPreviousValue())) {
                            helper.addCell(row, DTReportColumn.DUP_ALLOWANCE.index(), new Long(vals.getPreviousValue()));
                        } else {
                            helper.addUpdatedCell(row, DTReportColumn.DUP_ALLOWANCE.index(), new Long(vals.getPreviousValue()));
                        }
                    }
                }
                if ("D".equals(record.getType())) {
                    if (DTReportColumn.DESCRIPTION.updateKey().equals(vals.getColumn().getName())) {
                        helper.addCell(row, DTReportColumn.DESCRIPTION.index(), vals.getPreviousValue());// description1
                    }
                } else {
                    if (DTReportColumn.DESCRIPTION.updateKey().equals(vals.getColumn().getName())) {
                        if (vals.getCurrentValue().equals(vals.getPreviousValue())) {
                            helper.addCell(row, DTReportColumn.DESCRIPTION.index(), vals.getPreviousValue());// description1
                        } else {
                            helper.addUpdatedCell(row, DTReportColumn.DESCRIPTION.index(), vals.getPreviousValue());
                        }
                    }
                }

            }// end for
            if (dupSame && descriptionSame) {
                if (vaRowCreated != null) {
                    helper.removeRow(vaRowCreated);
                    --row;
                }
                if (latestRowCreated != null) {
                    helper.removeRow(latestRowCreated);
                    --row;
                }
                if (prevRowCreated != null) {
                    helper.removeRow(prevRowCreated);
                    --row;
                }
            } else {
                if ("C".equals(record.getType())) {
                    recordTypeCCounter = recordTypeCCounter + 1;
                }
                helper.createSeparatorRow(++row, DTReportColumn.headers.size());
            }
        }// end while
        if ((recordTypeACounter == 0) || ((recordTypeCCounter == 0) && (recordTypeDCounter == 0))) {
            helper.createNoDataFoundRow(++row);
        }

        return helper;
    }

    /**
     * Save workbook.
     * 
     * @param directory
     *            the directory
     * @param fileName
     *            the file name
     * @param workbook
     *            the workbook
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    public final void saveWorkbook(String directory, String fileName, Workbook workbook) throws IOException {
        FileOutputStream out = new FileOutputStream(new File(directory + "\\" + fileName));
        workbook.write(out);
        out.flush();
    }

}
