package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.fdbreports.FdbColumnUpdate;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.poi.xssf.usermodel.XSSFRow;
import org.springframework.stereotype.Component;

/**
 * The Class DoseRangeReportGenerator.
 */
@Component
public final class DoseRangeReportGenerator implements FdbUpdateReportGenerator {
    private static final String FDBDX_ID = "999";
    private static final String ADD = "A";
    private static final String DELETE = "D";

    /** The drFdbUpdateDataAgumentaion. */

    private static final Long CONCEPT_TYPE = 6L;

    @Resource
    private DrugConceptDao drugConceptDao;

    @Resource
    private DRFdbUpdateDataAgumentaion drFdbUpdateDataAgumentaion;

    /**
     * Instantiates a new dosing report generator.
     */
    public DoseRangeReportGenerator() {
    }

    @Override
    public ExcelUpdateReportHelper generateReport(String fileDate) throws Exception {
        FdbUpdateRecord fdbUpdateRecord;
        ExcelUpdateReportHelper helper = new ExcelUpdateReportHelper();
        int previousRow = 0;
        int currentRow = 0;
        helper.createWorkbook();
        helper.createWorksheet("DR FDB Comparison Report");// shows on the tab in excel
        helper.getWorkbook().getSheet("DR FDB Comparison Report").createFreezePane(1, 2);
        int row = helper.createHeader(DRReportColumn.headers, fileDate);
        int rowsInReport = 0;
        while ((fdbUpdateRecord = getDrFdbUpdateDataAgumentaion().nextRecord()) != null) {

            int changedColumns = 0;
            List<XSSFRow> rowsForRecord = new ArrayList<XSSFRow>();
            Long conceptType =
                    (Long) fdbUpdateRecord.getValue(DRReportColumn.CONCEPT_TYPE.updateKey()).getCurrentValueAsObject();
            // exclude records with concepttype != 6 or fdbdx != '999'
            if (!conceptType.equals(CONCEPT_TYPE)
                    || !fdbUpdateRecord.getValue(DRReportColumn.FDBDX.updateKey()).getCurrentValue().equals(FDBDX_ID)
                    || ADD.equals(fdbUpdateRecord.getType())) {
                continue;
            }
            getDrFdbUpdateDataAgumentaion().populateCorrespondingFdbValues(fdbUpdateRecord);

            List<DoseRangeDTO> dtos = getDrFdbUpdateDataAgumentaion().findCorrespondingVaObjects(fdbUpdateRecord);
            if (dtos.isEmpty()) {

                rowsForRecord.add(helper.createNotCustomizedRow(++row));
            } else {
                row = populateCellsWithDTOs(helper, row, dtos, rowsForRecord);
            }

            List<FdbColumnUpdate> mylstvalues = fdbUpdateRecord.getValues();
            currentRow = ++row;
            previousRow = ++row;
            rowsForRecord.add(helper.createLatestFdbRow(currentRow));
            rowsForRecord.add(helper.createPreviousFdbRow(previousRow));
            // displaying FDB record Datup will delete is not part of the record so it is outside the loop
            if (DELETE.equals(fdbUpdateRecord.getType())) {
                helper.addCell(currentRow, DRReportColumn.DATUP_WILL_DELETE.index(), "Yes");
                rowsInReport++;

            } else {
                helper.addCell(currentRow, DRReportColumn.DATUP_WILL_DELETE.index(), "");
            }

            changedColumns =
                    addPreviousAndCurrentFDB(helper, mylstvalues, currentRow, previousRow,
                            DELETE.equals(fdbUpdateRecord.getType()));
            if (changedColumns == 0 && !DELETE.equals(fdbUpdateRecord.getType())) {
                helper.removeRows(rowsForRecord);
                rowsInReport++;
                row = row - rowsForRecord.size();
                continue;
            }
            helper.createSeparatorRow(++row, DRReportColumn.headers.size());
        }
        if (rowsInReport == 0) {
            helper.createNoDataFoundRow(++row);
        }

        return helper;
    }

    /**
     * @param helper
     *            the excel helper
     * @param mylstvalues
     *            the list of FDB columns
     * @param currentRow
     *            the current row index
     * @param previousRow
     *            the previous row index
     * @param isDeleted
     *            true if the record is being deleted
     * @return the number of changed cells
     */
    private int addPreviousAndCurrentFDB(ExcelUpdateReportHelper helper, List<FdbColumnUpdate> mylstvalues, int currentRow,
            int previousRow, boolean isDeleted) {
        int changedColumns = 0;
        for (FdbColumnUpdate fdbColumnUpdate : mylstvalues) {
            for (DRReportColumn column : DRReportColumn.values()) {
                if (column.updateKey().equalsIgnoreCase(fdbColumnUpdate.getColumn().getName())) {
                    if (DRReportColumn.CONCEPT_ID_NUMBER.updateKey().equals(fdbColumnUpdate.getColumn().getName())) {
                        String conceptDescription =
                                drugConceptDao.getDrugConceptDescription(Long.parseLong(fdbColumnUpdate.getCurrentValue()),
                                        CONCEPT_TYPE);
                        helper.addCell(currentRow, DRReportColumn.CONCEPT_ID_DESCRIPTION.index(), conceptDescription);
                        helper.addCell(previousRow, DRReportColumn.CONCEPT_ID_DESCRIPTION.index(), conceptDescription);

                    }
                    boolean changed = false;
                    switch (fdbColumnUpdate.getColumn().getDataType()) {
                    case DOUBLE:
                        changed = addCellsAsDoubles(helper, previousRow, currentRow, fdbColumnUpdate, column, isDeleted);
                        break;
                    case INTEGER:
                        changed = addCellsAsInteger(helper, previousRow, currentRow, fdbColumnUpdate, column, isDeleted);
                        break;
                    case SMALLINT:
                        changed = addCellsAsSmallInt(helper, previousRow, currentRow, fdbColumnUpdate, column, isDeleted);
                        break;
                    case VARCHAR:
                        changed = addCellsAsString(helper, previousRow, currentRow, fdbColumnUpdate, column, isDeleted);
                        break;
                    default:
                        break;
                    }
                    // check to see if the values are the same
                    if (changed) {
                        changedColumns++;
                    }
                }

            }
        }
        return changedColumns;

    }

    /**
     * @param helper
     *            the excel helper
     * 
     * @param currentRow
     *            the current row index
     * @param previousRow
     *            the previous row index
     * @param fdbColumnUpdate
     *            the update record
     * @param column
     *            the Column Enum
     * @param isDeleted
     *            true if the record is being deleted
     * @return true if cell has changed
     */
    private boolean addCellsAsString(ExcelUpdateReportHelper helper, int previousRow, int currentRow,
            FdbColumnUpdate fdbColumnUpdate, DRReportColumn column, boolean isDeleted) {
        if (fdbColumnUpdate.isChanged() && !isDeleted) {
            helper.addUpdatedCell(currentRow, column.index(),
                    fdbColumnUpdate.getCurrentValue() == null ? "" : fdbColumnUpdate.getCurrentValue());
            helper.addUpdatedCell(previousRow, column.index(), fdbColumnUpdate.getPreviousValue() == null ? ""
                    : fdbColumnUpdate.getPreviousValue());
            return true;
        }
        helper.addCell(currentRow, column.index(),
                fdbColumnUpdate.getCurrentValue() == null ? "" : fdbColumnUpdate.getCurrentValue());
        helper.addCell(previousRow, column.index(),
                fdbColumnUpdate.getPreviousValue() == null ? "" : fdbColumnUpdate.getPreviousValue());
        return false;

    }

    /**
     * @param helper
     *            the excel helper
     * 
     * @param currentRow
     *            the current row index
     * @param previousRow
     *            the previous row index
     * @param fdbColumnUpdate
     *            the update record
     * @param column
     *            the Column Enum
     * @param isDeleted
     *            true if the record is being deleted
     * 
     * @return true if cell has changed
     * 
     */
    private boolean addCellsAsSmallInt(ExcelUpdateReportHelper helper, int previousRow, int currentRow,
            FdbColumnUpdate fdbColumnUpdate, DRReportColumn column, boolean isDeleted) {
        Integer currentValue = fdbColumnUpdate.getCurrentValue() == null ? 0 : new Integer(fdbColumnUpdate.getCurrentValue());
        Integer previousValue =
                fdbColumnUpdate.getPreviousValue() == null ? 0 : new Integer(fdbColumnUpdate.getPreviousValue());
        if (fdbColumnUpdate.isChanged() && !isDeleted) {
            helper.addUpdatedCell(currentRow, column.index(), currentValue);
            helper.addUpdatedCell(previousRow, column.index(), previousValue);
            return true;
        }
        helper.addCell(currentRow, column.index(), currentValue);
        helper.addCell(previousRow, column.index(), previousValue);
        return false;
    }

    /**
     * @param helper
     *            the excel helper
     * 
     * @param currentRow
     *            the current row index
     * @param previousRow
     *            the previous row index
     * @param fdbColumnUpdate
     *            the update record
     * @param column
     *            the Column Enum
     * @param isDeleted
     *            true if the record is being deleted
     * @return true if cell has changed
     */
    private boolean addCellsAsInteger(ExcelUpdateReportHelper helper, int previousRow, int currentRow,
            FdbColumnUpdate fdbColumnUpdate, DRReportColumn column, boolean isDeleted) {

        Long currentValue = fdbColumnUpdate.getCurrentValue() == null ? 0L : new Long(fdbColumnUpdate.getCurrentValue());
        Long previousValue = fdbColumnUpdate.getPreviousValue() == null ? 0L : new Long(fdbColumnUpdate.getPreviousValue());
        if (fdbColumnUpdate.isChanged() && !isDeleted) {
            helper.addUpdatedCell(currentRow, column.index(), currentValue);
            helper.addUpdatedCell(previousRow, column.index(), previousValue);
            return true;
        }
        helper.addCell(currentRow, column.index(), currentValue);
        helper.addCell(previousRow, column.index(), previousValue);

        return false;
    }

    /**
     * @param helper
     *            the excel helper
     * 
     * @param currentRow
     *            the current row index
     * @param previousRow
     *            the previous row index
     * @param fdbColumnUpdate
     *            the update record
     * @param column
     *            the Column Enum
     * @param isDeleted
     *            true if the record is being deleted
     * @return true if cell has changed
     */
    private boolean addCellsAsDoubles(ExcelUpdateReportHelper helper, int previousRow, int currentRow,
            FdbColumnUpdate fdbColumnUpdate, DRReportColumn column, boolean isDeleted) {
        Double currentValue = fdbColumnUpdate.getCurrentValue() == null ? 0.0 : new Double(fdbColumnUpdate.getCurrentValue());
        Double previousValue =
                fdbColumnUpdate.getPreviousValue() == null ? 0.0 : new Double(fdbColumnUpdate.getPreviousValue());
        if (fdbColumnUpdate.isChanged() && !isDeleted) {
            helper.addUpdatedCell(currentRow, column.index(), currentValue);
            helper.addUpdatedCell(previousRow, column.index(), previousValue);
            return true;

        }
        helper.addCell(currentRow, column.index(), currentValue);
        helper.addCell(previousRow, column.index(), previousValue);

        return false;
    }

    /**
     * @param helper
     *            the excel helper
     * @param row
     *            the row in the spreadsheet
     * @param dtos
     *            the objects we will iterate over
     * @param rowsForRecord
     *            the rows for this record so we can delete them if there is no changes
     * @return the next row
     */

    private int populateCellsWithDTOs(ExcelUpdateReportHelper helper, int row, List<DoseRangeDTO> dtos,
            List<XSSFRow> rowsForRecord) {
        for (DoseRangeDTO doseRangeDTO : dtos) {
            rowsForRecord.add(helper.createVaCustomRow(++row));
            helper.addCell(row, DRReportColumn.ACTION_STATUS.index(), doseRangeDTO.getStatus());
            helper.addCell(row, DRReportColumn.ACTION_DATE.index(), doseRangeDTO.getActionDate());
            helper.addCell(row, DRReportColumn.CONCEPT_ID_NUMBER.index(), doseRangeDTO.getConceptid());
            helper.addCell(row, DRReportColumn.CONCEPT_ID_DESCRIPTION.index(), doseRangeDTO.getConceptDescription());
            helper.addCell(row, DRReportColumn.CONCEPT_TYPE.index(), doseRangeDTO.getConcepttype());

            helper.addCell(row, DRReportColumn.AGE_LOW_IN_DAYS.index(), doseRangeDTO.getAgelowindays());
            helper.addCell(row, DRReportColumn.AGE_HIGH_IN_DAYS.index(), doseRangeDTO.getAgehighindays());
            helper.addCell(row, DRReportColumn.DOSE_ROUTE_ID.index(), doseRangeDTO.getDoserouteid());
            helper.addCell(row, DRReportColumn.DOSE_TYPE_ID.index(), doseRangeDTO.getDosetypeid());

            helper.addCell(row, DRReportColumn.FDBDX.index(), doseRangeDTO.getFdbdx());
            helper.addCell(row, DRReportColumn.HITTYPE.index(), doseRangeDTO.getHitType());

            helper.addCell(row, DRReportColumn.DXID.index(), doseRangeDTO.getDxid());
            helper.addCell(row, DRReportColumn.DOSE_LOW.index(), doseRangeDTO.getDoselow());
            helper.addCell(row, DRReportColumn.DOSE_LOW_UNITS.index(), doseRangeDTO.getDoselowunits());
            helper.addCell(row, DRReportColumn.DOSE_HIGH.index(), doseRangeDTO.getDosehigh());

            helper.addCell(row, DRReportColumn.DOSE_HIGH_UNITS.index(), doseRangeDTO.getDosehighunits());
            helper.addCell(row, DRReportColumn.DOSE_FORM_LOW.index(), doseRangeDTO.getDoseformlow());
            helper.addCell(row, DRReportColumn.DOSE_FORM_LOW_UNITS.index(), doseRangeDTO.getDoseformlowunits());
            helper.addCell(row, DRReportColumn.DOSE_FORM_HIGH.index(), doseRangeDTO.getDoseformhigh());
            helper.addCell(row, DRReportColumn.DOSE_FORM_HIGH_UNITS.index(), doseRangeDTO.getDoseformhighunits());

            helper.addCell(row, DRReportColumn.FREQUENCY_LOW.index(), doseRangeDTO.getFrequencylow());
            helper.addCell(row, DRReportColumn.FREQUENCY_HIGH.index(), doseRangeDTO.getFrequencyhigh());
            helper.addCell(row, DRReportColumn.DURATION_LOW.index(), doseRangeDTO.getDurationlow());
            helper.addCell(row, DRReportColumn.DURATION_HIGH.index(), doseRangeDTO.getDurationhigh());
            helper.addCell(row, DRReportColumn.MAXIMUM_DURATION.index(), doseRangeDTO.getMaxduration());
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE.index(), doseRangeDTO.getMaxsingledose());
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_UNITS.index(), doseRangeDTO.getMaxsingledoseunits());
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_FORM.index(), doseRangeDTO.getMaxsingledoseform());
            //@formatter:off
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_FORM_UNITS.index(),
                    doseRangeDTO.getMaxsingledoseformunits());
          //@formatter:on
            helper.addCell(row, DRReportColumn.MAX_SINGLE_NTE_DOSE.index(), doseRangeDTO.getMaxSingleNTEDose());
            helper.addCell(row, DRReportColumn.MAX_SINGLE_NTE_DOSE_UNIT.index(), doseRangeDTO.getMaxSingleNTEDoseUnit());
            helper.addCell(row, DRReportColumn.MAX_SINGLE_NTE_DOSE_FORM.index(), doseRangeDTO.getMaxSingleNTEDoseForm());
            helper.addCell(row, DRReportColumn.MAX_SINGLE_NTE_DOSE_FORM_UNIT.index(),
                    doseRangeDTO.getMaxSingleNTEDoseFormUnit());
            
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE.index(), doseRangeDTO.getMaxdailydose());
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_UNITS.index(), doseRangeDTO.getMaxdailydoseunits());
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_FORM.index(), doseRangeDTO.getMaxdailydoseform());
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_FORM_UNITS.index(), doseRangeDTO.getMaxdailydoseformunits());
            helper.addCell(row, DRReportColumn.MAXIMUM_LIFETIME_DOSE.index(), doseRangeDTO.getMaxlifetimedose());
            helper.addCell(row, DRReportColumn.MAXIMUM_LIFETIME_DOSE_UNITS.index(), doseRangeDTO.getMaxlifetimedoseunits());
            helper.addCell(row, DRReportColumn.MAXIMUM_LIFETIME_DOSE_FORM.index(), doseRangeDTO.getMaxlifetimedoseform());
            helper.addCell(row, DRReportColumn.MAXIMUM_LIFETIME_DOSE_FORM_UNITS.index(),
                    doseRangeDTO.getMaxlifetimedoseformunits());
            helper.addCell(row, DRReportColumn.DOSE_RATE_LOW.index(), doseRangeDTO.getDoseratelow());
            helper.addCell(row, DRReportColumn.DOSE_RATE_LOW_UNITS.index(), doseRangeDTO.getDoseratelowunits());
            helper.addCell(row, DRReportColumn.DOSE_RATE_HIGH.index(), doseRangeDTO.getDoseratehigh());
            helper.addCell(row, DRReportColumn.DOSE_RATE_HIGH_UNITS.index(), doseRangeDTO.getDoseratehighunits());
            helper.addCell(row, DRReportColumn.DOSE_FORM_RATE_LOW.index(), doseRangeDTO.getDoseformratelow());
            helper.addCell(row, DRReportColumn.DOSE_FORM_RATE_LOW_UNITS.index(), doseRangeDTO.getDoseformratelowunits());
            helper.addCell(row, DRReportColumn.DOSE_FORM_RATE_HIGH.index(), doseRangeDTO.getDoseformratehigh());
            helper.addCell(row, DRReportColumn.DOSE_FORM_RATE_HIGH_UNITS.index(), doseRangeDTO.getDoseformratehighunits());
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_RATE.index(), doseRangeDTO.getMaxsingledoserate());
            //@formatter:off
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_RATE_UNITS.index(), 
                    doseRangeDTO.getMaxsingledoserateunits());
          //@formatter:on
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_FORM_RATE.index(), doseRangeDTO.getMaxsingledoseformrate());
            helper.addCell(row, DRReportColumn.MAXIMUM_SINGLE_DOSE_FORM_RATE_UNITS.index(),
                    doseRangeDTO.getMaxsingledoseformrateunits());
            //@formatter:off
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_FORM_RATE.index(), 
                    doseRangeDTO.getMaxdailydoseformrate());
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_RATE.index(), 
                    doseRangeDTO.getMaxdailydoseformrateunits());
            //@formatter:on
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_FORM_RATE_UNITS.index(),
                    doseRangeDTO.getMaxdailydoseformrateunits());
            helper.addCell(row, DRReportColumn.MAXIMUM_DAILY_DOSE_RATE_UNITS.index(), doseRangeDTO.getMaxdailydoserateunits());
            //@formatter:off
            helper.addCell(row, DRReportColumn.HEPATIC_IMPAIRMENT_INDICATOR.index(), 
                    doseRangeDTO.getHepaticimpairmentind());
            //@formatter:on
            helper.addCell(row, DRReportColumn.RENAL_IMPAIRMENT_INDICATOR.index(), doseRangeDTO.getRenalimpairmentind());
            helper.addCell(row, DRReportColumn.CRCL_THRESHOLD.index(), doseRangeDTO.getCrclthreshhold());
            helper.addCell(row, DRReportColumn.CRCL_THRESHOLD_UNITS.index(), doseRangeDTO.getCrclthreshholdunits());
            helper.addCell(row, DRReportColumn.LOW_ELIMINATION_HALF_LIFE.index(), doseRangeDTO.getLoweliminationhalflife());
            helper.addCell(row, DRReportColumn.HIGH_ELIMINATION_HALF_LIFE.index(), doseRangeDTO.getHigheliminationhalflife());
            helper.addCell(row, DRReportColumn.HALF_LIFE_UNITS.index(), doseRangeDTO.getHalflifeunits());
            helper.addCell(row, DRReportColumn.WEIGHT_REQUIRED_INDICATOR.index(), doseRangeDTO.getWeightrequiredind());
            helper.addCell(row, DRReportColumn.BSA_REQUIRED_INDICATOR.index(), doseRangeDTO.getBsarequiredind());

        }
        return row;
    }

    /**
     * Gets the drugConceptDao.
     * 
     * @return the drugConceptDao
     */
    public DrugConceptDao getDrugConceptDao() {
        return drugConceptDao;
    }

    /**
     * Sets the drugConceptDao.
     * 
     * @param drugConceptDao
     *            the drugConceptDao to set
     */
    public void setDrugConceptDao(DrugConceptDao drugConceptDao) {
        this.drugConceptDao = drugConceptDao;
    }

    /**
     * Gets the drFdbUpdateDataAgumentaion.
     * 
     * @return the drFdbUpdateDataAgumentaion
     */
    public DRFdbUpdateDataAgumentaion getDrFdbUpdateDataAgumentaion() {
        return drFdbUpdateDataAgumentaion;
    }

    /**
     * Sets the drFdbUpdateDataAgumentaion.
     * 
     * @param drFdbUpdateDataAgumentaion
     *            the drFdbUpdateDataAgumentaion to set
     */
    public void setDrFdbUpdateDataAgumentaion(DRFdbUpdateDataAgumentaion drFdbUpdateDataAgumentaion) {
        this.drFdbUpdateDataAgumentaion = drFdbUpdateDataAgumentaion;
    }

}
