package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.Iterator;
import java.util.List;

/**
 * The Class DosingFdbUpdateDataAugmentation.
 */
public final class DosingFdbUpdateDataAugmentation extends FdbUpdateDataAugmentation {

    /** The Constant CONCEPTID. */
    private static final String CONCEPTID = "conceptid";

    /** The Constant DOSEROUTEID. */
    private static final String DOSEROUTEID = "doserouteid";

    /** The Constant DOSETYPEID. */
    private static final String DOSETYPEID = "dosetypeid";

    /** The Constant AGELOWINDAYS. */
    private static final String AGELOWINDAYS = "agelowindays";

    /** The Constant AGEHIGHINDAYS. */
    private static final String AGEHIGHINDAYS = "agehighindays";

    /** The service. */
    private final DoseRangeService service;

    /**
     * Instantiates a new dosing fdb update data augmentation.
     * 
     * @param service
     *            the service
     */
    public DosingFdbUpdateDataAugmentation(DoseRangeService service) {
        super(service.getDao(false));
        this.service = service;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.fdbreports.impl.FdbUpdateDataAugmentation#findCorrespondingVaObjects(gov.va.med.pharmacy
     * .pecs.fdbreports.FdbUpdateRecord)
     */
    @Override
    public List<DoseRangeDTO> findCorrespondingVaObjects(FdbUpdateRecord rec) {
        Long conceptId = (Long) rec.getValue(CONCEPTID).getCurrentValueAsObject();
        String doseRouteId = rec.getValue(DOSEROUTEID).getCurrentValueAsObject().toString();
        String doseTypeId = rec.getValue(DOSETYPEID).getCurrentValueAsObject().toString();
        List<DoseRangeDTO> dtos = service.getStagingData(conceptId, doseRouteId, doseTypeId);

        long ageLow = (Long) rec.getValue(AGELOWINDAYS).getCurrentValueAsObject();
        long ageHigh = (Long) rec.getValue(AGEHIGHINDAYS).getCurrentValueAsObject();
        for (Iterator<DoseRangeDTO> iter = dtos.iterator(); iter.hasNext();) {
            DoseRangeDTO dto = iter.next();
            if (!dto.isCurrent() || Status.STATUS_DELETED.equals(dto.getStatus())
                    || Status.STATUS_REJECTED.equals(dto.getStatus()) || !ageRangeOverlap(dto, ageLow, ageHigh)) {
                iter.remove();
            }
        }
        return dtos;
    }

    /**
     * Age range overlap.
     * 
     * @param dto
     *            the dto
     * @param ageLow
     *            the age low
     * @param ageHigh
     *            the age high
     * @return true, if successful
     */
    private boolean ageRangeOverlap(DoseRangeDTO dto, long ageLow, long ageHigh) {
        return dto.getAgehighindays() <= ageHigh && dto.getAgehighindays() >= ageLow || dto.getAgelowindays() <= ageHigh
                && dto.getAgelowindays() >= ageLow;
    }

}
