package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateParser;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

/**
 * Creates and saves the Duplicate Therapy Comparison report.
 * 
 * @author VHAISACARUSR
 * 
 */
@Component
public final class DuplicateTherapyComparisonReportProcessor extends AbstractComparisonReportProcessor implements
        InitializingBean {
    private static final String FDBDUPLICATETHERAPY_UPD = "FDBDUPLICATETHERAPY.UPD";

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(DuplicateTherapyComparisonReportProcessor.class);

    private static final String DT_CONCEPT_KEY = "DT";

    /** The dt cat service. */
    @Resource
    private DuplicateTherapyService dtCatService;

    /**
     * Process duplicate therapy.
     * 
     * @param parser
     *            the parser
     */
    @Override
    public void processComparisonReport(FdbUpdateParser parser) {
        try {
            File f = parser.parseConcept(FDBDUPLICATETHERAPY_UPD);
            ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
            DTFdbUpdateDataAugmentation aug = new DTFdbUpdateDataAugmentation(dtCatService);
            aug.setInput(in);
            DTReportGenerator reportGenerator = new DTReportGenerator(aug);
            ExcelUpdateReportHelper helper = reportGenerator.generateReport(parser.getFdbIssueDate());
            String fileNameToSave = "DTReport.xlsx";
            saveReport(helper, fileNameToSave, DT_CONCEPT_KEY);
            getComparisonReportUtility().cleanup();

        } catch (Exception ex) {
            String msg = "Failed to process Duplicate Therapy inside processDuplicate therapy.";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {

        if (dtCatService == null) {
            throw new Exception("DtCatService has not been injected into FdbUpdateReportServiceImpl.");
        }
    }
}
