package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdiminteraction;
import gov.va.med.pharmacy.pecs.fdbreports.FdbColumnUpdate;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 * Takes the methods for sorting drug pairs out of the {@link FdbUpdateReportGenerator}.
 * 
 * @author VHAISACARUSR
 * 
 */
public final class SortedDrugPairHelper {

    /**
     * 
     */
    private SortedDrugPairHelper() {

    }

    /**
     * Builds the dp array list.
     * 
     * @param augDP
     *            the aug dp
     * @return the array list
     * @throws Exception
     *             the exception
     */
    public static List<SortedDrugPair> buildDpArrayList(DPFdbUpdateDataAugmentation augDP) throws Exception {
        FdbUpdateRecord rec;
        List<SortedDrugPair> sortedUpdArrayList = new ArrayList<SortedDrugPair>();
        while ((rec = augDP.nextRecord()) != null) {
            List<FdbColumnUpdate> mylstvalues = rec.getValues();
            augDP.populateCorrespondingFdbValues(rec);
            SortedDrugPair sortedDPs = new SortedDrugPair();
            for (FdbColumnUpdate fdbColumnUpdate : mylstvalues) {
                createSortedDrugPairFromUpdate(sortedDPs, fdbColumnUpdate);
                setDatupActionFromUpdate(rec, sortedDPs);
            }
            sortedUpdArrayList.add(sortedDPs);
        }
        Collections.sort(sortedUpdArrayList, new Comparator<SortedDrugPair>() {
            @Override
            public int compare(SortedDrugPair o1, SortedDrugPair o2) {
                return o1.getInteractionid().compareTo(o2.getInteractionid());
            }
        });
        // Removing reverse DPs
        SortedDrugPair sortedDrugPair;
        List<SortedDrugPair> sortedDrugPairs = new ArrayList<SortedDrugPair>();

        for (Iterator<SortedDrugPair> it1 = sortedUpdArrayList.iterator(); it1.hasNext();) {
        sortedDrugPair = it1.next();
        if (sortedDrugPair.getInteractionid() <= FdbDdiminteraction.MAX_DDI_FORWARD_INTERACTION_ID) {
            sortedDrugPairs.add(sortedDrugPair);
        }
      }

        return sortedDrugPairs;
    }

    /**
     * @param fdbUpdateRecord
     *            the FDB Update Record
     * @param sortedDrugPair
     *            the sorted drug pair
     */
    private static void setDatupActionFromUpdate(FdbUpdateRecord fdbUpdateRecord, SortedDrugPair sortedDrugPair) {
        if (fdbUpdateRecord.getType().equals("D")) {
            sortedDrugPair.setDatupAction("Delete");
        } else if (fdbUpdateRecord.getType().equals("C")) {
            sortedDrugPair.setDatupAction("Change");
        } else if (fdbUpdateRecord.getType().equals("A")) {
            sortedDrugPair.setDatupAction("Add");
        }
    }

    /**
     * @param drugPair
     *            the drug pair to be manipulated
     * @param fdbColumnUpdate
     *            the update file
     */
    private static void createSortedDrugPairFromUpdate(SortedDrugPair drugPair, FdbColumnUpdate fdbColumnUpdate) {
        if (fdbColumnUpdate.getColumn().getName().equals("rtgenid1")) {
            drugPair.setRtgenid1(new Long(fdbColumnUpdate.getCurrentValue()));
        } else if (fdbColumnUpdate.getColumn().getName().equals("rtgenid2")) {
            drugPair.setRtgenid2(new Long(fdbColumnUpdate.getCurrentValue()));
        } else if (fdbColumnUpdate.getColumn().getName().equals("interactionid")) {
            drugPair.setInteractionid(new Long(fdbColumnUpdate.getCurrentValue()));
        }
    }

}
