package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.persistence.contact.ContactUs;
import gov.va.med.pharmacy.pecs.persistence.contact.ContactUsDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.contact.ContactUsDTO;
import gov.va.med.pharmacy.pecs.service.contact.ContactUsService;
import gov.va.med.pharmacy.pecs.util.BlobHandler;

import java.sql.SQLException;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service for updating and retrieving the contact us content.
 * 
 * @author VHAISACARUSR
 * 
 */
@Service
public final class ContactUsServiceImpl implements ContactUsService {
    private static final Logger LOGGER = Logger.getLogger(ContactUsServiceImpl.class);
    @Resource
    private ContactUsDAOImpl contactUsDAOImpl;
    @Resource
    private BlobHandler blobHandler;

    @Override
    @Transactional
    public ContactUsDTO findContactContent() {
        ContactUs contactUs = new ContactUs();
        ContactUsDTO dto = new ContactUsDTO();

        contactUs = contactUsDAOImpl.findLatestContactUs();
        if (contactUs == null) {
            return dto;
        }
        LOGGER.debug(contactUs.toString());
        dto.setContactId(contactUs.getContactId());
        dto.setCreatedBy(contactUs.getCreatedBy());
        dto.setCreatedDate(contactUs.getCreatedDate());
        try {
            dto.setPostData(blobHandler.toByteArray(contactUs.getContent()));
            LOGGER.debug(new String(dto.getPostData()));
        } catch (SQLException e) {

            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return dto;

    }

    @Override
    @Transactional
    public void saveContactContent(ContactUsDTO content) {
        ContactUs contactUs = new ContactUs();
        contactUs.setContactId(content.getContactId());
        contactUs.setCreatedBy(content.getCreatedBy());
        contactUs.setCreatedDate(content.getCreatedDate());
        contactUs.setContent(blobHandler.createBlob(content.getPostData()));
        contactUsDAOImpl.save(contactUs);
    }

    /**
     * Gets the contactUsDAOImpl.
     * 
     * @return the contactUsDAOImpl
     */
    public ContactUsDAOImpl getContactUsDAOImpl() {
        return contactUsDAOImpl;
    }

    /**
     * Sets the contactUsDAOImpl.
     * 
     * @param contactUsDAOImpl
     *            the contactUsDAOImpl to set
     */
    public void setContactUsDAOImpl(ContactUsDAOImpl contactUsDAOImpl) {
        this.contactUsDAOImpl = contactUsDAOImpl;
    }

    /**
     * Gets the blobHandler.
     * 
     * @return the blobHandler
     */
    public BlobHandler getBlobHandler() {
        return blobHandler;
    }

    /**
     * Sets the blobHandler.
     * 
     * @param blobHandler
     *            the blobHandler to set
     */
    public void setBlobHandler(BlobHandler blobHandler) {
        this.blobHandler = blobHandler;
    }
}
