package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.List;

/**
 * 
 * This class implements RequestCountSupport. Because we only care about drug pairs associated with an approved
 * DrugDrugInteraction we needed more specific queries than the GenericRequestCountSupport.
 * 
 * @author VHAISACARUSR
 * 
 */
public class DrugPairRequestCountSupport implements RequestCountSupport {
    // private static final Log LOGGER = LogFactory.getLog(DrugPairRequestCountSupport.class);
    private final HibernateDAO<?> dao;

    /**
     * This Constructor takes the DAO associated with the hibernate session for use in the class.
     * 
     * @param drugDrugDAO
     *            The DAO associated with the hibernate session.
     */
    DrugPairRequestCountSupport(DrugPairDAOImpl drugDrugDAO) {
        this.dao = drugDrugDAO;
    }

    @Override
    public final int getCountsByRequestorAndStatus(String userName, String status) {
        String query =
                "select count (distinct interaction)  from DrugDrugInteractionVa as interaction "
                        + "inner join interaction.drugPairs as pair where "
                        + "interaction.exportManager.currentVersion.status.name = '" + Status.STATUS_APPROVED + "'"
                        + "and pair.requestor = '" + userName + "'"
                        + (status == null ? "" : " and pair.exportManager.currentVersion.status.name = '" + status + "' ");

        Long count = (Long) dao.getSession().createQuery(query).iterate().next();
        return count.intValue();
    }

    @Override
    public final int getCountsByAssignedAndStatus(String userName, String status) {
        return getCountsByAssignedAndStatus(userName, status, null);
    }

    @Override
    public final int getCountsByAssignedAndStatus(String userName, String status, String actionUser) {
        if ("UNASSIGNED".equals(userName)) {
            String query =

                    "select count (distinct interaction.interactionId)  from DrugDrugInteractionVa as interaction "
                            + "inner join interaction.drugPairs as pair "
                            + "where pair.exportManager.currentVersion.assignedUser"
                            + (userName == null ? " is null " : " = '" + userName + "' ")
                            + " and pair.exportManager.currentVersion.status.name not in ('"
                            + Status.STATUS_APPROVED
                            + "','"
                            + Status.STATUS_DELETED
                            + "','"
                            + Status.STATUS_REJECTED
                            + "') "
                            + (actionUser == null ? "" : " and pair.exportManager.currentVersion.actionBy != '" + actionUser
                                    + "' ") + " and interaction.exportManager.currentVersion.status.name = '"
                            + Status.STATUS_APPROVED + "'";
            Long count = (Long) dao.getSession().createQuery(query).iterate().next();
            return count.intValue();
        } else {
            String query =
                    "select count (distinct interaction)  from DrugDrugInteractionVa as interaction "
                            + "inner join interaction.drugPairs as pair "
                            + "where pair.exportManager.currentVersion.assignedUser"
                            + (userName == null ? " is null " : " = '" + userName + "' ")
                            + (status == null ? "" : " and pair.exportManager.currentVersion.status.name = '" + status + "'")
                            + (actionUser == null ? "" : " and pair.exportManager.currentVersion.actionBy != '" + actionUser
                                    + "' ") + " and interaction.exportManager.currentVersion.status.name = '"
                            + Status.STATUS_APPROVED + "'";
            Long count = (Long) dao.getSession().createQuery(query).iterate().next();
            return count.intValue();
        }
    }

    @Override
    public final int getCountsByStatus(String status) {
        String query =
                "select count (distinct interaction)  from DrugDrugInteractionVa as interaction "
                        + "inner join interaction.drugPairs as pair "
                        + "where pair.exportManager.currentVersion.status.name = '" + status + "'"
                        + " and interaction.exportManager.currentVersion.status.name = '" + Status.STATUS_APPROVED + "'";
        Long count = (Long) dao.getSession().createQuery(query).iterate().next();
        return count.intValue();
    }

    @Override
    public final int getCountsByRequestorAndStatuses(String userName, List<String> statuses) {
        String inStatus = null;
        for (String status : statuses) {
            if (inStatus == null) {
                inStatus = "'" + status + "'";
                continue;
            }
            inStatus = inStatus + " ," + "'" + status + "'";
        }
        String query =
                "select count (distinct interaction)  from DrugDrugInteractionVa as interaction "
                        + "inner join interaction.drugPairs as pair where "
                        + "interaction.exportManager.currentVersion.status.name = '" + Status.STATUS_APPROVED + "'"
                        + "and pair.requestor = '" + userName + "'"
                        + (inStatus == null ? "" : " and pair.exportManager.currentVersion.status.name in (" + inStatus + ") ");

        Long count = (Long) dao.getSession().createQuery(query).iterate().next();
        return count.intValue();
    }

    @Override
    public final int getCountsByAssignedAndStatuses(String userName, List<String> statuses) {
        String inStatus = null;
        for (String status : statuses) {
            if (inStatus == null) {
                inStatus = "'" + status + "'";
                continue;
            }
            inStatus = inStatus + " ," + "'" + status + "'";
        }
        String query =
                "select count (distinct interaction)  from DrugDrugInteractionVa as interaction "
                        + "inner join interaction.drugPairs as pair " + "where pair.exportManager.currentVersion.assignedUser"
                        + (userName == null ? " is null " : " = '" + userName + "' ")
                        + (inStatus == null ? "" : " and pair.exportManager.currentVersion.status.name in (" + inStatus + ") ")
                        + " and interaction.exportManager.currentVersion.status.name = '" + Status.STATUS_APPROVED + "'";
        Long count = (Long) dao.getSession().createQuery(query).iterate().next();
        return count.intValue();
    }

}
