package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.List;

/**
 * The Class GenericRequestCountSupport.
 * 
 * @author VHAISACARUSR
 */
public class GenericRequestCountSupport implements RequestCountSupport {

    /** The cls. */
    private final Class<?> cls;

    /** The dao. */
    private final HibernateDAO<?> dao;

    /**
     * Instantiates a new generic request count support.
     * 
     * @param cls
     *            the cls
     * @param dao
     *            the dao
     */
    GenericRequestCountSupport(Class<?> cls, HibernateDAO<?> dao) {
        this.cls = cls;
        this.dao = dao;
    }

    /**
     * Gets the counts by requestor and status.
     * 
     * @param userName
     *            the user name
     * @param status
     *            the status
     * @return the counts by requestor and status
     */
    @Override
    public final int getCountsByRequestorAndStatus(String userName, String status) {
        String query =
                "from " + cls.getName() + " where requestor = '" + userName + "' "
                        + (status == null ? "" : " and exportManager" + ".currentVersion.status.name = '" + status + "'");
        int count = (int) dao.resultCount(query);
        return count;
    }

    /**
     * Gets the counts by assigned and status.
     * 
     * @param userName
     *            the user name
     * @param status
     *            the status
     * @return the counts by assigned and status
     */
    @Override
    public final int getCountsByAssignedAndStatus(String userName, String status) {
        return getCountsByAssignedAndStatus(userName, status, null);
    }

    /**
     * Gets the counts by assigned and status.
     * 
     * @param userName
     *            the user name
     * @param status
     *            the status
     * @param actionUser
     *            the action user
     * @return the counts by assigned and status
     */
    @Override
    public final int getCountsByAssignedAndStatus(String userName, String status, String actionUser) {
        if ("UNASSIGNED".equals(userName)) {
            String query =
                    "from "
                            + cls.getName()
                            + " where exportManager.currentVersion.assignedUser"
                            + (userName == null ? " is null " : " = '" + userName + "' ")
                            + " and exportManager.currentVersion.status.name != '"
                            + Status.STATUS_APPROVED
                            + "' "
                            + " and exportManager.currentVersion.status.name != '"
                            + Status.STATUS_DELETED
                            + "' "
                            + " and exportManager.currentVersion.status.name != '"
                            + Status.STATUS_REJECTED
                            + "' "
                            + (actionUser == null ? "" : " and exportManager.currentVersion.exportManager.actionBy != '"
                                    + actionUser + "' ");
            int count = (int) dao.resultCount(query);
            return count;
        } else {
            String query =
                    "from " + cls.getName() + " where exportManager.currentVersion.assignedUser"
                            + (userName == null ? " is null " : " = '" + userName + "' ")
                            + (status == null ? "" : " and exportManager.currentVersion.status.name = '" + status + "'")
                            + (actionUser == null ? "" : " and exportManager.currentVersion.actionBy != '" + actionUser + "' ");
            int count = (int) dao.resultCount(query);
            return count;
        }
    }

    /**
     * Gets the counts by status.
     * 
     * @param status
     *            the status
     * @return the counts by status
     */
    @Override
    public final int getCountsByStatus(String status) {
        String query = "from " + cls.getName() + " where exportManager.currentVersion.status.name = '" + status + "'";
        int count = (int) dao.resultCount(query);
        return count;
    }

    /**
     * Gets the counts by requestor and statuses.
     * 
     * @param userName
     *            the user name
     * @param statuses
     *            the statuses
     * @return the counts by requestor and statuses
     */
    @Override
    public final int getCountsByRequestorAndStatuses(String userName, List<String> statuses) {
        String inStatus = null;
        for (String status : statuses) {
            if (inStatus == null) {
                inStatus = "'" + status + "'";
                continue;
            }
            inStatus = inStatus + "," + "'" + status + "'";

        }
        String query =
                "from " + cls.getName() + " where requestor = '" + userName + "' "
                        + (inStatus == null ? "" : " and exportManager.currentVersion.status.name in (" + inStatus + ")");
        int count = (int) dao.resultCount(query);
        return count;
    }

    /**
     * Gets the counts by assigned and statuses.
     * 
     * @param userName
     *            the user name
     * @param statuses
     *            the statuses
     * @return the counts by assigned and statuses
     */
    @Override
    public final int getCountsByAssignedAndStatuses(String userName, List<String> statuses) {
        String inStatus = null;
        for (String status : statuses) {
            if (inStatus == null) {
                inStatus = "'" + status + "'";
                continue;
            }

            inStatus = inStatus + "," + "'" + status + "'";

        }
        String query =
                "from " + cls.getName() + " where exportManager.currentVersion.assignedUser"
                        + (userName == null ? " is null " : " = '" + userName + "' ")
                        + (inStatus == null ? "" : " and exportManager.currentVersion.status.name in (" + inStatus + ")");
        int count = (int) dao.resultCount(query);
        return count;
    }
}
