package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.lock.ConceptRecordId;
import gov.va.med.pharmacy.pecs.lock.Lock;
import gov.va.med.pharmacy.pecs.lock.LockDAO;
import gov.va.med.pharmacy.pecs.service.RecordLockingService;

import java.util.Calendar;
import java.util.List;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * This is the implementation class for <code>RecordLockingService</code>.
 * 
 * @author vhaisacarusr
 * 
 */
@Service
public class RecordLockingServiceImpl implements RecordLockingService {
    private static final int RECORD_TIMEOUT = 60 * 20 * 1000;

    @Resource
    private LockDAO lockDAO;
    private static final Logger LOGGER = Logger.getLogger(RecordLockingServiceImpl.class);

    private static final String DELETE_QUERY = "from Lock as lock where lock.user =  '";

    @Override
    @Transactional(readOnly = true)
    public final boolean isLocked(ConceptRecordId conceptRecordId) {
        Lock lock = getLockingInformation(conceptRecordId);
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug(lock);
        }
        if (lock == null) {
            return false;
        }
        long date = lock.getDateLocked().getTime();
        long currentDate = Calendar.getInstance().getTimeInMillis();
        long timelapsed = currentDate - date;
        if (timelapsed < RECORD_TIMEOUT) {
            return true;
        }
        deleteLock(lock);
        return false;
    }

    @Override
    @Transactional(readOnly = true)
    public final Lock getLockingInformation(ConceptRecordId conceptRecordId) {
        Lock lock = getLockDAO().findById(conceptRecordId);
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug(lock);
        }
        return lock;

    }

    @Override
    @Transactional
    public final void lockRecord(Lock lock) {
        getLockDAO().save(lock);
    }

    @Override
    @Transactional
    public final void deleteLock(Lock lock) {
        getLockDAO().delete(lock);
    }

    @Override
    @Transactional
    public final void deleteAllLocksForUser(String username) {
        List<Lock> locks = getLockDAO().findByQuery(DELETE_QUERY + username + "'", Lock.class);
        for (Lock lock : locks) {
            getLockDAO().delete(lock);
        }
    }

    /**
     * Gets the Lock data access object.
     * 
     * @return the lockDAO
     */
    public final LockDAO getLockDAO() {
        return lockDAO;
    }

    /**
     * Sets the lock data access object.
     * 
     * @param lockDAO
     *            the lockDAO to set
     */
    public final void setLockDAO(LockDAO lockDAO) {
        this.lockDAO = lockDAO;
    }

}
