package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.exception.CustomizationReportException;
import gov.va.med.pharmacy.pecs.service.ReportService;
import gov.va.med.pharmacy.pecs.service.reports.IReport;
import gov.va.med.pharmacy.pecs.service.reports.impl.ReportFactory;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * A service that serves up reports from the report factory.
 * 
 * @author vhaisacarusr
 * 
 * 
 */
@Service
public final class ReportServiceImpl implements ReportService, InitializingBean {

    // private static final Log LOGGER = LogFactory.getLog(ReportServiceImpl.class);
    @Resource
    private ReportFactory reportFactory = null;

    @Override
    @Transactional
    public List<List<Object>> findData(String reportName) throws CustomizationReportException {
        IReport report = reportFactory.createReport(reportName);
        return report.findData();
    }

    @Override
    public String findFileName(String reportName) throws CustomizationReportException {
        IReport report = reportFactory.createReport(reportName);
        return report.getFileName();
    }

    /**
     * Returns the assigned ReportFactory instance.
     * 
     * @return The assigned ReportFactory instance
     */
    public ReportFactory getReportFactory() {
        return reportFactory;
    }

    /**
     * Assigns the ReportFactory instance.
     * 
     * @param reportFactory
     *            the report factory
     */
    public void setReportFactory(ReportFactory reportFactory) {
        this.reportFactory = reportFactory;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (reportFactory == null) {
            throw new Exception("The reportFactory has not been injected to ReportServiceImpl.");
        }
    }
}
