package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.persistence.DrugDrugInteractionDAOImpl;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

/**
 * The Class DeletedMonographCustomizationReport.
 */
@Service
public class DeletedMonographCustomizationReport extends TotalCustomizationReportBase implements IReport {

    /** The Constant logger. */
    private static final Logger logger = Logger.getLogger(DeletedMonographCustomizationReport.class);

    /** The Constant MODULE_FILE_NAME. */
    private static final String MODULE_FILE_NAME = "Deleted_Monograph_Report";

    /** The default record batch size. */
    private static int DEFAULT_RECORD_BATCH_SIZE = 500;

    /** The ddi dao. */
    @Resource
    private DrugDrugInteractionDAOImpl ddiDAO;

    /**
     * Instantiates a new deleted monograph customization report.
     */
    public DeletedMonographCustomizationReport() {
        super();
        fileName = IReport.DELETED_MONOGRAPH_CUSTOMIZATION_REPORT;
        recordBatchSize = DEFAULT_RECORD_BATCH_SIZE;
    }

    /**
     * Gets the drug drug interaction dao.
     * 
     * @return the drug drug interaction dao
     */
    public DrugDrugInteractionDAOImpl getDrugDrugInteractionDAO() {
        return ddiDAO;
    }

    /**
     * Sets the drug drug interaction dao.
     * 
     * @param ddiDAO
     *            the new drug drug interaction dao
     */
    public void setDrugDrugInteractionDAO(DrugDrugInteractionDAOImpl ddiDAO) {
        this.ddiDAO = ddiDAO;
    }

    /**
     * Finds the data.
     * 
     * @return the report table data
     */
    public List<List<Object>> findData() {
        List<List<Object>> reportableData = new ArrayList<List<Object>>();
        final Object[] header = { "INTERACTION ID", "DESCRIPTION", "MONOGRAPH ID" };
        reportableData.add(Arrays.asList(header));

        List<DrugDrugInteractionVersionVa> data = ddiDAO.getDeleteMonographData();
        if (data.size() > 0) {
            for (DrugDrugInteractionVersionVa dataRow : data) {
                reportableData.add(getData(dataRow));
            }
        } else {
            reportableData.add(Arrays.asList(noDataFound));
        }
        return reportableData;
    }

    /**
     * Gets the file name.
     * 
     * @return the module file name
     */
    public String getFileName() {
        return MODULE_FILE_NAME;
    }

    /**
     * Gets the data.
     * 
     * @param version
     *            the version
     * @return the data
     */
    private List<Object> getData(DrugDrugInteractionVersionVa version) {
        List<Object> dataRow = new ArrayList<Object>();
        dataRow.add(version.getInteractionId());
        dataRow.add(version.getDescription());
        dataRow.add(version.getMonographId());
        return dataRow;
    }
}
