package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVersionVaDAO;
import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

/**
 * The Class NullDrugDrugPairsCustomizationReport.
 */
@Service
public final class NullDrugDrugPairsCustomizationReport extends TotalCustomizationReportBase implements IReport {

    /** The Constant MODULE_FILE_NAME. */
    private static final String MODULE_FILE_NAME = "Null_Drug_Pairs_Report";

    /** The default record batch size. */
    private static final int DEFAULT_RECORD_BATCH_SIZE = 500;

    /** The Constant . */
    private static final int DRUG_MISSING_COLUMN_POSITION = 3;

    /** The dao. */
    @Resource
    private DrugPairVersionVaDAO dao;

    @Resource
    private DrugPairDAOImpl vaDrugPairDAO;

    /**
     * Instantiates a new null drug drug pairs customization report.
     */
    public NullDrugDrugPairsCustomizationReport() {
        super();
        fileName = IReport.NULL_DRUG_DRUG_PAIRS_CUSTOMIZATION_REPORT;
        recordBatchSize = DEFAULT_RECORD_BATCH_SIZE;
    }

    /**
     * Finds the data.
     * 
     * @return the report table data
     */
    public List<List<Object>> findData() {
        List<List<Object>> reportableData = new ArrayList<List<Object>>();
        final Object[] header = { "INTERACTION ID", "RTGENID1", "RTGENID2", "DRUGMISSING" };
        reportableData.add(Arrays.asList(header));

        List<Object[]> data = dao.getNullDrugDrugPairsCustomizationReportData();
        if (data.size() > 0) {
            for (Object[] dataRow : data) {

                DrugPairVa vaDrugPair =
                        getVaDrugPairDAO().findDrugPair(((BigDecimal) dataRow[1]).longValue(),
                                ((BigDecimal) dataRow[2]).longValue(), ((BigDecimal) dataRow[0]).longValue());
                if (vaDrugPair != null) {
                    if (vaDrugPair.getRoutedGeneric2() == null) {
                        dataRow[DRUG_MISSING_COLUMN_POSITION] = new BigDecimal(2);
                    } else {
                        dataRow[DRUG_MISSING_COLUMN_POSITION] = new BigDecimal(1);
                    }
                }

                reportableData.add(Arrays.asList(dataRow));
            }
        } else {
            reportableData.add(Arrays.asList(noDataFound));
        }
        return reportableData;
    }

    /**
     * Gets the file name.
     * 
     * @return the module file name
     */
    public String getFileName() {
        return MODULE_FILE_NAME;
    }

    /**
     * Gets the drug pair version va dao.
     * 
     * @return the drug pair version va dao
     */
    public DrugPairVersionVaDAO getDrugPairVersionVaDAO() {
        return dao;
    }

    /**
     * Sets the drug pair version va dao.
     * 
     * @param drugPairVersionVaDAO
     *            the new drug pair version va dao
     */
    public void setDrugPairVersionVaDAO(DrugPairVersionVaDAO drugPairVersionVaDAO) {
        this.dao = drugPairVersionVaDAO;
    }

    /**
     * Gets the va drug pair dao.
     * 
     * @return the va drug pair dao
     */
    public DrugPairDAOImpl getVaDrugPairDAO() {
        return vaDrugPairDAO;
    }

    /**
     * Sets the va drug pair dao.
     * 
     * @param vaDrugPairDAO
     *            the new va drug pair dao
     */
    public void setVaDrugPairDAO(DrugPairDAOImpl vaDrugPairDAO) {
        this.vaDrugPairDAO = vaDrugPairDAO;
    }

}
