package gov.va.med.pharmacy.pecs.service.reports.impl;

import javax.annotation.Resource;

import gov.va.med.pharmacy.pecs.exception.CustomizationReportException;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

/**
 * A factory for creating Report objects.
 * 
 * @author rick.bloyer
 */
@Service
public class ReportFactory {

    /** The Constant logger. */
    private static final Logger logger = Logger.getLogger(DtcatTotalCustomizationReport.class);

    /** Total Customization Report variable. */
    @Resource(name = "dtcatTotalCustomizationReport")
    private IReport dtcatTotalCustomizationReport = null;

    /** The dosing total customization report. */
    @Resource(name = "dosingTotalCustomizationReport")
    private IReport dosingTotalCustomizationReport = null;
    // @Resource(name="ddimTotalCustomizationReport")
    // private IReport ddimTotalCustomizationReport = null;
    /** The ddiminteraction total customization report. */
    @Resource(name = "ddiminteractionTotalCustomizationReport")
    private IReport ddiminteractionTotalCustomizationReport = null;

    /** The monograph total customization report. */
    @Resource(name = "monographTotalCustomizationReport")
    private IReport monographTotalCustomizationReport = null;

    /** The deleted monograph customization report. */
    @Resource(name = "deletedMonographCustomizationReport")
    private IReport deletedMonographCustomizationReport = null;

    /** The null drug drug pairs customization report. */
    @Resource(name = "nullDrugDrugPairsCustomizationReport")
    private IReport nullDrugDrugPairsCustomizationReport = null;

    /**
     * Default constructor.
     */
    public ReportFactory() {
    }

    /**
     * Returns an instance of IReport specified by the reportName parameter.
     * 
     * @param reportName
     *            the implementation of the report to return as specified by IReport.
     * @return IReport report implementation.
     * @throws CustomizationReportException
     *             the customization report exception
     */
    public IReport createReport(String reportName) throws CustomizationReportException {
        if (reportName == null || "".equals(reportName))
            throw new IllegalArgumentException("Name indicating the report to be created cannot be null");

        if (logger.is   ugEnabled())
            logger.debug("Received request to return report: " + reportName);

        if (IReport.TOTAL_CUSTOMIZATION_DTCAT.equals(reportName))
            return dtcatTotalCustomizationReport;
        if (IReport.TOTAL_CUSTOMIZATION_DOSING.equals(reportName))
            return dosingTotalCustomizationReport;
        // if (IReport.TOTAL_CUSTOMIZATION_DDIM.equals(reportName))
        // return ddimTotalCustomizationReport;
        if (IReport.TOTAL_CUSTOMIZATION_DDIMINTERACTION.equals(reportName))
            return ddiminteractionTotalCustomizationReport;
        if (IReport.DELETED_MONOGRAPH_CUSTOMIZATION_REPORT.equals(reportName))
            return deletedMonographCustomizationReport;
        if (IReport.NULL_DRUG_DRUG_PAIRS_CUSTOMIZATION_REPORT.equals(reportName))
            return nullDrugDrugPairsCustomizationReport;
        if (IReport.TOTAL_CUSTOMIZATION_MONOGRAPH.equals(reportName))
            return monographTotalCustomizationReport;
        else
            throw new CustomizationReportException("Unable to determine what report to return using report name: " 
                  + reportName);
    }

    /**
     * Gets the dosing total customization report.
     * 
     * @return the dosing total customization report
     */
    public IReport getDosingTotalCustomizationReport() {
        return dosingTotalCustomizationReport;
    }

    /**
     * Sets the dosing total customization report.
     * 
     * @param dosingTotalCustomizationReport
     *            the new dosing total customization report
     */
    public void setDosingTotalCustomizationReport(IReport dosingTotalCustomizationReport) {
        this.dosingTotalCustomizationReport = dosingTotalCustomizationReport;
    }

    /**
     * Gets the dtcat total customization report.
     * 
     * @return the dtcat total customization report
     */
    public IReport getDtcatTotalCustomizationReport() {
        return dtcatTotalCustomizationReport;
    }

    /**
     * Sets the dtcat total customization report.
     * 
     * @param dtcatTotalCustomizationReport
     *            the new dtcat total customization report
     */
    public void setDtcatTotalCustomizationReport(IReport dtcatTotalCustomizationReport) {
        this.dtcatTotalCustomizationReport = dtcatTotalCustomizationReport;
    }

    // public IReport getDdimTotalCustomizationReport() {
    // return ddimTotalCustomizationReport;
    // }
    //
    // public void setDdimTotalCustomizationReport(IReport ddimTotalCustomizationReport) {
    // this.ddimTotalCustomizationReport = ddimTotalCustomizationReport;
    // }

    /**
     * Gets the ddiminteraction total customization report.
     * 
     * @return the ddiminteraction total customization report
     */
    public IReport getDdiminteractionTotalCustomizationReport() {
        return ddiminteractionTotalCustomizationReport;
    }

    /**
     * Sets the ddiminteraction total customization report.
     * 
     * @param ddiminteractionTotalCustomizationReport
     *            the new ddiminteraction total customization report
     */
    public void setDdiminteractionTotalCustomizationReport(IReport ddiminteractionTotalCustomizationReport) {
        this.ddiminteractionTotalCustomizationReport = ddiminteractionTotalCustomizationReport;
    }

    /**
     * Gets the monograph total customization report.
     * 
     * @return the monograph total customization report
     */
    public IReport getMonographTotalCustomizationReport() {
        return monographTotalCustomizationReport;
    }

    /**
     * Sets the monograph total customization report.
     * 
     * @param monographTotalCustomizationReport
     *            the new monograph total customization report
     */
    public void setMonographTotalCustomizationReport(IReport monographTotalCustomizationReport) {
        this.monographTotalCustomizationReport = monographTotalCustomizationReport;
    }

    /**
     * Gets the deleted monograph customization report.
     * 
     * @return the deleted monograph customization report
     */
    public IReport getDeletedMonographCustomizationReport() {
        return deletedMonographCustomizationReport;
    }

    /**
     * Sets the deleted monograph customization report.
     * 
     * @param deletedMonographCustomizationReport
     *            the new deleted monograph customization report
     */
    public void setDeletedMonographCustomizationReport(IReport deletedMonographCustomizationReport) {
        this.deletedMonographCustomizationReport = deletedMonographCustomizationReport;
    }

    /**
     * Gets the null drug drug pairs customization report.
     * 
     * @return the null drug drug pairs customization report
     */
    public IReport getNullDrugDrugPairsCustomizationReport() {
        return nullDrugDrugPairsCustomizationReport;
    }

    /**
     * Sets the null drug drug pairs customization report.
     * 
     * @param nullDrugDrugPairsCustomizationReport
     *            the new null drug drug pairs customization report
     */
    public void setNullDrugDrugPairsCustomizationReport(IReport nullDrugDrugPairsCustomizationReport) {
        this.nullDrugDrugPairsCustomizationReport = nullDrugDrugPairsCustomizationReport;
    }
}
