package gov.va.med.pharmacy.pecs.service.util;

/**
 * Used to hold the counts of different file types when generating a Full Custom Update file to be sent to regional/local sites.
 * This information is used in generating the filecounts.dat file that gets stored in the produced zip file.
 * 
 * getXXXFullUpdateRecord are methods used to generate the individual rows in the filecounts.dat file.
 * 
 * @author Gary Duff
 * 
 */
public class FullUpdateFileCounts {

    /** The dt cat count. */
    private Long dtCatCount;

    /** The dosing count. */
    private Long dosingCount;

    /** The monograph count. */
    private Long monographCount;

    /** The drug pairs count. */
    private Long drugPairsCount;

    /** The drug drug interaction count. */
    private Long drugDrugInteractionCount;

    /** The drug drug interaction string count. */
    private Long drugDrugInteractionStringCount;

    /**
     * Gets the dt cat count.
     * 
     * @return the dt cat count
     */
    public Long getDtCatCount() {
        return dtCatCount;
    }

    /**
     * Gets the dt cat full update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the dt cat full update record
     */
    public String getDtCatFullUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DTCAT", fileName, delimiter, this.dtCatCount, terminator);
    }

    /**
     * Sets the dt cat count.
     * 
     * @param dtCatCount
     *            the new dt cat count
     */
    public void setDtCatCount(Long dtCatCount) {
        this.dtCatCount = dtCatCount;
    }

    /**
     * Sets the dt cat count.
     * 
     * @param dtCatCount
     *            the new dt cat count
     */
    public void setDtCatCount(long dtCatCount) {
        this.dtCatCount = new Long(dtCatCount);
    }

    /**
     * Gets the monograph count.
     * 
     * @return the monograph count
     */
    public Long getMonographCount() {
        return monographCount;
    }

    /**
     * Gets the monograph update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the monograph update record
     */
    public String getMonographUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_MONOGRAPH", fileName, delimiter, this.monographCount, terminator);
    }

    /**
     * Sets the monograph count.
     * 
     * @param monographCount
     *            the new monograph count
     */
    public void setMonographCount(Long monographCount) {
        this.monographCount = monographCount;
    }

    /**
     * Sets the monograph count.
     * 
     * @param monographCount
     *            the new monograph count
     */
    public void setMonographCount(long monographCount) {
        this.monographCount = new Long(monographCount);
    }

    /**
     * Gets the drug pairs count.
     * 
     * @return the drug pairs count
     */
    public Long getDrugPairsCount() {
        return drugPairsCount;
    }

    /**
     * Gets the drug pairs full update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the drug pairs full update record
     */
    public String getDrugPairsFullUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DDIM", fileName, delimiter, this.drugPairsCount, terminator);
    }

    /**
     * Sets the drug pairs count.
     * 
     * @param drugPairsCount
     *            the new drug pairs count
     */
    public void setDrugPairsCount(Long drugPairsCount) {
        this.drugPairsCount = drugPairsCount;
    }

    /**
     * Sets the drug pairs count.
     * 
     * @param drugPairsCount
     *            the new drug pairs count
     */
    public void setDrugPairsCount(long drugPairsCount) {
        this.drugPairsCount = new Long(drugPairsCount);
    }

    /**
     * Gets the drug drug interaction count.
     * 
     * @return the drug drug interaction count
     */
    public Long getDrugDrugInteractionCount() {
        return drugDrugInteractionCount;
    }

    /**
     * Gets the drug drug interaction full update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the drug drug interaction full update record
     */
    public String getDrugDrugInteractionFullUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DDIMINTERACTION", fileName, delimiter, this.drugDrugInteractionCount, terminator);
    }

    /**
     * Sets the drug drug interaction count.
     * 
     * @param drugDrugInteractionCount
     *            the new drug drug interaction count
     */
    public void setDrugDrugInteractionCount(Long drugDrugInteractionCount) {
        this.drugDrugInteractionCount = drugDrugInteractionCount;
    }

    /**
     * Sets the drug drug interaction count.
     * 
     * @param drugDrugInteractionCount
     *            the new drug drug interaction count
     */
    public void setDrugDrugInteractionCount(long drugDrugInteractionCount) {
        this.drugDrugInteractionCount = new Long(drugDrugInteractionCount);
    }

    /**
     * Gets the drug drug interaction string count.
     * 
     * @return the drug drug interaction string count
     */
    public Long getDrugDrugInteractionStringCount() {
        return drugDrugInteractionStringCount;
    }

    /**
     * Gets the drug drug interaction string full update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the drug drug interaction string full update record
     */
    public String getDrugDrugInteractionStringFullUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DDIMSTRINGS", fileName, delimiter, this.drugDrugInteractionStringCount,
                terminator);
    }

    /**
     * Sets the drug drug interaction string count.
     * 
     * @param drugDrugInteractionStringCount
     *            the new drug drug interaction string count
     */
    public void setDrugDrugInteractionStringCount(Long drugDrugInteractionStringCount) {
        this.drugDrugInteractionStringCount = drugDrugInteractionStringCount;
    }

    /**
     * Sets the drug drug interaction string count.
     * 
     * @param drugDrugInteractionStringCount
     *            the new drug drug interaction string count
     */
    public void setDrugDrugInteractionStringCount(long drugDrugInteractionStringCount) {
        this.drugDrugInteractionStringCount = new Long(drugDrugInteractionStringCount);
    }

    /**
     * Gets the dosing count.
     * 
     * @return the dosing count
     */
    public Long getDosingCount() {
        return this.dosingCount;
    }

    /**
     * Gets the dosing full update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the dosing full update record
     */
    public String getDosingFullUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("FDB_CUSTOM_DOSING", fileName, delimiter, this.dosingCount, terminator);
    }

    /**
     * Sets the dosing count.
     * 
     * @param dosingCount
     *            the new dosing count
     */
    public void setDosingCount(Long dosingCount) {
        this.dosingCount = dosingCount;
    }

    /**
     * Sets the dosing count.
     * 
     * @param dosingCount
     *            the new dosing count
     */
    public void setDosingCount(long dosingCount) {
        this.dosingCount = new Long(dosingCount);
    }

    /**
     * Gets the ct version update record.
     * 
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param terminator
     *            the terminator
     * @return the ct version update record
     */
    public String getCtVersionUpdateRecord(String fileName, String delimiter, String terminator) {
        return formatUpdateRecord("CT_VERSION", fileName, delimiter, 1L, terminator);
    }

    /**
     * Format update record.
     * 
     * @param tableName
     *            the table name
     * @param fileName
     *            the file name
     * @param delimiter
     *            the delimiter
     * @param count
     *            the count
     * @param terminator
     *            the terminator
     * @return the string
     */
    private String formatUpdateRecord(String tableName, String fileName, String delimiter, Long count, String terminator) {
        StringBuffer sb = new StringBuffer();
        sb.append(fileName);
        sb.append(delimiter);
        sb.append(tableName);
        sb.append(delimiter);
        sb.append(count.longValue());
        sb.append(terminator);
        return sb.toString();
    }
}
