/*
 * 
 */
package gov.va.med.pharmacy.pecs.service.util;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.MessageSource;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;

/**
 * The Class MessageHelper.
 */
public final class MessageHelper implements InitializingBean {

    /** The messages. */
    private MessageSource serviceMessages;

    /**
     * Instantiates a new message helper.
     */
    private MessageHelper() {

    }

    /**
     * Gets the formatted message.
     * 
     * @param key
     *            the key
     * @param args
     *            the args
     * @return the formatted message
     */
    public String getFormattedMessage(String key, Object[] args) {
        String defaultMessage = "The message key " + key + " was not found.";
        return getServiceMessages().getMessage(key, args, defaultMessage, null);
    }

    /**
     * Gets the formatted message.
     * 
     * @param key
     *            the key
     * @return the formatted message
     */
    public String getMessage(String key) {
        return getFormattedMessage(key, null);
    }

    /**
     * Adds the message to binding result.
     * 
     * @param className
     *            the class name
     * @param message
     *            the message
     * @param args
     *            the args
     * @param results
     *            the results
     */
    public void addMessageToBindingResult(String className, String message, Object[] args, BindingResult results) {

        String formattedMessage = getFormattedMessage(message, args);

        results.addError(new ObjectError(className, formattedMessage));
    }

    /**
     * Adds the message to binding result.
     * 
     * @param className
     *            the class name
     * @param message
     *            the message
     * @param results
     *            the results
     */
    public void addMessageToBindingResult(String className, String message, BindingResult results) {

        addMessageToBindingResult(className, message, null, results);

    }

    /**
     * Gets the service messages.
     * 
     * @return the service messages
     */
    public MessageSource getServiceMessages() {
        return serviceMessages;
    }

    /**
     * Sets the service messages.
     * 
     * @param serviceMessages
     *            the new service messages
     */

    public void setServiceMessages(MessageSource serviceMessages) {
        this.serviceMessages = serviceMessages;

    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (serviceMessages == null) {
            throw new Exception("The property MessageSource in AbstractController has not been set.");
        }

    }

}
