package gov.va.med.pharmacy.pecs.web;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.util.FileCopyUtils;

/**
 * The Class AbstractDownloadableContentController.
 */
public abstract class AbstractDownloadableContentController extends AbstractController {

    /** The logger. */
    private static Log logger = LogFactory.getLog(AbstractDownloadableContentController.class);

    /** The excel content type. */
    private final String EXCEL_CONTENT_TYPE = "application/vnc.ms-excel";

    /** The update content type. */
    private final String UPDATE_CONTENT_TYPE = "application/zip";

    /** The content disposition key. */
    private final String CONTENT_DISPOSITION_KEY = "Content-Disposition";

    /**
     * Gets the temp dir path.
     * 
     * @param request
     *            the request
     * @return the temp dir path
     */
    protected final String getTempDirPath(HttpServletRequest request) {
        ServletContext context = request.getSession().getServletContext();
        File tmpdir = (File) context.getAttribute("javax.servlet.context.tempdir");
        return tmpdir.getAbsolutePath();
    }

    /**
     * Send excel file.
     * 
     * @param response
     *            the response
     * @param file
     *            the file
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    protected final void sendExcelFile(HttpServletResponse response, File file) throws IOException {
        try {
            buildResponse(response, file, EXCEL_CONTENT_TYPE);
        } catch (IOException ioe) {
            throw ioe;
        }
    }

    /**
     * Send update file.
     * 
     * @param response
     *            the response
     * @param file
     *            the file
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    protected final void sendUpdateFile(HttpServletResponse response, File file) throws IOException {
        try {
            buildResponse(response, file, UPDATE_CONTENT_TYPE);
        } catch (IOException ioe) {
            throw ioe;
        }
    }

    /**
     * Builds the response.
     * 
     * @param response
     *            the response
     * @param file
     *            the file
     * @param contentType
     *            the content type
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    private void buildResponse(HttpServletResponse response, File file, String contentType) throws IOException {
        try {
            response.setContentType(contentType);
            response.setContentLength(Long.valueOf(file.length()).intValue());
            response.setHeader(CONTENT_DISPOSITION_KEY, "attachment;filename=\"" + file.getName() + "\"");
            FileCopyUtils.copy(new FileInputStream(file), response.getOutputStream());
        } catch (IOException ioe) {
            logger.error(ioe.getMessage(), ioe);
            throw ioe;
        }
    }
}
