package gov.va.med.pharmacy.pecs.web.auth;

import gov.va.med.pharmacy.pecs.customupdate.CtVersionVa;
import gov.va.med.pharmacy.pecs.customupdate.FdbVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.CtUsersVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.service.CustomUpdateService;
import gov.va.med.pharmacy.pecs.service.UserService;
import gov.va.med.pharmacy.pecs.web.IConstants;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;

/**
 * The Class IndexController.
 */
@Controller
public class IndexController implements InitializingBean {

    /** The logger. */
    private static Log logger = LogFactory.getLog(IndexController.class);

    /** The user service. */
    @Resource
    private UserService userService;

    /** The update file service. */
    @Resource
    private CustomUpdateService updateFileService;

    /** The select unassigned. */
    private final String SELECT_UNASSIGNED = "UNASSIGNED";

    /** The tiles view. */
    private final String TILES_VIEW = "pecs.worklist";

    /**
     * Populate work list model.
     * 
     * @return the string
     */
    @ModelAttribute(value = "workListModel")
    public String populateWorkListModel() {
        return new String();
    }

    /**
     * Populate fdb update info.
     * 
     * @return the string
     */
    @ModelAttribute(value = "fdbUpdateInfo")
    public String populateFdbUpdateInfo() {
        FdbVersion fdbVersion = updateFileService.getFdbVersion();
        String formattedDate = "Unknown";
        String dbVersion = "Unknown";
        if (fdbVersion != null) {
            formattedDate = formatIssueDate(fdbVersion.getIssuedate());
            dbVersion = fdbVersion.getDbversion();
        }

        return formattedDate + " version: " + dbVersion;
    }

    /**
     * Format issue date.
     * 
     * @param issueDate
     *            the issue date
     * @return the string
     */
    private String formatIssueDate(final String issueDate) {
        SimpleDateFormat formatter = new SimpleDateFormat(IConstants.VA_DATE_FORMAT);
        int year = Integer.valueOf(issueDate.substring(0, 4));
        int month = Integer.valueOf(issueDate.substring(4, 6)) - 1;
        int date = Integer.valueOf(issueDate.substring(6, 8));
        Calendar calendar = new GregorianCalendar();
        calendar.set(year, month, date);
        return formatter.format(calendar.getTime());
    }

    /**
     * Populate pecs update info.
     * 
     * @return the string
     */
    @ModelAttribute(value = "pecsUpdateInfo")
    public String populatePecsUpdateInfo() {
        String pecsDate = "Unknown";
        List<CtVersionVa> ctVersions = updateFileService.findAllCtVersionVas();
        if (ctVersions != null && ctVersions.size() > 0) {
            pecsDate = ctVersions.get(0).getCtIssuedateDisplay();
        }

        return pecsDate;
    }

    /**
     * Index.
     * 
     * @param request
     *            the request
     * @return the string
     */
    @RequestMapping(value = "/index")
    public String index(HttpServletRequest request) {
        authenticate(request);
        return TILES_VIEW;
    }

    /**
     * Authenticate.
     * 
     * @param request
     *            the request
     */
    private void authenticate(HttpServletRequest request) {
        HttpSession session = request.getSession(false);
        UserProfile profile = null;
        List<String> userList = null;
        try {
            profile = userService.authenticate(request);
            if (profile == null) {
            }

            if (logger.is   ugEnabled())
                logger.debug("KAAJEE user: " + profile.getUserName01() + " is in roles: " + profile.getRoles());

            userList = getAdminUsers();
            if (logger.is   ugEnabled())
                logger.debug("Obtained APPROVER list of: " + userList.toString());
        } catch (CustomizationServiceException cse) {
        }

        session.setAttribute(IConstants.SESSION_APPROVERS, userList);
        session.removeAttribute(IConstants.SESSION_USER);
        session.setAttribute(IConstants.SESSION_USER, profile);

    }

    /**
     * Gets the admin users.
     * 
     * @return the admin users
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    private List<String> getAdminUsers() throws CustomizationServiceException {
        List<CtUsersVa> userList = userService.findApprovers();
        ArrayList<String> orderedList = new ArrayList<String>();
        for (CtUsersVa model : userList)
            orderedList.add(model.getUsername());

        orderedList.add(0, "");
        orderedList.add(SELECT_UNASSIGNED);
        return orderedList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (userService == null) {
            throw new Exception("The property userService in IndexController has not been set.");
        }
        if (updateFileService == null) {
            throw new Exception("The property updateFileService in IndexController has not been set.");
        }
    }
}
