package gov.va.med.pharmacy.pecs.web.auth;

import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.service.RecordLockingService;
import gov.va.med.pharmacy.pecs.web.IConstants;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

/**
 * The Class LogoutController.
 */
@Controller
public final class LogoutController {

    /** The record locking service. */
    @Resource
    private RecordLockingService recordLockingService;

    /**
     * Logout.
     * 
     * @param request
     *            the request
     * @return the model and view
     */
    @RequestMapping(value = "/logout")
    public ModelAndView logout(HttpServletRequest request) {

        HttpSession session = request.getSession(false);
        UserProfile profile = null;

        if (session != null) {
            profile = (UserProfile) session.getAttribute(IConstants.SESSION_USER);
            session.removeAttribute(IConstants.SESSION_CONFIDENTIALITY);
            session.removeAttribute(IConstants.SESSION_USER);
            session.invalidate();
        }

        String logoutMessage;
        if (profile != null) {
            logoutMessage = profile.getUsername() + " has been logged out of the Custom Tables application.";
            recordLockingService.deleteAllLocksForUser(profile.getUsername());
        } else {
            logoutMessage = "You have been logged out of the Custom Tables application.";
        }

        return new ModelAndView("logout", "logoutMessage", logoutMessage);
    }

    /**
     * Gets the record locking service.
     * 
     * @return the recordLockingService
     */
    public RecordLockingService getRecordLockingService() {
        return recordLockingService;
    }

    /**
     * Sets the record locking service.
     * 
     * @param recordLockingService
     *            the recordLockingService to set
     */
    public void setRecordLockingService(RecordLockingService recordLockingService) {
        this.recordLockingService = recordLockingService;
    }
}
