/**
 * 
 */
package gov.va.med.pharmacy.pecs.web.contact;

import gov.va.med.pharmacy.pecs.persistence.contact.ContactUsDTO;
import gov.va.med.pharmacy.pecs.service.contact.ContactUsService;
import gov.va.med.pharmacy.pecs.util.BlobHandler;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.detail.AjaxMessageResponse;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

/**
 * Controller for the Contact Us Page.
 * 
 * @author VHAISACARUSR
 * 
 */

@Controller
public final class ContactUsController extends AbstractController {
    private static final String CONTACT_VIEW = "pecs.contact";
    private static final String EDIT_VIEW = "pecs.contact.edit";

    private static final Logger LOGGER = Logger.getLogger(ContactUsController.class);
    private static final int READ_AHEAD_LIMIT = 1000;

    @Resource
    private ContactUsService contactUsService;
    @Resource
    private BlobHandler blobHandler;

    /**
     * displays the page for contact us.
     * 
     * @return a model and view
     */
    @RequestMapping(value = "contact", method = RequestMethod.GET)
    public ModelAndView showContactPage() {
        ModelAndView mav = new ModelAndView(CONTACT_VIEW);
        ContactUsDTO content = contactUsService.findContactContent();
        mav.addObject(content);
        byte[] data = content.getPostData();
        String html = "";
        if (data != null) {
            html = new String(content.getPostData());
        }

        mav.addObject("html", html);
        LOGGER.debug(html);
        return mav;
    }

    /**
     * displays the page to edit contact us information.
     * 
     * @return a model and view
     */
    @RequestMapping(value = "contact/edit", method = RequestMethod.GET)
    public ModelAndView showContactEditPage() {
        ModelAndView mav = new ModelAndView(EDIT_VIEW);
        ContactUsDTO content = contactUsService.findContactContent();
        LOGGER.debug("debugging");
        mav.addObject(content);
        String html = "";
        if (content.getPostData() != null) {
            html = new String(content.getPostData());
        }
        mav.addObject("html", html);
        LOGGER.debug(html);
        return mav;
    }

    /**
     * Saves the page for contact us.
     * 
     * @param request
     *            the request with the post body
     * @return nothing
     * 
     */
    @RequestMapping(value = "contact/edit", method = RequestMethod.POST)
    @ResponseBody
    public AjaxMessageResponse saveContactPage(HttpServletRequest request) {
        AjaxMessageResponse response = new AjaxMessageResponse();
        ContactUsDTO contactUs = new ContactUsDTO();
        try {
            String postString = getPostData(request);
            LOGGER.debug(postString);
            contactUs.setPostData(postString.getBytes());
            contactUs.setCreatedBy(getUserProfile(request).getUsername());
            contactUs.setCreatedDate(new Date());
            contactUsService.saveContactContent(contactUs);
            response.setSuccessMessage("success");
        } catch (Exception e) {
            List<String> errors = new ArrayList<String>();
            errors.add("An error occured in saving.");
            response.setErrorMessages(errors);
        }
        return response;
    }

    /**
     * @param request
     *            the request
     * @return a string
     */
    private String getPostData(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder();
        try {
            BufferedReader reader = request.getReader();
            reader.mark(READ_AHEAD_LIMIT);

            String line = reader.readLine();
            while (line != null) {

                sb.append(line);
                line = reader.readLine();

            }

            reader.reset();
            // do NOT close the reader here, or you won't be able to get the post data twice
        } catch (IOException e) {
            LOGGER.warn("getPostData couldn't.. get the post data", e); // This has happened if the request's reader is closed
        }
        LOGGER.debug(sb.toString());

        return sb.toString();
    }

    /**
     * Gets the contactUsService.
     * 
     * @return the contactUsService
     */
    public ContactUsService getContactUsService() {
        return contactUsService;
    }

    /**
     * Sets the contactUsService.
     * 
     * @param contactUsService
     *            the contactUsService to set
     */
    public void setContactUsService(ContactUsService contactUsService) {
        this.contactUsService = contactUsService;
    }

    /**
     * Gets the blobHandler.
     * 
     * @return the blobHandler
     */
    public BlobHandler getBlobHandler() {
        return blobHandler;
    }

    /**
     * Sets the blobHandler.
     * 
     * @param blobHandler
     *            the blobHandler to set
     */
    public void setBlobHandler(BlobHandler blobHandler) {
        this.blobHandler = blobHandler;
    }

}
