package gov.va.med.pharmacy.pecs.web.customization;

import gov.va.med.pharmacy.pecs.web.IConstants;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * The Class CustomizationQueryModel.
 */
public class CustomizationQueryModel {

    /** The concept selection. */
    private String conceptSelection;
    
    /** The record selection. */
    private String recordSelection;
    
    /** The column selection. */
    private String columnSelection;
    
    /** The constraint selection. */
    private String constraintSelection;
    
    /** The column value. */
    private String columnValue;
    
    /** The va records. */
    private List<List<String>> vaRecords;
    
    /** The fdb records. */
    private List<List<String>> fdbRecords;
    
    /** The saved queries. */
    private List<ViewQuery> savedQueries = new ArrayList<ViewQuery>();
    
    /** The last query. */
    private ViewQuery lastQuery;
    
    /** The table name in db. */
    private TableName tableNameInDB;
    
    /** The fdb table name in db. */
    private TableName fdbTableNameInDB;
    
    /** The query history. */
    private boolean queryHistory;
    
    /** The saved query history. */
    private boolean savedQueryHistory;

    /** The session. */
    @SuppressWarnings("rawtypes")
    protected Map session = null;
    
    /** The d b source. */
    protected DBSource dBSource = DBSource.Both;
    
    /** The d b source display. */
    protected String dBSourceDisplay = IConstants.DB_SOURCE_BOTH;
    
    /** The table source display. */
    protected String tableSourceDisplay = IConstants.DB_SOURCE_CUSTOM;
    
    /** The staging table name in db. */
    protected TableName stagingTableNameInDB;
    
    /** The table name. */
    protected final String tableName = "";
    
    /** The table url. */
    protected final String tableURL = "";

    /** The last query both. */
    protected ViewQuery lastQueryBoth;
    
    /** The selected query id. */
    protected long selectedQueryID;
    
    /** The custom query name. */
    protected String customQueryName;
    
    /** The whos query. */
    protected String whosQuery;
    
    /** The query type. */
    protected String queryType;
    
    /** The query as xml. */
    protected String queryAsXML;
    
    /** The query as xml both. */
    protected String queryAsXMLBoth;
    
    /** The query as hql. */
    protected String queryAsHQL;
    
    /** The is new query. */
    protected String isNewQuery;

    /** The wizard step. */
    protected int wizardStep;
    
    /** The wizard step both. */
    protected int wizardStepBoth;
    
    /** The record id. */
    protected String recordId = null;
    
    /** The page. */
    protected String page = "1";
    
    /** The page both. */
    protected String pageBoth = "1";
    
    /** The query prev page. */
    protected String queryPrevPage = "1";
    
    /** The query next page. */
    protected String queryNextPage = "2";
    
    /** The query cur page. */
    protected String queryCurPage = "1";
    
    /** The query prev page both. */
    protected String queryPrevPageBoth = "1";
    
    /** The query next page both. */
    protected String queryNextPageBoth = "2";
    
    /** The query cur page both. */
    protected String queryCurPageBoth = "1";

    /** The Constant DEFAULT_QUERY_NAME. */
    protected final static String DEFAULT_QUERY_NAME = "Unnamed Query";
    
    /** The Constant DEFAULT_QUERY_DISPLAY_COUNT. */
    protected final static int DEFAULT_QUERY_DISPLAY_COUNT = 10;
    
    /** The whos query my. */
    public final String WHOS_QUERY_MY = "MyQuery";
    
    /** The whos query other. */
    public final String WHOS_QUERY_OTHER = "OthersQueries";
    
    /** The query type dynamic. */
    public final String QUERY_TYPE_DYNAMIC = "Dynamic";
    
    /** The query type saved. */
    public final String QUERY_TYPE_SAVED = "Saved";

    /**
     * Gets the concept selection.
     *
     * @return the concept selection
     */
    public String getConceptSelection() {
        return conceptSelection;
    }

    /**
     * Sets the concept selection.
     *
     * @param conceptSelection the new concept selection
     */
    public void setConceptSelection(String conceptSelection) {
        this.conceptSelection = conceptSelection;
    }

    /**
     * Gets the record selection.
     *
     * @return the record selection
     */
    public String getRecordSelection() {
        return recordSelection;
    }

    /**
     * Sets the record selection.
     *
     * @param recordSelection the new record selection
     */
    public void setRecordSelection(String recordSelection) {
        this.recordSelection = recordSelection;
    }

    /**
     * Gets the column selection.
     *
     * @return the column selection
     */
    public String getColumnSelection() {
        return columnSelection;
    }

    /**
     * Sets the column selection.
     *
     * @param columnSelection the new column selection
     */
    public void setColumnSelection(String columnSelection) {
        this.columnSelection = columnSelection;
    }

    /**
     * Gets the constraint selection.
     *
     * @return the constraint selection
     */
    public String getConstraintSelection() {
        return constraintSelection;
    }

    /**
     * Sets the constraint selection.
     *
     * @param constraintSelection the new constraint selection
     */
    public void setConstraintSelection(String constraintSelection) {
        this.constraintSelection = constraintSelection;
    }

    /**
     * Gets the column value.
     *
     * @return the column value
     */
    public String getColumnValue() {
        return columnValue;
    }

    /**
     * Sets the column value.
     *
     * @param columnValue the new column value
     */
    public void setColumnValue(String columnValue) {
        this.columnValue = columnValue;
    }

    /**
     * Gets the va records.
     *
     * @return the va records
     */
    public List<List<String>> getVaRecords() {
        return vaRecords;
    }

    /**
     * Sets the va records.
     *
     * @param vaRecords the new va records
     */
    public void setVaRecords(List<List<String>> vaRecords) {
        this.vaRecords = vaRecords;
    }

    /**
     * Gets the fdb records.
     *
     * @return the fdb records
     */
    public List<List<String>> getFdbRecords() {
        return fdbRecords;
    }

    /**
     * Sets the fdb records.
     *
     * @param fdbRecords the new fdb records
     */
    public void setFdbRecords(List<List<String>> fdbRecords) {
        this.fdbRecords = fdbRecords;
    }

    /**
     * Gets the saved queries.
     *
     * @return the saved queries
     */
    public List<ViewQuery> getSavedQueries() {
        return savedQueries;
    }

    /**
     * Sets the saved queries.
     *
     * @param savedQueries the new saved queries
     */
    public void setSavedQueries(List<ViewQuery> savedQueries) {
        this.savedQueries = savedQueries;
    }

    /**
     * Gets the last query.
     *
     * @return the last query
     */
    public ViewQuery getLastQuery() {
        return lastQuery;
    }

    /**
     * Sets the last query.
     *
     * @param lastQuery the new last query
     */
    public void setLastQuery(ViewQuery lastQuery) {
        this.lastQuery = lastQuery;
    }

    /**
     * Gets the table name in db.
     *
     * @return the table name in db
     */
    public TableName getTableNameInDB() {
        return tableNameInDB;
    }

    /**
     * Sets the table name in db.
     *
     * @param tableNameInDB the new table name in db
     */
    public void setTableNameInDB(TableName tableNameInDB) {
        this.tableNameInDB = tableNameInDB;
    }

    /**
     * Gets the fdb table name in db.
     *
     * @return the fdb table name in db
     */
    public TableName getFdbTableNameInDB() {
        return fdbTableNameInDB;
    }

    /**
     * Sets the fdb table name in db.
     *
     * @param fdbTableNameInDB the new fdb table name in db
     */
    public void setFdbTableNameInDB(TableName fdbTableNameInDB) {
        this.fdbTableNameInDB = fdbTableNameInDB;
    }

    /**
     * Gets the whos query.
     *
     * @return the whos query
     */
    public String getWhosQuery() {
        return whosQuery;
    }

    /**
     * Sets the whos query.
     *
     * @param whosQuery the new whos query
     */
    public void setWhosQuery(String whosQuery) {
        this.whosQuery = whosQuery;
    }

    /**
     * Gets the query type.
     *
     * @return the query type
     */
    public String getQueryType() {
        return queryType;
    }

    /**
     * Sets the query type.
     *
     * @param queryType the new query type
     */
    public void setQueryType(String queryType) {
        this.queryType = queryType;
    }

    /**
     * Gets the query as xml.
     *
     * @return the query as xml
     */
    public String getQueryAsXML() {
        return queryAsXML;
    }

    /**
     * Sets the query as xml.
     *
     * @param queryAsXML the new query as xml
     */
    public void setQueryAsXML(String queryAsXML) {
        this.queryAsXML = queryAsXML;
    }

    /**
     * Gets the query history.
     *
     * @return the query history
     */
    public boolean getQueryHistory() {
        return queryHistory;
    }

    /**
     * Sets the query history.
     *
     * @param queryHistory the new query history
     */
    public void setQueryHistory(boolean queryHistory) {
        this.queryHistory = queryHistory;
    }

    /**
     * Gets the saved query history.
     *
     * @return the saved query history
     */
    public boolean getSavedQueryHistory() {
        return savedQueryHistory;
    }

    /**
     * Sets the saved query history.
     *
     * @param savedQueryHistory the new saved query history
     */
    public void setSavedQueryHistory(boolean savedQueryHistory) {
        this.savedQueryHistory = savedQueryHistory;
    }

    /**
     * Gets the query as hql.
     *
     * @return the query as hql
     */
    public String getQueryAsHQL() {
        return queryAsHQL;
    }

    /**
     * Sets the query as hql.
     *
     * @param queryAsHQL the new query as hql
     */
    public void setQueryAsHQL(String queryAsHQL) {
        this.queryAsHQL = queryAsHQL;
    }

}
