package gov.va.med.pharmacy.pecs.web.detail;

import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.RecordLockingService;
import gov.va.med.pharmacy.pecs.web.AbstractController;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * This class deals with ajax calls that are used for all of the detail pages.
 * 
 * @author vhaisacarusr
 */
@Controller
@RequestMapping(value = "details")
public final class MetadataController extends AbstractController implements InitializingBean {

    @Resource
    private MetadataService metadataService;
    @Resource
    private RecordLockingService recordLockingService;
    @Resource
    private DrugDrugInteractionService drugDrugInteractionService;
    @Resource
    private RecordLockingHelper recordLockingHelper;

    /**
     * Returns a response body in the form of a Json string.
     * 
     * @param conceptName
     *            the name of the concept we want to retrieve the order for
     * @return the field names in the order they are to be displayed
     */
    @RequestMapping(value = "order/{conceptName}")
    @ResponseBody
    public List<String> retrieveDetailOrder(@PathVariable(value = "conceptName") String conceptName) {
        ConceptMetadata conceptMetadata = getMetadataService().getMetadata(Concept.fromString(conceptName).getConceptClass());
        List<ConceptField> fields = conceptMetadata.getFields();
        List<String> list = new ArrayList<String>();
        for (ConceptField conceptField : fields) {
            if (conceptField.includeInDetailDisplay()) {
                list.add(conceptField.getName());
            }
        }

        return list;

    }

   

    /**
     * Gets the recordLockingService.
     * 
     * @return the recordLockingService
     */
    public final RecordLockingService getRecordLockingService() {
        return recordLockingService;
    }

    /**
     * Sets the recordLockingService.
     * 
     * @param recordLockingService
     *            the recordLockingService to set
     */
    public final void setRecordLockingService(RecordLockingService recordLockingService) {
        this.recordLockingService = recordLockingService;
    }

    /**
     * Gets the drugDrugInteractionService.
     * 
     * @return the drugDrugInteractionService
     */
    public final DrugDrugInteractionService getDrugDrugInteractionService() {
        return drugDrugInteractionService;
    }

    /**
     * Sets the drugDrugInteractionService.
     * 
     * @param drugDrugInteractionService
     *            the drugDrugInteractionService to set
     */
    public final void setDrugDrugInteractionService(DrugDrugInteractionService drugDrugInteractionService) {
        this.drugDrugInteractionService = drugDrugInteractionService;
    }

    /**
     * Gets the recordLockingHelper.
     * 
     * @return the recordLockingHelper
     */
    public final RecordLockingHelper getRecordLockingHelper() {
        return recordLockingHelper;
    }

    /**
     * Sets the recordLockingHelper.
     * 
     * @param recordLockingHelper
     *            the recordLockingHelper to set
     */
    public final void setRecordLockingHelper(RecordLockingHelper recordLockingHelper) {
        this.recordLockingHelper = recordLockingHelper;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (getMetadataService() == null) {
            throw new Exception("MetadataService not set");
        }
    }

}
