package gov.va.med.pharmacy.pecs.web.detail.drugdrug;

import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * The Class DDIMessageHelper.
 */
public class DDIMessageHelper {

    /**
     * Instantiates a new dDI message helper.
     */
    protected DDIMessageHelper() {

    }

    /**
     * Gets the messages.
     * 
     * @param updatedDDI
     *            the updated ddi
     * @param originalDDI
     *            the original ddi
     * @param action
     *            the action
     * @param drugPairsStatusCounts
     *            the drug pairs status counts
     * @return the messages
     */
    public static List<String> getMessages(final DrugDrugInteractionDTO updatedDDI, final DrugDrugInteractionDTO originalDDI,
            final String action, final Map<String, Long> drugPairsStatusCounts) {
        List<String> messages = new ArrayList<String>();

        /******************* New ************************/
        if (Status.STATUS_NEW.equals(updatedDDI.getStatus()) || "".equals(updatedDDI.getStatus())) {
            if (!"".equals(updatedDDI.getSeveritylevelcode())) {
                if (Integer.parseInt(updatedDDI.getSeveritylevelcode()) 
                > Integer.parseInt(originalDDI.getSeveritylevelcode())) {
                    messages.add("ddi.message.new.severity");
                }
            }
            if (!"".equals(updatedDDI.getStatus()) && !"Modify".equals(action) && !updatedDDI.isDuplicate()) {
                messages.add("ddi.message.new");
            }

        }
        /******************* Reviewed ************************/
        if (Status.STATUS_REVIEWED.equals(updatedDDI.getStatus())) {
            if (hasStatus(Status.STATUS_REVIEWED, drugPairsStatusCounts)) {
                messages.add("ddi.message.reviewed.reviewedpairs");
            }
            if (Status.STATUS_NEW.equals(originalDDI.getStatus()) || Status.STATUS_DELETED.equals(originalDDI.getStatus())
                    || Status.STATUS_REJECTED.equals(originalDDI.getStatus())
                    || Status.STATUS_MODIFIED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.reviewed");
            }
        }
        /******************* Approved ************************/
        if (Status.STATUS_APPROVED.equals(updatedDDI.getStatus())) {
            if (Status.STATUS_DELETE_REVIEWED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.approved.fromreviewed");
            } else if (Status.STATUS_MODIFIED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.approved.frommodified");
            } else if (Status.STATUS_APPROVED.equals(originalDDI.getStatus()) && "Modify".equals(action)) {
                // No message
            } else if (Status.STATUS_APPROVED.equals(originalDDI.getStatus()) && "Submit_for_Delete".equals(action)) {
                // No message
            } else if (Status.STATUS_REVIEWED.equals(originalDDI.getStatus()) && "Reject".equals(action)) {
                messages.add("message.us.information.approver.rejectapproved");
            } else {
                messages.add("ddi.message.approved");
            }
        }
        /******************* Delete Reviewed ************************/
        if (Status.STATUS_DELETE_REVIEWED.equals(updatedDDI.getStatus())) {
            messages.add("ddi.message.delete_reviewed");
        }
        /******************* Deleted ************************/
        if (Status.STATUS_DELETED.equals(updatedDDI.getStatus())) {
            if ("Reject".equals(action)) {
                if (Status.STATUS_REVIEWED.equals(originalDDI.getStatus())) {
                    messages.add("message.us.information.approver.reject3");
                } else if (!Status.STATUS_DELETED.equals(originalDDI.getStatus())) {
                    messages.add("message.us.information.approver.reject5");
                }
            }
            if ("Delete".equals(action)) {
                messages.add("ddi.message.deleted");
            }
        }
        // messages.add(MESSAGE_PREFIX + Status.STATUS_DELETED.toLowerCase()); */
        /******************* Modified ************************/
        if (Status.STATUS_MODIFIED.equals(updatedDDI.getStatus())) {
            if (Status.STATUS_DELETED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.modified.fromdeleted");
                messages.add("ddi.message.new.nopairs");
            }
            if (Status.STATUS_NEW.equals(originalDDI.getStatus()) || Status.STATUS_REVIEWED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.new");
            }
            if (Status.STATUS_APPROVED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.modified.fromapproved");
            }
            if (Status.STATUS_REJECTED.equals(originalDDI.getStatus())) {
                messages.add("ddi.message.modified");
            }
        }
        if (Status.STATUS_REJECTED.equals(updatedDDI.getStatus()) && !Status.STATUS_REJECTED.equals(originalDDI.getStatus())) {
            messages.add("ddi.message.rejected");
        }

        return messages;
    }

    /**
     * Gets the drug pair messages.
     * 
     * @param dto
     *            the fdb
     * @param drugPairsStatusCounts
     *            the drug pairs status counts
     * @return the drug pair messages
     */
    public static List<String> getDrugPairMessages(DrugDrugInteractionDTO dto, final Map<String, Long> drugPairsStatusCounts) {
        List<String> messages = new ArrayList<String>();
        if (dto.isCurrent()) {
            if (hasOnlyDeletedAndRejected(drugPairsStatusCounts) || drugPairsStatusCounts.isEmpty()) {
                if ((Status.STATUS_NEW.equals(dto.getStatus())) || (Status.STATUS_REVIEWED.equals(dto.getStatus()))
                        || (Status.STATUS_MODIFIED.equals(dto.getStatus()))) {
                    messages.add("ddi.message.new.nopairs");
                }
                // If there are not drug pairs there is no reason to continue.
                if (drugPairsStatusCounts.isEmpty()){                
                    return messages;
                }
            }

            boolean hasReviewedStatus = hasStatus(Status.STATUS_REVIEWED, drugPairsStatusCounts);
            boolean hasNewStatus = hasStatus(Status.STATUS_NEW, drugPairsStatusCounts);
            boolean hasModifiedStatus = hasStatus(Status.STATUS_MODIFIED, drugPairsStatusCounts);
            boolean hasApprovedStatus = hasStatus(Status.STATUS_APPROVED, drugPairsStatusCounts);
            boolean hasRejectedStatus = hasStatus(Status.STATUS_REJECTED, drugPairsStatusCounts);
            boolean hasDeletedStatus = hasStatus(Status.STATUS_DELETED, drugPairsStatusCounts);

            
            if (hasReviewedStatus) {

                if (hasNewStatus) {
                    messages.add("ddi.message.new.mixedpairs");
                } else if (hasModifiedStatus && hasApprovedStatus) {
                    messages.add("ddi.message.new.mixedpairs");
                } else if (hasModifiedStatus && (hasRejectedStatus || hasDeletedStatus)) {
                    messages.add("ddi.message.new.mixedpairs");
                } else if (!Status.STATUS_REVIEWED.equals(dto.getStatus()) && hasRejectedStatus || hasDeletedStatus) {
                    messages.add("ddi.message.new.reviewedpairs");
                }

            } else {

                if ((hasNewStatus && hasApprovedStatus) || (hasModifiedStatus)) {
                    messages.add("ddi.message.new.mixedpairs");
                }
            }
            //@formatter:off
            if (sameStatus(Status.STATUS_REVIEWED, drugPairsStatusCounts)
                    && (Status.STATUS_NEW.equals(dto.getStatus()) || Status.STATUS_MODIFIED.equals(dto.getStatus()) 
                            || Status.STATUS_REVIEWED
                            .equals(dto.getStatus()))) {
            // @formatter:on
                messages.add("ddi.message.new.reviewedpairs");
            }
            //@formatter:off
            
            if ((hasReviewedStatus && hasApprovedStatus && !hasNewStatus && !hasModifiedStatus) 
                    && (Status.STATUS_NEW.equals(dto.getStatus()) || Status.STATUS_MODIFIED.equals(dto.getStatus()) 
                            || Status.STATUS_REVIEWED
                            .equals(dto.getStatus()))) {
            // @formatter:on
                messages.add("ddi.message.new.reviewedpairs");
            }
            if (sameStatus(Status.STATUS_REJECTED, drugPairsStatusCounts)) {
                messages.add("ddi.message.rejected.drugpairs");
            }
            if (hasNewStatus) {
                if (sameStatus(Status.STATUS_NEW, drugPairsStatusCounts) && !Status.STATUS_REJECTED.equals(dto.getStatus())) {
                    messages.add("ddi.message.new.newpairs");
                } else if ((hasRejectedStatus) || hasDeletedStatus) {
                    messages.add("ddi.message.new.mixedpairs");
                }
            }
            if (Status.STATUS_REVIEWED.equals(dto.getStatus()) && hasReviewedStatus) {
                if (sameStatus(Status.STATUS_REVIEWED, drugPairsStatusCounts)) {
                    messages.add("ddi.message.reviewed.reviewedpairs");
                } else if (!(hasReviewedStatus && hasApprovedStatus)) {
                    messages.add("ddi.message.reviewed.mixedpairs");
                }
            }
            if (Status.STATUS_REJECTED.equals(dto.getStatus()) && !hasRejectedStatus) {
                messages.add("ddi.message.rejected.mixedpairs");
            }
        }
        return messages;
    }

    /**
     * Checks if any of the drug pairs contains the specified status.
     * 
     * @param status
     *            the status
     * @param drugPairsStatusCounts
     *            the drug pairs status counts
     * @return true, if at least one drug pair has the status that is specified; false otherwise
     */
    private static boolean hasStatus(String status, Map<String, Long> drugPairsStatusCounts) {
        if (drugPairsStatusCounts.containsKey(status) && drugPairsStatusCounts.get(status) > 0){        
                return true;
            }
        return false;
    }

    /**
     * Determines if all the drug pairs have the specified status and there are no drug pairs with another status.  
     * Deleted and rejected drug pairs are not considered in this determination.
     * 
     * @param status
     *            the status
     * @param drugPairsStatusCounts
     *            the drug pairs status counts
     * @return true, if all drug pairs contain only the specified status; false otherwise
     */
    protected static boolean sameStatus(String status, Map<String, Long> drugPairsStatusCounts) {
        Set<String> statuses = new HashSet<String>(); 
        statuses.addAll(drugPairsStatusCounts.keySet());
        //ignore deleted and rejected records unless we are  checking for deleted or rejected.
        if (!Status.STATUS_DELETED.equals(status) && !Status.STATUS_REJECTED.equals(status)){
            statuses.remove(Status.STATUS_DELETED);
            statuses.remove(Status.STATUS_REJECTED);
        }
        
        if (statuses.size() > 1){
            return false;
        }
                    
        if (statuses.contains(status) 
                && drugPairsStatusCounts.get(status) > 0){
            return true;
        }
        
        return false;         
    }
    
    /**
     * Determines if all the drug pairs are in deleted and/or rejected status.  
     * 
     * @param drugPairsStatusCounts
     *            the drug pairs status counts
     * @return true, if all drug pairs are in deleted or rejected status; false otherwise
     */
    protected static boolean hasOnlyDeletedAndRejected(Map<String, Long> drugPairsStatusCounts) {        
        Set<String> statuses = new HashSet<String>(); 
        statuses.addAll(drugPairsStatusCounts.keySet());
        //ignore deleted and rejected records
        statuses.remove(Status.STATUS_DELETED);
        statuses.remove(Status.STATUS_REJECTED);
        
        if (statuses.size() == 0){
            return true;
        }
                            
        return false;         
    }    
}
