package gov.va.med.pharmacy.pecs.web.detail.drugdrug;

import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.RecordType;
import gov.va.med.pharmacy.pecs.web.detail.LockingResult;
import gov.va.med.pharmacy.pecs.web.detail.RecordLockingHelper;
import gov.va.med.pharmacy.pecs.web.view.ReportHistoryXLSXViewer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.View;
import org.springframework.web.servlet.view.RedirectView;

/**
 * This is the controller class for the drug drug interaction concept.
 * 
 * @author vhaisacarusr
 * 
 */
@Controller
@RequestMapping(value = "/drugdrug")
public final class InteractionController extends AbstractController {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(InteractionController.class);

    /** The Constant SELECT_UNASSIGNED. */
    private static final String SELECT_UNASSIGNED = "UNASSIGNED";

    /** The Constant INTERACTION_VIEW. */
    private static final String INTERACTION_VIEW = "pecs.drugdrug.interaction.page";

    /** The ddi service. */
    @Resource
    private DrugDrugInteractionService ddiService = null;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /** The monograph service. */
    @Resource
    private MonographService monographService;

    /** The record locking helper. */
    @Resource
    private RecordLockingHelper recordLockingHelper;

    /**
     * 
     * Display interaction by id.
     * 
     * 
     * 
     * @param interactionId
     * 
     *            the interaction id
     * 
     * @param request
     * 
     *            the request
     * 
     * @param response
     * 
     *            the response
     * 
     * @return the model and view
     */

    @RequestMapping(value = "/interactionid/{interactionId}", method = RequestMethod.GET)
    public ModelAndView displayInteractionById(@PathVariable(value = "interactionId") String interactionId,

    HttpServletRequest request, HttpServletResponse response) {

        ModelAndView mav = new ModelAndView();

        DrugDrugInteractionDTO fdb = ddiService.getStagingByInteractionId(Long.parseLong(interactionId));

        String url =
                request.getContextPath() + request.getServletPath() + "/drugdrug/interaction/VA/"
                        + fdb.getDdiminteractionVaId();

        View view = new RedirectView(url, false, false, false);

        mav.setView(view);

        return mav;

    }

    /**
     * Display interaction.
     * 
     * @param recordId
     *            the record id
     * @param recordType
     *            the record type
     * @param request
     *            the request
     * @return the model and view
     */
    @RequestMapping(value = "/interaction/{recordType}/{recordId}", method = RequestMethod.GET)
    public ModelAndView displayInteraction(@PathVariable(value = "recordId") String recordId, @PathVariable(
            value = "recordType") String recordType, HttpServletRequest request) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("Received from ActionContext - recordid: " + recordId);
        }
        DrugDrugInteractionDTO fdb = null;
        String fdbIdSearch = "";
        String reverseFdbIdSearch = "";
        if (RecordType.FDB == RecordType.fromString(recordType)) {
            fdbIdSearch = recordId;
            Long reverseFDBId = 32000L - Long.valueOf(recordId);
            reverseFdbIdSearch = Long.toString(reverseFDBId);
            fdb = getDdiService().findFdbById(Long.valueOf(recordId));
        }
        if (RecordType.VA == RecordType.fromString(recordType)) {
            fdb = getDdiService().getStagingData(Long.valueOf(recordId));
            fdbIdSearch = Long.toString(fdb.getFdbId());
            reverseFdbIdSearch = Long.toString(fdb.getReverseFdbId());
            // List<DrugPairDTO> drugPairs =
            // drugPairService.getAllDrugPairsForInteraction(Long.valueOf(fdb.getInteractionid()));
            Map<String, Long> drugPairsStatusCounts =
                    drugPairService.getDrugPairsStatusCountsByInteractionId(Long.valueOf(fdb.getInteractionid()));
            List<String> messages = DDIMessageHelper.getDrugPairMessages(fdb, drugPairsStatusCounts);
            for (String message : messages) {
                addInformationMessage(message);
            }

        }

        List<DrugDrugInteractionDTO> list = ddiService.getStagingByFdbId(Long.parseLong(fdbIdSearch));
        Map<String, DrugDrugInteractionDTO> customizationList = new HashMap<String, DrugDrugInteractionDTO>();
        boolean vaRecExists = false;
        int customCount = 0;
        if (list != null && list.size() > 0) {
            for (DrugDrugInteractionDTO value : list) {
                if (!fdbIdSearch.equals(Long.toString(value.getInteractionid()))) {
                    customCount++;
                    vaRecExists = true;
                    String key = Long.toString(value.getInteractionid());
                    customizationList.put(key, value);
                }
            }
            DrugDrugInteractionDTO fdbHold = getDdiService().findFdbById(Long.valueOf(fdbIdSearch));
            customizationList.put(fdbIdSearch, fdbHold);
        }

        List<DrugDrugInteractionDTO> reverseList = ddiService.getStagingByFdbId(Long.valueOf(reverseFdbIdSearch));
        if (reverseList != null && reverseList.size() > 0) {
            for (DrugDrugInteractionDTO value : reverseList) {
                if (!reverseFdbIdSearch.equals(Long.toString(value.getInteractionid()))) {
                    customCount++;
                    vaRecExists = true;
                    String key = Long.toString(value.getInteractionid());
                    customizationList.put(key, value);
                }
            }
            // DrugDrugInteractionDTO fdbHold = getDdiService().findFdbById(Long.valueOf(reverseFdbIdSearch));
            // customizationList.put(reverseFdbIdSearch, fdbHold);
        }

        if (vaRecExists) {
            if (RecordType.FDB == RecordType.fromString(recordType)) {
                addInformationMessage("Following VA custom record(s) already exist for this FDB Drug-Drug Interaction.");
            }
            if (RecordType.VA == RecordType.fromString(recordType)) {
                if (!fdbIdSearch.equals(Long.toString(fdb.getInteractionid())) && customCount > 1) {
                    addInformationMessage("Following additional VA custom record(s) "
                            + "exist for the corresponding FDB Drug-Drug Interaction.");
                }
            }
        }
        if (fdb == null) {
            fdb = new DrugDrugInteractionDTO();
        }
        ConceptMetadata conceptMetadata = ddiService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        ModelAndView mav = new ModelAndView(INTERACTION_VIEW);
        mav.addObject("customizationList", customizationList);
        mav.addObject("fdb", fdb);
        mav.addObject("fieldMap", fieldMap);
        populateModel(request, fdb, mav);
        return mav;
    }

    /**
     * Save interaction.
     * 
     * @param recordId
     *            the record id
     * @param recordType
     *            the record type
     * @param request
     *            the request
     * @param fdb
     *            the fdb
     * @param bindingResult
     *            the binding result
     * @return the model and view
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    @RequestMapping(value = "/interaction/{recordType}/{recordId}", method = RequestMethod.POST)
    public ModelAndView saveInteraction(@PathVariable(value = "recordId") String recordId,
            @PathVariable(value = "recordType") String recordType, HttpServletRequest request,
            @ModelAttribute("fdb") DrugDrugInteractionDTO fdb, BindingResult bindingResult) throws IOException {
        final DrugDrugInteractionDTO originalDDI;
        final DrugDrugInteractionDTO updatedDDI;
        ModelAndView mav = new ModelAndView();
        fdb.setActionUser(getUserProfile(request));

        // formatAction(request);
        String action = request.getParameter("action");
        action = action.replace(' ', '_');
        action = action.replace("_As_", "_as_");
        action = action.replace("_For_", "_for_");
        if (fdb.getAssignedTo() == null || "".equals(fdb.getAssignedTo())) {
            fdb.setAssignedTo(SELECT_UNASSIGNED);
        }
        if (RecordType.FDB == RecordType.fromString(recordType)) {
            fdb.setInteractionid(0L);
            updatedDDI = ddiService.createNewRecord(fdb, bindingResult);
            originalDDI = ddiService.findFdbById(Long.parseLong(recordId));
        } else {
            updatedDDI = ddiService.applyAction(action, fdb, bindingResult);
            originalDDI = ddiService.getStagingData(Long.valueOf(recordId));
        }

        ConceptMetadata conceptMetadata = ddiService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();

        mav.addObject("fieldMap", fieldMap);
        populateModel(request, updatedDDI, mav);
        Map<String, Long> drugPairsStatusCounts =
                drugPairService.getDrugPairsStatusCountsByInteractionId(Long.valueOf(fdb.getInteractionid()));
        List<String> messages = DDIMessageHelper.getMessages(updatedDDI, originalDDI, action, drugPairsStatusCounts);
        if (updatedDDI.isDuplicate()) {
            addWarningMessage("message.duplicate.ddi.on.modify",
                    new Object[] { updatedDDI.getDescription1(), updatedDDI.getSeveritylevelcode() });
        }

        if (bindingResult.hasErrors()) {
            mav.addObject("fdb", fdb);
            mav.setViewName(INTERACTION_VIEW);

            return mav;

        }
        for (String message : messages) {
            addInformationMessage(findMessage(message));
        }

        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            cancelLock(recordType, recordId, request);
        } else {
            cancelLock(recordType, String.valueOf(updatedDDI.getFdbId()), request);
        }
        View redirectView =
                new RedirectView(request.getContextPath() + request.getServletPath() + "/drugdrug/interaction/VA/"
                        + updatedDDI.getDdiminteractionVaId(), false, false, false);
        mav.setView(redirectView);
        return mav;
    }

    /**
     * Locks the Record so that one user my alter it at a time.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @return the locking result
     */
    @RequestMapping(value = "interaction/edit/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public LockingResult lockRecord(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        DrugDrugInteractionDTO ddiminteractionVa = null;
        LockingResult result = new LockingResult();
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            ddiminteractionVa = ddiService.getStagingData(Long.parseLong(recordId));
            if (!ddiminteractionVa.isCurrent()) {
                result.setMessage(findMessage("detail.message.notcurrent"));
                DrugDrugInteractionDTO current = ddiService.getStagingByInteractionId(ddiminteractionVa.getInteractionid());
                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }
            recordId = String.valueOf(ddiminteractionVa.getMasterRecordId());
        }
        String message =
                recordLockingHelper.checkEdit(recordType, Concept.DRUG_DRUG_INTERACTION.toString(), recordId,
                        getUserProfile(request).getUsername());
        result.setMessage(message);
        return result;
    }

    /**
     * Cancel lock.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @return the string
     */
    @RequestMapping(value = "interaction/cancel/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public String cancelLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        DrugDrugInteractionDTO ddiminteractionVa = null;
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            ddiminteractionVa = ddiService.getStagingData(Long.parseLong(recordId));

            recordId = String.valueOf(ddiminteractionVa.getMasterRecordId());
        }
        return recordLockingHelper.cancelEdit(recordType, Concept.DRUG_DRUG_INTERACTION.toString(), recordId,
                getUserProfile(request).getUsername());
    }

    /**
     * Extend lock.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @return the locking result
     */
    @RequestMapping(value = "interaction/extend/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public LockingResult extendLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        DrugDrugInteractionDTO ddiminteractionVa = null;
        LockingResult result = new LockingResult();
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            ddiminteractionVa = ddiService.getStagingData(Long.parseLong(recordId));
            if (!ddiminteractionVa.isCurrent()) {
                DrugDrugInteractionDTO current = ddiService.getStagingByInteractionId(ddiminteractionVa.getInteractionid());
                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }

            recordId = String.valueOf(ddiminteractionVa.getMasterRecordId());
        } else {
            ddiminteractionVa = ddiService.findFdbById(Long.parseLong(recordId));

        }
        recordLockingHelper.extendLock(recordType, Concept.DRUG_DRUG_INTERACTION.toString(), recordId, getUserProfile(request)
                .getUsername());

        return result;
    }

    /**
     * Display interaction history.
     * 
     * @param recordId
     *            the record id
     * @param recordType
     *            the record type
     * @param request
     *            the request
     * @return the model and view
     */
    @RequestMapping(value = "/interaction/history/{recordType}/{recordId}", method = RequestMethod.GET)
    public ModelAndView displayInteractionHistory(@PathVariable(value = "recordId") String recordId, @PathVariable(
            value = "recordType") String recordType, HttpServletRequest request) {
        Map<String, Object> model = new HashMap<String, Object>();
        model.put(ReportHistoryXLSXViewer.METADATA_KEY, Concept.DRUG_DRUG_INTERACTION);
        model.put(ReportHistoryXLSXViewer.DATA_KEY, ddiService.findReportHistoryData(Long.valueOf(recordId)));
        model.put(ReportHistoryXLSXViewer.EXCEL_FILE_NAME_KEY, Concept.DRUG_DRUG_INTERACTION + "_HISTORY_REPORT");
        return new ModelAndView(ReportHistoryXLSXViewer.REPORT_HISTORY_VIEW, model);
    }

    /**
     * Populate model.
     * 
     * @param request
     *            the request
     * @param fdb
     *            the fdb
     * @param mav
     *            the mav
     */
    private void populateModel(HttpServletRequest request, DrugDrugInteractionDTO fdb, ModelAndView mav) {
        List<?> descriptions = ddiService.getInteractionDescriptions();
        List<LabelValue> descriptionList = new ArrayList<LabelValue>();
        for (Object object : descriptions) {
            Object[] string = (Object[]) object;
            LabelValue labelValue =
                    new LabelValue(string[1].toString() + " - " + string[0].toString(), string[0].toString() + " - "
                            + string[0].toString());
            descriptionList.add(labelValue);
        }

        mav.addObject("descriptions", descriptionList);
        mav.addObject("severityLevels", terminologyService.getSeverityLevels());
        mav.addObject("clinicalEffects", terminologyService.getClinicalEffects());
        // mav.addObject("monographs", monographService.getAllMonographs());
        mav.addObject("monographs", monographService.getAllActiveMonographs());
        mav.addObject("interactions", terminologyService.getEvaluationOfDrugInteractions());
        mav.addObject("onsets", terminologyService.getDrugInteractionFactOnsets());
        mav.addObject("factSeverities", terminologyService.getDrugInteractionFactSeverities());
        mav.addObject("factDocumentation", terminologyService.getDrugInteractionFactDocumentation());
        mav.addObject("microSeverities", terminologyService.getMicromedexSeverities());
        mav.addObject("microOnsets", terminologyService.getMicromedexOnsets());
        mav.addObject("microSubstantions", terminologyService.getMicromedexSubstantiations());
        mav.addObject("medilineHits", terminologyService.getMedlineHits());
        mav.addObject("packageInserts", terminologyService.getPackageInserts());
        mav.addObject("pbmCriteria", terminologyService.getPBMCriteria());
        mav.addObject("interactionSources", terminologyService.getInteractionSources());
        mav.addObject("interactionTypes", terminologyService.getInteractionTypes());
        mav.addObject("interactionFacts", terminologyService.getDrugInteractionFacts());
        mav.addObject("aidsGuidelines", terminologyService.getAIDSGuidelines());
        mav.addObject("levelOfEvidence", terminologyService.getLevelOfEvidence());
        mav.addObject("users", getAdminUsers());

        UserProfile profile = getUserProfile(request);

        // interrogating the state machine for allowable actions makes sense
        // if we're not displaying the record in read-only mode and it's not
        // and FDB record

        Set<String> actions = Collections.emptySet();
        if (fdb.isCurrent()) {
            if (fdb.getDdiminteractionVaId() != 0) {
                actions = ddiService.getAllowableActions(fdb, getUserProfile(request));
            }
            permissionValidation(profile, actions, mav, fdb.getDdiminteractionVaId());
        }
        mav.addObject("showCommentButton", shouldCommentButtonBeShown(fdb));
    }

    /**
     * Should comment button be shown.
     * 
     * @param fdb
     *            the fdb
     * @return true,button will be shown; false, button will not be shown
     */
    public boolean shouldCommentButtonBeShown(DrugDrugInteractionDTO fdb) {
        if (doesActiveCustomizedVAExist(fdb.getReverseFdbId())) {
            return false;
        }
        if (doesActiveCustomizedVAExist(fdb.getFdbId())) {
            return false;
        }
        if (fdb.getDdiminteractionVaId() != 0) {
            return false;
        }
        return true;
    }

    /**
     * Method Checks to see if the Reverse DDI has customized Records.
     * 
     * @param fdbId
     *            the fdb id
     * @return true or false
     */
    public boolean doesActiveCustomizedVAExist(Long fdbId) {
        List<DrugDrugInteractionDTO> vaList = getDdiService().getStagingByFdbId(fdbId);
        if (vaList != null && vaList.size() > 0) {
            for (DrugDrugInteractionDTO record : vaList) {
                if (!((PecsDTO) record).getStatus().equalsIgnoreCase(Status.STATUS_DELETED)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Sets the ddi service.
     * 
     * @param ddiService
     *            the new ddi service
     */
    public void setDdiService(DrugDrugInteractionService ddiService) {
        this.ddiService = ddiService;
    }

    /**
     * Gets the ddi service.
     * 
     * @return the ddi service
     */
    public DrugDrugInteractionService getDdiService() {
        return ddiService;
    }

    /**
     * Sets the terminology service.
     * 
     * @param terminologyService
     *            the new terminology service
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    /**
     * Gets the terminology service.
     * 
     * @return the terminology service
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the monograph service.
     * 
     * @param monographService
     *            the new monograph service
     */
    public void setMonographService(MonographService monographService) {
        this.monographService = monographService;
    }

    /**
     * Gets the monograph service.
     * 
     * @return the monograph service
     */
    public MonographService getMonographService() {
        return monographService;
    }

    /**
     * Display blank.
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @param fdb
     *            the fdb
     * @param bindingResult
     *            the binding result
     * @return the model and view
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    @RequestMapping(value = "/blank", method = RequestMethod.GET)
    public ModelAndView displayBlank(HttpServletRequest request, HttpServletResponse response,
            @ModelAttribute("fdb") DrugDrugInteractionDTO fdb, BindingResult bindingResult) throws IOException {
        ConceptMetadata conceptMetadata = ddiService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        ModelAndView mav = new ModelAndView(INTERACTION_VIEW);
        fdb.setActionUser(getUserProfile(request));
        fdb.setActionBy(getUserProfile(request).getUsername());
        fdb.setInteractionid(0L);
        mav.addObject("fdb", fdb);
        mav.addObject("fieldMap", fieldMap);
        populateModel(request, fdb, mav);
        return mav;
    }

    /**
     * Display blank submitted.
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @param fdb
     *            the fdb
     * @param bindingResult
     *            the binding result
     * @return the model and view
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    @RequestMapping(value = "/blank", method = RequestMethod.POST)
    public ModelAndView displayBlankSubmitted(HttpServletRequest request, HttpServletResponse response,
            @ModelAttribute("fdb") DrugDrugInteractionDTO fdb, BindingResult bindingResult) throws IOException {
        final DrugDrugInteractionDTO updatedDDI;
        if (fdb == null) {
            fdb = new DrugDrugInteractionDTO();
        }
        if (fdb.getAssignedTo() == null || "".equals(fdb.getAssignedTo())) {
            fdb.setAssignedTo(SELECT_UNASSIGNED);
        }
        ConceptMetadata conceptMetadata = ddiService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        ModelAndView mav = new ModelAndView(INTERACTION_VIEW);
        fdb.setActionUser(getUserProfile(request));
        fdb.setActionBy(getUserProfile(request).getUsername());
        fdb.setInteractionid(0L);
        mav.addObject("fdb", fdb);
        mav.addObject("fieldMap", fieldMap);
        updatedDDI = ddiService.createNewRecord(fdb, bindingResult);
        populateModel(request, updatedDDI, mav);

        if (bindingResult.hasErrors()) {
            return mav;
        }
        Map<String, Long> emptyDPStatusCountMap = Collections.emptyMap();
        List<String> messages = DDIMessageHelper.getMessages(updatedDDI, fdb, "Customize", emptyDPStatusCountMap);
        for (String message : messages) {
            addInformationMessage(message);
        }

        View view = new RedirectView("interaction/VA/" + updatedDDI.getDdiminteractionVaId(), true, false, false);
        mav.setView(view);

        return mav;
    }

    /**
     * Gets the record locking helper.
     * 
     * @return the recordLockingHelper
     */
    public RecordLockingHelper getRecordLockingHelper() {
        return recordLockingHelper;
    }

    /**
     * Sets the record locking helper.
     * 
     * @param recordLockingHelper
     *            the recordLockingHelper to set
     */
    public void setRecordLockingHelper(RecordLockingHelper recordLockingHelper) {
        this.recordLockingHelper = recordLockingHelper;
    }
}
