/**
 *
 */
package gov.va.med.pharmacy.pecs.web.detail.drugpair;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdbLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.detail.LockingResult;
import gov.va.med.pharmacy.pecs.web.detail.RecordLockingHelper;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.validation.BindingResult;
import org.springframework.web.servlet.ModelAndView;

/**
 * 
 * Abstract class that holds common methods shared by the Drug Pair Customization pages.
 * 
 * @author VHAISACARUSR 
 * 
 */
public abstract class AbstractDrugPairController extends AbstractController implements InitializingBean {

    /** The Constant EXPECTED_SPLIT. */
    private static final int EXPECTED_SPLIT = 3;

    /** The Constant SELECT_UNASSIGNED. */
    private static final String SELECT_UNASSIGNED = "UNASSIGNED";

    /** The Constant SEP. */
    private static final String SEP = "|";
    /**
     * type of record being locked.
     */
    protected static final String RECORD_TYPE = "VA";

    /** The drug drug interaction service. */
    @Resource
    private DrugDrugInteractionService drugDrugInteractionService;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The record locking helper. */
    @Resource
    private RecordLockingHelper recordLockingHelper;

    /**
     * Collects all of the data needed to display the page and adds to the <code>ModelAndView</code>.
     * 
     * @param interactionId
     *            The ID of the interaction we are customizing
     * @param modelAndView
     *            Holds the objects for display
     * @param profile
     *            the profile of the user performing the action
     */
    final void prepareModelAndViewForDisplay(Long interactionId, ModelAndView modelAndView, UserProfile profile) {
        DrugDrugInteractionDTO vaInteraction = getDrugDrugInteractionService().getStagingByInteractionId(interactionId);
        List<ConceptField> columnList = drugPairService.getMetadata().getQueryFields(true);
        DrugDrugInteractionDTO fdbInteraction = null;
        if (vaInteraction.getFdbId() != 0) {
            fdbInteraction = getDrugDrugInteractionService().findFdbById(vaInteraction.getFdbId());
        }
        List<DrugDrugInteractionDTO> otherVaInteractions =
                getDrugDrugInteractionService().getStagingByFdbId(vaInteraction.getFdbId());
        removeCurrentInteraction(vaInteraction, otherVaInteractions);

        List<DrugPairDTO> filteredFdbDrugPairs = drugPairService.getFilteredFdbData(vaInteraction.getFdbId());
        List<DrugPairDTO> customDrugParis = drugPairService.getDrugPairsForInteraction(interactionId, profile);

        Map<String, List<String>> customDrugPairsForDisplay = prepareRowsVaDrugPairs(customDrugParis, columnList);
        showExistingDrugPairsMesssage(modelAndView, vaInteraction);
        modelAndView.addObject("vaInteraction", vaInteraction);
        modelAndView.addObject("fdbInteraction", fdbInteraction);
        modelAndView.addObject("otherVaInteractions", otherVaInteractions);

        modelAndView.addObject("fdbDrugPairs", filteredFdbDrugPairs);
        modelAndView.addObject("columnList", columnList);
        modelAndView.addObject("customDrugPairs", customDrugParis);
        modelAndView.addObject("customDrugPairsForDisplay", customDrugPairsForDisplay);

        modelAndView.addObject("user", profile);

        createModelWithForms(interactionId, vaInteraction.getAssignedTo(), modelAndView);

        List<RoutedGenericFdbLite> routedGenerics = getDrugPairService().getRoutedGenericDrugsLite();
        modelAndView.addObject("routedGenerics", routedGenerics);
    }

    /**
     * Show existing drug pairs messsage.
     * 
     * @param modelAndView
     *            Holds the message for display on the page
     * @param vaInteraction
     *            The interaction that is being customized
     */
    private void showExistingDrugPairsMesssage(ModelAndView modelAndView, DrugDrugInteractionDTO vaInteraction) {
        if (drugPairService.hasFdbDrugPairsOnOtherInteraction(vaInteraction.getFdbId())) {
            String drugPairsExistMsg = "Existing customized Drug Pairs for this FDB Drug-Drug Interaction are not displayed.";
            modelAndView.addObject("drugPairsExistMsg", drugPairsExistMsg);
        }
    }

    /**
     * Removes the current interaction.
     * 
     * @param vaInteraction
     *            The interaction that is being customized
     * @param otherVaInteractions
     *            all of the interactions associated with the FDB interaction of the interaction we are customizing
     */
    private void
            removeCurrentInteraction(DrugDrugInteractionDTO vaInteraction, List<DrugDrugInteractionDTO> otherVaInteractions) {
        for (DrugDrugInteractionDTO drugDrugInteractionDTO : otherVaInteractions) {
            if (drugDrugInteractionDTO.getInteractionid() == vaInteraction.getInteractionid()) {
                otherVaInteractions.remove(drugDrugInteractionDTO);
                break;
            }
        }
    }

    /**
     * Creates the model with forms.
     * 
     * @param interactionId
     *            The ID of the interaction we are customizing
     * @param assignedTo
     *            the name of the user the interaction is assignedTo
     * @param modelAndView
     *            Holds the objects for display
     * @return the model and view
     */
    private ModelAndView createModelWithForms(Long interactionId, String assignedTo, ModelAndView modelAndView) {

        addGenericDrugPairsForm(interactionId, assignedTo, modelAndView);

        addRoutedGenericForm(interactionId, assignedTo, modelAndView);

        addFdbDrugPairsForm(interactionId, assignedTo, modelAndView);

        addDrugPairModificationForm(assignedTo, modelAndView);

        return modelAndView;
    }

    /**
     * Method to add the FDB Drug Pair Form. If the form already exists then the interaction id parameter is checked against the
     * form interaction id to determine which form to use.
     * 
     * @param interactionId
     *            the interaction id
     * @param assignedTo
     *            the assigned to
     * @param modelAndView
     *            the model and view
     */
    private void addFdbDrugPairsForm(Long interactionId, String assignedTo, ModelAndView modelAndView) {
        FdbDrugPairsForm fdbDrugPairsForm;
        if (modelAndView.getModelMap().containsAttribute("fdbDrugPairsForm")) {
            fdbDrugPairsForm = (FdbDrugPairsForm) modelAndView.getModelMap().get("fdbDrugPairsForm");
            if (fdbDrugPairsForm.getInteractionId() != interactionId.longValue()) {
                fdbDrugPairsForm = new FdbDrugPairsForm();
            }
        } else {
            fdbDrugPairsForm = new FdbDrugPairsForm();
        }

        fdbDrugPairsForm.setAssignedTo(assignedTo);
        fdbDrugPairsForm.setInteractionId(interactionId);

        fdbDrugPairsForm.setFdbDrugPairs(null);
        modelAndView.addObject("fdbDrugPairsForm", fdbDrugPairsForm);
    }

    /**
     * Method to add the Routed Generic Form. If the form already exists then the interaction id parameter is checked against
     * the form interaction id to determine which form to use.
     * 
     * @param interactionId
     *            the interaction id
     * @param assignedTo
     *            the assigned to
     * @param modelAndView
     *            the model and view
     */
    private void addRoutedGenericForm(Long interactionId, String assignedTo, ModelAndView modelAndView) {
        RoutedGenericsForm routedGenericsForm;
        if (modelAndView.getModelMap().containsAttribute("routedGenericsForm")) {
            routedGenericsForm = (RoutedGenericsForm) modelAndView.getModelMap().get("routedGenericsForm");
            if (routedGenericsForm.getInteractionId() != interactionId.longValue()) {
                routedGenericsForm = new RoutedGenericsForm();
            }
        } else {
            routedGenericsForm = new RoutedGenericsForm();
        }

        routedGenericsForm.setInteractionId(interactionId);
        routedGenericsForm.setAssignedTo(assignedTo);
        modelAndView.addObject("routedGenericsForm", routedGenericsForm);
    }

    /**
     * Method to add the Generic Drug Pairs Form. If the form already exists then the interaction id parameter is checked
     * against the form interaction id to determine which form to use.
     * 
     * @param interactionId
     *            the interaction id
     * @param assignedTo
     *            the assigned to
     * @param modelAndView
     *            the model and view
     */
    private void addGenericDrugPairsForm(Long interactionId, String assignedTo, ModelAndView modelAndView) {
        GenericDrugPairsForm genericDrugPairsForm;

        if (modelAndView.getModelMap().containsAttribute("genericDrugPairsForm")) {
            genericDrugPairsForm = (GenericDrugPairsForm) modelAndView.getModelMap().get("genericDrugPairsForm");
            if (genericDrugPairsForm.getInteractionId() != interactionId.longValue()) {
                genericDrugPairsForm = new GenericDrugPairsForm();
            }
        } else {
            genericDrugPairsForm = new GenericDrugPairsForm();
        }
        genericDrugPairsForm.setInteractionId(interactionId);
        genericDrugPairsForm.setAssignedTo(assignedTo);
        modelAndView.addObject("genericDrugPairsForm", genericDrugPairsForm);
    }

    /**
     * Method to add the Drug Pair Modification Form. If the form already exists then it will be used.
     * 
     * @param assignedTo
     *            the assigned to
     * @param modelAndView
     *            the model and view
     */
    private void addDrugPairModificationForm(String assignedTo, ModelAndView modelAndView) {
        DrugPairModificationForm drugPairModificationForm;

        if (modelAndView.getModelMap().containsAttribute("drugPairModificationForm")) {
            drugPairModificationForm = (DrugPairModificationForm) modelAndView.getModelMap().get("drugPairModificationForm");

        } else {
            drugPairModificationForm = new DrugPairModificationForm();
        }

        drugPairModificationForm.setAssignedTo(assignedTo);
        modelAndView.addObject("drugPairModificationForm", drugPairModificationForm);
    }

    /**
     * Takes the routed generic ID's and uses them to create a list of drug pairs to be modified.
     * 
     * 
     * @param drugPairModificationForm
     *            the form that has been submitted
     * @param profile
     *            the profile for the user
     * @param action
     *            the action to be performed
     * @return a list of <code>DrugPairDTO</code>. Never <code>null</code>
     */
    final List<DrugPairDTO> createModifiableDrugPairs(DrugPairModificationForm drugPairModificationForm, UserProfile profile,
            String action) {

        DrugDrugInteractionDTO interaction =
                getDrugDrugInteractionService().getStagingByInteractionId(drugPairModificationForm.getInteractionId());
        return populateModifiableDrugPairs(drugPairModificationForm, profile, interaction.getSeveritylevelcode(), action);
    }

    /**
     * Takes input from the form and populates the drug pair objects to be passed to he service tier.
     * 
     * @param drugPairModificationForm
     *            the form that holds the drug pair information
     * @param profile
     *            the profile for the user
     * @param severityLevelCode
     *            the severity level from the interaction
     * @param action
     *            the action to be performed;
     * @return a list of drug pairs. Never <code>null</code>
     */

    final List<DrugPairDTO> populateModifiableDrugPairs(DrugPairModificationForm drugPairModificationForm, UserProfile profile,
            String severityLevelCode, String action) {

        List<DrugPairDTO> drugPairs = new ArrayList<DrugPairDTO>();
        String[] drugPairsFromForm = drugPairModificationForm.getSelectedList();
        for (String key : drugPairsFromForm) {
            long routedGeneric1 = new Long(0);
            long routedGeneric2 = new Long(0);

            String[] keys = key.split("\\|");
            if (keys.length > 2) {

                routedGeneric1 = Long.parseLong(keys[1]);
                routedGeneric2 = Long.parseLong(keys[2]);
            } else {
                routedGeneric1 = Long.parseLong(keys[0]);
                routedGeneric2 = Long.parseLong(keys[1]);
            }

            if (drugPairModificationForm.getAssignedTo().equals(profile.getUsername())
                    && (action.equalsIgnoreCase("Submit_As_Reviewed") || action.equalsIgnoreCase("Submit_For_Delete"))) {
                drugPairModificationForm.setAssignedTo(SELECT_UNASSIGNED);
            }
            DrugPairDTO drugPairDTO =
                    populateDrugPair(routedGeneric1, routedGeneric2, drugPairModificationForm.getAssignedTo(),
                            drugPairModificationForm.getInteractionId(), profile, severityLevelCode);
            drugPairs.add(drugPairDTO);
        }
        return drugPairs;
    }

    /**
     * Populates the required fields to create a drug pair.
     * 
     * @param routedGeneric1
     *            the routed generic 1 ID
     * @param routedGeneric2
     *            the routed generic 2 ID
     * @param assignedTo
     *            the drug pair assignee
     * @param interactionId
     *            the interaction ID the drug pair is associated with
     * @param profile
     *            the user profile for the user performing the action
     * @param severityLevelCode
     *            the severity level of the interaction
     * @return a drug pair
     */
    private DrugPairDTO populateDrugPair(long routedGeneric1, long routedGeneric2, String assignedTo, long interactionId,
            UserProfile profile, String severityLevelCode) {
        DrugPairDTO drugPairDTO = new DrugPairDTO();
        drugPairDTO.setRtgenid1(routedGeneric1);
        drugPairDTO.setRtgenid2(routedGeneric2);

        drugPairDTO.setAssignedTo(assignedTo);
        drugPairDTO.setInteractionid(interactionId);
        drugPairDTO.setSeveritylevelcode(severityLevelCode);

        drugPairDTO.setActionBy(profile.getUsername());
        drugPairDTO.setActionUser(profile);
        return drugPairDTO;
    }

    /**
     * Creates and populates a new <code>DrugPairDTO</code> object.
     * 
     * @param routedGeneric1
     *            the routed generic 1 ID
     * @param routedGeneric2
     *            the routed generic 2 ID
     * @param interactionId
     *            the interaction ID the drug pair is associated with
     * @param profile
     *            the <code>Profile</code> for the user performing the action
     * @param assignedTo
     *            the drug pair assignee
     * @param currentActionReason
     *            the reason for adding the drug pair
     * @param referenceText
     *            the reference text for the drug pair
     * @return a populated <code>DrugPairDTO</code>
     */
    final DrugPairDTO createNewDrugPair(long routedGeneric1, long routedGeneric2, long interactionId, UserProfile profile,
            String assignedTo, String currentActionReason, String referenceText) {
        DrugPairDTO drugPair = new DrugPairDTO();

        drugPair.setRtgenid1(routedGeneric1);
        drugPair.setRtgenid2(routedGeneric2);
        drugPair.setInteractionid(interactionId);
        drugPair.setActionBy(profile.getUsername());
        drugPair.setActionUser(profile);
        drugPair.setCurrentActionReason(currentActionReason);
        drugPair.setAssignedTo(assignedTo);
        drugPair.setReferenceText(referenceText);

        return drugPair;
    }

    /**
     * Takes the drug pairs from the form and creates a list of objects from them.
     * 
     * @param profile
     *            the user profile
     * @param drugPairsForm
     *            the form from the web page.
     * @return a list of <code>DrugPairDTO</code>. Never <code>null</code>
     */
    final List<DrugPairDTO> convertFormToDrugPairs(UserProfile profile, FdbDrugPairsForm drugPairsForm) {
        List<DrugPairDTO> selectedFdbDdims = new ArrayList<DrugPairDTO>();

        String[] selectedFdbList = drugPairsForm.getFdbDrugPairs();
        if (selectedFdbList == null) {
            return selectedFdbDdims;
        }

        Long id = Long.valueOf(drugPairsForm.getInteractionId());

        for (String k : selectedFdbList) {

            if ("SelectDeselectAll".equalsIgnoreCase(k)) {
                continue;
            }
            String[] vals = null;
            vals = k.split("\\|");

            if (vals.length < EXPECTED_SPLIT) {
                return selectedFdbDdims;
            }

            Long fdbid = Long.valueOf(vals[0]);
            // Long revfdbid = FDB_BASE_VALUE - fdbid;
            Long d1 = Long.valueOf(vals[1]);
            Long d2 = Long.valueOf(vals[2]);

            DrugPairDTO ddim = new DrugPairDTO();

            // Set Requestor
            ddim.setRequestor(profile.getUsername());

            ddim.setActionUser(profile);

            ddim.setAssignedTo(drugPairsForm.getAssignedTo());

            // Set ref text and action reason from UI
            ddim.setReferenceText(drugPairsForm.getReferenceText());
            ddim.setCurrentActionReason(drugPairsForm.getCurrentActionReason());

            // Set key fields
            ddim.setFdbId(fdbid);
            ddim.setInteractionid(id);
            ddim.setRtgenid1(d1);
            ddim.setRtgenid2(d2);

            selectedFdbDdims.add(ddim);
        }

        return selectedFdbDdims;
    }

    /**
     * Prepare rows va drug pairs.
     * 
     * @param commitedDrugPairs
     *            VA drug pairs
     * @param columnList
     *            List of columns to be displayed
     * @return a double linked list that represents the table
     */
    private Map<String, List<String>> prepareRowsVaDrugPairs(/* List<DrugPairDTO> druglinkList, */
    List<DrugPairDTO> commitedDrugPairs, List<ConceptField> columnList) {
        Map<String, List<String>> modifyDdims = new HashMap<String, List<String>>();

        for (DrugPairDTO ddim : commitedDrugPairs) {
            List<String> preparedColumnList = prepareDisplayColumns(columnList, ddim);
            modifyDdims.put(
                    toDelimitedString(ddim.getAllowableActions()) + SEP + ddim.getRtgenid1() + SEP + ddim.getRtgenid2(),
                    preparedColumnList);

        }
        return modifyDdims;
    }

    /**
     * Prepare display columns.
     * 
     * @param fields
     *            Concept Fields to be displayed
     * @param ddim
     *            the Drug pair to be displayed on the row
     * @return a list of string values
     */
    private final List<String> prepareDisplayColumns(List<ConceptField> fields, DrugPairDTO ddim) {
        ArrayList<String> fieldValues = new ArrayList<String>();
        for (ConceptField field : fields) {
            fieldValues.add(field.getValueAsString(ddim));
        }
        return fieldValues;
    }

    /**
     * To delimited string.
     * 
     * @param list
     *            List of values that make up the key
     * @return the key string
     */
    private String toDelimitedString(Collection<String> list) {
        StringBuffer res = new StringBuffer();
        for (Object elem : list) {
            if (res.length() == 0) {
                res.append(elem);
            } else {
                res.append('~').append(elem);
            }
        }
        return res.toString();
    }

    /**
     * Creates a lock on the Interaction so that it cannot be edited by another user.
     * 
     * @param recordType
     *            type of record either VA or FDB
     * @param recordId
     *            the VA ID of the record being locked
     * @param userProfile
     *            the <code>UserProfile</code> for the user performing the action
     * @return a <code>LockingResult</code> to be processed by the browser
     */
    final LockingResult lockRecord(String recordType, Long recordId, UserProfile userProfile) {
        DrugDrugInteractionDTO vaRecord = getDrugDrugInteractionService().getStagingByInteractionId(recordId);
        LockingResult lockingResult = new LockingResult();
        String message =
                getRecordLockingHelper().checkEdit(recordType, Concept.DRUG_DRUG_INTERACTION.toString(),
                        String.valueOf(vaRecord.getMasterRecordId()), userProfile.getUsername());
        lockingResult.setMessage(message);
        lockingResult.setCurrent(true);
        return lockingResult;
    }

    /**
     * Cancels a lock on the Interaction so that it can be edited by another user.
     * 
     * @param recordType
     *            type of record either VA or FDB
     * @param recordId
     *            the VA ID of the record being locked
     * @param userProfile
     *            the <code>UserProfile</code> for the user performing the action
     * @return a String response to be processed by the browser
     */
    final String cancelLock(String recordType, Long recordId, UserProfile userProfile) {
        String response = null;
        DrugDrugInteractionDTO vaRecord = getDrugDrugInteractionService().getStagingByInteractionId(recordId);
        response =
                getRecordLockingHelper().cancelEdit(recordType, Concept.DRUG_DRUG_INTERACTION.toString(),
                        String.valueOf(vaRecord.getMasterRecordId()), userProfile.getUsername());
        return response;
    }

    /**
     * Extends a lock on the Interaction so that it cannot be edited by another user.
     * 
     * @param recordType
     *            type of record either VA or FDB
     * @param recordId
     *            the VA ID of the record being locked
     * @param userProfile
     *            the <code>UserProfile</code> for the user performing the action
     */
    final void extendLock(String recordType, Long recordId, UserProfile userProfile) {
        DrugDrugInteractionDTO vaRecord = getDrugDrugInteractionService().getStagingByInteractionId(recordId);
        getRecordLockingHelper().extendLock(recordType, Concept.DRUG_DRUG_INTERACTION.toString(),
                String.valueOf(vaRecord.getMasterRecordId()), userProfile.getUsername());
    }

    /**
     * Gets the drugPairService.
     * 
     * @return the drugPairService
     */
    public final DrugPairService getDrugPairService() {
        return drugPairService;
    }

    /**
     * Sets the drugPairService.
     * 
     * @param drugPairService
     *            the drugPairService to set
     */
    public final void setDrugPairService(DrugPairService drugPairService) {
        this.drugPairService = drugPairService;
    }

    /**
     * Gets the drugDrugInteractionService.
     * 
     * @return the drugDrugInteractionService
     */
    public final DrugDrugInteractionService getDrugDrugInteractionService() {
        return drugDrugInteractionService;
    }

    /**
     * Sets the drugDrugInteractionService.
     * 
     * @param drugDrugInteractionService
     *            the drugDrugInteractionService to set
     */
    public final void setDrugDrugInteractionService(DrugDrugInteractionService drugDrugInteractionService) {
        this.drugDrugInteractionService = drugDrugInteractionService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.AbstractController#afterPropertiesSet()
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (getDrugPairService() == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "DrugPairService" }));
        }
        if (getDrugDrugInteractionService() == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "DrugDrugInteractionService" }));
        }
    }

    /**
     * Gets the recordLockingHelper.
     * 
     * @return the recordLockingHelper
     */
    public final RecordLockingHelper getRecordLockingHelper() {
        return recordLockingHelper;
    }

    /**
     * Sets the recordLockingHelper.
     * 
     * @param recordLockingHelper
     *            the recordLockingHelper to set
     */
    public final void setRecordLockingHelper(RecordLockingHelper recordLockingHelper) {
        this.recordLockingHelper = recordLockingHelper;
    }

    /**
     * Checks if is reject or delete action.
     * 
     * @param action
     *            the action
     * @return true, if is reject or delete action
     */
    protected final boolean isRejectOrDeleteAction(String action) {
        if (action.equalsIgnoreCase("Delete") || action.equalsIgnoreCase("Reject")
                || action.equalsIgnoreCase("Submit_For_Delete"))
            return true;
        return false;
    }

    /**
     * Validate interaction status.
     * 
     * @param interactionId
     *            the interaction id
     * @param bindingResult
     *            the binding result
     */
    protected final void validateInteractionStatus(Long interactionId, BindingResult bindingResult) {
        DrugDrugInteractionVa interaction = getDrugDrugInteractionService().getDDIByInteractionId(interactionId);

        checkInteractionStatus(interaction, bindingResult);
    }

}
