package gov.va.med.pharmacy.pecs.web.detail.drugpair;

/**
 * This represents the drug pair grid for FDB drug pairs.
 * 
 * @author vhaisacarusr
 * 
 */

public class DrugPairsGridBean implements Comparable<DrugPairsGridBean> {
    
    /** The key. */
    private String key;
    
    /** The rtgenid1 desc. */
    private String rtgenid1Desc;
    
    /** The rtgenid2 desc. */
    private String rtgenid2Desc;

    /**
     * Full constructor.
     * 
     * @param interactionId
     *            fdb drug pair interaction id
     * 
     * @param rtgenid1
     *            routed generic 1 id
     * @param rtgenid2
     *            routed generic 2 id
     * @param rtgenid1Desc
     *            routed generic 1 description
     * @param rtgenid2Desc
     *            routed generic 2 description
     */
    public DrugPairsGridBean(long interactionId, long rtgenid1, long rtgenid2, String rtgenid1Desc, String rtgenid2Desc) {
        this.rtgenid1Desc = rtgenid1Desc;
        this.rtgenid2Desc = rtgenid2Desc;
        setKey(interactionId, rtgenid1, rtgenid2);
    }

    /**
     * Returns the key.
     * 
     * @return the key
     */
    public final String getKey() {
        return key;
    }

    /**
     * Components that make up the key to identify the drug pair.
     *
     * @param interactionId fdb drug pair interaction id
     * @param rtgenid1 routed generic 1 id
     * @param rtgenid2 routed generic 2 id
     */
    private void setKey(long interactionId, long rtgenid1, long rtgenid2) {
        this.key =
                String.valueOf(interactionId).concat("|").concat(String.valueOf(rtgenid1)).concat("|")
                        .concat(String.valueOf(rtgenid2));
    }

    /**
     * Description of the routed generic drug 1.
     * 
     * @return the rtgenid1Desc
     */
    public final String getRtgenid1Desc() {
        return rtgenid1Desc;
    }

    /**
     * Description of the routed generic drug 1.
     * 
     * @param rtgenid1Desc
     *            the rtgenid1Desc to set
     */
    public final void setRtgenid1Desc(String rtgenid1Desc) {
        this.rtgenid1Desc = rtgenid1Desc;
    }

    /**
     * Description of the routed generic drug 2.
     * 
     * @return the rtgenid2Desc
     */
    public final String getRtgenid2Desc() {
        return rtgenid2Desc;
    }

    /**
     * Description of the routed generic drug 2.
     * 
     * @param rtgenid2Desc
     *            the rtgenid2Desc to set
     */
    public final void setRtgenid2Desc(String rtgenid2Desc) {
        this.rtgenid2Desc = rtgenid2Desc;
    }

    
    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public final int compareTo(DrugPairsGridBean o) {
        return this.rtgenid1Desc.compareTo(o.rtgenid1Desc);
    }

}
