/**
 * 
 */
package gov.va.med.pharmacy.pecs.web.detail.drugpair;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Supports the sorting of the grid.
 * 
 * @author vhaisacarusr
 * 
 */
public class FdbDrugPairSortSupport {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(FdbDrugPairSortSupport.class);

    /** The pattern. */
    private static Pattern pattern = Pattern.compile("(\\[([\\\\/])@(\\[[^\\[\\]]+\\]]))?\\[(\\d*):(\\d+)\\]");

    /** The Constant THREE. */
    private static final int THREE = 3;

    /** The drug pairs grid beans. */
    private final List<DrugPairsGridBean> drugPairsGridBeans;

    /** The sort asc. */
    private boolean sortAsc = true;

    /** The sort column. */
    private String sortColumn;

    /**
     * Constructor.
     * 
     * @param drugPairsGridBeans
     *            the list to sort
     * @param json
     *            the sort information
     */
    public FdbDrugPairSortSupport(List<DrugPairsGridBean> drugPairsGridBeans, String json) {
        this.drugPairsGridBeans = drugPairsGridBeans;
        if ("/".equals(json)) {
            sortAsc = true;
            sortColumn = "rtgenid1Desc";
        } else {
            Matcher matcher = pattern.matcher(json);
            if (matcher.find()) {
                if (matcher.group(1) != null) {
                    if ("/".equals(matcher.group(2))) {
                        sortAsc = true;
                    } else {
                        sortAsc = false;
                    }
                    sortColumn = matcher.group(THREE).substring(2, matcher.group(THREE).length() - THREE);
                    LOGGER.debug(sortColumn);
                }

            } else {
                LOGGER.error("Couldn't parse extra query information: " + json);
            }
        }

    }

    /**
     * Sorts the list.
     * 
     * @return the sorted list
     */
    public final List<DrugPairsGridBean> sort() {
        Collections.sort(drugPairsGridBeans, getDrugPairsComparator());
        return drugPairsGridBeans;

    }

    /**
     * Determines the sorting.
     * 
     * @return the appropriate comparator
     */
    private Comparator<DrugPairsGridBean> getDrugPairsComparator() {
        if ("rtgenid2Desc".equals(sortColumn)) {
            if (sortAsc) {
                return new DrugPairsRtgenId2Comparator();
            }
            return Collections.reverseOrder(new DrugPairsRtgenId2Comparator());

        } else {
            if (sortAsc) {
                return new DrugPairsRtgenId1Comparator();
            }
            return Collections.reverseOrder(new DrugPairsRtgenId1Comparator());

        }
    }

    /**
     * Class is used to compare routed generic description 1.
     * 
     * 
     */
    private class DrugPairsRtgenId1Comparator implements Comparator<DrugPairsGridBean> {

        /*
         * (non-Javadoc)
         * 
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        @Override
        public final int compare(DrugPairsGridBean gridBean1, DrugPairsGridBean gridBean2) {
            return gridBean1.getRtgenid1Desc().compareTo(gridBean2.getRtgenid1Desc());
        }
    }

    /**
     * Class is used to compare routed generic description 2.
     * 
     */
    private class DrugPairsRtgenId2Comparator implements Comparator<DrugPairsGridBean> {

        /**
         * (non-Javadoc).
         * 
         * @param gridBean1
         *            the grid bean1
         * @param gridBean2
         *            the grid bean2
         * @return the int
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        @Override
        public final int compare(DrugPairsGridBean gridBean1, DrugPairsGridBean gridBean2) {
            return gridBean1.getRtgenid2Desc().compareTo(gridBean2.getRtgenid2Desc());
        }

    }

}
