package gov.va.med.pharmacy.pecs.web.easy;

import gov.va.med.pharmacy.pecs.domain.impl.easy.DosingInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugCheckResult;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugDrugInteractionInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DuplicateTherapyInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.PecsDrugInfo;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.EasySearchSelectType;

import java.util.List;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.hibernate.validator.constraints.NotEmpty;

/**
 * The Class EasySearchModel.
 */
@JsonIgnoreProperties({ "dtCheckResult", "drCheckResult", "drugList", "drugsSearched", "duplicateTherapyCheckResult",
        "drugDrugCheckResults", "expandPM", "drugsDRSearched", "doseRangeCheckResults" })
public class EasySearchModel {

    /** The concept selection. */
    @NotEmpty
    private String conceptSelection;

    /** The concept selection list. */
    private List<EasySearchSelectType> conceptSelectionList;

    /** The ddi selected. */
    private boolean ddiSelected;

    /** The dt selected. */
    private boolean dtSelected;

    /** The ddi severity level. */
    private String ddiSeverityLevel;

    /** The ddidtcriteria. */
    private String ddidtcriteria;

    /** The drcriteria. */
    private String drcriteria;

    /** The prospective drug list. */
    private List<String> prospectiveDrugList;

    /** The drug list. */
    private List<PecsDrugInfo> drugList;

    /** The drugs searched. */
    private List<PecsDrugInfo> drugsSearched;

    /** The drug dr list. */
    private List<PecsDrugInfo> drugDRList;

    /** The drugs dr searched. */
    private List<PecsDrugInfo> drugsDRSearched;

    /** The prospective dr drug list. */
    private List<String> prospectiveDRDrugList;

    /** The drug dose type list. */
    private List<DoseType> drugDoseTypeList;

    /** The drug dose route list. */
    private List<DoseRoute> drugDoseRouteList;

    /** The drug dose unit list. */
    private List<DoseUnitType> drugDoseUnitList;

    /** The concept dr selection. */
    private String conceptDRSelection;

    /** The dt check result. */
    @JsonIgnore
    private DrugCheckResult<DuplicateTherapyInfo> dtCheckResult;

    /** The drug drug check results. */
    @JsonIgnore
    private DrugCheckResult<DrugDrugInteractionInfo> drugDrugCheckResults;

    /** The dr check result. */
    @JsonIgnore
    private DrugCheckResult<DosingInfo> drCheckResult;

    /** The drugs checked. */
    private String drugsChecked;

    /** The professional monograph results set. */
    private String professionalMonographResultsSet;

    /** The drug drug results set. */
    private String drugDrugResultsSet;

    /** The dr selected. */
    private boolean drSelected;

    /** The dr dose range selection. */
    private String drDoseRangeSelection;

    /** The dose type selection. */
    private String doseTypeSelection;

    /** The dose route selection. */
    private String doseRouteSelection;

    /** The demographic weight selection. */
    private String demographicWeightSelection;

    /** The demographic height selection. */
    private String demographicHeightSelection;

    /** The dose unit selection. */
    private String doseUnitSelection;

    /** The dose unit selected. */
    private DoseUnitType doseUnitSelected;

    /** The dose rate unit selection. */
    private String doseRateUnitSelection;

    /** The frequency. */
    private String frequency;

    /** The single dose. */
    private String singleDose;

    /** The dose range selected drug. */
    private String doseRangeSelectedDrug;

    /** The dose bsa. */
    private String doseBSA;

    /** The dose weight. */
    private String doseWeight;

    /** The dose age days. */
    private String doseAgeDays;

    /** The dose height. */
    private String doseHeight;

    /** The drug search dr results list. */
    private DrugCheckResult<DosingInfo> drugSearchDRResultsList;

    /** The drug identifier value. */
    private String drugIdentifierValue;

    /** The drug identifier type. */
    private String drugIdentifierType;

    /** The is custom. */
    private boolean isCustom;

    /**
     * Gets the drug search dr results list.
     * 
     * @return the drug search dr results list
     */
    public DrugCheckResult<DosingInfo> getDrugSearchDRResultsList() {
        return drugSearchDRResultsList;
    }

    /**
     * Sets the drug search dr results list.
     * 
     * @param drugSearchDRResultsList
     *            the new drug search dr results list
     */
    public void setDrugSearchDRResultsList(DrugCheckResult<DosingInfo> drugSearchDRResultsList) {
        this.drugSearchDRResultsList = drugSearchDRResultsList;
    }

    /**
     * Gets the dose bsa.
     * 
     * @return the dose bsa
     */
    public String getDoseBSA() {
        return doseBSA;
    }

    /**
     * Gets the dr dose range selection.
     * 
     * @return the dr dose range selection
     */
    public String getDrDoseRangeSelection() {
        return drDoseRangeSelection;
    }

    /**
     * Sets the dr dose range selection.
     * 
     * @param drDoseRangeSelection
     *            the new dr dose range selection
     */
    public void setDrDoseRangeSelection(String drDoseRangeSelection) {
        this.drDoseRangeSelection = drDoseRangeSelection;
    }

    /**
     * Sets the dose bsa.
     * 
     * @param doseBSA
     *            the new dose bsa
     */
    public void setDoseBSA(String doseBSA) {
        this.doseBSA = doseBSA;
    }

    /**
     * Gets the dose weight.
     * 
     * @return the dose weight
     */
    public String getDoseWeight() {
        return doseWeight;
    }

    /**
     * Sets the dose weight.
     * 
     * @param doseWeight
     *            the new dose weight
     */
    public void setDoseWeight(String doseWeight) {
        this.doseWeight = doseWeight;
    }

    /**
     * Gets the dose age days.
     * 
     * @return the dose age days
     */
    public String getDoseAgeDays() {
        return doseAgeDays;
    }

    /**
     * Sets the dose age days.
     * 
     * @param doseAgeDays
     *            the new dose age days
     */
    public void setDoseAgeDays(String doseAgeDays) {
        this.doseAgeDays = doseAgeDays;
    }

    /**
     * Gets the dose height.
     * 
     * @return the dose height
     */
    public String getDoseHeight() {
        return doseHeight;
    }

    /**
     * Sets the dose height.
     * 
     * @param doseHeight
     *            the new dose height
     */
    public void setDoseHeight(String doseHeight) {
        this.doseHeight = doseHeight;
    }

    /**
     * Gets the single dose.
     * 
     * @return the single dose
     */
    public String getSingleDose() {
        return singleDose;
    }

    /**
     * Sets the single dose.
     * 
     * @param singleDose
     *            the new single dose
     */
    public void setSingleDose(String singleDose) {
        this.singleDose = singleDose;
    }

    /**
     * Gets the drug dose type list.
     * 
     * @return the drug dose type list
     */
    public List<DoseType> getDrugDoseTypeList() {
        return drugDoseTypeList;
    }

    /**
     * Sets the drug dose type list.
     * 
     * @param drugDoseTypeList
     *            the new drug dose type list
     */
    public void setDrugDoseTypeList(List<DoseType> drugDoseTypeList) {
        this.drugDoseTypeList = drugDoseTypeList;
    }

    /**
     * Gets the concept dr selection.
     * 
     * @return the concept dr selection
     */
    public String getConceptDRSelection() {
        return conceptDRSelection;
    }

    /**
     * Sets the concept dr selection.
     * 
     * @param conceptDRSelection
     *            the new concept dr selection
     */
    public void setConceptDRSelection(String conceptDRSelection) {
        this.conceptDRSelection = conceptDRSelection;
    }

    /**
     * Gets the drug dose unit list.
     * 
     * @return the drug dose unit list
     */
    public List<DoseUnitType> getDrugDoseUnitList() {
        return drugDoseUnitList;
    }

    /**
     * Sets the drug dose unit list.
     * 
     * @param drugDoseUnitList
     *            the new drug dose unit list
     */
    public void setDrugDoseUnitList(List<DoseUnitType> drugDoseUnitList) {
        this.drugDoseUnitList = drugDoseUnitList;
    }

    /**
     * Sets the drug dose route list.
     * 
     * @param drugDoseRouteList
     *            the new drug dose route list
     */
    public void setDrugDoseRouteList(List<DoseRoute> drugDoseRouteList) {
        this.drugDoseRouteList = drugDoseRouteList;
    }

    /**
     * Gets the drug dose route list.
     * 
     * @return the drug dose route list
     */
    public List<DoseRoute> getDrugDoseRouteList() {
        return drugDoseRouteList;
    }

    /**
     * Gets the dose range selected drug.
     * 
     * @return the dose range selected drug
     */
    public String getDoseRangeSelectedDrug() {
        doseRangeSelectedDrug = "xxx";
        if (drugsSearched != null) {
            for (PecsDrugInfo check : drugsSearched) {
                doseRangeSelectedDrug = check.getUiLabel();
            }
        }
        return doseRangeSelectedDrug;
    }

    /**
     * Sets the dose range selected drug.
     * 
     * @param doseRangeSelectedDrug
     *            the new dose range selected drug
     */
    public void setDoseRangeSelectedDrug(String doseRangeSelectedDrug) {
        this.doseRangeSelectedDrug = doseRangeSelectedDrug;
    }

    /**
     * Gets the frequency.
     * 
     * @return the frequency
     */
    public String getFrequency() {
        return frequency;
    }

    /**
     * Sets the frequency.
     * 
     * @param frequency
     *            the new frequency
     */
    public void setFrequency(String frequency) {
        this.frequency = frequency;
    }

    /**
     * Gets the dose unit selection.
     * 
     * @return the dose unit selection
     */
    public String getDoseUnitSelection() {
        return doseUnitSelection;
    }

    /**
     * Sets the dose unit selection.
     * 
     * @param doseUnitSelection
     *            the new dose unit selection
     */
    public void setDoseUnitSelection(String doseUnitSelection) {
        this.doseUnitSelection = doseUnitSelection;
    }

    /**
     * Gets the dose type selection.
     * 
     * @return the dose type selection
     */
    public String getDoseTypeSelection() {
        return doseTypeSelection;
    }

    /**
     * Sets the dose type selection.
     * 
     * @param doseTypeSelection
     *            the new dose type selection
     */
    public void setDoseTypeSelection(String doseTypeSelection) {
        this.doseTypeSelection = doseTypeSelection;
    }

    /**
     * Gets the dose route selection.
     * 
     * @return the dose route selection
     */
    public String getDoseRouteSelection() {
        return doseRouteSelection;
    }

    /**
     * Sets the dose route selection.
     * 
     * @param doseRouteSelection
     *            the new dose route selection
     */
    public void setDoseRouteSelection(String doseRouteSelection) {
        this.doseRouteSelection = doseRouteSelection;
    }

    /**
     * Gets the demographic weight selection.
     * 
     * @return the demographic weight selection
     */
    public String getDemographicWeightSelection() {
        return demographicWeightSelection;
    }

    /**
     * Sets the demographic weight selection.
     * 
     * @param demographicWeightSelection
     *            the new demographic weight selection
     */
    public void setDemographicWeightSelection(String demographicWeightSelection) {
        this.demographicWeightSelection = demographicWeightSelection;
    }

    /**
     * Gets the demographic height selection.
     * 
     * @return the demographic height selection
     */
    public String getDemographicHeightSelection() {
        return demographicHeightSelection;
    }

    /**
     * Sets the demographic height selection.
     * 
     * @param demographicHeightSelection
     *            the new demographic height selection
     */
    public void setDemographicHeightSelection(String demographicHeightSelection) {
        this.demographicHeightSelection = demographicHeightSelection;
    }

    /**
     * Gets the dose rate unit selection.
     * 
     * @return the dose rate unit selection
     */
    public String getDoseRateUnitSelection() {
        return doseRateUnitSelection;
    }

    /**
     * Sets the dose rate unit selection.
     * 
     * @param doseRateUnitSelection
     *            the new dose rate unit selection
     */
    public void setDoseRateUnitSelection(String doseRateUnitSelection) {
        this.doseRateUnitSelection = doseRateUnitSelection;
    }

    /**
     * Gets the dose unit selected.
     * 
     * @return the dose unit selected
     */
    public DoseUnitType getDoseUnitSelected() {
        return doseUnitSelected;
    }

    /**
     * Sets the dose unit selected.
     * 
     * @param doseUnitSelected
     *            the new dose unit selected
     */
    public void setDoseUnitSelected(DoseUnitType doseUnitSelected) {
        this.doseUnitSelected = doseUnitSelected;
    }

    /**
     * Gets the dt check result.
     * 
     * @return the dt check result
     */
    public DrugCheckResult<DuplicateTherapyInfo> getDtCheckResult() {
        return dtCheckResult;
    }

    /**
     * Sets the dt check result.
     * 
     * @param dtCheckResult
     *            the new dt check result
     */
    public void setDtCheckResult(DrugCheckResult<DuplicateTherapyInfo> dtCheckResult) {
        this.dtCheckResult = dtCheckResult;
    }

    /**
     * Gets the drugs checked.
     * 
     * @return the drugs checked
     */
    public String getDrugsChecked() {
        return drugsChecked;
    }

    /**
     * Sets the drugs checked.
     * 
     * @param drugsChecked
     *            the new drugs checked
     */
    public void setDrugsChecked(String drugsChecked) {
        this.drugsChecked = drugsChecked;
    }

    /**
     * Gets the professional monograph results set.
     * 
     * @return the professional monograph results set
     */
    public String getProfessionalMonographResultsSet() {
        return professionalMonographResultsSet;
    }

    /**
     * Sets the professional monograph results set.
     * 
     * @param professionalMonographResultsSet
     *            the new professional monograph results set
     */
    public void setProfessionalMonographResultsSet(String professionalMonographResultsSet) {
        this.professionalMonographResultsSet = professionalMonographResultsSet;
    }

    /**
     * Gets the drug drug results set.
     * 
     * @return the drug drug results set
     */
    public String getDrugDrugResultsSet() {
        return drugDrugResultsSet;
    }

    /**
     * Sets the drug drug results set.
     * 
     * @param drugDrugResultsSet
     *            the new drug drug results set
     */
    public void setDrugDrugResultsSet(String drugDrugResultsSet) {
        this.drugDrugResultsSet = drugDrugResultsSet;
    }

    /**
     * Checks if is dr selected.
     * 
     * @return true, if is dr selected
     */
    public boolean isDrSelected() {
        return drSelected;
    }

    /**
     * Sets the dr selected.
     * 
     * @param drSelected
     *            the new dr selected
     */
    public void setDrSelected(boolean drSelected) {
        this.drSelected = drSelected;
    }

    /**
     * Gets the dose range check results.
     * 
     * @return the dose range check results
     */
    public DrugCheckResult<DosingInfo> getDoseRangeCheckResults() {
        return drCheckResult;
    }

    /**
     * Gets the duplicate therapy check result.
     * 
     * @return the duplicate therapy check result
     */
    public DrugCheckResult<DuplicateTherapyInfo> getDuplicateTherapyCheckResult() {
        return dtCheckResult;
    }

    /**
     * Gets the drug drug check results.
     * 
     * @return the drug drug check results
     */
    public DrugCheckResult<DrugDrugInteractionInfo> getDrugDrugCheckResults() {
        return drugDrugCheckResults;
    }

    /**
     * Sets the drug drug check results.
     * 
     * @param drugDrugCheckResults
     *            the new drug drug check results
     */
    public void setDrugDrugCheckResults(DrugCheckResult<DrugDrugInteractionInfo> drugDrugCheckResults) {
        this.drugDrugCheckResults = drugDrugCheckResults;
    }

    /**
     * Format parser.
     * 
     * @param valueToChange
     *            the value to change
     * @param template
     *            the template
     * @param leadingParse
     *            the leading parse
     * @return the string
     */
    public String formatParser(String valueToChange, String template, String leadingParse) {
        String formattedOutput = "";

        return formattedOutput;
    }

    /**
     * Sets the dose range check results.
     * 
     * @param result
     *            the new dose range check results
     */
    public void setDoseRangeCheckResults(DrugCheckResult<DosingInfo> result) {
        drCheckResult = result;
    }

    /**
     * Sets the duplicate therapy check result.
     * 
     * @param result
     *            the new duplicate therapy check result
     */
    public void setDuplicateTherapyCheckResult(DrugCheckResult<DuplicateTherapyInfo> result) {
        dtCheckResult = result;
    }

    /**
     * Gets the concept selection.
     * 
     * @return the concept selection
     */
    public String getConceptSelection() {
        return conceptSelection;
    }

    /**
     * Sets the concept selection.
     * 
     * @param conceptSelection
     *            the new concept selection
     */
    public void setConceptSelection(String conceptSelection) {
        this.conceptSelection = conceptSelection;
    }

    /**
     * Gets the concept selection list.
     * 
     * @return the concept selection list
     */
    public List<EasySearchSelectType> getConceptSelectionList() {
        return conceptSelectionList;
    }

    /**
     * Sets the concept selection list.
     * 
     * @param conceptSelectionList
     *            the new concept selection list
     */
    public void setConceptSelectionList(List<EasySearchSelectType> conceptSelectionList) {
        this.conceptSelectionList = conceptSelectionList;
    }

    /**
     * Checks if is ddi selected.
     * 
     * @return true, if is ddi selected
     */
    public boolean isDdiSelected() {
        return ddiSelected;
    }

    /**
     * Sets the ddi selected.
     * 
     * @param ddiSelected
     *            the new ddi selected
     */
    public void setDdiSelected(boolean ddiSelected) {
        this.ddiSelected = ddiSelected;
    }

    /**
     * Checks if is dt selected.
     * 
     * @return true, if is dt selected
     */
    public boolean isDtSelected() {
        return dtSelected;
    }

    /**
     * Sets the dt selected.
     * 
     * @param dtSelected
     *            the new dt selected
     */
    public void setDtSelected(boolean dtSelected) {
        this.dtSelected = dtSelected;
    }

    /**
     * Gets the ddi severity level.
     * 
     * @return the ddi severity level
     */
    public String getDdiSeverityLevel() {
        return ddiSeverityLevel;
    }

    /**
     * Sets the ddi severity level.
     * 
     * @param ddiSeverityLevel
     *            the new ddi severity level
     */
    public void setDdiSeverityLevel(String ddiSeverityLevel) {
        this.ddiSeverityLevel = ddiSeverityLevel;
    }

    /**
     * Gets the prospective drug list.
     * 
     * @return the prospective drug list
     */
    public List<String> getProspectiveDrugList() {
        return prospectiveDrugList;
    }

    /**
     * Sets the prospective drug list.
     * 
     * @param prospectiveDrugList
     *            the new prospective drug list
     */
    public void setProspectiveDrugList(List<String> prospectiveDrugList) {
        this.prospectiveDrugList = prospectiveDrugList;
    }

    /**
     * Gets the drug list.
     * 
     * @return the drug list
     */
    public List<PecsDrugInfo> getDrugList() {
        return drugList;
    }

    /**
     * Sets the drug list.
     * 
     * @param drugList
     *            the new drug list
     */
    public void setDrugList(List<PecsDrugInfo> drugList) {
        this.drugList = drugList;
    }

    /**
     * Gets the drugs searched.
     * 
     * @return the drugs searched
     */
    public List<PecsDrugInfo> getDrugsSearched() {
        return drugsSearched;
    }

    /**
     * Sets the drugs searched.
     * 
     * @param drugsSearched
     *            the new drugs searched
     */
    public void setDrugsSearched(List<PecsDrugInfo> drugsSearched) {
        this.drugsSearched = drugsSearched;
    }

    /** The expand pm. */
    @JsonIgnore
    int expandPM = 0;

    /**
     * Gets the expand pm.
     * 
     * @return the expand pm
     */
    public int getExpandPM() {
        return expandPM;
    }

    /**
     * Sets the expand pm.
     * 
     * @param expandPM
     *            the new expand pm
     */
    public void setExpandPM(int expandPM) {
        this.expandPM = expandPM;
    }

    /**
     * Change expand pm.
     * 
     * @return the int
     */
    public int changeExpandPM() {
        return expandPM;
    }

    /**
     * Gets the ddidtcriteria.
     * 
     * @return the ddidtcriteria
     */
    public String getDdidtcriteria() {
        return ddidtcriteria;
    }

    /**
     * Sets the ddidtcriteria.
     * 
     * @param ddidtcriteria
     *            the new ddidtcriteria
     */
    public void setDdidtcriteria(String ddidtcriteria) {
        this.ddidtcriteria = ddidtcriteria;
    }

    /**
     * Gets the drcriteria.
     * 
     * @return the drcriteria
     */
    public String getDrcriteria() {
        return drcriteria;
    }

    /**
     * Sets the drcriteria.
     * 
     * @param drcriteria
     *            the new drcriteria
     */
    public void setDrcriteria(String drcriteria) {
        this.drcriteria = drcriteria;
    }

    /**
     * Gets the drug dr list.
     * 
     * @return the drug dr list
     */
    public List<PecsDrugInfo> getDrugDRList() {
        return drugDRList;
    }

    /**
     * Sets the drug dr list.
     * 
     * @param drugDRList
     *            the new drug dr list
     */
    public void setDrugDRList(List<PecsDrugInfo> drugDRList) {
        this.drugDRList = drugDRList;
    }

    /**
     * Gets the drugs dr searched.
     * 
     * @return the drugs dr searched
     */
    public List<PecsDrugInfo> getDrugsDRSearched() {
        return drugsDRSearched;
    }

    /**
     * Sets the drugs dr searched.
     * 
     * @param drugsDRSearched
     *            the new drugs dr searched
     */
    public void setDrugsDRSearched(List<PecsDrugInfo> drugsDRSearched) {
        this.drugsDRSearched = drugsDRSearched;
    }

    /**
     * Gets the prospective dr drug list.
     * 
     * @return the prospective dr drug list
     */
    public List<String> getProspectiveDRDrugList() {
        return prospectiveDRDrugList;
    }

    /**
     * Sets the prospective dr drug list.
     * 
     * @param prospectiveDRDrugList
     *            the new prospective dr drug list
     */
    public void setProspectiveDRDrugList(List<String> prospectiveDRDrugList) {
        this.prospectiveDRDrugList = prospectiveDRDrugList;
    }

    /**
     * Gets the dr check result.
     * 
     * @return the dr check result
     */
    public DrugCheckResult<DosingInfo> getDrCheckResult() {
        return drCheckResult;
    }

    /**
     * Sets the dr check result.
     * 
     * @param drCheckResult
     *            the new dr check result
     */
    public void setDrCheckResult(DrugCheckResult<DosingInfo> drCheckResult) {
        this.drCheckResult = drCheckResult;
    }

    /**
     * Sets the drug identifier value.
     * 
     * @param drugIdentifierValue
     *            the new drug identifier value
     */
    public void setDrugIdentifierValue(String drugIdentifierValue) {
        this.drugIdentifierValue = drugIdentifierValue;
    }

    /**
     * Gets the drug identifier value.
     * 
     * @return the drug identifier value
     */
    public String getDrugIdentifierValue() {
        return drugIdentifierValue;
    }

    /**
     * Sets the drug identifier type.
     * 
     * @param drugIdentifierType
     *            the new drug identifier type
     */
    public void setDrugIdentifierType(String drugIdentifierType) {
        this.drugIdentifierType = drugIdentifierType;
    }

    /**
     * Gets the drug identifier type.
     * 
     * @return the drug identifier type
     */
    public String getDrugIdentifierType() {
        return drugIdentifierType;
    }

    /**
     * Sets the custom.
     * 
     * @param isCustom
     *            the new custom
     */
    public void setCustom(boolean isCustom) {
        this.isCustom = isCustom;
    }

    /**
     * Checks if is custom.
     * 
     * @return true, if is custom
     */
    public boolean isCustom() {
        return isCustom;
    }

}
