/**
 * 
 */
package gov.va.med.pharmacy.pecs.web.easy;

import gov.va.med.pharmacy.pecs.domain.impl.easy.DoseCheck;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DosingInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugCheckResult;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugDrugInteractionInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifierType;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DuplicateTherapyInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.PecsDrugInfo;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.service.easy.EasySearchService;
import gov.va.med.pharmacy.pecs.web.view.ReportDrugInteractionsXLSXViewer;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

/**
 * new easy search controller.
 * 
 * @author VHAISACARUSR
 * 
 */
@Controller
@RequestMapping(value = "/search")
public final class EasySearchSelectionController {
    private static final Log LOGGER = LogFactory.getLog(EasySearchController.class);
    private static final String TILES_VIEW = "pecs.easysearch";

    private static final String DDI_FOR_SINGLE_DRUG_VIEW = "pecs.easysearch.ddiforsingledrug";
    private static final String DOSE_RANGE_VIEW = "pecs.easysearch.doserange";
    private static final String DOSE_RANGE_RESULTS_VIEW = "pecs.easysearch.doserange.results";
    private static final String MULTIPLE_CONCEPT_VIEW = "pecs.easysearch.multipleconcept";
    private static final String MULTIPLE_CONCEPT_RESULTS_VIEW = "pecs.easysearch.multipleconcept.results";

    private static final int DAYS_IN_YEAR = 365;
    private static final double METRIC = 2.20462262185;
    private static final String ALL_SEVERITIES = "Display_Severity_Levels_all";

    @Resource
    private TerminologyService terminologyService;
    @Resource
    private EasySearchService easySearchService;

    /**
     * Handles the initial GET to display the drop down menu.
     * 
     * @param modelAndView
     *            the model to display the page
     * @return the Model and View
     */
    @RequestMapping(method = RequestMethod.GET)
    public ModelAndView get(ModelAndView modelAndView) {
        modelAndView = new ModelAndView(TILES_VIEW);
        modelAndView.addObject("searchTypes", getTerminologyService().getEasySearchSelectTypes());

        return modelAndView;
    }

    /**
     * 
     * Gets the drug drug query section of the page.
     * 
     * @param dispensableGenericForm
     *            the form to be processed
     * @param modelAndView
     *            the model to display the page
     * @return the drug query section of the page
     */
    @RequestMapping(value = "/ddiforsingledrug", method = RequestMethod.GET)
    public ModelAndView getDDIForSingleDrug(ModelAndView modelAndView, DispensableGenericForm dispensableGenericForm) {
        modelAndView = new ModelAndView(DDI_FOR_SINGLE_DRUG_VIEW);
        return modelAndView;
    }

    /**
     * 
     * Submits the DDI for a single dispensable generic form.
     * 
     * @param modelAndView
     *            the model to display the page
     * @param dispensableGenericForm
     *            the form to be processed
     * @param bindingResult
     *            holds the errors if there are any
     * 
     * @return the drug query section of the page
     */
    @RequestMapping(value = "/dispensable/results", method = RequestMethod.POST)
    public ModelAndView postDispensable(ModelAndView modelAndView,
            @ModelAttribute DispensableGenericForm dispensableGenericForm, BindingResult bindingResult) {

        DrugIdentifier id = DrugIdentifierKeyHelper.createDrugIdentifier(dispensableGenericForm.getDrug());

        Map<String, Object> model = new HashMap<String, Object>();
        model.put(ReportDrugInteractionsXLSXViewer.METADATA_KEY, Concept.DRUG_PAIR);
        try {
            List<DrugPairDTO> list = easySearchService.generateReports(id, dispensableGenericForm.getSeverityLevel());
            model.put(ReportDrugInteractionsXLSXViewer.DATA_KEY, list);
            model.put(ReportDrugInteractionsXLSXViewer.EXCEL_FILE_NAME_KEY, Concept.DRUG_PAIR + "_INTERACTION_REPORT");
        } catch (Exception ex) {
            LOGGER.debug("exception thrown during excel creation" + ex.getMessage());
        }

        return new ModelAndView(ReportDrugInteractionsXLSXViewer.REPORT_DRUG_INTER_VIEW, model);

    }

    /**
     * 
     * Gets the drug drug query section of the page.
     * 
     * @param criteria
     *            the search string
     * @return the drug query section of the page
     */
    @RequestMapping(value = "/dispensable/critera/{criteria}", method = RequestMethod.POST)
    @ResponseBody
    public Collection<PecsDrugInfo> postDispensableResults(@PathVariable(value = "criteria") String criteria) {
        Collection<PecsDrugInfo> drugs = easySearchService.searchDrugs(criteria);
        return drugs;
    }

    /**
     * 
     * Gets the dose range query section of the page.
     * 
     * @param doseRangeForm
     *            the form to be processed
     * @param modelAndView
     *            the model to display the page
     * @return the drug query section of the page
     */
    @RequestMapping(value = "/doserange", method = RequestMethod.GET)
    public ModelAndView getDoseRange(ModelAndView modelAndView, DoseRangeForm doseRangeForm) {
        modelAndView = new ModelAndView(DOSE_RANGE_VIEW);

        return modelAndView;
    }

    /**
     * 
     * Gets the dose range query section of the page.
     * 
     * @param modelAndView
     *            the model to display the page
     * @param doseRangeForm
     *            the form to be processed
     * @param results
     *            the errors if there are any
     * @return the drug query section of the page
     * 
     */
    @RequestMapping(value = "/doserange", method = RequestMethod.POST)
    public ModelAndView postDoseRange(ModelAndView modelAndView, @ModelAttribute DoseRangeForm doseRangeForm,
            BindingResult results) {

        ModelAndView resultsView = new ModelAndView(DOSE_RANGE_RESULTS_VIEW);
        DoseRangeResults resultsModel = new DoseRangeResults();
        try {
            drSearch(doseRangeForm, resultsModel);
        } catch (Exception e) {
            String message = "Failed Dose Range Search";
            LOGGER.error(message, e);
            throw new CustomizationServiceException(message, e);

        }
        resultsView.addObject("doseRangeFormResults", resultsModel);
        resultsView.addObject("jsonBackQuery", null);

        return resultsView;
    }

    /**
     * 
     * Gets the Drug Drug Interaction with Duplicate Therapy or Professional Monograph section of the page.
     * 
     * @param multipleConceptForm
     *            the form to be processed
     * @param modelAndView
     *            the model to display the page
     * @return the drug query section of the page
     */
    @RequestMapping(value = "/multipleconcept", method = RequestMethod.GET)
    public ModelAndView getMultipleConcept(ModelAndView modelAndView, MultipleConceptForm multipleConceptForm) {
        modelAndView = new ModelAndView(MULTIPLE_CONCEPT_VIEW);

        return modelAndView;
    }

    /**
     * 
     * Gets the Drug Drug Interaction with Duplicate Therapy or Professional Monograph section of the page.
     * 
     * @param modelAndView
     *            the model to display the page
     * @param multipleConceptForm
     *            the form to be processed
     * @param results
     *            the errors if there are any
     * @return the drug query section of the page
     * 
     */
    @RequestMapping(value = "/multipleconcept", method = RequestMethod.POST)
    public ModelAndView postMultipleConcept(ModelAndView modelAndView, @ModelAttribute MultipleConceptForm multipleConceptForm,
            BindingResult results) {

        ModelAndView resultsView = new ModelAndView(MULTIPLE_CONCEPT_RESULTS_VIEW);
        MultipleConceptResults multipleConceptResults = new MultipleConceptResults();
        try {
            ddidtSearch(multipleConceptForm, multipleConceptResults);
        } catch (Exception e) {
            e.printStackTrace();
        }

        resultsView.addObject("multipleConceptResults", multipleConceptResults);
        resultsView.addObject("jsonBackQuery", null);
        return resultsView;

    }

    /**
     * @param multipleConceptForm
     *            the form to be processed
     * 
     * @param multipleConceptResults
     *            the model populated with results
     * @throws Exception
     *             if an error occurs
     */
    private void ddidtSearch(MultipleConceptForm multipleConceptForm, MultipleConceptResults multipleConceptResults)
            throws Exception {
        Collection<DrugIdentifier> ids = buildRoutedDrugList(multipleConceptForm.getProspectiveDrugList());
        multipleConceptResults.setDrugsSearched(buildDrugsSearched(ids));
        if (multipleConceptForm.isDdiSelected()) {
            searchForDrugDrugInteraction(ids, multipleConceptForm, multipleConceptResults);
            multipleConceptResults.setDdiSelected(multipleConceptForm.isDdiSelected());
        }
        if (multipleConceptForm.isDtSelected()) {
            searchForDuplicateTherapy(ids, multipleConceptResults);
            multipleConceptResults.setDtSelected(multipleConceptForm.isDtSelected());
        }
    }

    /**
     * @param ids
     *            a list of drug identifiers
     * @param multipleConceptForm
     *            the form to be processed
     * @param multipleConceptResults
     *            the model populated with results
     * @throws Exception
     *             if an error occurs
     */
    private void searchForDrugDrugInteraction(Collection<DrugIdentifier> ids, MultipleConceptForm multipleConceptForm,
            MultipleConceptResults multipleConceptResults) throws Exception {
        DrugCheckResult<DrugDrugInteractionInfo> result = getEasySearchService().checkDrugDrugInteraction(ids);
        if (!ALL_SEVERITIES.equals(multipleConceptForm.getDdiSeverityLevel())) {
            List<SeverityLevel> vistaSeverities = getEasySearchService().getVistaSeverityLevels();
            for (Iterator<DrugDrugInteractionInfo> iter = result.getChecks().iterator(); iter.hasNext();) {
                if (!vistaSeverities.contains(iter.next().getSeverity())) {
                    iter.remove();
                }
            }
        }
        multipleConceptResults.setDrugDrugCheckResults(result);

    }

    /**
     * @param ids
     *            a list of drug identifiers
     * @param multipleConceptResults
     *            the model populated with results
     * @throws Exception
     *             if an error occurs
     */
    private void searchForDuplicateTherapy(Collection<DrugIdentifier> ids, MultipleConceptResults multipleConceptResults)
            throws Exception {
        DrugCheckResult<DuplicateTherapyInfo> result = getEasySearchService().checkDuplicateTherapy(ids);
        multipleConceptResults.setDuplicateTherapyCheckResult(result);

    }

    /**
     * Dose range search.
     * 
     * @param form
     *            the form
     * @param doseRangeResults
     *            the dose range results
     * @throws Exception
     *             the exception
     */
    private void drSearch(DoseRangeForm form, DoseRangeResults doseRangeResults) throws Exception {
        DoseCheck doseCheck = new DoseCheck();

        doseRangeResults.setDoseRangeForm(form);
        doseCheck.setDrugId(new DrugIdentifier(form.getDrugIdentifierValue(), DrugIdentifierType.fromString(form
                .getDrugIdentifierType())));
        doseCheck.setSingleDoseAmount(Double.valueOf(form.getSingleDose()));
        doseCheck.setDoseUnit(form.getDoseUnitSelection());
        doseCheck.setDoseRateUnit(form.getDoseRateUnitSelection());
        doseCheck.setDoseType(form.getDoseTypeSelection());
        doseCheck.setDoseRoute(form.getDoseRouteSelection());
        doseCheck.setDuration(1);
        doseCheck.setDurationRate("DAY");
        doseCheck.setFrequency(form.getFrequency());
        long ageInYears = Long.parseLong(form.getDoseAgeDays());

        doseCheck.setAgeInDays(ageInYears * DAYS_IN_YEAR);
        double weight = Double.parseDouble(form.getDoseWeight());
        form.getDemographicWeightSelection();
        if ("LB".equalsIgnoreCase(form.getDemographicWeightSelection())) {
            weight = weight / METRIC;
        }
        doseCheck.setWeightInKg(weight);
        doseCheck.setBsaInSqM(Double.parseDouble(form.getDoseBSA()));

        DrugCheckResult<DosingInfo> checkDosing = getEasySearchService().checkDosing(doseCheck);
        if (!checkDosing.getChecks().isEmpty()) {
            doseRangeResults.setCustom(checkDosing.getChecks().iterator().next().isCustom());
        }
        doseRangeResults.setDrCheckResult(checkDosing);

        Collection<DrugIdentifier> ids = new ArrayList<DrugIdentifier>();
        ids.add(doseCheck.getDrugId());
        form.setDrugsDRSearched(buildDrugsSearched(ids));
        doseRangeResults.setDoseRangeSelection(form.getDrugsDRSearched().get(0).getUiLabel());
        form.setDoseUnitSelected(getTerminologyService().getDoseUnitTypeByCode(form.getDoseUnitSelection()));
    }

    /**
     * @param drugs
     *            a list of drug identifiers
     * @return a list of drug information
     */
    private List<PecsDrugInfo> buildDrugsSearched(Collection<DrugIdentifier> drugs) {
        List<PecsDrugInfo> drugsSearched = new ArrayList<PecsDrugInfo>();
        for (DrugIdentifier id : drugs) {
            try {
                drugsSearched.add(getEasySearchService().getDrug(id));
            } catch (Exception e) {
                LOGGER.info("Received exception in EasySearchController while obtaining drugs searched: ", e);
            }
        }
        return drugsSearched;
    }

    /**
     * @param drugIdentifiers
     *            a list of drug id's as strings
     * @return a collection of drug identifiers
     */
    private Collection<DrugIdentifier> buildRoutedDrugList(List<String> drugIdentifiers) {
        return buildDrugList(drugIdentifiers);
    }

    /**
     * @param ids
     *            a list of drug id's as strings
     * 
     * @return a collection of drug identifiers
     */
    private Collection<DrugIdentifier> buildDrugList(List<String> ids) {
        Collection<DrugIdentifier> idList = new ArrayList<DrugIdentifier>();
        if (ids == null) {
            return idList;
        }
        for (String id : ids) {
            idList.add(DrugIdentifierKeyHelper.createDrugIdentifier(id));
        }
        return idList;
    }

    /**
     * Gets the terminologyService.
     * 
     * @return the terminologyService
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminologyService.
     * 
     * @param terminologyService
     *            the terminologyService to set
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    /**
     * Gets the easySearchService.
     * 
     * @return the easySearchService
     */
    public EasySearchService getEasySearchService() {
        return easySearchService;
    }

    /**
     * Sets the easySearchService.
     * 
     * @param easySearchService
     *            the easySearchService to set
     */
    public void setEasySearchService(EasySearchService easySearchService) {
        this.easySearchService = easySearchService;
    }

}
