package gov.va.med.pharmacy.pecs.web.interceptor;

import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.util.InformationMessageSupport;
import gov.va.med.pharmacy.pecs.web.util.WarningMessageSupport;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

/**
 * The Class UserMessageInterceptor.
 */
public class UserMessageInterceptor extends HandlerInterceptorAdapter {

    /** The logger. */
    private static Log logger = LogFactory.getLog(UserMessageInterceptor.class);

    /**
     * Instantiates a new user message interceptor.
     */
    public UserMessageInterceptor() {
        super();
        if (logger.is   ugEnabled()) {
            logger.debug("InformationMessageInterceptor initialized");
        }
    }

    /**
     * If the Object is an instance of WarningMessageSupport or InformationMessageSupport, the appropriate messages are
     * retrieved from the Object and attached to the ModelAndView as "userMessages".
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @param object
     *            the object
     * @param modelAndView
     *            the model and view
     * @throws Exception
     *             the exception
     */
    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object object, ModelAndView modelAndView)
            throws Exception {

        if (object instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) object;

            if (modelAndView != null) {
                if (handlerMethod.getBean() instanceof WarningMessageSupport) {
                    if (modelAndView.hasView() && modelAndView.getViewName() != null
                            && !modelAndView.getViewName().startsWith("redirect:")) {
                        WarningMessageSupport wms = (WarningMessageSupport) handlerMethod.getBean();
                        modelAndView.addObject(AbstractController.WARNING_MESSAGE, wms.getWarningMessages());
                        wms.clearWarningMessages();
                    }
                }
                if (handlerMethod.getBean() instanceof InformationMessageSupport) {
                    if (modelAndView.hasView() && modelAndView.getViewName() != null
                            && !modelAndView.getViewName().startsWith("redirect:")) {
                        InformationMessageSupport ims = (InformationMessageSupport) handlerMethod.getBean();
                        modelAndView.addObject(AbstractController.INFO_MESSAGE, ims.getInformationMessages());
                        ims.clearInformationMessages();
                    }
                }
            }
        }
    }
}
