package gov.va.med.pharmacy.pecs.web.lookup.drugpair;

import org.hibernate.validator.constraints.ScriptAssert;

/**
 * The Class DrugPairLookupCriteria.
 */
@ScriptAssert(lang = "javascript", script = "!((_this.drugA == null || _this.drugA.isEmpty()) "
        + "&& (_this.drugB == null || _this.drugB.isEmpty()) "
        + "&& (_this.interaction == null || _this.interaction.isEmpty()) "
        + "&& (_this.severityLevel == null || _this.severityLevel.isEmpty()))")
public class DrugPairLookupCriteria {

    /** The drug a. */
    private String drugA;

    /** The drug b. */
    private String drugB;

    /** The interaction. */
    private String interaction;

    /** The severity level. */
    private String severityLevel;

    /**
     * Allows for the retrieval of the entered value.
     * 
     * @return the drug a
     */
    public String getDrugA() {
        return drugA;
    }

    /**
     * Allows for a user to entered either a Routed Generic Identifier or Routed Generic Description. If all numeric values are
     * entered, the value is treated as a Routed Generic Identifier. If not all numeric, the value is treated as a Routed
     * Generic Description.
     * 
     * @param drugA
     *            the new drug a
     */
    public void setDrugA(String drugA) {
        this.drugA = drugA;
    }

    /**
     * Allows for the retrieval of the entered value.
     * 
     * @return the drug b
     */
    public String getDrugB() {
        return drugB;
    }

    /**
     * Allows for a user to entered either a Routed Generic Identifier or Routed Generic Description. If all numeric values are
     * entered, the value is treated as a Routed Generic Identifier. If not all numeric, the value is treated as a Routed
     * Generic Description.
     * 
     * @param drugB
     *            the new drug b
     */
    public void setDrugB(String drugB) {
        this.drugB = drugB;
    }

    /**
     * Allows for the retrieval of the entered value.
     * 
     * @return the interaction
     */
    public String getInteraction() {
        return interaction;
    }

    /**
     * Allows for a user to entered either a Drug Drug Interaction Identifier or a Drug Drug Interaction Description. If all
     * numeric values are entered, the value is treated as a Drug Drug Interaction Identifier. If not all numeric, the value is
     * treated as a Drug Drug Interaction Description.
     * 
     * @param interaction
     *            the new interaction
     */
    public void setInteraction(String interaction) {
        this.interaction = interaction;
    }

    /**
     * Allows for the retrieval of the entered Severity Level Code.
     * 
     * @return the severity level
     */
    public String getSeverityLevel() {
        return severityLevel;
    }

    /**
     * Allows for a user to enter a Severity Level Code.
     * 
     * @param severityLevel
     *            the new severity level
     */
    public void setSeverityLevel(String severityLevel) {
        this.severityLevel = severityLevel;
    }

}
