package gov.va.med.pharmacy.pecs.web.lookup.drugpair;

import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.web.RecordType;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * The Class FieldMetadataController.
 */
@Controller
@RequestMapping(value = "fieldMetadata")
public class FieldMetadataController implements InitializingBean {

    /** The Constant logger. */
    private static final Log logger = LogFactory.getLog(FieldMetadataController.class);

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /**
     * Field metadata to json.
     * 
     * @param recordType
     *            the record type
     * @param conceptName
     *            the concept name
     * @return the list
     */
    @RequestMapping(value = "{recordType}/{conceptName}")
    @ResponseBody
    public List<JsonConceptField> fieldMetadataToJson(@PathVariable("recordType") String recordType,
            @PathVariable("conceptName") String conceptName) {
        ConceptMetadata cm = metadataService.getMetadata(Concept.fromString(conceptName).getConceptClass());
        return createJsonConceptFields(cm.getQueryFields((RecordType.VA.equals(RecordType.fromString(recordType)) ? true
                : false)));
    }

    /**
     * Creates the json concept fields.
     * 
     * @param fields
     *            the fields
     * @return the list
     */
    private List<JsonConceptField> createJsonConceptFields(List<ConceptField> fields) {
        List<JsonConceptField> json = new ArrayList<JsonConceptField>();
        for (ConceptField field : fields) {
            json.add(new JsonConceptField(field));
        }
        return json;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (metadataService == null) {
            throw new Exception("MetadatService has not been injected in to DataGridResultsController.");
        }
    }
}
