package gov.va.med.pharmacy.pecs.web.lookup.drugpair;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;

/**
 * The Class JsonConceptField.
 */
public class JsonConceptField {

    /** The required. */
    private boolean required;

    /** The display name. */
    private String displayName;

    /** The name. */
    private String name;

    /** The internal. */
    private boolean internal;

    /** The include in reports. */
    private boolean includeInReports;

    /** The include in query results. */
    private boolean includeInQueryResults;

    /** The include in detail display. */
    private boolean includeInDetailDisplay;

    /** The display order. */
    private int displayOrder;

    /** The property name. */
    private String propertyName;

    /** The editable. */
    private boolean editable;

    /** The display lines. */
    private int displayLines;

    /** The data type. */
    private String dataType;

    /**
     * Instantiates a new json concept field.
     */
    public JsonConceptField() {

    }

    /**
     * Instantiates a new json concept field.
     * 
     * @param field
     *            the field
     */
    public JsonConceptField(ConceptField field) {
        this.required = field.isRequired();
        this.displayName = field.getDisplayName();
        this.name = field.getName();
        this.internal = field.isInternal();
        this.includeInReports = field.includeInReports();
        this.includeInQueryResults = field.includeInQueryResults();
        this.includeInDetailDisplay = field.includeInDetailDisplay();
        this.displayOrder = field.getDisplayOrder();
        this.propertyName = field.getPropertyName();
        this.editable = field.isEditable();
        this.displayLines = field.getDisplayLines();
        this.dataType = field.getType().getSimpleName();
    }

    /**
     * Checks if is required.
     * 
     * @return true, if is required
     */
    public boolean isRequired() {
        return required;
    }

    /**
     * Gets the display name.
     * 
     * @return the display name
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * Gets the name.
     * 
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * Checks if is internal.
     * 
     * @return true, if is internal
     */
    public boolean isInternal() {
        return internal;
    }

    /**
     * Checks if is include in reports.
     * 
     * @return true, if is include in reports
     */
    public boolean isIncludeInReports() {
        return includeInReports;
    }

    /**
     * Checks if is include in query results.
     * 
     * @return true, if is include in query results
     */
    public boolean isIncludeInQueryResults() {
        return includeInQueryResults;
    }

    /**
     * Checks if is include in detail display.
     * 
     * @return true, if is include in detail display
     */
    public boolean isIncludeInDetailDisplay() {
        return includeInDetailDisplay;
    }

    /**
     * Gets the display order.
     * 
     * @return the display order
     */
    public int getDisplayOrder() {
        return displayOrder;
    }

    /**
     * Gets the property name.
     * 
     * @return the property name
     */
    public String getPropertyName() {
        return propertyName;
    }

    /**
     * Checks if is editable.
     * 
     * @return true, if is editable
     */
    public boolean isEditable() {
        return editable;
    }

    /**
     * Gets the display lines.
     * 
     * @return the display lines
     */
    public int getDisplayLines() {
        return displayLines;
    }

    /**
     * Gets the data type.
     * 
     * @return the data type
     */
    public String getDataType() {
        return dataType;
    }
}
