/**
 * 
 */
package gov.va.med.pharmacy.pecs.web.monograph.professional;

import gov.va.med.pharmacy.pecs.domain.impl.va.MonographKey;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.web.util.BaseRecordKeyHelper;

import org.apache.log4j.Logger;

/**
 * The Class MonographCheckRecordKeyHelper.
 * 
 * @author DOMAINOVERSD
 */
public class MonographCheckRecordKeyHelper {

    /** The Constant LOG. */
    private static final Logger LOG = Logger.getLogger(MonographCheckRecordKeyHelper.class);

    /** The Constant MONOGRAPH_ID. */
    private static final int MONOGRAPH_ID = 0;

    /** The Constant LANGUAGE_ID. */
    private static final int LANGUAGE_ID = 1;

    /** The Constant SEQUENCE_NUMBER. */
    private static final int SEQUENCE_NUMBER = 2;

    /** The Constant VERSION_CODE. */
    private static final int VERSION_CODE = 3;

    /** The monograph id. */
    private Long monographId;

    /** The language id. */
    private Long languageId;

    /** The sequence number. */
    private Long sequenceNumber;

    /** The version code. */
    private String versionCode;

    /**
     * Parses the record key.
     * 
     * example 12|6|0|29|064|01|999|1
     * 
     * @param recordKey
     *            the pipe delimited key
     */
    public MonographCheckRecordKeyHelper(String recordKey) {
        if (recordKey == null || recordKey.equals(""))
            throw new IllegalArgumentException("Record ID parameter is null or could not be determined from: " + recordKey);

        if (LOG.is   ugEnabled())
            LOG.debug("Parsing recordkey: " + recordKey);

        try {
            String[] result = recordKey.split("\\" + BaseRecordKeyHelper.WIZARD_RECORDID_DELIMITER);
            if (result.length > 0) {

                setMonographId(Long.parseLong(result[MONOGRAPH_ID]));
                setLanguageId(Long.parseLong(result[LANGUAGE_ID]));
                setSequenceNumber(Long.parseLong(result[SEQUENCE_NUMBER]));
                setVersionCode(result[VERSION_CODE]);
            }
        } catch (Exception ex) {
            LOG.error("Unable to parse record id: " + recordKey, ex);
            throw new CustomizationServiceException("Unable to parse record key", ex);
        }

    }

    /**
     * Creates the monograph key.
     * 
     * @return the monograph key
     */
    public MonographKey createMonographKey() {
        MonographKey key = new MonographKey();
        key.setLanguageId(getLanguageId());
        key.setMonographId(getMonographId());
        key.setSequenceNumber(getSequenceNumber());
        key.setVersionCode(getVersionCode());
        return key;
    }

    /**
     * Gets the monograph id.
     * 
     * @return the monographId
     */
    public Long getMonographId() {
        return monographId;
    }

    /**
     * Sets the monograph id.
     * 
     * @param monographId
     *            the monographId to set
     */
    public void setMonographId(Long monographId) {
        this.monographId = monographId;
    }

    /**
     * Gets the language id.
     * 
     * @return the languageId
     */
    public Long getLanguageId() {
        return languageId;
    }

    /**
     * Sets the language id.
     * 
     * @param languageId
     *            the languageId to set
     */
    public void setLanguageId(Long languageId) {
        this.languageId = languageId;
    }

    /**
     * Gets the sequence number.
     * 
     * @return the sequenceNumber
     */
    public Long getSequenceNumber() {
        return sequenceNumber;
    }

    /**
     * Sets the sequence number.
     * 
     * @param sequenceNumber
     *            the sequenceNumber to set
     */
    public void setSequenceNumber(Long sequenceNumber) {
        this.sequenceNumber = sequenceNumber;
    }

    /**
     * Gets the version code.
     * 
     * @return the versionCode
     */
    public String getVersionCode() {
        return versionCode;
    }

    /**
     * Sets the version code.
     * 
     * @param versionCode
     *            the versionCode to set
     */
    public void setVersionCode(String versionCode) {
        this.versionCode = versionCode;
    }
}
