package gov.va.med.pharmacy.pecs.web.queryresults;

import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.ConceptQueryFactory;
import gov.va.med.pharmacy.pecs.query.DoJoGridSuport;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.service.PecsService;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

/**
 * The Class QueryResultsController.
 */
@Controller
public class QueryResultsController implements InitializingBean {

    /** The factory. */
    @Resource(name = "jsonConceptQueryFactory")
    private ConceptQueryFactory factory;

    /** The ddi service. */
    @Resource
    private DrugDrugInteractionService ddiService;

    /** The dp service. */
    @Resource
    private DrugPairService dpService;

    /** The dt cat service. */
    @Resource
    private DuplicateTherapyService dtCatService;

    /** The dr service. */
    @Resource
    private DoseRangeService drService;

    /** The monograph service. */
    @Resource
    private MonographService monographService;

    /** The Constant FDB_PREFIX. */
    public static final String FDB_PREFIX = "FDB";

    /** The Constant VA_PREFIX. */
    public static final String VA_PREFIX = "VA";

    /** The Constant VIEW. */
    private static final String VIEW = "queryResultsViewer";

    /** The service map. */
    private Map<String, PecsService<?>> serviceMap;

    /** The Constant JSON_PAGING_INFO. */
    private static final String JSON_PAGING_INFO = "/[0:0]";

    /**
     * Execute query.
     * 
     * @param recordType
     *            the record type
     * @param concept
     *            the concept
     * @param json
     *            the json
     * @param history
     *            the history
     * @return the model and view
     */
    @RequestMapping(value = "/export/{recordType}/{concept}")
    public ModelAndView executeQuery(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "concept") String concept, @RequestParam(value = "json") String json, @RequestParam(
                    value = "history") boolean history) {
        boolean va = false;
        Map<String, Object> model = new HashMap<String, Object>();
        @SuppressWarnings("rawtypes")
        List items = null;

        if (VA_PREFIX.equals(recordType)) {
            va = true;
        } else if (FDB_PREFIX.equals(recordType)) {
            va = false;
        } else {
            throw new IllegalArgumentException("Invalid record type: " + recordType);
        }
        DoJoGridSuport support = new DoJoGridSuport(json + JSON_PAGING_INFO);

        if (support.isParsed()) {
            PecsService<?> service = getService(concept);
            ConceptQuery query = factory.createQuery(service.getMetadata(), support.getJson());
            query.setIncludeHistory(history);
            if (support.hasSorting()) {
                ConceptField field = service.getMetadata().getFieldByProperty(support.getSortColumn());
                query.setSortFields(Arrays.asList(new ConceptField[] { field }));
                query.setSortOrder(support.sortAsc());
            } else {
                ConceptField field = service.getMetadata().getField(PecsDTO.ACTION_DATE);
                query.setSortFields(Arrays.asList(new ConceptField[] { field }));
                query.setSortOrder(false);
            }
            if (va) {
                items = service.getStagingData(query);
            } else {
                items = service.getFdbData(query);
            }
            model.put(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY, concept.toUpperCase());
            model.put(AbstractXLSXViewer.CRITERIA_STRING, query.toText());
            model.put(recordType + " " + concept.toUpperCase(), items);
        }

        return new ModelAndView(VIEW, model);
    }

    /**
     * Gets the service.
     * 
     * @param concept
     *            the concept
     * @return the service
     */
    private PecsService<?> getService(String concept) {
        return serviceMap.get(concept);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (ddiService == null) {
            throw new Exception("The property drugDrugInteractionService in QueryResultsController has not been set.");
        }
        if (dpService == null) {
            throw new Exception("The property drugPairService in QueryResultsController has not been set.");
        }
        if (dtCatService == null) {
            throw new Exception("The property DtCatTherapyService in QueryResultsController has not been set.");
        }
        if (drService == null) {
            throw new Exception("The property DoseRangeService in QueryResultsController has not been set.");
        }
        if (monographService == null) {
            throw new Exception("The property MonographService in QueryResultsController has not been set.");
        }
        serviceMap = new HashMap<String, PecsService<?>>();
        serviceMap.put(Concept.DRUG_DRUG_INTERACTION.toString(), ddiService);
        serviceMap.put(Concept.DRUG_PAIR.toString(), dpService);
        serviceMap.put(Concept.DUPLICATE_THERAPY.toString(), dtCatService);
        serviceMap.put(Concept.DOSE_RANGE.toString(), drService);
        serviceMap.put(Concept.PROFESSIONAL_MONOGRAPH.toString(), monographService);
    }

}
