package gov.va.med.pharmacy.pecs.web.settings;


import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import gov.va.med.pharmacy.pecs.service.DeletedRoutedGenericService;
import gov.va.med.pharmacy.pecs.web.AbstractController;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

/**
 * A controller to handle the processing of the Routed Generics administration process.
 * 
 * @author vhaisaduffg
 *
 */
@Controller
@RequestMapping(value = "/routedGenericProcess")
public class RoutedGenericController extends AbstractController implements InitializingBean {

    @Resource
    private DeletedRoutedGenericService deletedRoutedGenericService;
    
    
    /**
     *Method to start the Routed Generics administration process.
     *
     * @param request the request
     * @return the model and view
     */
    @RequestMapping(method = RequestMethod.POST)
    public final ModelAndView process(HttpServletRequest request) {
        ModelAndView view = new ModelAndView(SettingsController.TILES_VIEW);
        try {
            deletedRoutedGenericService.process(getUserProfile(request));
            addInformationMessage("us.message.drugpairremoval.complete");
            view.addObject(CONFIGURABLE_CONCEPTS_ATTRIBUTE, populateConfigurableConcepts());
        } catch( Exception e ) {
            addWarningMessage("us.message.drugpairremoval.error");
        }
        return view;
    }
    
    /**
     * Sets the deletedRoutedGenericService.
     *
     * @param deletedRoutedGenericService the deletedRoutedGenericService to set
     */
    public final void setDeletedRoutedGenericService(DeletedRoutedGenericService deletedRoutedGenericService) {
        this.deletedRoutedGenericService = deletedRoutedGenericService;
    }
    
    
    /**
     * Allows the class to check at runtime to make sure all Spring beans have been injected.
     * 
     * @throws Exception if a bean has not been injected
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        super.afterPropertiesSet();
        if (deletedRoutedGenericService == null) {
            throw new Exception("The property DeletedRoutedGenericService has " 
                    + "not been injected in to RoutedGenericController.");
        }
    }
}
