package gov.va.med.pharmacy.pecs.web.updates;

import gov.va.med.pharmacy.pecs.customupdate.CustomUpdateFileVa;
import gov.va.med.pharmacy.pecs.service.CustomUpdateService;

import java.io.IOException;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.multiaction.MultiActionController;

/**
 * The Class FilesController.
 */
@Controller
@RequestMapping("/download")
public class FilesController extends MultiActionController implements InitializingBean {

    /** The logger. */
    private final Log logger = LogFactory.getLog(FilesController.class);
    /** The update file service. */
    @Resource
    private CustomUpdateService updateFileService;

    /** The Constant FILE_TYPE. */
    public final static String FILE_TYPE = "fileType";

    /** The Constant IDENTIFIER. */
    public final static String IDENTIFIER = "id";

    /** The content type. */
    private final String CONTENT_TYPE = "application/zip";

    /** The content disposition key. */
    private final String CONTENT_DISPOSITION_KEY = "Content-Disposition";

    /**
     * Download.
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @return the model and view
     * @throws Exception
     *             the exception
     */
    @RequestMapping(method = RequestMethod.GET)
    public ModelAndView download(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String type = ServletRequestUtils.getRequiredStringParameter(request, FILE_TYPE);
        long id = ServletRequestUtils.getRequiredLongParameter(request, IDENTIFIER);

        if (type != null && type.equals("ctVersion")) {
            try {
                CustomUpdateFileVa updateFile = updateFileService.findUpdateFile(id);
                copyByteArrayToResponse(response, updateFile.getZipFileName(), updateFile.getZipFileContent());
            } catch (NullPointerException e) {
                logger.error("The Custom Update File with id: " + id + "you are trying to download does not exist", e);
                String errorMessage =
                        "The Custom Update File you are trying to download does not exist please select another file.";
                ModelAndView model = new ModelAndView("updates");
                model.addObject("errorMessage", errorMessage);
                return model;
            }
        }

        return null;
    }

    /**
     * Copy byte array to response.
     * 
     * @param response
     *            the response
     * @param fileName
     *            the file name
     * @param file
     *            the file
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    private void copyByteArrayToResponse(HttpServletResponse response, String fileName, byte[] file) throws IOException {
        response.setContentType(CONTENT_TYPE);
        response.setContentLength(file.length);
        response.setHeader(CONTENT_DISPOSITION_KEY, "attachment;filename=\"" + fileName + "\"");
        FileCopyUtils.copy(file, response.getOutputStream());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (updateFileService == null) {
            throw new Exception("The property UpdateFileService in UpdateController has not been set.");
        }
    }

}
