package gov.va.med.pharmacy.pecs.web.view;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.web.queryresults.QueryResultsController;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.web.servlet.view.document.AbstractExcelView;

/**
 * The Class QueryResultsExcelViewer.
 */
public class QueryResultsExcelViewer extends AbstractExcelView implements InitializingBean {

    /** The Constant logger. */
    private static final Log logger = LogFactory.getLog(QueryResultsExcelViewer.class);

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.web.servlet.view.document.AbstractExcelView#buildExcelDocument(java.util.Map,
     * org.apache.poi.hssf.usermodel.HSSFWorkbook, javax.servlet.http.HttpServletRequest,
     * javax.servlet.http.HttpServletResponse)
     */
    @Override
    protected void buildExcelDocument(Map<String, Object> model, HSSFWorkbook workbook, HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        CreationHelper createHelper = workbook.getCreationHelper();
        CellStyle cellStyle = workbook.createCellStyle();
        cellStyle.setDataFormat(createHelper.createDataFormat().getFormat("yyyy-MM-dd HH:mm:ss"));
        HSSFFont headerFont = workbook.createFont();
        headerFont.setBoldweight((short) 2);
        HSSFCellStyle headerStyle = workbook.createCellStyle();
        headerStyle.setFont(headerFont);

        if (model.containsKey(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY)) {
            String fileName = (String) model.get(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY);
            response.addHeader("Content-Disposition", "attachment; filename=\"" + fileName + ".xls" + "\"");
            model.remove(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY);
        }
        Iterator<Entry<String, Object>> iter = model.entrySet().iterator();
        while (iter.hasNext()) {
            Entry<String, Object> entry = iter.next();

            createSheet(workbook, headerStyle, cellStyle, entry);
        }
    }

    /**
     * Creates the sheet.
     * 
     * @param workbook
     *            the workbook
     * @param headerStyle
     *            the header style
     * @param cellStyle
     *            the cell style
     * @param entry
     *            the entry
     */
    private void createSheet(HSSFWorkbook workbook, CellStyle headerStyle, CellStyle cellStyle, Entry<String, Object> entry) {
        @SuppressWarnings({ "rawtypes" })
        List data = (List) entry.getValue();

        Sheet sheet = workbook.createSheet(entry.getKey());
        int rowNum = 0;
        Row row = sheet.createRow(rowNum++);
        int colNum = 0;
        boolean custom = true;
        if (entry.getKey().startsWith(QueryResultsController.FDB_PREFIX)) {
            custom = false;
        }

        if (logger.is   ugEnabled()) {
            logger.debug("Trying to write " + data.size() + " rows to spreadsheet.");
        }
        try {
            // header
            ConceptMetadata meta = metadataService.getMetadata(Class.forName(data.get(0).getClass().getName()));

            for (ConceptField f : meta.getQueryFields(custom)) {
                Cell cell = row.createCell(colNum++);
                cell.setCellValue(f.getDisplayName());
                cell.setCellStyle(headerStyle);
            }
            sheet.createFreezePane(0, 1, 0, 1);

            for (Object d : data) {
                row = sheet.createRow(rowNum++);
                colNum = 0;
                for (ConceptField f : meta.getQueryFields(custom)) {
                    Cell cell = row.createCell(colNum++);
                    setCellValue(cell, f.getValue(d), cellStyle);
                }
                if (logger.is   ugEnabled() && (rowNum % 1000) == 0) {
                    logger.debug("Placed " + rowNum + " rows in the spreadsheet.");
                }
            }

            // Size columns - This has to be done after the data is populated.
            int column = 0;
            int numberOfColumns = meta.getQueryFields(custom).size();
            for (int i = 0; i < numberOfColumns; i++) {
                sheet.autoSizeColumn(column++);
            }
        } catch (ClassNotFoundException cnfe) {
            // TODO What to do what to do.
        }
    }

    /**
     * Sets the cell value.
     * 
     * @param cell
     *            the cell
     * @param val
     *            the val
     * @param dateCellStyle
     *            the date cell style
     */
    private void setCellValue(Cell cell, Object val, CellStyle dateCellStyle) {
        if (val == null) {
            cell.setCellValue("");
        } else if (val instanceof Date) {
            cell.setCellValue((Date) val);
            cell.setCellStyle(dateCellStyle);
        } else if (val instanceof Number) {
            cell.setCellValue(((Number) val).doubleValue());
        } else {
            cell.setCellValue(val.toString());
        }
    }

    /**
     * Sets the metadata service.
     * 
     * @param service
     *            the new metadata service
     */
    public void setMetadataService(MetadataService service) {
        metadataService = service;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (metadataService == null) {
            throw new Exception("MetadataService has not been injected in to ExcelViewer.");
        }
    }

}
